@@ -30,9 +30,17 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 '''
-import inkex, simplestyle, simpletransform, cubicsuperpath, coloreffect, dxf_templates, math
-import gettext
-_ = gettext.gettext
+# standard library
+import math
+# local library
+import inkex
+import simplestyle
+import simpletransform
+import cubicsuperpath
+import coloreffect
+import dxf_templates
+
+inkex.localize()
 
 try:
     from numpy import *
@@ -60,16 +68,33 @@ def get_matrix(u, i, j):
 class MyEffect(inkex.Effect):
     def __init__(self):
         inkex.Effect.__init__(self)
-        self.OptionParser.add_option("-R", "--ROBO", action="store", type="string", dest="ROBO")
-        self.OptionParser.add_option("-P", "--POLY", action="store", type="string", dest="POLY")
-        self.OptionParser.add_option("--units", action="store", type="string", dest="units")
-        self.OptionParser.add_option("--encoding", action="store", type="string", dest="char_encode", default="latin_1")
-        self.OptionParser.add_option("--tab", action="store", type="string", dest="tab")
-        self.OptionParser.add_option("--inputhelp", action="store", type="string", dest="inputhelp")
+        self.OptionParser.add_option("-R", "--ROBO", action="store",
+                                     type="string", dest="ROBO",
+                                     default=False)
+        self.OptionParser.add_option("-P", "--POLY", action="store",
+                                     type="string", dest="POLY",
+                                     default=True)
+        self.OptionParser.add_option("--units", action="store",
+                                     type="string", dest="units",
+                                     default="72./90") # Points
+        self.OptionParser.add_option("--encoding", action="store",
+                                     type="string", dest="char_encode",
+                                     default="latin_1")
+        self.OptionParser.add_option("--tab", action="store",
+                                     type="string", dest="tab")
+        self.OptionParser.add_option("--inputhelp", action="store",
+                                     type="string", dest="inputhelp")
+        self.OptionParser.add_option("--layer_option", action="store",
+                                     type="string", dest="layer_option",
+                                     default="all")
+        self.OptionParser.add_option("--layer_name", action="store",
+                                     type="string", dest="layer_name")
+                                     
         self.dxf = []
         self.handle = 255                       # handle for DXF ENTITY
         self.layers = ['0']
         self.layer = '0'                        # mandatory layer
+        self.layernames = []
         self.csp_old = [[0.0,0.0]]*4            # previous spline
         self.d = array([0], float)              # knot vector
         self.poly = [[0.0,0.0]]                 # LWPOLYLINE data
@@ -243,7 +268,16 @@ class MyEffect(inkex.Effect):
 
     def process_group(self, group):
         if group.get(inkex.addNS('groupmode', 'inkscape')) == 'layer':
+            style = group.get('style')
+            if style:
+                style = simplestyle.parseStyle(style)
+                if style.has_key('display'):
+                    if style['display'] == 'none' and self.options.layer_option and self.options.layer_option=='visible':
+                        return
             layer = group.get(inkex.addNS('label', 'inkscape'))
+            if self.options.layer_name and self.options.layer_option and self.options.layer_option=='name' and not layer.lower() in self.options.layer_name:
+                return
+              
             layer = layer.replace(' ', '_')
             if layer in self.layers:
                 self.layer = layer
@@ -261,6 +295,14 @@ class MyEffect(inkex.Effect):
             self.groupmat.pop()
 
     def effect(self):
+        #Warn user if name match field is empty
+        if self.options.layer_option and self.options.layer_option=='name' and not self.options.layer_name:
+            inkex.errormsg(_("Error: Field 'Layer match name' must be filled when using 'By name match' option"))
+            inkex.sys.exit()
+        #Split user layer data into a list: "layerA,layerb,LAYERC" becomes ["layera", "layerb", "layerc"]
+        if self.options.layer_name:
+            self.options.layer_name = self.options.layer_name.lower().split(',')
+			
         #References:   Minimum Requirements for Creating a DXF File of a 3D Model By Paul Bourke
         #              NURB Curves: A Guide for the Uninitiated By Philip J. Schneider
         #              The NURBS Book By Les Piegl and Wayne Tiller (Springer, 1995)
@@ -269,6 +311,9 @@ class MyEffect(inkex.Effect):
         for node in self.document.getroot().xpath('//svg:g', namespaces=inkex.NSS):
             if node.get(inkex.addNS('groupmode', 'inkscape')) == 'layer':
                 layer = node.get(inkex.addNS('label', 'inkscape'))
+                self.layernames.append(layer.lower())
+                if self.options.layer_name and self.options.layer_option and self.options.layer_option=='name' and not layer.lower() in self.options.layer_name:
+                    continue
                 layer = layer.replace(' ', '_')
                 if layer and not layer in self.layers:
                     self.layers.append(layer)
@@ -280,7 +325,7 @@ class MyEffect(inkex.Effect):
         scale = eval(self.options.units)
         if not scale:
             scale = 25.4/90     # if no scale is specified, assume inch as baseunit
-        h = inkex.unittouu(self.document.getroot().xpath('@height', namespaces=inkex.NSS)[0])
+        h = self.unittouu(self.document.getroot().xpath('@height', namespaces=inkex.NSS)[0])
         self.groupmat = [[[scale, 0.0, 0.0], [0.0, -scale, h*scale]]]
         doc = self.document.getroot()
         self.process_group(doc)
@@ -289,6 +334,11 @@ class MyEffect(inkex.Effect):
         if self.options.POLY == 'true':
             self.LWPOLY_output()
         self.dxf_add(dxf_templates.r14_footer)
+		#Warn user if layer data seems wrong
+        if self.options.layer_name and self.options.layer_option and self.options.layer_option=='name':
+            for layer in self.options.layer_name:
+                if not layer in self.layernames:
+                    inkex.errormsg(_("Warning: Layer '%s' not found!") % (layer))
 
 if __name__ == '__main__':
     e = MyEffect()
