@@ -1,7 +1,8 @@
 #!/usr/bin/env python 
 '''
 Copyright (C) 2006 Aaron Spike, aaron@ekips.org
-Copyright (C) 2010-2011 Nicolas Dufour, nicoduf@yahoo.fr (Windows support and various fixes)
+Copyright (C) 2010-2012 Nicolas Dufour, nicoduf@yahoo.fr
+(Windows support and various fixes)
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,27 +18,53 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 '''
+# standard library
+import os
+import shutil
+from subprocess import Popen, PIPE
+import sys
+import tempfile
+# local library
 import inkex
-import sys, os, tempfile, shutil
-import gettext
 
-try:
-    from subprocess import Popen, PIPE
-    bsubprocess = True
-except:
-    bsubprocess = False
+inkex.localize()
+
+# Define extension exceptions
+class GimpXCFError(Exception): pass
+
+class GimpXCFExpectedIOError(GimpXCFError): pass
+
+class GimpXCFGimpNotInstalled(GimpXCFError):
+    def __init__(self):
+        inkex.errormsg(_('Gimp must be installed and set in your path variable.'))
+
+class GimpXCFScriptFuError(GimpXCFError):
+    def __init__(self):
+        inkex.errormsg(_('An error occurred while processing the XCF file.'))
+
 
 class MyEffect(inkex.Effect):
     def __init__(self):
         inkex.Effect.__init__(self)
+        self.OptionParser.add_option("--tab",
+                                     action="store", type="string",
+                                     dest="tab")
         self.OptionParser.add_option("-d", "--guides",
-                                         action="store", type="inkbool",
-                                         dest="saveGuides", default=False,
-                                         help="Save the Guides with the .XCF")
+                                     action="store", type="inkbool",
+                                     dest="saveGuides", default=False,
+                                     help="Save the Guides with the .XCF")
         self.OptionParser.add_option("-r", "--grid",
-                                         action="store", type="inkbool",
-                                         dest="saveGrid", default=False,
-                                         help="Save the Grid with the .XCF")
+                                     action="store", type="inkbool",
+                                     dest="saveGrid", default=False,
+                                     help="Save the Grid with the .XCF")
+        self.OptionParser.add_option("-b", "--background",
+                                     action="store", type="inkbool",
+                                     dest="layerBackground", default=False,
+                                     help="Add background color to each layer")
+        self.OptionParser.add_option("-i", "--dpi",
+                                     action="store", type="string",
+                                     dest="resolution", default="90",
+                                     help="File resolution")
 
     def output(self):
         pass
@@ -51,31 +78,31 @@ class MyEffect(inkex.Effect):
         docname = ttmp_orig.get(inkex.addNS('docname',u'sodipodi'))
         if docname is None: docname = self.args[-1]
 
-        pageHeight = int(inkex.unittouu(self.xpathSingle('/svg:svg/@height').split('.')[0]))
-        pageWidth = int(inkex.unittouu(self.xpathSingle('/svg:svg/@width').split('.')[0]))
+        pageHeight = int(self.unittouu(self.xpathSingle('/svg:svg/@height').split('.')[0]))
+        pageWidth = int(self.unittouu(self.xpathSingle('/svg:svg/@width').split('.')[0]))
 
-        #create os temp dir
+        # Create os temp dir (to store exported pngs and Gimp log file)
         self.tmp_dir = tempfile.mkdtemp()
 
         # Guides
         hGuides = []
         vGuides = []
-        valid = 0
         if self.options.saveGuides:
-            guideXpath = "sodipodi:namedview/sodipodi:guide" #grab all guide tags in the namedview tag
+            # Grab all guide tags in the namedview tag
+            guideXpath = "sodipodi:namedview/sodipodi:guide"
             for guideNode in self.document.xpath(guideXpath, namespaces=inkex.NSS):
                 ori = guideNode.get('orientation')
                 if  ori == '0,1':
-                    #this is a horizontal guide
+                    # This is a horizontal guide
                     pos = int(guideNode.get('position').split(',')[1].split('.')[0])
-                    #GIMP doesn't like guides that are outside of the image
+                    # GIMP doesn't like guides that are outside of the image
                     if pos > 0 and pos < pageHeight:
-                        #the origin is at the top in GIMP land
+                        # The origin is at the top in GIMP land
                         hGuides.append(str(pageHeight - pos))
                 elif ori == '1,0':
-                    #this is a vertical guide
+                    # This is a vertical guide
                     pos = int(guideNode.get('position').split(',')[0].split('.')[0])
-                    #GIMP doesn't like guides that are outside of the image
+                    # GIMP doesn't like guides that are outside of the image
                     if pos > 0 and pos < pageWidth:
                         vGuides.append(str(pos))
 
@@ -85,12 +112,12 @@ class MyEffect(inkex.Effect):
         # Grid
         gridSpacingFunc = ''
         gridOriginFunc = '' 
-        #GIMP only allows one rectangular grid
+        # GIMP only allows one rectangular grid
         gridXpath = "sodipodi:namedview/inkscape:grid[@type='xygrid' and (not(@units) or @units='px')]"
         if (self.options.saveGrid and self.document.xpath(gridXpath, namespaces=inkex.NSS)):
             gridNode = self.xpathSingle(gridXpath)
             if gridNode != None:
-                #these attributes could be nonexistant
+                # These attributes could be nonexistant
                 spacingX = gridNode.get('spacingx')
                 if spacingX == None: spacingX = '1  '
 
@@ -108,45 +135,53 @@ class MyEffect(inkex.Effect):
 
         # Layers
         area = '--export-area-page'
+        opacity = '--export-background-opacity='
+        resolution = '--export-dpi=' + self.options.resolution
+        
+        if self.options.layerBackground:
+            opacity += "1"
+        else:
+            opacity += "0"
         pngs = []
         names = []
+        self.valid = 0
         path = "/svg:svg/*[name()='g' or @style][@id]"
         for node in self.document.xpath(path, namespaces=inkex.NSS):
             if len(node) > 0: # Get rid of empty layers
-                valid=1
+                self.valid = 1
                 id = node.get('id')
                 if node.get("{" + inkex.NSS["inkscape"] + "}label"):
                     name = node.get("{" + inkex.NSS["inkscape"] + "}label")
                 else:
                     name = id
                 filename = os.path.join(self.tmp_dir, "%s.png" % id)
-                command = "inkscape -i \"%s\" -j %s -e \"%s\" %s " % (id, area, filename, svg_file)
-                if bsubprocess:
-                    p = Popen(command, shell=True, stdout=PIPE, stderr=PIPE)
-                    return_code = p.wait()
-                    f = p.stdout
-                    err = p.stderr
-                else:
-                    _,f,err = os.popen3(command,'r')
+                command = "inkscape -i \"%s\" -j %s %s -e \"%s\" %s %s" % (id, area, opacity, filename, svg_file, resolution)
+               
+                p = Popen(command, shell=True, stdin=PIPE, stdout=PIPE, stderr=PIPE)
+                return_code = p.wait()
+                f = p.stdout
+                err = p.stderr
+                stdin = p.stdin
                 f.read()
                 f.close()
                 err.close()
+                stdin.close()
+                
                 if os.name == 'nt':
                     filename = filename.replace("\\", "/")
                 pngs.append(filename)
                 names.append(name.encode('utf-8'))
 
-        if (valid==0):
-            inkex.errormsg(gettext.gettext('This extension requires at least one non empty layer.'))
+        if (self.valid == 0):
             self.clear_tmp()
-            sys.exit(0)
-
-        filelist = '"%s"' % '" "'.join(pngs)
-        namelist = '"%s"' % '" "'.join(names)
-        xcf = os.path.join(self.tmp_dir, "%s.xcf" % docname)
-        if os.name == 'nt':
-            xcf = xcf.replace("\\", "/")
-        script_fu = """
+            inkex.errormsg(_('This extension requires at least one non empty layer.'))
+        else:
+            filelist = '"%s"' % '" "'.join(pngs)
+            namelist = '"%s"' % '" "'.join(names)
+            xcf = os.path.join(self.tmp_dir, "%s.xcf" % docname)
+            if os.name == 'nt':
+                xcf = xcf.replace("\\", "/")
+            script_fu = """
 (tracing 1)
 (define
   (png-to-layer img png_filename layer_name)
@@ -164,6 +199,7 @@ class MyEffect(inkex.Effect):
   (
     (img (car (gimp-image-new 200 200 RGB)))
   )
+  (gimp-image-set-resolution img %s %s)
   (gimp-image-undo-disable img)
   (for-each
     (lambda (names)
@@ -194,27 +230,47 @@ class MyEffect(inkex.Effect):
   (gimp-image-undo-enable img)
   (gimp-file-save RUN-NONINTERACTIVE img (car (gimp-image-get-active-layer img)) "%s" "%s"))
 (gimp-quit 0)
-        """ % (filelist, namelist, hGList, vGList, gridSpacingFunc, gridOriginFunc, xcf, xcf)
+            """ % (self.options.resolution, self.options.resolution, filelist, namelist, hGList, vGList, gridSpacingFunc, gridOriginFunc, xcf, xcf)
 
-        junk = os.path.join(self.tmp_dir, 'junk_from_gimp.txt')
-        f = os.popen('gimp -i --batch-interpreter plug-in-script-fu-eval -b - > %s 2>&1' % junk,'w')
-        f.write(script_fu)
-        f.close()
-        # uncomment these lines to see the output from gimp
-        #err = open(junk, 'r')
-        #inkex.debug(err.read())
-        #err.close()
+            junk = os.path.join(self.tmp_dir, 'junk_from_gimp.txt')
+            command = 'gimp -i --batch-interpreter plug-in-script-fu-eval -b - > %s 2>&1' % junk
+
+            p = Popen(command, shell=True, stdin=PIPE, stdout=PIPE, stderr=PIPE)
+            f = p.stdin
+            out = p.stdout
+            err = p.stderr
+            f.write(script_fu.encode('utf-8'))
+            return_code = p.wait()
+            
+            if p.returncode != 0:
+                self.clear_tmp()
+                raise GimpXCFGimpNotInstalled
+
+            f.close()
+            err.close()
+            out.close()
+            # Uncomment these lines to see the output from gimp
+            #err = open(junk, 'r')
+            #inkex.debug(err.read())
+            #err.close()
 
-        x = open(xcf, 'rb')
-        if os.name == 'nt':
             try:
-                import msvcrt
-                msvcrt.setmode(1, os.O_BINARY)
+                x = open(xcf, 'rb')
             except:
-                pass
-        sys.stdout.write(x.read())
-        x.close()
-        self.clear_tmp()
+                self.clear_tmp()
+                raise GimpXCFScriptFuError
+
+            if os.name == 'nt':
+                try:
+                    import msvcrt
+                    msvcrt.setmode(1, os.O_BINARY)
+                except:
+                    pass
+            try:
+                sys.stdout.write(x.read())
+            finally:
+                x.close()
+                self.clear_tmp()
 
 
 if __name__ == '__main__':
