@@ -1,6 +1,11 @@
 #!/usr/bin/env python
-import webbrowser, threading
+# standard library
+import webbrowser
+import threading
 from optparse import OptionParser
+# local library
+import inkex
+inkex.localize()
 
 class VisitWebSiteWithoutLockingInkscape(threading.Thread):
     def __init__(self):
@@ -12,10 +17,10 @@ class VisitWebSiteWithoutLockingInkscape(threading.Thread):
         (self.options, args) = parser.parse_args()
 
     def run(self):
-        webbrowser.open(self.options.url)
+        webbrowser.open(_(self.options.url))
 
 vwswli = VisitWebSiteWithoutLockingInkscape()
 vwswli.start()
 
 
-# vim: expandtab shiftwidth=4 tabstop=8 softtabstop=4 encoding=utf-8 textwidth=99
+# vim: expandtab shiftwidth=4 tabstop=8 softtabstop=4 fileencoding=utf-8 textwidth=99
