@@ -19,7 +19,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 import inkex, simplestyle, pturtle, random
 
 def stripme(s):
-    return s.strip()	
+    return s.strip()
 
 class LSystem(inkex.Effect):
     def __init__(self):
@@ -56,6 +56,9 @@ class LSystem(inkex.Effect):
                         action="store", type="string", 
                         dest="rules", default="F=FF-[-F+F+F]+[+F-F-F]",
                         help="replacement rules")
+        self.OptionParser.add_option("-t", "--tab",
+                        action="store", type="string",
+                        dest="tab")
         self.stack = []
         self.turtle = pturtle.pTurtle()
     def iterate(self):
@@ -101,15 +104,17 @@ class LSystem(inkex.Effect):
         return level_string
             
     def effect(self):
-        new = self.document.createElement('svg:path')
-        s = {'stroke-linejoin': 'miter', 'stroke-width': '1.0px', 
+        self.options.step = self.unittouu(str(self.options.step) + 'px')
+        s = {'stroke-linejoin': 'miter', 'stroke-width': str(self.unittouu('1px')), 
             'stroke-opacity': '1.0', 'fill-opacity': '1.0', 
             'stroke': '#000000', 'stroke-linecap': 'butt', 
             'fill': 'none'}
-        new.setAttribute('style', simplestyle.formatStyle(s))
-        new.setAttribute('d', self.iterate())
-        self.current_layer.appendChild(new)
+        attribs = {'style':simplestyle.formatStyle(s),'d':self.iterate()}
+        inkex.etree.SubElement(self.current_layer,inkex.addNS('path','svg'),attribs)
 
-e = LSystem()
-e.affect()
+if __name__ == '__main__':
+    e = LSystem()
+    e.affect()
 
+
+# vim: expandtab shiftwidth=4 tabstop=8 softtabstop=4 fileencoding=utf-8 textwidth=99
