@@ -18,10 +18,10 @@ class SVG_and_Media_ZIP_Output_BasicTest(unittest.TestCase):
 
   def test_run_without_parameters(self):
     args = [ 'minimal-blank.svg' ]
-    e = SVG_and_Media_ZIP_Output()
+    e = CompressedMediaOutput()
     e.affect( args, False )
     #self.assertEqual( e.something, 'some value', 'A commentary about that.' )
-    e.clear_tmp()
+    #e.clear_tmp()
 
 if __name__ == '__main__':
   unittest.main()
