@@ -5,9 +5,9 @@
 
 namespace Geom {
 
-Region Region::operator*(Matrix const &m) const {
+Region Region::operator*(Affine const &m) const {
     Region r((m.flips() ? boundary.reverse() : boundary) * m, fill);
-    if(box && m.onlyScaleAndTranslation()) r.box = (*box) * m;
+    if(box && m.isZoom()) r.box = (*box) * m;
     return r;
 }
 
@@ -42,4 +42,4 @@ unsigned outer_index(Regions const &ps) {
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
