@@ -68,20 +68,16 @@ find_parametric_bezier_roots(Geom::Point const *w, /* The control points  */
         break;
     }
 
-    /*
-     * Otherwise, solve recursively after subdividing control polygon
-     * New left and right control polygons
-     */
-    Geom::Point *Left  = new Geom::Point[degree+1];
-    Geom::Point *Right = new Geom::Point[degree+1];
+    /* Otherwise, solve recursively after subdividing control polygon  */
 
-    Bezier(w, degree, 0.5, &Left[0], &Right[0]);
+    //Geom::Point Left[degree+1],	/* New left and right  */
+    //    Right[degree+1];	/* control polygons  */
+    std::vector<Geom::Point> Left( degree+1 ), Right(degree+1);
+
+    Bezier(w, degree, 0.5, Left.data(), Right.data());
     total_subs ++;
-    find_parametric_bezier_roots(&Left[0],  degree, solutions, depth + 1);
-    find_parametric_bezier_roots(&Right[0], degree, solutions, depth + 1);
-
-    delete[] Left;
-    delete[] Right;
+    find_parametric_bezier_roots(Left.data(),  degree, solutions, depth+1);
+    find_parametric_bezier_roots(Right.data(), degree, solutions, depth+1);
 }
 
 
@@ -130,7 +126,8 @@ control_poly_flat_enough(Geom::Point const *V, /* Control points	*/
 
     const double abSquared = (a * a) + (b * b);
 
-    double distance[degree]; /* Distances from pts to line */
+    //double distance[degree]; /* Distances from pts to line */
+    std::vector<double> distance(degree); /* Distances from pts to line */
     for (unsigned i = 1; i < degree; i++) {
         /* Compute distance from each of the points to that line */
         double & dist(distance[i-1]);
@@ -198,13 +195,13 @@ Bezier(Geom::Point const *V, /* Control pts	*/
        Geom::Point *Left,	/* RETURN left half ctl pts */
        Geom::Point *Right)	/* RETURN right half ctl pts */
 {
-    Geom::Point **Vtemp = new Geom::Point* [degree+1];
-
-    for (unsigned int i = 0; i < degree+1; ++i)
-        Vtemp[i] = new Geom::Point[degree+1];
+    //Geom::Point Vtemp[degree+1][degree+1];
+    std::vector<std::vector<Geom::Point> > Vtemp(degree+1);
+    for ( size_t i = 0; i < degree + 1; ++i )
+        Vtemp.reserve(degree+1);
 
     /* Copy control points	*/
-    std::copy(V, V+degree+1, Vtemp[0]);
+    std::copy(V, V+degree+1, Vtemp[0].begin());
 
     /* Triangle computation	*/
     for (unsigned i = 1; i <= degree; i++) {	
@@ -218,14 +215,7 @@ Bezier(Geom::Point const *V, /* Control pts	*/
     for (unsigned j = 0; j <= degree; j++)
         Right[j] = Vtemp[degree-j][j];
 
-    Geom::Point return_value = Vtemp[degree][0];
-
-    for (unsigned int i = 0; i < degree+1; ++i)
-        delete[] Vtemp[i];
-
-    delete[] Vtemp;
-
-    return return_value;
+    return (Vtemp[degree][0]);
 }
 
 };
@@ -239,4 +229,4 @@ Bezier(Geom::Point const *V, /* Control pts	*/
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
