@@ -150,6 +150,7 @@ struct {char const *attr; bool supported;} const all_attrs[] = {
     {"in", true},
     {"in2", true},
     {"intercept", true},
+    {"isolation", true},
     {"k", true},
     {"k1", true},
     {"k2", true},
@@ -181,6 +182,7 @@ struct {char const *attr; bool supported;} const all_attrs[] = {
     {"media", false},
     {"method", false},
     {"min", true},
+    {"mix-blend-mode", true},
     {"mode", true},
     {"name", true},
     {"numOctaves", true},
@@ -213,8 +215,9 @@ struct {char const *attr; bool supported;} const all_attrs[] = {
     {"overflow", true},
     {"overline-position", true},
     {"overline-thickness", true},
+    {"paint-order", true},
     {"panose-1", true},
-    {"path", false},
+    {"path", true},
     {"pathLength", false},
     {"patternContentUnits", true},
     {"patternTransform", true},
@@ -276,6 +279,9 @@ struct {char const *attr; bool supported;} const all_attrs[] = {
     {"text-align", true},
     {"text-anchor", true},
     {"text-decoration", true},
+    {"text-decoration-line", true},
+    {"text-decoration-style", true},
+    {"text-decoration-color", true},
     {"text-indent", true},
     {"text-rendering", true},
     {"text-transform", true},
@@ -342,6 +348,10 @@ struct {char const *attr; bool supported;} const all_attrs[] = {
     {"sodipodi:version", false},
     {"inkscape:version", true},
     {"inkscape:object-paths", true},
+    {"inkscape:snap-perpendicular", true},
+    {"inkscape:snap-tangential", true},
+    {"inkscape:snap-path-clip", true},
+    {"inkscape:snap-path-mask", true},
     {"inkscape:object-nodes", true},
     {"inkscape:bbox-paths", true},
     {"inkscape:bbox-nodes", true},
@@ -349,14 +359,14 @@ struct {char const *attr; bool supported;} const all_attrs[] = {
     {"inkscape:snap-global", true},
     {"inkscape:snap-bbox", true},
     {"inkscape:snap-nodes", true},
-    {"inkscape:snap-from-guide", true},
+    {"inkscape:snap-others", true},
     {"inkscape:snap-center", true},
     {"inkscape:snap-smooth-nodes", true},
     {"inkscape:snap-midpoints", true},
     {"inkscape:snap-object-midpoints", true},
+    {"inkscape:snap-text-baseline", true},
     {"inkscape:snap-bbox-edge-midpoints", true},
     {"inkscape:snap-bbox-midpoints", true},
-    //{"inkscape:snap-intersection-grid-guide", true},
     {"inkscape:snap-grids", true},
     {"inkscape:snap-to-guides", true},
     {"inkscape:snap-intersection-paths", true},
@@ -379,6 +389,8 @@ struct {char const *attr; bool supported;} const all_attrs[] = {
     {"inkscape:connection-start", true},
     {"inkscape:connection-end", true},
     {"inkscape:connection-points", true},
+    {"inkscape:connection-start-point", true},
+    {"inkscape:connection-end-point", true},
     {"inkscape:connector-curvature", true},
     {"inkscape:connector-avoid", true},
     {"inkscape:connector-spacing", true},
@@ -425,8 +437,17 @@ struct {char const *attr; bool supported;} const all_attrs[] = {
     {"inkscape:dstColumn", true},
     {"inkscape:excludeShape", true},
     {"inkscape:layoutOptions", true},
+    {"osb:paint", true},
+
+    /* SPMeshPatch */
+    {"tensor", true},
 
     /* SPNamedView */
+    {"fit-margin-top", true},
+    {"fit-margin-left", true},
+    {"fit-margin-right", true},
+    {"fit-margin-bottom", true},
+    {"units", true},
     {"viewonly", true},
     {"showgrid", true},
 //    {"gridtype", true},
@@ -540,4 +561,4 @@ struct {char const *attr; bool supported;} const all_attrs[] = {
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
