@@ -1,6 +1,4 @@
-#define __SP_COLOR_C__
-
-/** \file
+/*
  * Colors.
  *
  * Author:
@@ -63,6 +61,10 @@ SPColor::~SPColor()
 
 SPColor& SPColor::operator= (SPColor const& other)
 {
+    if (this == &other){
+        return *this;
+    }
+    
     SVGICCColor* tmp_icc = other.icc ? new SVGICCColor(*other.icc) : 0;
 
     v.c[0] = other.v.c[0];
@@ -82,9 +84,10 @@ SPColor& SPColor::operator= (SPColor const& other)
  */
 bool SPColor::operator == (SPColor const& other) const
 {
-    bool match = (v.c[0] != other.v.c[0])
-        && (v.c[1] != other.v.c[1])
-        && (v.c[2] != other.v.c[2]);
+    bool match =
+        (v.c[0] == other.v.c[0]) &&
+        (v.c[1] == other.v.c[1]) &&
+        (v.c[2] == other.v.c[2]);
 
     match &= profileMatches( icc, other.icc );
 
@@ -97,9 +100,7 @@ bool SPColor::operator == (SPColor const& other) const
  */
 bool SPColor::isClose( SPColor const& other, float epsilon ) const
 {
-    bool match = false;
-
-    match = (fabs((v.c[0]) - (other.v.c[0])) < epsilon)
+    bool match = (fabs((v.c[0]) - (other.v.c[0])) < epsilon)
         && (fabs((v.c[1]) - (other.v.c[1])) < epsilon)
         && (fabs((v.c[2]) - (other.v.c[2])) < epsilon);
 
@@ -185,8 +186,6 @@ guint32 SPColor::toRGBA32( gdouble alpha ) const
 std::string SPColor::toString() const
 {
     CSSOStringStream css;
-
-    std::string result;
     char tmp[64] = {0};
 
     sp_svg_write_color(tmp, sizeof(tmp), toRGBA32(0x0ff));
@@ -352,7 +351,7 @@ sp_color_rgb_to_hsl_floatv (float *hsl, float r, float g, float b)
     }
 }
 
-float
+static float
 hue_2_rgb (float v1, float v2, float h)
 {
     if (h < 0) h += 6.0;
@@ -452,4 +451,4 @@ sp_color_cmyk_to_rgb_floatv (float *rgb, float c, float m, float y, float k)
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
