@@ -1,9 +1,7 @@
-#ifndef __SP_COLOR_H__
-#define __SP_COLOR_H__
+#ifndef SEEN_SP_COLOR_H
+#define SEEN_SP_COLOR_H
 
-/** \file
- * Colors.
- *
+/*
  * Author:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *   bulia byak <buliabyak@users.sf.net>
@@ -48,6 +46,7 @@ struct SPColor {
     SPColor& operator= (SPColor const& other);
 
     bool operator == ( SPColor const& other ) const;
+    bool operator != ( SPColor const& other ) const { return !(*this == other); };
     bool isClose( SPColor const& other, float epsilon ) const;
 
     void set( float r, float g, float b );
@@ -82,5 +81,4 @@ void sp_color_rgb_to_cmyk_floatv (float *cmyk, float r, float g, float b);
 void sp_color_cmyk_to_rgb_floatv (float *rgb, float c, float m, float y, float k);
 
 
-#endif
-
+#endif // SEEN_SP_COLOR_H
