@@ -14,8 +14,18 @@
 #include <gdk/gdk.h>
 #include <2geom/forward.h>
 
-struct SPDesktop;
-struct SPItem;
+class  SPDesktop;
+class  SPItem;
+
+namespace Inkscape {
+namespace UI {
+namespace Tools {
+
+class ToolBase;
+
+}
+}
+}
 
 const double goldenratio = 1.61803398874989484820; // golden ratio
 
@@ -29,7 +39,7 @@ extern bool have_viable_layer(SPDesktop *desktop, MessageContext *message);
 extern bool have_viable_layer(SPDesktop *desktop, MessageStack *message);
 Geom::Rect snap_rectangular_box(SPDesktop const *desktop, SPItem *item,
                               Geom::Point const &pt, Geom::Point const &center, int state);
-Geom::Point setup_for_drag_start(SPDesktop *desktop, SPEventContext* ec, GdkEvent *ev);
+Geom::Point setup_for_drag_start(SPDesktop *desktop, Inkscape::UI::Tools::ToolBase* ec, GdkEvent *ev);
 
 }
 
@@ -44,4 +54,4 @@ Geom::Point setup_for_drag_start(SPDesktop *desktop, SPEventContext* ec, GdkEven
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
