@@ -13,6 +13,7 @@
 #include <glib.h>
 #include <glibmm/ustring.h>
 #include "debug/simple-event.h"
+#include "timestamp.h"
 
 namespace Inkscape {
 
@@ -41,4 +42,4 @@ Util::ptr_shared<char> timestamp() {
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
