@@ -16,9 +16,9 @@
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
 
-class SPDesktop;
-class SPDesktopWidget;
-class SPCanvasItem;
+class  SPDesktop;
+struct SPDesktopWidget;
+struct SPCanvasItem;
 
 /* Item handlers */
 
@@ -51,4 +51,4 @@ gint sp_dt_guide_event (SPCanvasItem *item, GdkEvent *event, gpointer data);
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
