@@ -1,5 +1,5 @@
-#ifndef __SP_DESKTOP_HANDLES_H__
-#define __SP_DESKTOP_HANDLES_H__
+#ifndef SEEN_SP_DESKTOP_HANDLES_H
+#define SEEN_SP_DESKTOP_HANDLES_H
 
 /*
  * Frontends
@@ -13,8 +13,24 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include "display/display-forward.h"
-#include "forward.h"
+
+class SPDesktop;
+class SPDocument;
+
+namespace Inkscape {
+namespace UI {
+namespace Tools {
+
+class ToolBase;
+
+}
+}
+}
+
+class SPNamedView;
+struct SPCanvas;
+struct SPCanvasGroup;
+struct SPCanvasItem;
 
 namespace Inkscape { 
     class MessageStack;
@@ -25,10 +41,10 @@ namespace Inkscape {
 #define SP_DESKTOP_ZOOM_MIN 0.01
 
 #define SP_COORDINATES_UNDERLINE_NONE (0)
-#define SP_COORDINATES_UNDERLINE_X (1 << NR::X)
-#define SP_COORDINATES_UNDERLINE_Y (1 << NR::Y)
+#define SP_COORDINATES_UNDERLINE_X (1 << Geom::X)
+#define SP_COORDINATES_UNDERLINE_Y (1 << Geom::Y)
 
-SPEventContext * sp_desktop_event_context (SPDesktop const * desktop);
+//ToolBase * sp_desktop_event_context (SPDesktop const * desktop);
 Inkscape::Selection * sp_desktop_selection (SPDesktop const * desktop);
 SPDocument * sp_desktop_document (SPDesktop const * desktop);
 SPCanvas * sp_desktop_canvas (SPDesktop const * desktop);
@@ -39,10 +55,11 @@ SPCanvasGroup * sp_desktop_guides (SPDesktop const * desktop);
 SPCanvasItem *sp_desktop_drawing (SPDesktop const *desktop);
 SPCanvasGroup * sp_desktop_sketch (SPDesktop const * desktop);
 SPCanvasGroup * sp_desktop_controls (SPDesktop const * desktop);
+SPCanvasGroup * sp_desktop_tempgroup (SPDesktop const * desktop);
 Inkscape::MessageStack * sp_desktop_message_stack (SPDesktop const * desktop);
 SPNamedView * sp_desktop_namedview (SPDesktop const * desktop);
 
-#endif
+#endif // SEEN_SP_DESKTOP_HANDLES_H
 
 /*
   Local Variables:
@@ -53,4 +70,4 @@ SPNamedView * sp_desktop_namedview (SPDesktop const * desktop);
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
