@@ -1,5 +1,6 @@
-/** @file
- * @brief  Event handler for dialog windows
+/**
+ * @file
+ * Event handler for dialog windows.
  */
 /* Authors:
  *   bulia byak <bulia@dr.com>
@@ -14,31 +15,30 @@
 # include "config.h"
 #endif
 
+#if GLIBMM_DISABLE_DEPRECATED && HAVE_GLIBMM_THREADS_H
+#include <glibmm/threads.h>
+#endif
+
+#include <gtkmm/entry.h>
+#include <gtkmm/window.h>
 #include <gdk/gdkkeysyms.h>
 #include "macros.h"
 #include <gtk/gtk.h>
 #include "desktop.h"
 #include "inkscape-private.h"
 #include "preferences.h"
-#include "event-context.h"
+#include "ui/tools/tool-base.h"
 
 #include "dialog-events.h"
 
 
-#include <gtkmm/entry.h>
-
-
-
 /**
- * \brief  Remove focus from window to whoever it is transient for...
- *
+ * Remove focus from window to whoever it is transient for.
  */
-void
-sp_dialog_defocus_cpp (Gtk::Window *win)
+void sp_dialog_defocus_cpp(Gtk::Window *win)
 {
-    Gtk::Window *w;
     //find out the document window we're transient for
-    w = win->get_transient_for();
+    Gtk::Window *w = win->get_transient_for();
 
     //switch to it
     if (w) {
@@ -51,7 +51,7 @@ sp_dialog_defocus (GtkWindow *win)
 {
     GtkWindow *w;
     //find out the document window we're transient for
-    w = gtk_window_get_transient_for ((GtkWindow *) win);
+    w = gtk_window_get_transient_for(GTK_WINDOW(win));
     //switch to it
 
     if (w) {
@@ -61,20 +61,17 @@ sp_dialog_defocus (GtkWindow *win)
 
 
 /**
- * \brief Callback to defocus a widget's parent dialog.
- *
+ * Callback to defocus a widget's parent dialog.
  */
-void
-sp_dialog_defocus_callback_cpp (Gtk::Entry *e)
+void sp_dialog_defocus_callback_cpp(Gtk::Entry *e)
 {
     sp_dialog_defocus_cpp(dynamic_cast<Gtk::Window *>(e->get_toplevel()));
 }
 
 void
-sp_dialog_defocus_callback (GtkWindow */*win*/, gpointer data)
+sp_dialog_defocus_callback (GtkWindow * /*win*/, gpointer data)
 {
-    sp_dialog_defocus ((GtkWindow *)
-        gtk_widget_get_toplevel ((GtkWidget *) data));
+    sp_dialog_defocus( GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(data))) );
 }
 
 
@@ -99,7 +96,7 @@ sp_dialog_event_handler (GtkWindow *win, GdkEvent *event, gpointer data)
 {
 
 // if the focus is inside the Text and Font textview, do nothing
-    GObject *dlg = (GObject *) data;
+    GObject *dlg = G_OBJECT(data);
     if (g_object_get_data (dlg, "eatkeys")) {
         return FALSE;
     }
@@ -110,16 +107,16 @@ sp_dialog_event_handler (GtkWindow *win, GdkEvent *event, gpointer data)
 
         case GDK_KEY_PRESS:
 
-            switch (get_group0_keyval (&event->key)) {
-                case GDK_Escape:
+            switch (Inkscape::UI::Tools::get_group0_keyval (&event->key)) {
+                case GDK_KEY_Escape:
                     sp_dialog_defocus (win);
                     ret = TRUE;
                     break;
-                case GDK_F4:
-                case GDK_w:
-                case GDK_W:
+                case GDK_KEY_F4:
+                case GDK_KEY_w:
+                case GDK_KEY_W:
                     // close dialog
-                    if (MOD__CTRL_ONLY) {
+                    if (MOD__CTRL_ONLY(event)) {
 
                         /* this code sends a delete_event to the dialog,
                          * instead of just destroying it, so that the
@@ -127,12 +124,12 @@ sp_dialog_event_handler (GtkWindow *win, GdkEvent *event, gpointer data)
                          * its position.
                          */
                         GdkEventAny event;
-                        GtkWidget *widget = (GtkWidget *) win;
+                        GtkWidget *widget = GTK_WIDGET(win);
                         event.type = GDK_DELETE;
-                        event.window = widget->window;
+                        event.window = gtk_widget_get_window (widget);
                         event.send_event = TRUE;
                         g_object_ref (G_OBJECT (event.window));
-                        gtk_main_do_event ((GdkEvent*)&event);
+                        gtk_main_do_event(reinterpret_cast<GdkEvent*>(&event));
                         g_object_unref (G_OBJECT (event.window));
 
                         ret = TRUE;
@@ -152,11 +149,10 @@ sp_dialog_event_handler (GtkWindow *win, GdkEvent *event, gpointer data)
 
 
 /**
- * \brief  Make the argument dialog transient to the currently active document
-           window.
+ * Make the argument dialog transient to the currently active document
+ * window.
  */
-void
-sp_transientize (GtkWidget *dialog)
+void sp_transientize(GtkWidget *dialog)
 {
     Inkscape::Preferences *prefs = Inkscape::Preferences::get();
 #ifndef WIN32  // FIXME: Temporary Win32 special code to enable transient dialogs
@@ -223,9 +219,9 @@ void on_dialog_unhide (GtkWidget *w)
 }
 
 gboolean
-sp_dialog_hide (GtkObject */*object*/, gpointer data)
+sp_dialog_hide(GObject * /*object*/, gpointer data)
 {
-    GtkWidget *dlg = (GtkWidget *) data;
+    GtkWidget *dlg = GTK_WIDGET(data);
 
     if (dlg)
         gtk_widget_hide (dlg);
@@ -236,9 +232,9 @@ sp_dialog_hide (GtkObject */*object*/, gpointer data)
 
 
 gboolean
-sp_dialog_unhide (GtkObject */*object*/, gpointer data)
+sp_dialog_unhide(GObject * /*object*/, gpointer data)
 {
-    GtkWidget *dlg = (GtkWidget *) data;
+    GtkWidget *dlg = GTK_WIDGET(data);
 
     if (dlg)
         gtk_widget_show (dlg);
@@ -256,4 +252,4 @@ sp_dialog_unhide (GtkObject */*object*/, gpointer data)
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
