@@ -2,39 +2,31 @@
 #define CANVAS_AXONOMGRID_H
 
 /*
- * Copyright (C) 2006-2007 Johan Engelen  <johan@shouraizou.nl>
+ * Authors:
+ *    Johan Engelen <j.b.c.engelen@alumnus.utwente.nl>
  *
-  */
+ * Copyright (C) 2006-2012 Authors
+ * Released under GNU GPL, read the file 'COPYING' for more information
+ */
 
-#include <display/sp-canvas.h>
-#include "xml/repr.h"
-#include <gtkmm/box.h>
-
-#include <gtkmm.h>
-#include "ui/widget/color-picker.h"
-#include "ui/widget/scalar-unit.h"
-
-#include "ui/widget/registered-widget.h"
-#include "ui/widget/registry.h"
-
-#include "xml/node-event-vector.h"
-
-#include "snapper.h"
 #include "line-snapper.h"
-
 #include "canvas-grid.h"
 
-struct SPDesktop;
-struct SPNamedView;
+struct SPCanvasBuf;
+class  SPDesktop;
+class SPNamedView;
 
 namespace Inkscape {
+namespace XML {
+    class Node;
+};
 
 class CanvasAxonomGrid : public CanvasGrid {
 public:
     CanvasAxonomGrid(SPNamedView * nv, Inkscape::XML::Node * in_repr, SPDocument * in_doc);
     virtual ~CanvasAxonomGrid();
 
-    void Update (Geom::Matrix const &affine, unsigned int flags);
+    void Update (Geom::Affine const &affine, unsigned int flags);
     void Render (SPCanvasBuf *buf);
 
     void readRepr();
@@ -47,6 +39,9 @@ public:
 
     bool scaled;          /**< Whether the grid is in scaled mode */
 
+protected:
+    friend class CanvasAxonomGridSnapper;
+
     Geom::Point ow;         /**< Transformed origin by the affine for the zoom */
     double lyw;           /**< Transformed length y by the affine for the zoom */
     double lxw_x;
@@ -55,7 +50,6 @@ public:
 
     Geom::Point sw;          /**< the scaling factors of the affine transform */
 
-protected:
     virtual Gtk::Widget * newSpecificWidget();
 
 private:
@@ -74,12 +68,13 @@ public:
     bool ThisSnapperMightSnap() const;
 
     Geom::Coord getSnapperTolerance() const; //returns the tolerance of the snapper in screen pixels (i.e. independent of zoom)
-	bool getSnapperAlwaysSnap() const; //if true, then the snapper will always snap, regardless of its tolerance
+    bool getSnapperAlwaysSnap() const; //if true, then the snapper will always snap, regardless of its tolerance
 
 private:
     LineList _getSnapLines(Geom::Point const &p) const;
-    void _addSnappedLine(SnappedConstraints &sc, Geom::Point const snapped_point, Geom::Coord const snapped_distance, SnapSourceType const &source, long source_num, Geom::Point const normal_to_line, const Geom::Point point_on_line) const;
-    void _addSnappedPoint(SnappedConstraints &sc, Geom::Point const snapped_point, Geom::Coord const snapped_distance, SnapSourceType const &source, long source_num, bool constrained_snap) const;
+    void _addSnappedLine(IntermSnapResults &isr, Geom::Point const &snapped_point, Geom::Coord const &snapped_distance, SnapSourceType const &source, long source_num, Geom::Point const &normal_to_line, const Geom::Point &point_on_line) const;
+    void _addSnappedPoint(IntermSnapResults &isr, Geom::Point const &snapped_point, Geom::Coord const &snapped_distance, SnapSourceType const &source, long source_num, bool constrained_snap) const;
+    void _addSnappedLinePerpendicularly(IntermSnapResults &isr, Geom::Point const &snapped_point, Geom::Coord const &snapped_distance, SnapSourceType const &source, long source_num, bool constrained_snap) const;
 
     CanvasAxonomGrid *grid;
 };
