@@ -1,5 +1,5 @@
 /** @file
- * @brief Cartesian grid item for the Inkscape canvas
+ * Cartesian grid item for the Inkscape canvas.
  */
 /* Copyright (C) Johan Engelen 2006-2007 <johan@shouraizou.nl>
  * Copyright (C) Lauris Kaplinski 2000
@@ -8,28 +8,29 @@
 #ifndef INKSCAPE_CANVAS_GRID_H
 #define INKSCAPE_CANVAS_GRID_H
 
-#include <cstring>
-#include <string>
-
-#include <gtkmm/box.h>
-#include <gtkmm.h>
-
-#include "display/sp-canvas.h"
-#include "xml/repr.h"
-#include "ui/widget/color-picker.h"
-#include "ui/widget/scalar-unit.h"
-#include "ui/widget/registered-widget.h"
+#include "sp-canvas-item.h"
 #include "ui/widget/registry.h"
-#include "xml/node-event-vector.h"
-#include "snapper.h"
 #include "line-snapper.h"
 
-struct SPDesktop;
-struct SPNamedView;
-class SPDocument;
+class  SPDesktop;
+class SPNamedView;
+struct SPCanvasBuf;
+class  SPDocument;
+
+namespace Gtk {
+	class Widget;
+}
 
 namespace Inkscape {
+class Snapper;
 
+namespace XML {
+class Node;
+}
+
+namespace Util {
+class Unit;
+}
 
 enum GridType {
     GRID_RECTANGULAR = 0,
@@ -38,15 +39,14 @@ enum GridType {
 #define GRID_MAXTYPENR 1
 
 #define INKSCAPE_TYPE_GRID_CANVASITEM            (Inkscape::grid_canvasitem_get_type ())
-#define INKSCAPE_GRID_CANVASITEM(obj)            (GTK_CHECK_CAST ((obj), INKSCAPE_TYPE_GRID_CANVASITEM, GridCanvasItem))
-#define INKSCAPE_GRID_CANVASITEM_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), INKSCAPE_TYPE_GRID_CANVASITEM, GridCanvasItem))
-#define INKSCAPE_IS_GRID_CANVASITEM(obj)         (GTK_CHECK_TYPE ((obj), INKSCAPE_TYPE_GRID_CANVASITEM))
-#define INKSCAPE_IS_GRID_CANVASITEM_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), INKSCAPE_TYPE_GRID_CANVASITEM))
+#define INKSCAPE_GRID_CANVASITEM(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), INKSCAPE_TYPE_GRID_CANVASITEM, GridCanvasItem))
+#define INKSCAPE_GRID_CANVASITEM_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), INKSCAPE_TYPE_GRID_CANVASITEM, GridCanvasItem))
+#define INKSCAPE_IS_GRID_CANVASITEM(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), INKSCAPE_TYPE_GRID_CANVASITEM))
+#define INKSCAPE_IS_GRID_CANVASITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), INKSCAPE_TYPE_GRID_CANVASITEM))
 
 class CanvasGrid;
 
-/** \brief  All the variables that are tracked for a grid specific
-            canvas item. */
+/** All the variables that are tracked for a grid specific canvas item. */
 struct GridCanvasItem : public SPCanvasItem{
     CanvasGrid *grid; // the owning grid object
 };
@@ -56,7 +56,7 @@ struct GridCanvasItemClass {
 };
 
 /* Standard Gtk function */
-GtkType grid_canvasitem_get_type (void);
+GType grid_canvasitem_get_type (void);
 
 
 
@@ -65,9 +65,9 @@ public:
     virtual ~CanvasGrid();
 
     // TODO: see effect.h and effect.cpp from live_effects how to link enums to SVGname to typename properly. (johan)
-    const char * getName();
-    const char * getSVGName();
-    GridType     getGridType();
+    const char * getName() const;
+    const char * getSVGName() const;
+    GridType     getGridType() const;
     static const char * getName(GridType type);
     static const char * getSVGName(GridType type);
     static GridType     getGridTypeFromSVGName(const char * typestr);
@@ -78,7 +78,7 @@ public:
 
     GridCanvasItem * createCanvasItem(SPDesktop * desktop);
 
-    virtual void Update (Geom::Matrix const &affine, unsigned int flags) = 0;
+    virtual void Update (Geom::Affine const &affine, unsigned int flags) = 0;
     virtual void Render (SPCanvasBuf *buf) = 0;
 
     virtual void readRepr() = 0;
@@ -92,7 +92,7 @@ public:
     guint32 empcolor;     /**< Color for emphasis lines */
     gint empspacing;      /**< Spacing between emphasis lines */
 
-    SPUnit const* gridunit;
+    Inkscape::Util::Unit const* gridunit;  /**< points to Unit object in UnitTable (so don't delete it) */
 
     Inkscape::XML::Node * repr;
     SPDocument *doc;
@@ -101,8 +101,8 @@ public:
 
     static void on_repr_attr_changed (Inkscape::XML::Node * repr, const gchar *key, const gchar *oldval, const gchar *newval, bool is_interactive, void * data);
 
-    bool isVisible() { return (isEnabled() &&visible); };
-    bool isEnabled();
+    bool isVisible() const { return (isEnabled() &&visible); };
+    bool isEnabled() const;
 
 protected:
     CanvasGrid(SPNamedView * nv, Inkscape::XML::Node * in_repr, SPDocument *in_doc, GridType type);
@@ -129,7 +129,7 @@ public:
     CanvasXYGrid(SPNamedView * nv, Inkscape::XML::Node * in_repr, SPDocument * in_doc);
     virtual ~CanvasXYGrid();
 
-    void Update (Geom::Matrix const &affine, unsigned int flags);
+    void Update (Geom::Affine const &affine, unsigned int flags);
     void Render (SPCanvasBuf *buf);
 
     void readRepr();
@@ -167,8 +167,9 @@ public:
 
 private:
     LineList _getSnapLines(Geom::Point const &p) const;
-    void _addSnappedLine(SnappedConstraints &sc, Geom::Point const snapped_point, Geom::Coord const snapped_distance,  SnapSourceType const &source, long source_num, Geom::Point const normal_to_line, const Geom::Point point_on_line) const;
-    void _addSnappedPoint(SnappedConstraints &sc, Geom::Point const snapped_point, Geom::Coord const snapped_distance, SnapSourceType const &source, long source_num, bool constrained_snap) const;
+    void _addSnappedLine(IntermSnapResults &isr, Geom::Point const &snapped_point, Geom::Coord const &snapped_distance,  SnapSourceType const &source, long source_num, Geom::Point const &normal_to_line, const Geom::Point &point_on_line) const;
+    void _addSnappedPoint(IntermSnapResults &isr, Geom::Point const &snapped_point, Geom::Coord const &snapped_distance, SnapSourceType const &source, long source_num, bool constrained_snap) const;
+    void _addSnappedLinePerpendicularly(IntermSnapResults &isr, Geom::Point const &snapped_point, Geom::Coord const &snapped_distance, SnapSourceType const &source, long source_num, bool constrained_snap) const;
     CanvasXYGrid *grid;
 };
 
