@@ -1,5 +1,5 @@
-#ifndef __SP_GUIDELINE_H__
-#define __SP_GUIDELINE_H__
+#ifndef SEEN_SP_GUIDELINE_H
+#define SEEN_SP_GUIDELINE_H
 
 /*
  * The visual representation of SPGuide.
@@ -14,21 +14,24 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include "sp-canvas.h"
 #include <2geom/point.h>
+#include "sp-canvas-item.h"
 
 #define SP_TYPE_GUIDELINE (sp_guideline_get_type())
-#define SP_GUIDELINE(o) (GTK_CHECK_CAST((o), SP_TYPE_GUIDELINE, SPGuideLine))
-#define SP_IS_GUIDELINE(o) (GTK_CHECK_TYPE((o), SP_TYPE_GUIDELINE))
+#define SP_GUIDELINE(o) (G_TYPE_CHECK_INSTANCE_CAST((o), SP_TYPE_GUIDELINE, SPGuideLine))
+#define SP_IS_GUIDELINE(o) (G_TYPE_CHECK_INSTANCE_TYPE((o), SP_TYPE_GUIDELINE))
 
-class SPCtrlPoint;
+struct SPCtrlPoint;
 
 struct SPGuideLine {
     SPCanvasItem item;
+    Geom::Affine affine;
+
     SPCtrlPoint *origin; // unlike 'item', this is only held locally
 
     guint32 rgba;
 
+    char* label;
     Geom::Point normal_to_line;
     Geom::Point point_on_line;
     double angle;
@@ -45,15 +48,16 @@ struct SPGuideLineClass {
 
 GType sp_guideline_get_type();
 
-SPCanvasItem *sp_guideline_new(SPCanvasGroup *parent, Geom::Point point_on_line, Geom::Point normal);
+SPCanvasItem *sp_guideline_new(SPCanvasGroup *parent, char* label, Geom::Point point_on_line, Geom::Point normal);
 
+void sp_guideline_set_label(SPGuideLine *gl, const char* label);
 void sp_guideline_set_position(SPGuideLine *gl, Geom::Point point_on_line);
 void sp_guideline_set_normal(SPGuideLine *gl, Geom::Point normal_to_line);
 void sp_guideline_set_color(SPGuideLine *gl, unsigned int rgba);
 void sp_guideline_set_sensitive(SPGuideLine *gl, int sensitive);
 void sp_guideline_delete(SPGuideLine *gl);
 
-#endif
+#endif // SEEN_SP_GUIDELINE_H
 
 /*
   Local Variables:
