@@ -1,7 +1,8 @@
 #ifndef __NR_FILTER_TYPES_H__
 #define __NR_FILTER_TYPES_H__
 
-namespace NR {
+namespace Inkscape {
+namespace Filters {
 
 enum FilterPrimitiveType {
     NR_FILTER_BLEND,
@@ -34,10 +35,19 @@ enum FilterSlotType {
     NR_FILTER_STROKEPAINT = -7,
     NR_FILTER_UNNAMED_SLOT = -8
 };
-/* Unnamed slot is for NR::FilterSlot internal use. Passing it as
- * parameter to NR::FilterSlot accessors may have unforeseen consequences. */
+/* Unnamed slot is for Inkscape::Filters::FilterSlot internal use. Passing it as
+ * parameter to Inkscape::Filters::FilterSlot accessors may have unforeseen consequences. */
 
-} /* namespace NR */
+enum FilterQuality {
+    FILTER_QUALITY_BEST = 2,
+    FILTER_QUALITY_BETTER = 1,
+    FILTER_QUALITY_NORMAL = 0,
+    FILTER_QUALITY_WORSE = -1,
+    FILTER_QUALITY_WORST = -2
+};
+
+} /* namespace Filters */
+} /* namespace Inkscape */
 
 #endif // __NR_FILTER_TYPES_H__
 /*
@@ -49,4 +59,4 @@ enum FilterSlotType {
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
