@@ -1,5 +1,4 @@
 #include "config.h"
-#ifdef ENABLE_SVG_FONTS
 /*
  * SVGFonts rendering implementation
  *
@@ -14,12 +13,24 @@
 
 #include <2geom/pathvector.h>
 #include <2geom/transforms.h>
-#include "../style.h"
 #include <cairo.h>
 #include <vector>
+#include "sp-object.h"
 #include "svg/svg.h"
-#include "inkscape-cairo.h"
-#include "nr-svgfonts.h"
+#include "display/cairo-utils.h"
+#include "display/nr-svgfonts.h"
+#include "display/nr-svgfonts.h"
+#include "sp-path.h"
+#include "sp-object-group.h"
+#include "sp-use.h"
+#include "sp-use-reference.h"
+#include "display/curve.h"
+#include "xml/repr.h"
+#include "sp-font-face.h"
+#include "sp-glyph.h"
+#include "sp-missing-glyph.h"
+#include "sp-font.h"
+#include "sp-glyph-kerning.h"
 
 //*************************//
 // UserFont Implementation //
@@ -31,10 +42,9 @@
 static cairo_user_data_key_t key;
 
 static cairo_status_t font_init_cb (cairo_scaled_font_t  *scaled_font,
-                                    cairo_t */*cairo*/, cairo_font_extents_t *metrics){
-    cairo_font_face_t*  face;
-    face = cairo_scaled_font_get_font_face(scaled_font);
-    SvgFont* instance = (SvgFont*) cairo_font_face_get_user_data(face, &key);
+                                    cairo_t * /*cairo*/, cairo_font_extents_t *metrics){
+    cairo_font_face_t* face = cairo_scaled_font_get_font_face(scaled_font);
+    SvgFont* instance = static_cast<SvgFont*>(cairo_font_face_get_user_data(face, &key));
     return instance->scaled_font_init(scaled_font, metrics);
 }
 
@@ -46,9 +56,8 @@ static cairo_status_t font_text_to_glyphs_cb ( cairo_scaled_font_t  *scaled_font
                                                cairo_text_cluster_t **clusters,
                                                int                  *num_clusters,
                                                cairo_text_cluster_flags_t *flags){
-    cairo_font_face_t*  face;
-    face = cairo_scaled_font_get_font_face(scaled_font);
-    SvgFont* instance = (SvgFont*) cairo_font_face_get_user_data(face, &key);
+    cairo_font_face_t* face = cairo_scaled_font_get_font_face(scaled_font);
+    SvgFont* instance = static_cast<SvgFont*>(cairo_font_face_get_user_data(face, &key));
     return instance->scaled_font_text_to_glyphs(scaled_font, utf8, utf8_len, glyphs, num_glyphs, clusters, num_clusters, flags);
 }
 
@@ -56,9 +65,8 @@ static cairo_status_t font_render_glyph_cb (cairo_scaled_font_t  *scaled_font,
                                             unsigned long         glyph,
                                             cairo_t              *cr,
                                             cairo_text_extents_t *metrics){
-    cairo_font_face_t*  face;
-    face = cairo_scaled_font_get_font_face(scaled_font);
-    SvgFont* instance = (SvgFont*) cairo_font_face_get_user_data(face, &key);
+    cairo_font_face_t* face = cairo_scaled_font_get_font_face(scaled_font);
+    SvgFont* instance = static_cast<SvgFont*>(cairo_font_face_get_user_data(face, &key));
     return instance->scaled_font_render_glyph(scaled_font, glyph, cr, metrics);
 }
 
@@ -104,15 +112,15 @@ unsigned int size_of_substring(const char* substring, gchar* str){
 }
 
 //TODO: in these macros, verify what happens when using unicode strings.
-#define Match_VKerning_Rule (((SPVkern*)node)->u1->contains(previous_unicode[0])\
-                || ((SPVkern*)node)->g1->contains(previous_glyph_name)) &&\
-                (((SPVkern*)node)->u2->contains(this->glyphs[i]->unicode[0])\
-                || ((SPVkern*)node)->g2->contains(this->glyphs[i]->glyph_name.c_str()))
+#define Match_VKerning_Rule ((SP_VKERN(node))->u1->contains(previous_unicode[0])\
+                || (SP_VKERN(node))->g1->contains(previous_glyph_name)) &&\
+                ((SP_VKERN(node))->u2->contains(this->glyphs[i]->unicode[0])\
+                || (SP_VKERN(node))->g2->contains(this->glyphs[i]->glyph_name.c_str()))
 
-#define Match_HKerning_Rule (((SPHkern*)node)->u1->contains(previous_unicode[0])\
-                || ((SPHkern*)node)->g1->contains(previous_glyph_name)) &&\
-                (((SPHkern*)node)->u2->contains(this->glyphs[i]->unicode[0])\
-                || ((SPHkern*)node)->g2->contains(this->glyphs[i]->glyph_name.c_str()))
+#define Match_HKerning_Rule ((SP_HKERN(node))->u1->contains(previous_unicode[0])\
+                || (SP_HKERN(node))->g1->contains(previous_glyph_name)) &&\
+                ((SP_HKERN(node))->u2->contains(this->glyphs[i]->unicode[0])\
+                || (SP_HKERN(node))->g2->contains(this->glyphs[i]->glyph_name.c_str()))
 
 cairo_status_t
 SvgFont::scaled_font_text_to_glyphs (cairo_scaled_font_t  */*scaled_font*/,
@@ -175,14 +183,14 @@ SvgFont::scaled_font_text_to_glyphs (cairo_scaled_font_t  */*scaled_font*/,
                 for(SPObject* node = this->font->children;previous_unicode && node;node=node->next){
                     //apply glyph kerning if appropriate
                     if (SP_IS_HKERN(node) && is_horizontal_text && Match_HKerning_Rule ){
-                        x -= (((SPHkern*)node)->k / 1000.0);//TODO: use here the height of the font
+                        x -= ((SP_HKERN(node))->k / 1000.0);//TODO: use here the height of the font
                     }
                     if (SP_IS_VKERN(node) && !is_horizontal_text && Match_VKerning_Rule ){
-                        y -= (((SPVkern*)node)->k / 1000.0);//TODO: use here the "height" of the font
+                        y -= ((SP_VKERN(node))->k / 1000.0);//TODO: use here the "height" of the font
                     }
                 }
-                previous_unicode = (char*) this->glyphs[i]->unicode.c_str();//used for kerning checking
-                previous_glyph_name = (char*) this->glyphs[i]->glyph_name.c_str();//used for kerning checking
+                previous_unicode = const_cast<char*>(this->glyphs[i]->unicode.c_str());//used for kerning checking
+                previous_glyph_name = const_cast<char*>(this->glyphs[i]->glyph_name.c_str());//used for kerning checking
                 (*glyphs)[count].index = i;
                 (*glyphs)[count].x = x;
                 (*glyphs)[count++].y = y;
@@ -212,6 +220,47 @@ SvgFont::scaled_font_text_to_glyphs (cairo_scaled_font_t  */*scaled_font*/,
     return CAIRO_STATUS_SUCCESS;
 }
 
+void
+SvgFont::render_glyph_path(cairo_t* cr, Geom::PathVector* pathv){
+    if (!pathv->empty()){
+        //This glyph has a path description on its d attribute, so we render it:
+        cairo_new_path(cr);
+
+        //adjust scale of the glyph
+//        Geom::Scale s(1.0/((SPFont*) node->parent)->horiz_adv_x);
+        Geom::Scale s(1.0/1000);//TODO: use here the units-per-em attribute?
+
+        Geom::Rect area( Geom::Point(0,0), Geom::Point(1,1) ); //I need help here!    (reaction: note that the 'area' parameter is an *optional* rect, so you can pass an empty Geom::OptRect() )
+
+        feed_pathvector_to_cairo (cr, *pathv, s, area, false, 0);
+        cairo_fill(cr);
+    }
+}
+
+void
+SvgFont::glyph_modified(SPObject* /* blah */, unsigned int /* bleh */){
+    this->refresh();
+    //TODO: update rendering on svgfonts preview widget (in the svg fonts dialog)
+}
+
+Geom::PathVector
+SvgFont::flip_coordinate_system(SPFont* spfont, Geom::PathVector pathv){
+    double units_per_em = 1000;
+    SPObject* obj;
+    for (obj = (SP_OBJECT(spfont))->children; obj; obj=obj->next){
+        if (SP_IS_FONTFACE(obj)){
+            //XML Tree being directly used here while it shouldn't be.
+            sp_repr_get_double(obj->getRepr(), "units_per_em", &units_per_em);
+        }
+    }
+
+    double baseline_offset = units_per_em - spfont->horiz_origin_y;
+
+    //This matrix flips y-axis and places the origin at baseline
+    Geom::Affine m(Geom::Coord(1),Geom::Coord(0),Geom::Coord(0),Geom::Coord(-1),Geom::Coord(0),Geom::Coord(baseline_offset));
+    return pathv*m;
+}
+
 cairo_status_t
 SvgFont::scaled_font_render_glyph (cairo_scaled_font_t  */*scaled_font*/,
                                    unsigned long         glyph,
@@ -229,42 +278,58 @@ SvgFont::scaled_font_render_glyph (cairo_scaled_font_t  */*scaled_font*/,
     SPObject* node;
     if (glyph == this->glyphs.size()){
         if (!this->missingglyph) return CAIRO_STATUS_SUCCESS;
-        node = (SPObject*) this->missingglyph;
+        node = SP_OBJECT(this->missingglyph);
     } else {
-        node = (SPObject*) this->glyphs[glyph];
+        node = SP_OBJECT(this->glyphs[glyph]);
+    }
+
+    if (!SP_IS_GLYPH(node) && !SP_IS_MISSING_GLYPH(node)) {
+        return CAIRO_STATUS_SUCCESS;  // FIXME: is this the right code to return?
+    }
+
+    SPFont* spfont = SP_FONT(node->parent);
+    if (!spfont) {
+        return CAIRO_STATUS_SUCCESS;  // FIXME: is this the right code to return?
     }
 
     //glyphs can be described by arbitrary SVG declared in the childnodes of a glyph node
     // or using the d attribute of a glyph node.
     // pathv stores the path description from the d attribute:
     Geom::PathVector pathv;
-    if (SP_IS_GLYPH(node) && ((SPGlyph*)node)->d) {
-        pathv = sp_svg_read_pathv(((SPGlyph*)node)->d);
-    } else if (SP_IS_MISSING_GLYPH(node) && ((SPMissingGlyph*)node)->d) {
-        pathv = sp_svg_read_pathv(((SPMissingGlyph*)node)->d);
-    } else {
-        return CAIRO_STATUS_SUCCESS;  // FIXME: is this the right code to return?
+    if (SP_IS_GLYPH(node) && (SP_GLYPH(node))->d) {
+        pathv = sp_svg_read_pathv((SP_GLYPH(node))->d);
+        pathv = flip_coordinate_system(spfont, pathv);
+        this->render_glyph_path(cr, &pathv);
+    } else if (SP_IS_MISSING_GLYPH(node) && (SP_MISSING_GLYPH(node))->d) {
+        pathv = sp_svg_read_pathv((SP_MISSING_GLYPH(node))->d);
+        pathv = flip_coordinate_system(spfont, pathv);
+        this->render_glyph_path(cr, &pathv);
     }
 
-    if (!pathv.empty()){
-        //This glyph has a path description on its d attribute, so we render it:
-        cairo_new_path(cr);
-        //adjust scale of the glyph
-//        Geom::Scale s(1.0/((SPFont*) node->parent)->horiz_adv_x);
-        Geom::Scale s(1.0/1000);//TODO: use here the units-per-em attribute?
-        //This matrix flips the glyph vertically
-        Geom::Matrix m(Geom::Coord(1),Geom::Coord(0),Geom::Coord(0),Geom::Coord(-1),Geom::Coord(0),Geom::Coord(0));
-        //then we offset it
-//      pathv += Geom::Point(Geom::Coord(0),Geom::Coord(-((SPFont*) node->parent)->horiz_adv_x));
-        pathv += Geom::Point(Geom::Coord(0),Geom::Coord(-1000));//TODO: use here the units-per-em attribute?
+    if (node->hasChildren()){
+        //render the SVG described on this glyph's child nodes.
+        for(node = node->children; node; node=node->next){
+            if (SP_IS_PATH(node)){
+                pathv = (SP_SHAPE(node))->_curve->get_pathvector();
+                pathv = flip_coordinate_system(spfont, pathv);
+                this->render_glyph_path(cr, &pathv);
+            }
+            if (SP_IS_OBJECTGROUP(node)){
+                g_warning("TODO: svgfonts: render OBJECTGROUP");
+            }
+            if (SP_IS_USE(node)){
+                SPItem* item = SP_USE(node)->ref->getObject();
+                if (SP_IS_PATH(item)){
+                    pathv = (SP_SHAPE(item))->_curve->get_pathvector();
+                    pathv = flip_coordinate_system(spfont, pathv);
+                    this->render_glyph_path(cr, &pathv);
+                }
 
-        Geom::Rect area( Geom::Point(0,0), Geom::Point(1,1) ); //I need help here!    (reaction: note that the 'area' parameter is an *optional* rect, so you can pass an empty Geom::OptRect() )
-
-        feed_pathvector_to_cairo (cr, pathv, s*m, area, false, 0);
-        cairo_fill(cr);
+                glyph_modified_connection = (SP_OBJECT(item))->connectModified(sigc::mem_fun(*this, &SvgFont::glyph_modified));
+            }
+        }
     }
 
-    //TODO: render the SVG described on this glyph's child nodes.
     return CAIRO_STATUS_SUCCESS;
 }
 
@@ -273,10 +338,10 @@ SvgFont::get_font_face(){
     if (!this->userfont) {
         for(SPObject* node = this->font->children;node;node=node->next){
             if (SP_IS_GLYPH(node)){
-                this->glyphs.push_back((SPGlyph*)node);
+                this->glyphs.push_back(SP_GLYPH(node));
             }
             if (SP_IS_MISSING_GLYPH(node)){
-                this->missingglyph=(SPMissingGlyph*)node;
+                this->missingglyph=SP_MISSING_GLYPH(node);
             }
         }
         this->userfont = new UserFont(this);
@@ -290,8 +355,6 @@ void SvgFont::refresh(){
     this->userfont = NULL;
 }
 
-#endif //#ifdef ENABLE_SVG_FONTS
-
 /*
   Local Variables:
   mode:c++
@@ -301,4 +364,4 @@ void SvgFont::refresh(){
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
