@@ -12,8 +12,13 @@ namespace Inkscape {
 enum RenderMode {
     RENDERMODE_NORMAL,
     RENDERMODE_NO_FILTERS,
-    RENDERMODE_OUTLINE,
-    RENDERMODE_PRINT_COLORS_PREVIEW
+    RENDERMODE_OUTLINE
+};
+
+enum ColorMode {
+    COLORMODE_NORMAL,
+    COLORMODE_GRAYSCALE,
+    COLORMODE_PRINT_COLORS_PREVIEW
 };
 
 }
