@@ -1,4 +1,4 @@
-/** \file
+/*
  * Provides a class that shows a temporary indicator on the canvas of where the snap was, and what kind of snap
  *
  * Authors:
@@ -6,7 +6,7 @@
  *   Diederik van Lierop
  *
  * Copyright (C) Johan Engelen 2009 <j.b.c.engelen@utwente.nl>
- * Copyright (C) Diederik van Lierop 2010 <mail@diedenrezi.nl>
+ * Copyright (C) Diederik van Lierop 2010 - 2012 <mail@diedenrezi.nl>
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
@@ -22,6 +22,8 @@
 #include "knot.h"
 #include "preferences.h"
 #include <glibmm/i18n.h>
+#include "tools-switch.h"
+#include "enums.h"
 
 namespace Inkscape {
 namespace Display {
@@ -51,7 +53,6 @@ SnapIndicator::set_new_snaptarget(Inkscape::SnappedPoint const &p, bool pre_snap
     g_assert(_desktop != NULL);
 
     if (!p.getSnapped()) {
-        g_warning("No snapping took place, so no snap target will be displayed");
         return; // If we haven't snapped, then it is of no use to draw a snapindicator
     }
 
@@ -72,6 +73,7 @@ SnapIndicator::set_new_snaptarget(Inkscape::SnappedPoint const &p, bool pre_snap
         switch (p.getTarget()) {
             case SNAPTARGET_UNDEFINED:
                 target_name = _("UNDEFINED");
+                g_warning("Snap target has not been specified");
                 break;
             case SNAPTARGET_GRID:
                 target_name = _("grid line");
@@ -79,6 +81,9 @@ SnapIndicator::set_new_snaptarget(Inkscape::SnappedPoint const &p, bool pre_snap
             case SNAPTARGET_GRID_INTERSECTION:
                 target_name = _("grid intersection");
                 break;
+            case SNAPTARGET_GRID_PERPENDICULAR:
+                target_name = _("grid line (perpendicular)");
+                break;
             case SNAPTARGET_GUIDE:
                 target_name = _("guide");
                 break;
@@ -88,6 +93,9 @@ SnapIndicator::set_new_snaptarget(Inkscape::SnappedPoint const &p, bool pre_snap
             case SNAPTARGET_GUIDE_ORIGIN:
                 target_name = _("guide origin");
                 break;
+            case SNAPTARGET_GUIDE_PERPENDICULAR:
+                target_name = _("guide (perpendicular)");
+                break;
             case SNAPTARGET_GRID_GUIDE_INTERSECTION:
                 target_name = _("grid-guide intersection");
                 break;
@@ -100,9 +108,24 @@ SnapIndicator::set_new_snaptarget(Inkscape::SnappedPoint const &p, bool pre_snap
             case SNAPTARGET_PATH:
                 target_name = _("path");
                 break;
+            case SNAPTARGET_PATH_PERPENDICULAR:
+                target_name = _("path (perpendicular)");
+                break;
+            case SNAPTARGET_PATH_TANGENTIAL:
+                target_name = _("path (tangential)");
+                break;
             case SNAPTARGET_PATH_INTERSECTION:
                 target_name = _("path intersection");
                 break;
+            case SNAPTARGET_PATH_GUIDE_INTERSECTION:
+                target_name = _("guide-path intersection");
+                break;
+            case SNAPTARGET_PATH_CLIP:
+                target_name = _("clip-path");
+                break;
+            case SNAPTARGET_PATH_MASK:
+                target_name = _("mask-path");
+                break;
             case SNAPTARGET_BBOX_CORNER:
                 target_name = _("bounding box corner");
                 break;
@@ -121,9 +144,6 @@ SnapIndicator::set_new_snaptarget(Inkscape::SnappedPoint const &p, bool pre_snap
             case SNAPTARGET_ROTATION_CENTER:
                 target_name = _("object rotation center");
                 break;
-            case SNAPTARGET_HANDLE:
-                target_name = _("handle");
-                break;
             case SNAPTARGET_BBOX_EDGE_MIDPOINT:
                 target_name = _("bounding box side midpoint");
                 break;
@@ -133,18 +153,16 @@ SnapIndicator::set_new_snaptarget(Inkscape::SnappedPoint const &p, bool pre_snap
             case SNAPTARGET_PAGE_CORNER:
                 target_name = _("page corner");
                 break;
-            case SNAPTARGET_CONVEX_HULL_CORNER:
-                target_name = _("convex hull corner");
-                break;
             case SNAPTARGET_ELLIPSE_QUADRANT_POINT:
                 target_name = _("quadrant point");
                 break;
-            case SNAPTARGET_CENTER:
-                target_name = _("center");
-                break;
-            case SNAPTARGET_CORNER:
+            case SNAPTARGET_RECT_CORNER:
+            case SNAPTARGET_IMG_CORNER:
                 target_name = _("corner");
                 break;
+            case SNAPTARGET_TEXT_ANCHOR:
+                target_name = _("text anchor");
+                break;
             case SNAPTARGET_TEXT_BASELINE:
                 target_name = _("text baseline");
                 break;
@@ -155,7 +173,7 @@ SnapIndicator::set_new_snaptarget(Inkscape::SnappedPoint const &p, bool pre_snap
                 target_name = _("constraint");
                 break;
             default:
-                g_warning("Snap target has not yet been defined!");
+                g_warning("Snap target not in SnapTargetType enum");
                 break;
         }
 
@@ -163,6 +181,7 @@ SnapIndicator::set_new_snaptarget(Inkscape::SnappedPoint const &p, bool pre_snap
         switch (p.getSource()) {
             case SNAPSOURCE_UNDEFINED:
                 source_name = _("UNDEFINED");
+                g_warning("Snap source has not been specified");
                 break;
             case SNAPSOURCE_BBOX_CORNER:
                 source_name = _("Bounding box corner");
@@ -207,20 +226,18 @@ SnapIndicator::set_new_snaptarget(Inkscape::SnappedPoint const &p, bool pre_snap
             case SNAPSOURCE_ELLIPSE_QUADRANT_POINT:
                 source_name = _("Quadrant point");
                 break;
-            case SNAPSOURCE_CENTER:
-                source_name = _("Center");
-                break;
-            case SNAPSOURCE_CORNER:
+            case SNAPSOURCE_RECT_CORNER:
+            case SNAPSOURCE_IMG_CORNER:
                 source_name = _("Corner");
                 break;
-            case SNAPSOURCE_TEXT_BASELINE:
-                source_name = _("Text baseline");
+            case SNAPSOURCE_TEXT_ANCHOR:
+                source_name = _("Text anchor");
                 break;
             case SNAPSOURCE_GRID_PITCH:
                 source_name = _("Multiple of grid spacing");
                 break;
             default:
-                g_warning("Snap source has not yet been defined!");
+                g_warning("Snap source not in SnapSourceType enum");
                 break;
         }
         //std::cout << "Snapped " << source_name << " to " << target_name << std::endl;
@@ -230,14 +247,14 @@ SnapIndicator::set_new_snaptarget(Inkscape::SnappedPoint const &p, bool pre_snap
         // Display the snap indicator (i.e. the cross)
         SPCanvasItem * canvasitem = NULL;
         canvasitem = sp_canvas_item_new(sp_desktop_tempgroup (_desktop),
-										SP_TYPE_CTRL,
-										"anchor", GTK_ANCHOR_CENTER,
-										"size", 10.0,
-										"stroked", TRUE,
-										"stroke_color", pre_snap ? 0x7f7f7fff : 0xff0000ff,
-										"mode", SP_KNOT_MODE_XOR,
-										"shape", SP_KNOT_SHAPE_CROSS,
-										NULL );
+                                        SP_TYPE_CTRL,
+                                        "anchor", SP_ANCHOR_CENTER,
+                                        "size", 10.0,
+                                        "stroked", TRUE,
+                                        "stroke_color", pre_snap ? 0x7f7f7fff : 0xff0000ff,
+                                        "mode", SP_KNOT_MODE_XOR,
+                                        "shape", SP_KNOT_SHAPE_CROSS,
+                                        NULL );
 
         const int timeout_val = 1200; // TODO add preference for snap indicator timeout?
 
@@ -247,21 +264,37 @@ SnapIndicator::set_new_snaptarget(Inkscape::SnappedPoint const &p, bool pre_snap
 
         // Display the tooltip, which reveals the type of snap source and the type of snap target
         gchar *tooltip_str = NULL;
-        if (p.getSource() != SNAPSOURCE_GRID_PITCH) {
+        if ( (p.getSource() != SNAPSOURCE_GRID_PITCH) && (p.getTarget() != SNAPTARGET_UNDEFINED) ) {
             tooltip_str = g_strconcat(source_name, _(" to "), target_name, NULL);
-        } else {
+        } else if (p.getSource() != SNAPSOURCE_UNDEFINED) {
             tooltip_str = g_strdup(source_name);
         }
-        Geom::Point tooltip_pos = p.getPoint() + _desktop->w2d(Geom::Point(15, -15));
+        double fontsize = prefs->getInt("/tools/measure/fontsize");
 
-        SPCanvasItem *canvas_tooltip = sp_canvastext_new(sp_desktop_tempgroup(_desktop), _desktop, tooltip_pos, tooltip_str);
-        if (pre_snap) {
-            SP_CANVASTEXT(canvas_tooltip)->rgba = 0x7f7f7fff;
+        if (tooltip_str) {
+            Geom::Point tooltip_pos = p.getPoint();
+            if (tools_isactive(_desktop, TOOLS_MEASURE)) {
+                // Make sure that the snap tooltips do not overlap the ones from the measure tool
+                tooltip_pos += _desktop->w2d(Geom::Point(0, -3*fontsize));
+            } else {
+                tooltip_pos += _desktop->w2d(Geom::Point(0, -2*fontsize));
+            }
+
+            SPCanvasItem *canvas_tooltip = sp_canvastext_new(sp_desktop_tempgroup(_desktop), _desktop, tooltip_pos, tooltip_str);
+            sp_canvastext_set_fontsize(SP_CANVASTEXT(canvas_tooltip), fontsize);
+            SP_CANVASTEXT(canvas_tooltip)->rgba = 0xffffffff;
+            SP_CANVASTEXT(canvas_tooltip)->outline = false;
+            SP_CANVASTEXT(canvas_tooltip)->background = true;
+            if (pre_snap) {
+                SP_CANVASTEXT(canvas_tooltip)->rgba_background = 0x33337f40;
+            } else {
+                SP_CANVASTEXT(canvas_tooltip)->rgba_background = 0x33337f7f;
+            }
+            SP_CANVASTEXT(canvas_tooltip)->anchor_position = TEXT_ANCHOR_CENTER;
+            g_free(tooltip_str);
+
+            _snaptarget_tooltip = _desktop->add_temporary_canvasitem(canvas_tooltip, timeout_val);
         }
-        g_free(tooltip_str);
-
-        sp_canvastext_set_anchor((SPCanvasText* )canvas_tooltip, -1, 1);
-        _snaptarget_tooltip = _desktop->add_temporary_canvasitem(canvas_tooltip, timeout_val);
 
         // Display the bounding box, if we snapped to one
         Geom::OptRect const bbox = p.getTargetBBox();
@@ -309,7 +342,7 @@ SnapIndicator::set_new_snapsource(Inkscape::SnapCandidatePoint const &p)
 {
     remove_snapsource();
 
-    g_assert(_desktop != NULL);
+    g_assert(_desktop != NULL); // If this fails, then likely setup() has not been called on the snap manager (see snap.cpp -> setup())
 
     Inkscape::Preferences *prefs = Inkscape::Preferences::get();
     bool value = prefs->getBool("/options/snapindicator/value", true);
@@ -317,7 +350,7 @@ SnapIndicator::set_new_snapsource(Inkscape::SnapCandidatePoint const &p)
     if (value) {
         SPCanvasItem * canvasitem = sp_canvas_item_new( sp_desktop_tempgroup (_desktop),
                                                         SP_TYPE_CTRL,
-                                                        "anchor", GTK_ANCHOR_CENTER,
+                                                        "anchor", SP_ANCHOR_CENTER,
                                                         "size", 6.0,
                                                         "stroked", TRUE,
                                                         "stroke_color", 0xff0000ff,
@@ -330,6 +363,25 @@ SnapIndicator::set_new_snapsource(Inkscape::SnapCandidatePoint const &p)
     }
 }
 
+void
+SnapIndicator::set_new_debugging_point(Geom::Point const &p)
+{
+    g_assert(_desktop != NULL);
+    SPCanvasItem * canvasitem = sp_canvas_item_new( sp_desktop_tempgroup (_desktop),
+                                                    SP_TYPE_CTRL,
+                                                    "anchor", SP_ANCHOR_CENTER,
+                                                    "size", 10.0,
+                                                    "fill_color", 0x00ff00ff,
+                                                    "stroked", FALSE,
+                                                    "mode", SP_KNOT_MODE_XOR,
+                                                    "shape", SP_KNOT_SHAPE_DIAMOND,
+                                                    NULL );
+
+    SP_CTRL(canvasitem)->moveto(p);
+    _debugging_points.push_back(_desktop->add_temporary_canvasitem(canvasitem, 5000));
+
+}
+
 void
 SnapIndicator::remove_snapsource()
 {
@@ -339,6 +391,16 @@ SnapIndicator::remove_snapsource()
     }
 }
 
+void
+SnapIndicator::remove_debugging_points()
+{
+    for (std::list<TemporaryItem *>::const_iterator i = _debugging_points.begin(); i != _debugging_points.end(); ++i) {
+        _desktop->remove_temporary_canvasitem(*i);
+    }
+    _debugging_points.clear();
+}
+
+
 } //namespace Display
 } /* namespace Inkscape */
 
