@@ -1,10 +1,11 @@
-#ifndef __INKSCAPE_CTRLRECT_H__
-#define __INKSCAPE_CTRLRECT_H__
+#ifndef SEEN_INKSCAPE_CTRLRECT_H
+#define SEEN_INKSCAPE_CTRLRECT_H
 
 /**
- * \file sodipodi-ctrlrect.h
- * \brief Simple non-transformed rectangle, usable for rubberband
- *
+ * @file
+ * Simple non-transformed rectangle, usable for rubberband.
+ */
+/*
  * Authors:
  *   Lauris Kaplinski <lauris@ximian.com>
  *   Carl Hetherington <inkscape@carlh.net>
@@ -17,13 +18,17 @@
  */
 
 #include <glib.h>
-#include "sp-canvas.h"
+#include "sp-canvas-item.h"
+#include <2geom/rect.h>
+#include <2geom/int-rect.h>
+
+struct SPCanvasBuf;
 
 #define SP_TYPE_CTRLRECT (sp_ctrlrect_get_type ())
-#define SP_CTRLRECT(obj) (GTK_CHECK_CAST((obj), SP_TYPE_CTRLRECT, CtrlRect))
-#define SP_CTRLRECT_CLASS(c) (GTK_CHECK_CLASS_CAST((c), SP_TYPE_CTRLRECT, SPCtrlRectClass))
-#define SP_IS_CTRLRECT(obj) (GTK_CHECK_TYPE ((obj), SP_TYPE_CTRLRECT))
-#define SP_IS_CTRLRECT_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), SP_TYPE_CTRLRECT))
+#define SP_CTRLRECT(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), SP_TYPE_CTRLRECT, CtrlRect))
+#define SP_CTRLRECT_CLASS(c) (G_TYPE_CHECK_CLASS_CAST((c), SP_TYPE_CTRLRECT, SPCtrlRectClass))
+#define SP_IS_CTRLRECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SP_TYPE_CTRLRECT))
+#define SP_IS_CTRLRECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SP_TYPE_CTRLRECT))
 
 class CtrlRect : public SPCanvasItem
 {
@@ -36,7 +41,7 @@ public:
     void setDashed(bool d);
 
     void render(SPCanvasBuf *buf);
-    void update(Geom::Matrix const &affine, unsigned int flags);
+    void update(Geom::Affine const &affine, unsigned int flags);
     
 private:
     void _requestUpdate();
@@ -44,7 +49,7 @@ private:
     Geom::Rect _rect;
     bool _has_fill;
     bool _dashed;
-    NRRectL _area;
+    Geom::OptIntRect _area;
     gint _shadow_size;
     guint32 _border_color;
     guint32 _fill_color;
@@ -54,9 +59,9 @@ private:
 
 struct SPCtrlRectClass : public SPCanvasItemClass {};
 
-GtkType sp_ctrlrect_get_type();
+GType sp_ctrlrect_get_type();
 
-#endif
+#endif // SEEN_RUBBERBAND_H
 
 /*
   Local Variables:
@@ -67,4 +72,4 @@ GtkType sp_ctrlrect_get_type();
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
