@@ -1,5 +1,3 @@
-
-
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  *
  */
@@ -44,16 +42,15 @@
 #include <cmath>
 #include <string.h>
 
-#include <gdk/gdkkeysyms.h>
-#include <gtk/gtk.h>
-
 #include "widgets/icon.h"
+#include <gdk/gdkkeysyms.h>
+
 #include "icon-size.h"
 #include "ege-adjustment-action.h"
+#include "ui/widget/gimpspinscale.h"
+#include "ui/icon-names.h"
 
 
-static void ege_adjustment_action_class_init( EgeAdjustmentActionClass* klass );
-static void ege_adjustment_action_init( EgeAdjustmentAction* action );
 static void ege_adjustment_action_finalize( GObject* object );
 static void ege_adjustment_action_get_property( GObject* obj, guint propId, GValue* value, GParamSpec * pspec );
 static void ege_adjustment_action_set_property( GObject* obj, guint propId, const GValue *value, GParamSpec* pspec );
@@ -72,28 +69,25 @@ static void ege_adjustment_action_defocus( EgeAdjustmentAction* action );
 static void egeAct_free_description( gpointer data, gpointer user_data );
 static void egeAct_free_all_descriptions( EgeAdjustmentAction* action );
 
-
-static GtkActionClass* gParentClass = 0;
+static EgeCreateAdjWidgetCB gFactoryCb = 0;
 static GQuark gDataName = 0;
 
 enum {
     APPEARANCE_UNKNOWN = -1,
     APPEARANCE_NONE = 0,
-    APPEARANCE_FULL,    // label, then all choices represented by separate buttons
-    APPEARANCE_COMPACT, // label, then choices in a drop-down menu
-    APPEARANCE_MINIMAL, // no label, just choices in a drop-down menu
+    APPEARANCE_FULL,    /* label, then all choices represented by separate buttons */
+    APPEARANCE_COMPACT, /* label, then choices in a drop-down menu */
+    APPEARANCE_MINIMAL, /* no label, just choices in a drop-down menu */
 };
 
-#if GTK_CHECK_VERSION(2,12,0)
 /* TODO need to have appropriate icons setup for these: */
 static const gchar *floogles[] = {
-    GTK_STOCK_REMOVE,
-    GTK_STOCK_ADD,
-    GTK_STOCK_GO_DOWN,
-    GTK_STOCK_ABOUT,
-    GTK_STOCK_GO_UP,
+    INKSCAPE_ICON("list-remove"),
+    INKSCAPE_ICON("list-add"),
+    INKSCAPE_ICON("go-down"),
+    INKSCAPE_ICON("help-about"),
+    INKSCAPE_ICON("go-up"),
     0};
-#endif /* GTK_CHECK_VERSION(2,12,0) */
 
 typedef struct _EgeAdjustmentDescr EgeAdjustmentDescr;
 
@@ -106,7 +100,6 @@ struct _EgeAdjustmentDescr
 struct _EgeAdjustmentActionPrivate
 {
     GtkAdjustment* adj;
-    GtkTooltips* toolTips;
     GtkWidget* focusWidget;
     gdouble climbRate;
     guint digits;
@@ -123,6 +116,7 @@ struct _EgeAdjustmentActionPrivate
     gchar* appearance;
     gchar* iconId;
     Inkscape::IconSize iconSize;
+    Inkscape::UI::Widget::UnitTracker *unitTracker;
 };
 
 #define EGE_ADJUSTMENT_ACTION_GET_PRIVATE( o ) ( G_TYPE_INSTANCE_GET_PRIVATE( (o), EGE_ADJUSTMENT_ACTION_TYPE, EgeAdjustmentActionPrivate ) )
@@ -136,7 +130,8 @@ enum {
     PROP_TOOL_POST,
     PROP_APPEARANCE,
     PROP_ICON_ID,
-    PROP_ICON_SIZE
+    PROP_ICON_SIZE,
+    PROP_UNIT_TRACKER
 };
 
 enum {
@@ -150,38 +145,16 @@ enum {
     BUMP_CUSTOM = 100
 };
 
-GType ege_adjustment_action_get_type( void )
-{
-    static GType myType = 0;
-    if ( !myType ) {
-        static const GTypeInfo myInfo = {
-            sizeof( EgeAdjustmentActionClass ),
-            NULL, /* base_init */
-            NULL, /* base_finalize */
-            (GClassInitFunc)ege_adjustment_action_class_init,
-            NULL, /* class_finalize */
-            NULL, /* class_data */
-            sizeof( EgeAdjustmentAction ),
-            0, /* n_preallocs */
-            (GInstanceInitFunc)ege_adjustment_action_init,
-            NULL
-        };
-
-        myType = g_type_register_static( GTK_TYPE_ACTION, "EgeAdjustmentAction", &myInfo, (GTypeFlags)0 );
-    }
-
-    return myType;
-}
-
+G_DEFINE_TYPE(EgeAdjustmentAction, ege_adjustment_action, GTK_TYPE_ACTION);
 
 static void ege_adjustment_action_class_init( EgeAdjustmentActionClass* klass )
 {
     if ( klass ) {
-        gParentClass = GTK_ACTION_CLASS( g_type_class_peek_parent( klass ) );
         GObjectClass * objClass = G_OBJECT_CLASS( klass );
 
         gDataName = g_quark_from_string("ege-adj-action");
 
+  
         objClass->finalize = ege_adjustment_action_finalize;
 
         objClass->get_property = ege_adjustment_action_get_property;
@@ -264,15 +237,26 @@ static void ege_adjustment_action_class_init( EgeAdjustmentActionClass* klass )
                                                            (int)Inkscape::ICON_SIZE_SMALL_TOOLBAR,
                                                            (GParamFlags)(G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT) ) );
 
+        g_object_class_install_property( objClass,
+                                         PROP_UNIT_TRACKER,
+                                         g_param_spec_pointer( "unit_tracker",
+                                                               "Unit Tracker",
+                                                               "The widget that keeps track of the unit",
+                                                               (GParamFlags)(G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT) ) );
+
         g_type_class_add_private( klass, sizeof(EgeAdjustmentActionClass) );
     }
 }
 
+void ege_adjustment_action_set_compact_tool_factory( EgeCreateAdjWidgetCB factoryCb )
+{
+    gFactoryCb = factoryCb;
+}
+
 static void ege_adjustment_action_init( EgeAdjustmentAction* action )
 {
     action->private_data = EGE_ADJUSTMENT_ACTION_GET_PRIVATE( action );
     action->private_data->adj = 0;
-    action->private_data->toolTips = 0;
     action->private_data->focusWidget = 0;
     action->private_data->climbRate = 0.0;
     action->private_data->digits = 2;
@@ -289,6 +273,7 @@ static void ege_adjustment_action_init( EgeAdjustmentAction* action )
     action->private_data->appearance = 0;
     action->private_data->iconId = 0;
     action->private_data->iconSize = Inkscape::ICON_SIZE_SMALL_TOOLBAR;
+    action->private_data->unitTracker = NULL;
 }
 
 static void ege_adjustment_action_finalize( GObject* object )
@@ -299,15 +284,16 @@ static void ege_adjustment_action_finalize( GObject* object )
 
     action = EGE_ADJUSTMENT_ACTION( object );
 
-    if ( action->private_data->format ) {
-        g_free( action->private_data->format );
-        action->private_data->format = 0;
-    }
+    // g_free(NULL) does nothing
+    g_free( action->private_data->format );
+    g_free( action->private_data->selfId );
+    g_free( action->private_data->appearance );
+    g_free( action->private_data->iconId );
 
     egeAct_free_all_descriptions( action );
 
-    if ( G_OBJECT_CLASS(gParentClass)->finalize ) {
-        (*G_OBJECT_CLASS(gParentClass)->finalize)(object);
+    if ( G_OBJECT_CLASS(ege_adjustment_action_parent_class)->finalize ) {
+        (*G_OBJECT_CLASS(ege_adjustment_action_parent_class)->finalize)(object);
     }
 }
 
@@ -317,7 +303,8 @@ EgeAdjustmentAction* ege_adjustment_action_new( GtkAdjustment* adjustment,
                                                 const gchar *tooltip,
                                                 const gchar *stock_id,
                                                 gdouble climb_rate,
-                                                guint digits )
+                                                guint digits,
+                                                Inkscape::UI::Widget::UnitTracker *unit_tracker )
 {
     GObject* obj = (GObject*)g_object_new( EGE_ADJUSTMENT_ACTION_TYPE,
                                            "name", name,
@@ -327,6 +314,7 @@ EgeAdjustmentAction* ege_adjustment_action_new( GtkAdjustment* adjustment,
                                            "adjustment", adjustment,
                                            "climb-rate", climb_rate,
                                            "digits", digits,
+                                           "unit_tracker", unit_tracker,
                                            NULL );
 
     EgeAdjustmentAction* action = EGE_ADJUSTMENT_ACTION( obj );
@@ -368,11 +356,15 @@ static void ege_adjustment_action_get_property( GObject* obj, guint propId, GVal
 
         case PROP_ICON_ID:
             g_value_set_string( value, action->private_data->iconId );
-	    break;
+            break;
 
         case PROP_ICON_SIZE:
             g_value_set_int( value, action->private_data->iconSize );
-	    break;
+            break;
+
+        case PROP_UNIT_TRACKER:
+            g_value_set_pointer( value, action->private_data->unitTracker );
+            break;
 
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID( obj, propId, pspec );
@@ -475,6 +467,12 @@ void ege_adjustment_action_set_property( GObject* obj, guint propId, const GValu
         }
         break;
 
+        case PROP_UNIT_TRACKER:
+        {
+            action->private_data->unitTracker = (Inkscape::UI::Widget::UnitTracker*)g_value_get_pointer( value );
+        }
+        break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID( obj, propId, pspec );
     }
@@ -567,7 +565,7 @@ void ege_adjustment_action_set_appearance( EgeAdjustmentAction* action, gchar co
 static void process_menu_action( GtkWidget* obj, gpointer data )
 {
     GtkCheckMenuItem* item = GTK_CHECK_MENU_ITEM(obj);
-    if ( item->active ) {
+    if ( gtk_check_menu_item_get_active (item)) {
         EgeAdjustmentAction* act = (EgeAdjustmentAction*)g_object_get_qdata( G_OBJECT(obj), gDataName );
         gint what = GPOINTER_TO_INT(data);
 
@@ -757,30 +755,28 @@ static GtkWidget* create_menu_item( GtkAction* action )
     if ( IS_EGE_ADJUSTMENT_ACTION(action) ) {
         EgeAdjustmentAction* act = EGE_ADJUSTMENT_ACTION( action );
         GValue value;
-        const gchar*  sss = 0;
         GtkWidget*  subby = 0;
 
         memset( &value, 0, sizeof(value) );
         g_value_init( &value, G_TYPE_STRING );
         g_object_get_property( G_OBJECT(action), "label", &value );
 
-        sss = g_value_get_string( &value );
-
-        item = gtk_menu_item_new_with_label( sss );
+        item = gtk_menu_item_new_with_label( g_value_get_string( &value ) );
 
         subby = create_popup_number_menu( act );
         gtk_menu_item_set_submenu( GTK_MENU_ITEM(item), subby );
         gtk_widget_show_all( subby );
+        g_value_unset( &value );
     } else {
-        item = gParentClass->create_menu_item( action );
+        item = GTK_ACTION_CLASS(ege_adjustment_action_parent_class)->create_menu_item( action );
     }
 
     return item;
 }
 
-void value_changed_cb( GtkSpinButton* spin, EgeAdjustmentAction* act )
+static void value_changed_cb( GtkSpinButton* spin, EgeAdjustmentAction* act )
 {
-    if ( GTK_WIDGET_HAS_FOCUS( GTK_WIDGET(spin) ) ) {
+    if ( gtk_widget_has_focus( GTK_WIDGET(spin) ) ) {
         gint start = 0, end = 0;
         if (GTK_IS_EDITABLE(spin) && gtk_editable_get_selection_bounds (GTK_EDITABLE(spin), &start, &end)
                 && start != end) {
@@ -810,13 +806,6 @@ static gboolean event_cb( EgeAdjustmentAction* act, GdkEvent* evt )
     return handled;
 }
 
-static gchar*
-slider_format_falue (GtkScale* scale, gdouble value, gchar *label)
-{
-    (void)scale;
-    return g_strdup_printf("%s %d", label, (int) round(value));
-}
-
 static GtkWidget* create_tool_item( GtkAction* action )
 {
     GtkWidget* item = 0;
@@ -824,29 +813,32 @@ static GtkWidget* create_tool_item( GtkAction* action )
     if ( IS_EGE_ADJUSTMENT_ACTION(action) ) {
         EgeAdjustmentAction* act = EGE_ADJUSTMENT_ACTION( action );
         GtkWidget* spinbutton = 0;
+#if GTK_CHECK_VERSION(3,0,0)
+	GtkWidget* hb = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
+	gtk_box_set_homogeneous(GTK_BOX(hb), FALSE);
+#else
         GtkWidget* hb = gtk_hbox_new( FALSE, 5 );
-
+#endif
         GValue value;
         memset( &value, 0, sizeof(value) );
         g_value_init( &value, G_TYPE_STRING );
         g_object_get_property( G_OBJECT(action), "short_label", &value );
-        const gchar* sss = g_value_get_string( &value );
 
         if ( act->private_data->appearanceMode == APPEARANCE_FULL ) {
-	    // Slider
-	    spinbutton = gtk_hscale_new( act->private_data->adj);
+            /* Slider */
+            spinbutton  = gimp_spin_scale_new (act->private_data->adj, g_value_get_string( &value ), 0);
             gtk_widget_set_size_request(spinbutton, 100, -1);
-            gtk_scale_set_digits (GTK_SCALE(spinbutton), 0);
-            gtk_signal_connect(GTK_OBJECT(spinbutton), "format-value", GTK_SIGNAL_FUNC(slider_format_falue), (void *) sss);
 
-#if GTK_CHECK_VERSION(2,12,0)
         } else if ( act->private_data->appearanceMode == APPEARANCE_MINIMAL ) {
             spinbutton = gtk_scale_button_new( GTK_ICON_SIZE_MENU, 0, 100, 2, 0 );
             gtk_scale_button_set_adjustment( GTK_SCALE_BUTTON(spinbutton), act->private_data->adj );
             gtk_scale_button_set_icons( GTK_SCALE_BUTTON(spinbutton), floogles );
-#endif /* GTK_CHECK_VERSION(2,12,0) */
         } else {
-            spinbutton = gtk_spin_button_new( act->private_data->adj, act->private_data->climbRate, act->private_data->digits );
+            if ( gFactoryCb ) {
+                spinbutton = gFactoryCb( act->private_data->adj, act->private_data->climbRate, act->private_data->digits, act->private_data->unitTracker );
+            } else {
+                spinbutton = gtk_spin_button_new( act->private_data->adj, act->private_data->climbRate, act->private_data->digits );
+            }
         }
 
         item = GTK_WIDGET( gtk_tool_item_new() );
@@ -858,26 +850,24 @@ static GtkWidget* create_tool_item( GtkAction* action )
             g_object_get_property( G_OBJECT(action), "tooltip", &tooltip );
             const gchar* tipstr = g_value_get_string( &tooltip );
             if ( tipstr && *tipstr ) {
-                if ( !act->private_data->toolTips ) {
-                    act->private_data->toolTips = gtk_tooltips_new();
-                }
-                gtk_tooltips_set_tip( act->private_data->toolTips, spinbutton, tipstr, 0 );
+                gtk_widget_set_tooltip_text( spinbutton, tipstr );
             }
+            g_value_unset( &tooltip );
         }
 
         if ( act->private_data->appearanceMode != APPEARANCE_FULL ) {
-	    GtkWidget* filler1 = gtk_label_new(" ");
-	    gtk_box_pack_start( GTK_BOX(hb), filler1, FALSE, FALSE, 0 );
+            GtkWidget* filler1 = gtk_label_new(" ");
+            gtk_box_pack_start( GTK_BOX(hb), filler1, FALSE, FALSE, 0 );
 
-	    // Use an icon if available or use short-label
-	    if ( act->private_data->iconId && strcmp( act->private_data->iconId, "" ) != 0 ) {
+            /* Use an icon if available or use short-label */
+            if ( act->private_data->iconId && strcmp( act->private_data->iconId, "" ) != 0 ) {
                 GtkWidget* icon = sp_icon_new( act->private_data->iconSize, act->private_data->iconId );
                 gtk_box_pack_start( GTK_BOX(hb), icon, FALSE, FALSE, 0 );
-	    } else {
-                GtkWidget* lbl = gtk_label_new( sss ? sss : "wwww" );
+            } else {
+                GtkWidget* lbl = gtk_label_new( g_value_get_string( &value ) ? g_value_get_string( &value ) : "wwww" );
                 gtk_misc_set_alignment( GTK_MISC(lbl), 1.0, 0.5 );
                 gtk_box_pack_start( GTK_BOX(hb), lbl, FALSE, FALSE, 0 );
-	    }
+            }
         }
 
         if ( act->private_data->appearanceMode == APPEARANCE_FULL ) {
@@ -901,10 +891,8 @@ static GtkWidget* create_tool_item( GtkAction* action )
         g_signal_connect_swapped( G_OBJECT(spinbutton), "event", G_CALLBACK(event_cb), action );
         if ( act->private_data->appearanceMode == APPEARANCE_FULL ) {
             /* */
-#if GTK_CHECK_VERSION(2,12,0)
         } else if ( act->private_data->appearanceMode == APPEARANCE_MINIMAL ) {
             /* */
-#endif /* GTK_CHECK_VERSION(2,12,0) */
         } else {
             gtk_entry_set_width_chars( GTK_ENTRY(spinbutton), act->private_data->digits + 3 );
         }
@@ -915,8 +903,10 @@ static GtkWidget* create_tool_item( GtkAction* action )
         if ( act->private_data->toolPost ) {
             act->private_data->toolPost( item );
         }
+
+        g_value_unset( &value );
     } else {
-        item = gParentClass->create_tool_item( action );
+        item = GTK_ACTION_CLASS(ege_adjustment_action_parent_class)->create_tool_item( action );
     }
 
     return item;
@@ -924,12 +914,12 @@ static GtkWidget* create_tool_item( GtkAction* action )
 
 static void connect_proxy( GtkAction *action, GtkWidget *proxy )
 {
-    gParentClass->connect_proxy( action, proxy );
+    GTK_ACTION_CLASS(ege_adjustment_action_parent_class)->connect_proxy( action, proxy );
 }
 
 static void disconnect_proxy( GtkAction *action, GtkWidget *proxy )
 {
-    gParentClass->disconnect_proxy( action, proxy );
+    GTK_ACTION_CLASS(ege_adjustment_action_parent_class)->disconnect_proxy( action, proxy );
 }
 
 void ege_adjustment_action_defocus( EgeAdjustmentAction* action )
@@ -948,10 +938,8 @@ gboolean focus_in_cb( GtkWidget *widget, GdkEventKey *event, gpointer data )
         EgeAdjustmentAction* action = EGE_ADJUSTMENT_ACTION( data );
         if ( GTK_IS_SPIN_BUTTON(widget) ) {
             action->private_data->lastVal = gtk_spin_button_get_value( GTK_SPIN_BUTTON(widget) );
-#if GTK_CHECK_VERSION(2,12,0)
         } else if ( GTK_IS_SCALE_BUTTON(widget) ) {
             action->private_data->lastVal = gtk_scale_button_get_value( GTK_SCALE_BUTTON(widget) );
-#endif /* GTK_CHECK_VERSION(2,12,0) */
         } else if (GTK_IS_RANGE(widget) ) {
             action->private_data->lastVal = gtk_range_get_value( GTK_RANGE(widget) );
         }
@@ -995,11 +983,11 @@ static gboolean process_tab( GtkWidget* widget, int direction )
                     if ( mid && GTK_IS_TOOL_ITEM(mid->data) ) {
                         /* potential target */
                         GtkWidget* child = gtk_bin_get_child( GTK_BIN(mid->data) );
-                        if ( child && GTK_IS_HBOX(child) ) { /* could be ours */
+                        if ( child && GTK_IS_BOX(child) ) { /* could be ours */
                             GList* subChildren = gtk_container_get_children( GTK_CONTAINER(child) );
                             if ( subChildren ) {
                                 GList* last = g_list_last(subChildren);
-                                if ( last && GTK_IS_SPIN_BUTTON(last->data) && GTK_WIDGET_IS_SENSITIVE( GTK_WIDGET(last->data) ) ) {
+                                if ( last && GTK_IS_SPIN_BUTTON(last->data) && gtk_widget_is_sensitive( GTK_WIDGET(last->data) ) ) {
                                     gtk_widget_grab_focus( GTK_WIDGET(last->data) );
                                     handled = TRUE;
                                     mid = 0; /* to stop loop */
@@ -1028,7 +1016,7 @@ gboolean keypress_cb( GtkWidget *widget, GdkEventKey *event, gpointer data )
                                          0, &key, 0, 0, 0 );
 
     switch ( key ) {
-        case GDK_Escape:
+        case GDK_KEY_Escape:
         {
             action->private_data->transferFocus = TRUE;
             gtk_spin_button_set_value( GTK_SPIN_BUTTON(widget), action->private_data->lastVal );
@@ -1037,8 +1025,8 @@ gboolean keypress_cb( GtkWidget *widget, GdkEventKey *event, gpointer data )
         }
         break;
 
-        case GDK_Return:
-        case GDK_KP_Enter:
+        case GDK_KEY_Return:
+        case GDK_KEY_KP_Enter:
         {
             action->private_data->transferFocus = TRUE;
             ege_adjustment_action_defocus( action );
@@ -1046,22 +1034,22 @@ gboolean keypress_cb( GtkWidget *widget, GdkEventKey *event, gpointer data )
         }
         break;
 
-        case GDK_Tab:
+        case GDK_KEY_Tab:
         {
             action->private_data->transferFocus = FALSE;
             wasConsumed = process_tab( widget, 1 );
         }
         break;
 
-        case GDK_ISO_Left_Tab:
+        case GDK_KEY_ISO_Left_Tab:
         {
             action->private_data->transferFocus = FALSE;
             wasConsumed = process_tab( widget, -1 );
         }
         break;
 
-        case GDK_Up:
-        case GDK_KP_Up:
+        case GDK_KEY_Up:
+        case GDK_KEY_KP_Up:
         {
             action->private_data->transferFocus = FALSE;
             gdouble val = gtk_spin_button_get_value( GTK_SPIN_BUTTON(widget) );
@@ -1070,8 +1058,8 @@ gboolean keypress_cb( GtkWidget *widget, GdkEventKey *event, gpointer data )
         }
         break;
 
-        case GDK_Down:
-        case GDK_KP_Down:
+        case GDK_KEY_Down:
+        case GDK_KEY_KP_Down:
         {
             action->private_data->transferFocus = FALSE;
             gdouble val = gtk_spin_button_get_value( GTK_SPIN_BUTTON(widget) );
@@ -1080,8 +1068,8 @@ gboolean keypress_cb( GtkWidget *widget, GdkEventKey *event, gpointer data )
         }
         break;
 
-        case GDK_Page_Up:
-        case GDK_KP_Page_Up:
+        case GDK_KEY_Page_Up:
+        case GDK_KEY_KP_Page_Up:
         {
             action->private_data->transferFocus = FALSE;
             gdouble val = gtk_spin_button_get_value( GTK_SPIN_BUTTON(widget) );
@@ -1090,8 +1078,8 @@ gboolean keypress_cb( GtkWidget *widget, GdkEventKey *event, gpointer data )
         }
         break;
 
-        case GDK_Page_Down:
-        case GDK_KP_Page_Down:
+        case GDK_KEY_Page_Down:
+        case GDK_KEY_KP_Page_Down:
         {
             action->private_data->transferFocus = FALSE;
             gdouble val = gtk_spin_button_get_value( GTK_SPIN_BUTTON(widget) );
@@ -1100,8 +1088,8 @@ gboolean keypress_cb( GtkWidget *widget, GdkEventKey *event, gpointer data )
         }
         break;
 
-        case GDK_z:
-        case GDK_Z:
+        case GDK_KEY_z:
+        case GDK_KEY_Z:
         {
             action->private_data->transferFocus = FALSE;
             gtk_spin_button_set_value( GTK_SPIN_BUTTON(widget), action->private_data->lastVal );
@@ -1113,3 +1101,13 @@ gboolean keypress_cb( GtkWidget *widget, GdkEventKey *event, gpointer data )
 
     return wasConsumed;
 }
+/*
+  Local Variables:
+  mode:c++
+  c-file-style:"stroustrup"
+  c-file-offsets:((innamespace . 0)(inline-open . 0)(case-label . +))
+  indent-tabs-mode:nil
+  fill-column:99
+  End:
+*/
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
