@@ -63,6 +63,14 @@ typedef struct _EgeAdjustmentAction      EgeAdjustmentAction;
 typedef struct _EgeAdjustmentActionClass EgeAdjustmentActionClass;
 typedef struct _EgeAdjustmentActionPrivate EgeAdjustmentActionPrivate;
 
+namespace Inkscape {
+    namespace UI {
+        namespace Widget {
+            class UnitTracker;
+        }
+    }
+}
+
 /**
  * Instance structure of EgeAdjustmentAction.
  */
@@ -87,6 +95,24 @@ struct _EgeAdjustmentActionClass
 /** Standard Gtk type function */
 GType ege_adjustment_action_get_type( void );
 
+
+/*
+ * Note: This normally could be implemented via a GType property for the class to construct,
+ * but gtkmm classes implemented in C++ only will often not funciton properly.
+ *
+ */
+
+/** Callback type for widgets creation factory */
+typedef GtkWidget* (*EgeCreateAdjWidgetCB)( GtkAdjustment *adjustment, gdouble climb_rate, guint digits, Inkscape::UI::Widget::UnitTracker *unit_tracker );
+
+/**
+ * Sets a factory callback to be used to create the specific widget.
+ *
+ * @param factoryCb the callback to use to create custom widgets, NULL to use the default.
+ */
+void ege_adjustment_action_set_compact_tool_factory( EgeCreateAdjWidgetCB factoryCb );
+
+
 /**
  * Creates a new EgeAdjustmentAction instance.
  * This is a GtkAction subclass that manages a value stored in a
@@ -99,6 +125,7 @@ GType ege_adjustment_action_get_type( void );
  * @param stock_id Icon id to use.
  * @param climb_rate Used for created widgets.
  * @param digits Used for created widgets.
+ * @param unit_tracker Used to store unit.
  */
 EgeAdjustmentAction* ege_adjustment_action_new( GtkAdjustment* adjustment,
                                                 const gchar *name,
@@ -106,7 +133,8 @@ EgeAdjustmentAction* ege_adjustment_action_new( GtkAdjustment* adjustment,
                                                 const gchar *tooltip,
                                                 const gchar *stock_id,
                                                 gdouble climb_rate,
-                                                guint digits
+                                                guint digits,
+                                                Inkscape::UI::Widget::UnitTracker *unit_tracker
                                                 );
 /**
  * Returns a pointer to the GtkAdjustment represented by the given
