@@ -1,6 +1,38 @@
-/**
- * Inkscape::EventLog
+/*
+ * Author:
+ *   Gustav Broberg <broberg@kth.se>
+ *   Jon A. Cruz <jon@joncruz.org>
  *
+ * Copyright (c) 2014 Authors
+ *
+ * Released under GNU GPL, read the file 'COPYING' for more information
+ */
+
+#ifndef INKSCAPE_EVENT_LOG_H
+#define INKSCAPE_EVENT_LOG_H
+
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#if GLIBMM_DISABLE_DEPRECATED && HAVE_GLIBMM_THREADS_H
+#include <glibmm/threads.h>
+#endif
+
+#include <gtkmm/treestore.h>
+#include <glibmm/refptr.h>
+#include <gtkmm/treeselection.h>
+#include <gtkmm/treeview.h>
+#include <sigc++/trackable.h>
+
+#include "undo-stack-observer.h"
+#include "event.h"
+
+namespace Inkscape {
+
+class EventLogPrivate;
+
+/**
  * A simple log for maintaining a history of commited, undone and redone events along with their
  * type. It implements the UndoStackObserver and should be registered with a
  * CompositeUndoStackObserver for each document. The event log is then notified on all commit, undo
@@ -12,33 +44,9 @@
  * If a Gtk::TreeView is connected to the event log, the TreeView's selection and its nodes
  * expanded/collapsed state will be updated as events are commited, undone and redone. Whenever
  * this happens, the event log will block the TreeView's callbacks to prevent circular updates.
- *
- * Author:
- *   Gustav Broberg <broberg@kth.se>
- *
- * Copyright (c) 2006, 2007 Authors
- *
- * Released under GNU GPL, read the file 'COPYING' for more information
  */
-
-#ifndef INKSCAPE_EVENT_LOG_H
-#define INKSCAPE_EVENT_LOG_H
-
-#include <gdkmm/pixbuf.h>
-#include <glibmm/refptr.h>
-#include <gtkmm/treestore.h>
-#include <gtkmm/treeselection.h>
-#include <gtkmm/treeview.h>
-
-#include "undo-stack-observer.h"
-#include "event.h"
-
-namespace Inkscape {
-
-/**
- *     
- */
-class EventLog : public UndoStackObserver {
+class EventLog : public UndoStackObserver, public sigc::trackable
+{
         
 public:
     typedef Gtk::TreeModel::iterator iterator;
@@ -50,7 +58,6 @@ public:
     /**
      * Event datatype
      */
-
     struct EventModelColumns : public Gtk::TreeModelColumnRecord
     {
         Gtk::TreeModelColumn<Event *> event;
@@ -64,20 +71,18 @@ public:
         }
     };
 
-    /**
-     * Implementation of Inkscape::UndoStackObserver methods
-     * \brief Modifies the log's entries and the view's selection when triggered
-     */
+    // Implementation of Inkscape::UndoStackObserver methods
 
+    /**
+     * Modifies the log's entries and the view's selection when triggered.
+     */
     void notifyUndoEvent(Event *log);
     void notifyRedoEvent(Event *log);
     void notifyUndoCommitEvent(Event *log);
     void notifyClearUndoEvent();
     void notifyClearRedoEvent();
 
-    /**
-     * Accessor functions
-     */
+    // Accessor functions
 
     Glib::RefPtr<Gtk::TreeModel> getEventListStore() const { return _event_list_store; }
     const EventModelColumns& getColumns() const            { return _columns; }
@@ -89,9 +94,7 @@ public:
     void blockNotifications(bool status=true)  { _notifications_blocked = status; }
     void rememberFileSave()                    { _last_saved = _curr_event; }
 
-    /* 
-     * Callback types for TreeView changes.
-     */
+    // Callback types for TreeView changes.
 
     enum CallbackTypes { 
         CALLB_SELECTION_CHANGE, 
@@ -105,7 +108,12 @@ public:
     /**
      * Connect with a TreeView.
      */
-    void connectWithDialog(Gtk::TreeView *event_list_view, CallbackMap *callback_connections);
+    void addDialogConnection(Gtk::TreeView *event_list_view, CallbackMap *callback_connections);
+
+    /**
+     * Disconnect from a TreeView.
+     */
+    void removeDialogConnection(Gtk::TreeView *event_list_view, CallbackMap *callback_connections);
 
     /*
      * Updates the sensitivity and names of SP_VERB_EDIT_UNDO and SP_VERB_EDIT_REDO to reflect the
@@ -114,14 +122,13 @@ public:
     void updateUndoVerbs();
 
 private:
-    bool _connected;             //< connected with dialog
+    EventLogPrivate *_priv;
+
     SPDocument *_document;       //< document that is logged
 
     const EventModelColumns _columns;
 
     Glib::RefPtr<Gtk::TreeStore> _event_list_store; 
-    Glib::RefPtr<Gtk::TreeSelection> _event_list_selection;
-    Gtk::TreeView *_event_list_view;
 
     iterator _curr_event;        //< current event in _event_list_store
     iterator _last_event;        //< end position in _event_list_store
@@ -131,12 +138,7 @@ private:
 
     bool _notifications_blocked; //< if notifications should be handled
 
-    // Map of connections used to temporary block/unblock callbacks in a TreeView
-    CallbackMap *_callback_connections;
-
-    /**
-     * Helper functions
-     */
+    // Helper functions
 
     const_iterator _getUndoEvent() const; //< returns the current undoable event or NULL if none
     const_iterator _getRedoEvent() const; //< returns the current redoable event or NULL if none
@@ -149,7 +151,6 @@ private:
     // noncopyable, nonassignable
     EventLog(EventLog const &other);
     EventLog& operator=(EventLog const &other);
-
 };
 
 } // namespace Inkscape
@@ -165,4 +166,4 @@ private:
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
