@@ -12,12 +12,11 @@
 #include "config.h"
 #endif
 
+#include "ui/dialog/extensions.h"
+
 #include <glibmm/i18n.h>
-
-
 #include "inkscape.h"
 #include "preferences.h"
-#include "ui/dialog/extensions.h"
 #include "extension/extension.h"
 
 #include "error-file.h"
@@ -40,7 +39,7 @@ namespace Extension {
     probably good to check anyway).
 */
 ErrorFileNotice::ErrorFileNotice (void) :
-    Gtk::MessageDialog::MessageDialog(
+    Gtk::MessageDialog(
             "",                    /* message */
             false,                 /* use markup */
             Gtk::MESSAGE_WARNING,  /* dialog type */
@@ -56,7 +55,11 @@ ErrorFileNotice::ErrorFileNotice (void) :
     g_free(ext_error_file);
     set_message(dialog_text, true);
 
-    Gtk::VBox * vbox = get_vbox();
+#if WITH_GTKMM_3_0
+    Gtk::Box * vbox = get_content_area();
+#else
+    Gtk::Box * vbox = get_vbox();
+#endif
 
     /* This is some filler text, needs to change before relase */
     Inkscape::Preferences *prefs = Inkscape::Preferences::get();
