@@ -1,5 +1,5 @@
-#ifndef __INK_EXTENSION_H__
-#define __INK_EXTENSION_H__
+#ifndef INK_EXTENSION_H
+#define INK_EXTENSION_H
 
 /** \file
  * Frontend to certain, possibly pluggable, actions.
@@ -17,12 +17,20 @@
 #include <ostream>
 #include <fstream>
 #include <vector>
-#include <gtkmm/widget.h>
-#include <gtkmm/box.h>
-#include <gtkmm/table.h>
 #include <glibmm/ustring.h>
 #include "xml/repr.h"
-#include "extension/extension-forward.h"
+#include <sigc++/signal.h>
+
+namespace Gtk {
+#if WITH_GTKMM_3_0
+	class Grid;
+#else
+	class Table;
+#endif
+
+	class VBox;
+	class Widget;
+}
 
 /** The key that is used to identify that the I/O should be autodetected */
 #define SP_MODULE_KEY_AUTODETECT "autodetect"
@@ -51,15 +59,6 @@
 #define SP_MODULE_KEY_PRINT_LATEX    "org.inkscape.print.latex"
 /** Defines the key for printing with GNOME Print */
 #define SP_MODULE_KEY_PRINT_GNOME "org.inkscape.print.gnome"
-/** Defines the key for printing under Win32 */
-#define SP_MODULE_KEY_PRINT_WIN32 "org.inkscape.print.win32"
-#ifdef WIN32
-/** Defines the default printing to use */
-#define SP_MODULE_KEY_PRINT_DEFAULT  SP_MODULE_KEY_PRINT_WIN32
-#else
-/** Defines the default printing to use */
-#define SP_MODULE_KEY_PRINT_DEFAULT  SP_MODULE_KEY_PRINT_PS
-#endif
 
 /** Mime type for SVG */
 #define MIME_SVG "image/svg+xml"
@@ -72,11 +71,22 @@
 #define INKSCAPE_EXTENSION_NS_NC "extension"
 #define INKSCAPE_EXTENSION_NS    "extension:"
 
-struct SPDocument;
+class SPDocument;
 
 namespace Inkscape {
 namespace Extension {
 
+class Dependency;
+class ExpirationTimer;
+class ExpirationTimer;
+class Parameter;
+
+namespace Implementation
+{
+class Implementation;
+}
+
+
 /** The object that is the basis for the Extension system.  This object
     contains all of the information that all Extension have.  The
     individual items are detailed within. This is the interface that
@@ -99,6 +109,7 @@ private:
     state_t    _state;                    /**< Which state the Extension is currently in */
     std::vector<Dependency *>  _deps;     /**< Dependencies for this extension */
     static std::ofstream error_file;      /**< This is the place where errors get reported */
+    bool silent;
     bool _gui;
 
 protected:
@@ -120,6 +131,7 @@ public:
     gchar *       get_name     (void);
     /** \brief  Gets the help string for this extension */
     gchar const * get_help     (void) { return _help; }
+    bool          is_silent (void);
     void          deactivate   (void);
     bool          deactivated  (void);
     void          printFailure (Glib::ustring reason);
@@ -132,12 +144,12 @@ private:
                               extension */
 
 public:
-    /** \brief  A function to get the the number of parameters that
+    /** \brief  A function to get the number of parameters that
                 the extension has.
         \return The number of parameters. */
     unsigned int param_count ( ) { return parameters == NULL ? 0 :
                                               g_slist_length(parameters); };
-    /** \brief  A function to get the the number of parameters that
+    /** \brief  A function to get the number of parameters that
                 are visible to the user that the extension has.
         \return The number of visible parameters.
 
@@ -169,10 +181,30 @@ public:
 
 private:
     void             make_param       (Inkscape::XML::Node * paramrepr);
-#if 0
-    inline param_t * param_shared     (const gchar * name,
-                                       GSList * list);
-#endif
+    
+    /**
+     * This function looks through the linked list for a parameter
+     * structure with the name of the passed in name.
+     *
+     * This is an inline function that is used by all the get_param and
+     * set_param functions to find a param_t in the linked list with
+     * the passed in name.
+     *
+     * This function can throw a 'param_not_exist' exception if the
+     * name is not found.
+     *
+     * The first thing that this function checks is if the list is NULL.
+     * It could be NULL because there are no parameters for this extension
+     * or because all of them have been checked.  If the list
+     * is NULL then the 'param_not_exist' exception is thrown.
+     *
+     * @param name The name to search for.
+     * @return Parameter structure with a name of 'name'.
+     */
+     Parameter *get_param(const gchar * name);
+
+     Parameter const *get_param(const gchar * name) const;
+
 public:
     bool             get_param_bool   (const gchar * name,
                                        const SPDocument *   doc = NULL,
@@ -186,21 +218,36 @@ public:
                                        const SPDocument *   doc = NULL,
                                        const Inkscape::XML::Node * node = NULL);
 
-    const gchar *    get_param_string (const gchar * name,
-                                       const SPDocument *   doc = NULL,
-                                       const Inkscape::XML::Node * node = NULL);
+    /**
+     * Gets a parameter identified by name with the string placed in value.
+     * It isn't duplicated into the value string. Look up in the parameters list,
+     * then execute the function on that found parameter.
+     *
+     * @param name The name of the parameter to get.
+     * @param doc The document to look in for document specific parameters.
+     * @param node The node to look in for a specific parameter.
+     * @return A constant pointer to the string held by the parameters.
+     */
+    gchar const *get_param_string(gchar const *name,
+                                  SPDocument const *doc = NULL,
+                                  Inkscape::XML::Node const *node = NULL) const;
 
     guint32          get_param_color  (const gchar * name,
                                        const SPDocument *   doc = NULL,
-                                       const Inkscape::XML::Node * node = NULL);
+                                       const Inkscape::XML::Node * node = NULL) const;
 
     const gchar *    get_param_enum   (const gchar * name,
                                        const SPDocument *   doc = NULL,
-                                       const Inkscape::XML::Node * node = NULL);
+                                       const Inkscape::XML::Node * node = NULL) const;
 
     gchar const *get_param_optiongroup( gchar const * name,
                                         SPDocument const *   doc = 0,
-                                        Inkscape::XML::Node const * node = 0);
+                                        Inkscape::XML::Node const * node = 0) const;
+
+    bool             get_param_enum_contains(gchar const * name,
+                                             gchar const * value,
+                                             SPDocument  * doc = 0x0,
+                                             Inkscape::XML::Node * node = 0x0) const;
 
     bool             set_param_bool   (const gchar * name,
                                        bool          value,
@@ -227,6 +274,11 @@ public:
                                         SPDocument * doc = 0,
                                         Inkscape::XML::Node * node = 0);
 
+    gchar const *    set_param_enum   (gchar const * name,
+                                       gchar const * value,
+                                       SPDocument * doc = 0x0,
+                                       Inkscape::XML::Node * node = 0x0);
+
     guint32          set_param_color  (const gchar * name,
                                        guint32 color,
                                        SPDocument *   doc = NULL,
@@ -248,8 +300,11 @@ public:
     Gtk::VBox *    get_help_widget(void);
     Gtk::VBox *    get_params_widget(void);
 protected:
+#if WITH_GTKMM_3_0
+    inline static void add_val(Glib::ustring labelstr, Glib::ustring valuestr, Gtk::Grid * table, int * row);
+#else
     inline static void add_val(Glib::ustring labelstr, Glib::ustring valuestr, Gtk::Table * table, int * row);
-
+#endif
 };
 
 
@@ -277,7 +332,7 @@ public:
 }  /* namespace Extension */
 }  /* namespace Inkscape */
 
-#endif /* __INK_EXTENSION_H__ */
+#endif // INK_EXTENSION_H
 
 /*
   Local Variables:
