@@ -14,7 +14,10 @@
 #define INKSCAPE_EXTENSION_IMPEMENTATION_SCRIPT_H_SEEN
 
 #include "implementation.h"
-#include <gtkmm/messagedialog.h>
+#include <gtkmm/enums.h>
+#include <glibmm/main.h>
+#include <glibmm/spawn.h>
+#include <glibmm/fileutils.h>
 
 namespace Inkscape {
 namespace XML {
@@ -28,7 +31,6 @@ namespace Extension {
 namespace Implementation {
 
 
-
 /**
  * Utility class used for loading and launching script extensions
  */
