@@ -11,11 +11,11 @@
 # include "config.h"
 #endif
 
+#include "prefdialog.h"
 #include "implementation/implementation.h"
 #include "timer.h"
 #include "input.h"
 #include "io/sys.h"
-#include "prefdialog.h"
 
 /* Inkscape::Extension::Input */
 
@@ -49,11 +49,11 @@ Input::Input (Inkscape::XML::Node * in_repr, Implementation::Implementation * in
     if (repr != NULL) {
         Inkscape::XML::Node * child_repr;
 
-        child_repr = sp_repr_children(repr);
+        child_repr = repr->firstChild();
 
         while (child_repr != NULL) {
             if (!strcmp(child_repr->name(), INKSCAPE_EXTENSION_NS "input")) {
-                child_repr = sp_repr_children(child_repr);
+                child_repr = child_repr->firstChild();
                 while (child_repr != NULL) {
                     char const * chname = child_repr->name();
 					if (!strncmp(chname, INKSCAPE_EXTENSION_NS_NC, strlen(INKSCAPE_EXTENSION_NS_NC))) {
@@ -63,32 +63,32 @@ Input::Input (Inkscape::XML::Node * in_repr, Implementation::Implementation * in
                         chname++;
                     if (!strcmp(chname, "extension")) {
                         g_free (extension);
-                        extension = g_strdup(sp_repr_children(child_repr)->content());
+                        extension = g_strdup(child_repr->firstChild()->content());
                     }
                     if (!strcmp(chname, "mimetype")) {
                         g_free (mimetype);
-                        mimetype = g_strdup(sp_repr_children(child_repr)->content());
+                        mimetype = g_strdup(child_repr->firstChild()->content());
                     }
                     if (!strcmp(chname, "filetypename")) {
                         g_free (filetypename);
-                        filetypename = g_strdup(sp_repr_children(child_repr)->content());
+                        filetypename = g_strdup(child_repr->firstChild()->content());
                     }
                     if (!strcmp(chname, "filetypetooltip")) {
                         g_free (filetypetooltip);
-                        filetypetooltip = g_strdup(sp_repr_children(child_repr)->content());
+                        filetypetooltip = g_strdup(child_repr->firstChild()->content());
                     }
                     if (!strcmp(chname, "output_extension")) {
                         g_free (output_extension);
-                        output_extension = g_strdup(sp_repr_children(child_repr)->content());
+                        output_extension = g_strdup(child_repr->firstChild()->content());
                     }
 
-                    child_repr = sp_repr_next(child_repr);
+                    child_repr = child_repr->next();
                 }
 
                 break;
             }
 
-            child_repr = sp_repr_next(child_repr);
+            child_repr = child_repr->next();
         }
 
     }
@@ -152,6 +152,10 @@ Input::open (const gchar *uri)
 
     SPDocument *const doc = imp->open(this, uri);
 
+    if (imp->wasCancelled()) {
+        throw Input::open_cancelled();
+    }
+
     return doc;
 }
 
@@ -227,8 +231,7 @@ Input::prefs (const gchar *uri)
 
     delete dialog;
 
-    if (response == Gtk::RESPONSE_OK) return true;
-    return false;
+    return (response == Gtk::RESPONSE_OK);
 }
 
 } }  /* namespace Inkscape, Extension */
