@@ -10,6 +10,7 @@
 #include "extension/system.h"
 
 #include "contrast.h"
+#include <Magick++.h>
 
 namespace Inkscape {
 namespace Extension {
@@ -38,7 +39,7 @@ Contrast::init(void)
 		"<inkscape-extension xmlns=\"" INKSCAPE_EXTENSION_URI "\">\n"
 			"<name>" N_("Contrast") "</name>\n"
 			"<id>org.inkscape.effect.bitmap.contrast</id>\n"
-			"<param name=\"sharpen\" gui-text=\"" N_("Adjust") "\" type=\"int\" min=\"0\" max=\"10\">0</param>\n"
+			"<param name=\"sharpen\" gui-text=\"" N_("Adjust:") "\" type=\"int\" min=\"0\" max=\"10\">0</param>\n"
 			"<effect>\n"
 				"<object-type>all</object-type>\n"
 				"<effects-menu>\n"
