@@ -10,6 +10,7 @@
 #include "extension/system.h"
 
 #include "despeckle.h"
+#include <Magick++.h>
 
 namespace Inkscape {
 namespace Extension {
@@ -22,7 +23,7 @@ Despeckle::applyEffect(Magick::Image *image) {
 }
 
 void
-Despeckle::refreshParameters(Inkscape::Extension::Effect *module) {	
+Despeckle::refreshParameters(Inkscape::Extension::Effect */*module*/) {	
 }
 
 #include "../clear-n_.h"
@@ -39,7 +40,7 @@ Despeckle::init(void)
 				"<effects-menu>\n"
 					"<submenu name=\"" N_("Raster") "\" />\n"
 				"</effects-menu>\n"
-				"<menu-tip>" N_("Reduce speckle noise of selected bitmap(s).") "</menu-tip>\n"
+				"<menu-tip>" N_("Reduce speckle noise of selected bitmap(s)") "</menu-tip>\n"
 			"</effect>\n"
 		"</inkscape-extension>\n", new Despeckle());
 }
