@@ -1,5 +1,5 @@
-#ifndef __INKSCAPE_EXTENSION_INTERNAL_BITMAP_IMAGEMAGICK_H__
-#define __INKSCAPE_EXTENSION_INTERNAL_BITMAP_IMAGEMAGICK_H__
+#ifndef INKSCAPE_EXTENSION_INTERNAL_BITMAP_IMAGEMAGICK_H
+#define INKSCAPE_EXTENSION_INTERNAL_BITMAP_IMAGEMAGICK_H
 
 /*
  * Copyright (C) 2007 Authors:
@@ -10,11 +10,19 @@
  */
 
 #include "extension/implementation/implementation.h"
-#include "extension/extension-forward.h"
-#include <Magick++.h>
+
+class SPItem;
+
+namespace Magick {
+class Image;
+}
 
 namespace Inkscape {
 namespace Extension {
+
+class Effect;
+class Extension;
+
 namespace Internal {
 namespace Bitmap {
 
@@ -23,6 +31,7 @@ public:
     /* Functions to be implemented by subclasses */
     virtual void applyEffect(Magick::Image */*image*/) { };
     virtual void refreshParameters(Inkscape::Extension::Effect */*module*/) { };
+    virtual void postEffect(Magick::Image */*image*/, SPItem */*item*/) { };
 
     /* Functions implemented from ::Implementation */
     bool load(Inkscape::Extension::Extension *module);
@@ -36,4 +45,4 @@ public:
 }; /* namespace Extension */
 }; /* namespace Inkscape */
 
-#endif /* __INKSCAPE_EXTENSION_INTERNAL_BITMAP_IMAGEMAGICK_H__ */
+#endif // INKSCAPE_EXTENSION_INTERNAL_BITMAP_IMAGEMAGICK_H
