@@ -10,6 +10,7 @@
 #include "extension/system.h"
 
 #include "levelChannel.h"
+#include <Magick++.h>
 
 namespace Inkscape {
 namespace Extension {
@@ -50,7 +51,7 @@ LevelChannel::init(void)
 		"<inkscape-extension xmlns=\"" INKSCAPE_EXTENSION_URI "\">\n"
 			"<name>" N_("Level (with Channel)") "</name>\n"
 			"<id>org.inkscape.effect.bitmap.levelChannel</id>\n"
-			"<param name=\"channel\" gui-text=\"" N_("Channel") "\" type=\"enum\" >\n"
+			"<param name=\"channel\" gui-text=\"" N_("Channel:") "\" type=\"enum\" >\n"
 				"<_item value='Red Channel'>" N_("Red Channel") "</_item>\n"
 				"<_item value='Green Channel'>" N_("Green Channel") "</_item>\n"
 				"<_item value='Blue Channel'>" N_("Blue Channel") "</_item>\n"
@@ -61,15 +62,15 @@ LevelChannel::init(void)
 				"<_item value='Opacity Channel'>" N_("Opacity Channel") "</_item>\n"
 				"<_item value='Matte Channel'>" N_("Matte Channel") "</_item>\n"
 			"</param>\n"
-			"<param name=\"blackPoint\" gui-text=\"" N_("Black Point") "\" type=\"float\" min=\"0.0\" max=\"100.0\">0.0</param>\n"			
-			"<param name=\"whitePoint\" gui-text=\"" N_("White Point") "\" type=\"float\" min=\"0.0\" max=\"100.0\">100.0</param>\n"			
-			"<param name=\"midPoint\" gui-text=\"" N_("Gamma Correction") "\" type=\"float\" min=\"0.0\" max=\"10.0\">1.0</param>\n"
+			"<param name=\"blackPoint\" gui-text=\"" N_("Black Point:") "\" type=\"float\" min=\"0.0\" max=\"100.0\">0.0</param>\n"			
+			"<param name=\"whitePoint\" gui-text=\"" N_("White Point:") "\" type=\"float\" min=\"0.0\" max=\"100.0\">100.0</param>\n"			
+			"<param name=\"midPoint\" gui-text=\"" N_("Gamma Correction:") "\" type=\"float\" min=\"0.0\" max=\"10.0\">1.0</param>\n"
 			"<effect>\n"
 				"<object-type>all</object-type>\n"
 				"<effects-menu>\n"
 					"<submenu name=\"" N_("Raster") "\" />\n"
 				"</effects-menu>\n"
-				"<menu-tip>" N_("Level the specified channel of selected bitmap(s) by scaling values falling between the given ranges to the full color range.") "</menu-tip>\n"
+				"<menu-tip>" N_("Level the specified channel of selected bitmap(s) by scaling values falling between the given ranges to the full color range") "</menu-tip>\n"
 			"</effect>\n"
 		"</inkscape-extension>\n", new LevelChannel());
 }
