@@ -10,6 +10,7 @@
 #include "extension/system.h"
 
 #include "oilPaint.h"
+#include <Magick++.h>
 
 namespace Inkscape {
 namespace Extension {
@@ -35,13 +36,13 @@ OilPaint::init(void)
 		"<inkscape-extension xmlns=\"" INKSCAPE_EXTENSION_URI "\">\n"
 			"<name>" N_("Oil Paint") "</name>\n"
 			"<id>org.inkscape.effect.bitmap.oilPaint</id>\n"
-			"<param name=\"radius\" gui-text=\"" N_("Radius") "\" type=\"int\" min=\"0\" max=\"50\">3</param>\n"			
+			"<param name=\"radius\" gui-text=\"" N_("Radius:") "\" type=\"int\" min=\"0\" max=\"50\">3</param>\n"			
 			"<effect>\n"
 				"<object-type>all</object-type>\n"
 				"<effects-menu>\n"
 					"<submenu name=\"" N_("Raster") "\" />\n"
 				"</effects-menu>\n"
-				"<menu-tip>" N_("Stylize selected bitmap(s) so that they appear to be painted with oils.") "</menu-tip>\n"
+				"<menu-tip>" N_("Stylize selected bitmap(s) so that they appear to be painted with oils") "</menu-tip>\n"
 			"</effect>\n"
 		"</inkscape-extension>\n", new OilPaint());
 }
