@@ -10,6 +10,7 @@
 #include "extension/system.h"
 
 #include "reduceNoise.h"
+#include <Magick++.h>
 
 namespace Inkscape {
 namespace Extension {
@@ -35,16 +36,16 @@ void
 ReduceNoise::init(void)
 {
 	Inkscape::Extension::build_from_mem(
-		"<inkscape-extension>\n"
+		"<inkscape-extension xmlns=\"" INKSCAPE_EXTENSION_URI "\">\n"
 			"<name>" N_("Reduce Noise") "</name>\n"
 			"<id>org.inkscape.effect.bitmap.reduceNoise</id>\n"
-			"<param name=\"order\" gui-text=\"" N_("Order") "\" type=\"int\" min=\"-1\" max=\"100\">-1</param>\n"
+			"<param name=\"order\" gui-text=\"" N_("Order:") "\" type=\"int\" min=\"-1\" max=\"100\">-1</param>\n"
 			"<effect>\n"
 				"<object-type>all</object-type>\n"
 				"<effects-menu>\n"
 					"<submenu name=\"" N_("Raster") "\" />\n"
 				"</effects-menu>\n"
-				"<menu-tip>" N_("Reduce noise in selected bitmap(s) using a noise peak elimination filter.") "</menu-tip>\n"
+				"<menu-tip>" N_("Reduce noise in selected bitmap(s) using a noise peak elimination filter") "</menu-tip>\n"
 			"</effect>\n"
 		"</inkscape-extension>\n", new ReduceNoise());
 }
