@@ -10,6 +10,7 @@
 #include "extension/system.h"
 
 #include "swirl.h"
+#include <Magick++.h>
 
 namespace Inkscape {
 namespace Extension {
@@ -35,13 +36,13 @@ Swirl::init(void)
 		"<inkscape-extension xmlns=\"" INKSCAPE_EXTENSION_URI "\">\n"
 			"<name>" N_("Swirl") "</name>\n"
 			"<id>org.inkscape.effect.bitmap.swirl</id>\n"
-			"<param name=\"degrees\" gui-text=\"" N_("Degrees") "\" type=\"int\" min=\"-360\" max=\"360\">30</param>\n"
+			"<param name=\"degrees\" gui-text=\"" N_("Degrees:") "\" type=\"int\" min=\"-360\" max=\"360\">30</param>\n"
 			"<effect>\n"
 				"<object-type>all</object-type>\n"
 				"<effects-menu>\n"
 					"<submenu name=\"" N_("Raster") "\" />\n"
 				"</effects-menu>\n"
-				"<menu-tip>" N_("Swirl selected bitmap(s) around center point.") "</menu-tip>\n"
+				"<menu-tip>" N_("Swirl selected bitmap(s) around center point") "</menu-tip>\n"
 			"</effect>\n"
 		"</inkscape-extension>\n", new Swirl());
 }
