@@ -20,19 +20,22 @@
 
 /* System includes */
 #include <glibmm/i18n.h>
+#include <glibmm/fileutils.h>
 
 namespace Inkscape {
 namespace Extension {
 namespace Internal {
 namespace Filter {
 
-void
-Filter::filters_all_files (void)
+void Filter::filters_all_files(void)
 {
-	filters_load_dir(INKSCAPE_FILTERDIR, _("Bundled"));
-	filters_load_dir(profile_path("filters"), _("Personal"));
+	gchar *filtersProfilePath = profile_path("filters");
 
-	return;
+	filters_load_dir(INKSCAPE_FILTERDIR, _("Bundled"));
+	filters_load_dir(filtersProfilePath, _("Personal"));
+
+	g_free(filtersProfilePath);
+	filtersProfilePath = 0;
 }
 
 #define INKSCAPE_FILTER_FILE  ".svg"
@@ -41,7 +44,7 @@ void
 Filter::filters_load_dir (gchar const * dirname, gchar * menuname)
 {
     if (!dirname) {
-        g_warning(_("Null external module directory name.  Filters will not be loaded."));
+        g_warning("%s", _("Null external module directory name.  Filters will not be loaded."));
         return;
     }
 
