@@ -11,7 +11,7 @@
 #ifndef EXTENSION_INTERNAL_LATEX_OUT_H
 #define EXTENSION_INTERNAL_LATEX_OUT_H
 
-#include <extension/implementation/implementation.h>
+#include "extension/implementation/implementation.h"
 
 namespace Inkscape {
 namespace Extension {
@@ -28,7 +28,7 @@ public:
 
     void save(Inkscape::Extension::Output *mod, // Save the given document to the given filename
               SPDocument *doc,
-              gchar const *uri);
+              gchar const *filename);
 
     static void init(void);//Initialize the class
 };
