@@ -1,10 +1,9 @@
-#define __SP_LATEX_C__
-
 /*
  * LaTeX Printing
  *
  * Author:
  *  Michael Forbes <miforbes@mbhs.edu>
+ *   Abhishek Sharma
  *
  * Copyright (C) 2004 Authors
  * 
@@ -16,33 +15,32 @@
 #endif
 
 
-#include <signal.h>
-#include <errno.h>
-
 #include <2geom/pathvector.h>
 #include <2geom/sbasis-to-bezier.h>
 #include <2geom/bezier-curve.h>
 #include <2geom/hvlinesegment.h>
+#include <errno.h>
+#include <signal.h>
+#include "util/units.h"
 #include "helper/geom-curves.h"
 
-#include "sp-item.h"
-
-#include "style.h"
-
-#include "latex-pstricks.h"
-
-#include <unit-constants.h>
-
-#include "extension/system.h"
 #include "extension/print.h"
-
+#include "extension/system.h"
 #include "io/sys.h"
+#include "latex-pstricks.h"
+#include "sp-item.h"
+#include "style.h"
+#include "document.h"
+#include <cstring>
 
 namespace Inkscape {
 namespace Extension {
 namespace Internal {
 
-PrintLatex::PrintLatex (void): _stream(NULL)
+PrintLatex::PrintLatex (void):
+    _width(0),
+    _height(0),
+    _stream(NULL)
 {
 }
 
@@ -57,30 +55,23 @@ PrintLatex::~PrintLatex (void)
 	return;
 }
 
-unsigned int
-PrintLatex::setup (Inkscape::Extension::Print *mod)
+unsigned int PrintLatex::setup(Inkscape::Extension::Print * /*mod*/)
 {
     return TRUE;
 }
 
-unsigned int
-PrintLatex::begin (Inkscape::Extension::Print *mod, SPDocument *doc)
+unsigned int PrintLatex::begin (Inkscape::Extension::Print *mod, SPDocument *doc)
 {
     Inkscape::SVGOStringStream os;
     int res;
-    FILE *osf, *osp;
-    const gchar * fn;
-
-    os.setf(std::ios::fixed);
-
-    fn = mod->get_param_string("destination");
-
-    osf = NULL;
-    osp = NULL;
-
+    FILE *osf = NULL;
+    const gchar * fn = NULL;
     gsize bytesRead = 0;
     gsize bytesWritten = 0;
     GError* error = NULL;
+
+    os.setf(std::ios::fixed);
+    fn = mod->get_param_string("destination");
     gchar* local_fn = g_filename_from_utf8( fn,
                                             -1,  &bytesRead,  &bytesWritten, &error);
     fn = local_fn;
@@ -95,8 +86,8 @@ PrintLatex::begin (Inkscape::Extension::Print *mod, SPDocument *doc)
         Inkscape::IO::dump_fopen_call(fn, "K");
         osf = Inkscape::IO::fopen_utf8name(fn, "w+");
         if (!osf) {
-            fprintf(stderr, "inkscape: fopen(%s): %s\n",
-                    fn, strerror(errno));
+            fprintf(stderr, "inkscape: fopen(%s): %s\n", fn, strerror(errno));
+            g_free(local_fn);
             return 0;
         }
         _stream = osf;
@@ -104,12 +95,10 @@ PrintLatex::begin (Inkscape::Extension::Print *mod, SPDocument *doc)
 
     g_free(local_fn);
 
-    if (_stream) {
-        /* fixme: this is kinda icky */
+    /* fixme: this is kinda icky */
 #if !defined(_WIN32) && !defined(__WIN32__)
-        (void) signal(SIGPIPE, SIG_IGN);
+    (void) signal(SIGPIPE, SIG_IGN);
 #endif
-    }
 
     res = fprintf(_stream, "%%LaTeX with PSTricks extensions\n");
     /* flush this to test output stream as early as possible */
@@ -128,8 +117,8 @@ PrintLatex::begin (Inkscape::Extension::Print *mod, SPDocument *doc)
     }
 
     // width and height in pt
-    _width = sp_document_width(doc) * PT_PER_PX;
-    _height = sp_document_height(doc) * PT_PER_PX;
+    _width = doc->getWidth().value("pt");
+    _height = doc->getHeight().value("pt");
 
     if (res >= 0) {
 
@@ -139,66 +128,63 @@ PrintLatex::begin (Inkscape::Extension::Print *mod, SPDocument *doc)
         os << "\\psset{xunit=.5pt,yunit=.5pt,runit=.5pt}\n";
         // from now on we can output px, but they will be treated as pt
     
-        os << "\\begin{pspicture}(" << sp_document_width(doc) << "," << sp_document_height(doc) << ")\n";
+        os << "\\begin{pspicture}(" << doc->getWidth().value("px") << "," << doc->getHeight().value("px") << ")\n";
     }
 
-    m_tr_stack.push( Geom::Scale(1, -1) * Geom::Translate(0, sp_document_height(doc)));
+    m_tr_stack.push( Geom::Scale(1, -1) * Geom::Translate(0, doc->getHeight().value("px")));  /// @fixme hardcoded doc2dt transform
 
     return fprintf(_stream, "%s", os.str().c_str());
 }
 
-unsigned int
-PrintLatex::finish (Inkscape::Extension::Print *mod)
+unsigned int PrintLatex::finish(Inkscape::Extension::Print * /*mod*/)
 {
-    int res;
+    if (_stream) {
+        fprintf(_stream, "\\end{pspicture}\n");
 
-    if (!_stream) return 0;
+        // Flush stream to be sure.
+        fflush(_stream);
 
-    res = fprintf(_stream, "\\end{pspicture}\n");
-
-    /* Flush stream to be sure. */
-    (void) fflush(_stream);
-
-    fclose(_stream);
-    _stream = NULL;
+        fclose(_stream);
+        _stream = NULL;
+    }
     return 0;
 }
 
-unsigned int
-PrintLatex::bind(Inkscape::Extension::Print *mod, Geom::Matrix const *transform, float opacity)
+unsigned int PrintLatex::bind(Inkscape::Extension::Print * /*mod*/, Geom::Affine const &transform, float /*opacity*/)
 {
-    Geom::Matrix tr = *transform;
-    
-    if(m_tr_stack.size()){
-        Geom::Matrix tr_top = m_tr_stack.top();
-        m_tr_stack.push(tr * tr_top);
-    }else
-        m_tr_stack.push(tr);
+    if (!m_tr_stack.empty()) {
+        Geom::Affine tr_top = m_tr_stack.top();
+        m_tr_stack.push(transform * tr_top);
+    } else {
+        m_tr_stack.push(transform);
+    }
 
     return 1;
 }
 
-unsigned int
-PrintLatex::release(Inkscape::Extension::Print *mod)
+unsigned int PrintLatex::release(Inkscape::Extension::Print * /*mod*/)
 {
     m_tr_stack.pop();
     return 1;
 }
 
-unsigned int PrintLatex::comment (Inkscape::Extension::Print * module,
-		                  const char * comment)
+unsigned int PrintLatex::comment(Inkscape::Extension::Print * /*mod*/,
+                                 const char * comment)
 {
-    if (!_stream) return 0; // XXX: fixme, returning -1 as unsigned.
+    if (!_stream) {
+        return 0; // XXX: fixme, returning -1 as unsigned.
+    }
 
     return fprintf(_stream, "%%! %s\n",comment);
 }
 
-unsigned int
-PrintLatex::fill(Inkscape::Extension::Print *mod,
-        Geom::PathVector const &pathv, Geom::Matrix const *transform, SPStyle const *style,
-        NRRect const *pbox, NRRect const *dbox, NRRect const *bbox)
+unsigned int PrintLatex::fill(Inkscape::Extension::Print * /*mod*/,
+                              Geom::PathVector const &pathv, Geom::Affine const &transform, SPStyle const *style,
+                              Geom::OptRect const & /*pbox*/, Geom::OptRect const & /*dbox*/, Geom::OptRect const & /*bbox*/)
 {
-    if (!_stream) return 0; // XXX: fixme, returning -1 as unsigned.
+    if (!_stream) {
+        return 0; // XXX: fixme, returning -1 as unsigned.
+    }
 
     if (style->fill.isColor()) {
         Inkscape::SVGOStringStream os;
@@ -227,17 +213,19 @@ PrintLatex::fill(Inkscape::Extension::Print *mod,
     return 0;
 }
 
-unsigned int
-PrintLatex::stroke (Inkscape::Extension::Print *mod, Geom::PathVector const &pathv, const Geom::Matrix *transform, const SPStyle *style,
-			      const NRRect *pbox, const NRRect *dbox, const NRRect *bbox)
+unsigned int PrintLatex::stroke(Inkscape::Extension::Print * /*mod*/,
+                                Geom::PathVector const &pathv, Geom::Affine const &transform, SPStyle const *style,
+                                Geom::OptRect const & /*pbox*/, Geom::OptRect const & /*dbox*/, Geom::OptRect const & /*bbox*/)
 {
-    if (!_stream) return 0; // XXX: fixme, returning -1 as unsigned.
+    if (!_stream) {
+        return 0; // XXX: fixme, returning -1 as unsigned.
+    }
 
     if (style->stroke.isColor()) {
         Inkscape::SVGOStringStream os;
         float rgb[3];
         float stroke_opacity;
-        Geom::Matrix tr_stack = m_tr_stack.top();
+        Geom::Affine tr_stack = m_tr_stack.top();
         double const scale = tr_stack.descrim();
         os.setf(std::ios::fixed);
 
@@ -251,16 +239,13 @@ PrintLatex::stroke (Inkscape::Extension::Print *mod, Geom::PathVector const &pat
             os<<",strokeopacity="<<stroke_opacity;
         }
 
-        if (style->stroke_dasharray_set &&
-                style->stroke_dash.n_dash &&
-                style->stroke_dash.dash) {
-            int i;
+        if (style->stroke_dasharray.set &&  !style->stroke_dasharray.values.empty()) {
             os << ",linestyle=dashed,dash=";
-            for (i = 0; i < style->stroke_dash.n_dash; i++) {
+            for (unsigned i = 0; i < style->stroke_dasharray.values.size(); i++) {
                 if ((i)) {
                     os << " ";
                 }
-                os << style->stroke_dash.dash[i];
+                os << style->stroke_dasharray.values[i];
             }
         }
 
@@ -278,13 +263,13 @@ PrintLatex::stroke (Inkscape::Extension::Print *mod, Geom::PathVector const &pat
 
 // FIXME: why is 'transform' argument not used?
 void
-PrintLatex::print_pathvector(SVGOStringStream &os, Geom::PathVector const &pathv_in, const Geom::Matrix * /*transform*/)
+PrintLatex::print_pathvector(SVGOStringStream &os, Geom::PathVector const &pathv_in, const Geom::Affine & /*transform*/)
 {
     if (pathv_in.empty())
         return;
 
-//    Geom::Matrix tf=*transform;   // why was this here?
-    Geom::Matrix tf_stack=m_tr_stack.top(); // and why is transform argument not used?
+//    Geom::Affine tf=transform;   // why was this here?
+    Geom::Affine tf_stack=m_tr_stack.top(); // and why is transform argument not used?
     Geom::PathVector pathv = pathv_in * tf_stack; // generates new path, which is a bit slow, but this doesn't have to be performance optimized
 
     os << "\\newpath\n";
@@ -305,7 +290,7 @@ PrintLatex::print_pathvector(SVGOStringStream &os, Geom::PathVector const &pathv
 }
 
 void
-PrintLatex::print_2geomcurve(SVGOStringStream &os, Geom::Curve const & c )
+PrintLatex::print_2geomcurve(SVGOStringStream &os, Geom::Curve const &c)
 {
     using Geom::X;
     using Geom::Y;
@@ -338,22 +323,17 @@ PrintLatex::textToPath(Inkscape::Extension::Print * ext)
 
 #include "clear-n_.h"
 
-void
-PrintLatex::init (void)
+void PrintLatex::init(void)
 {
-	Inkscape::Extension::Extension * ext;
-	
-	/* SVG in */
-    ext = Inkscape::Extension::build_from_mem(
-		"<inkscape-extension xmlns=\"" INKSCAPE_EXTENSION_URI "\">\n"
-			"<name>" N_("LaTeX Print") "</name>\n"
-			"<id>" SP_MODULE_KEY_PRINT_LATEX "</id>\n"
-        		"<param name=\"destination\" type=\"string\"></param>\n"
-                        "<param name=\"textToPath\" type=\"boolean\">true</param>\n"
-			"<print/>\n"
-		"</inkscape-extension>", new PrintLatex());
-
-	return;
+    /* SVG in */
+    Inkscape::Extension::build_from_mem(
+        "<inkscape-extension xmlns=\"" INKSCAPE_EXTENSION_URI "\">\n"
+        "<name>" N_("LaTeX Print") "</name>\n"
+        "<id>" SP_MODULE_KEY_PRINT_LATEX "</id>\n"
+        "<param name=\"destination\" type=\"string\"></param>\n"
+        "<param name=\"textToPath\" type=\"boolean\">true</param>\n"
+        "<print/>\n"
+        "</inkscape-extension>", new PrintLatex());
 }
 
 }  /* namespace Internal */
@@ -369,5 +349,5 @@ PrintLatex::init (void)
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
 
