@@ -1,8 +1,9 @@
- /** \file
+/*
  * Native PDF import using libpoppler.
  *
  * Authors:
  *   miklos erdelyi
+ *   Abhishek Sharma
  *
  * Copyright (C) 2007 Authors
  *
@@ -14,14 +15,15 @@
 # include <config.h>
 #endif
 
-#ifdef HAVE_POPPLER
+#include "pdf-input.h"
 
-#include "goo/GooString.h"
-#include "ErrorCodes.h"
-#include "GlobalParams.h"
-#include "PDFDoc.h"
-#include "Page.h"
-#include "Catalog.h"
+#ifdef HAVE_POPPLER
+#include <poppler/goo/GooString.h>
+#include <poppler/ErrorCodes.h>
+#include <poppler/GlobalParams.h>
+#include <poppler/PDFDoc.h>
+#include <poppler/Page.h>
+#include <poppler/Catalog.h>
 
 #ifdef HAVE_POPPLER_CAIRO
 #include <poppler/glib/poppler.h>
@@ -29,17 +31,30 @@
 #include <poppler/glib/poppler-page.h>
 #endif
 
-#include "pdf-input.h"
+#include <gtkmm/alignment.h>
+#include <gtkmm/checkbutton.h>
+#include <gtkmm/comboboxtext.h>
+#include <gtkmm/drawingarea.h>
+#include <gtkmm/frame.h>
+#include <gtkmm/scale.h>
+
 #include "extension/system.h"
 #include "extension/input.h"
 #include "svg-builder.h"
 #include "pdf-parser.h"
 
 #include "document-private.h"
-#include "application/application.h"
+#include "document-undo.h"
+#include "inkscape.h"
+#include "util/units.h"
 
 #include "dialogs/dialog-events.h"
 #include <gtk/gtk.h>
+#include "ui/widget/spinbutton.h"
+#include "ui/widget/frame.h"
+#include <glibmm/i18n.h>
+
+#include <gdkmm/general.h>
 
 namespace Inkscape {
 namespace Extension {
@@ -72,9 +87,14 @@ PdfImportDialog::PdfImportDialog(PDFDoc *doc, const gchar */*uri*/)
     _labelSelect = Gtk::manage(new class Gtk::Label(_("Select page:")));
 
     // Page number
+#if WITH_GTKMM_3_0
+    Glib::RefPtr<Gtk::Adjustment> _pageNumberSpin_adj = Gtk::Adjustment::create(1, 1, _pdf_doc->getNumPages(), 1, 10, 0);
+    _pageNumberSpin = Gtk::manage(new Inkscape::UI::Widget::SpinButton(_pageNumberSpin_adj, 1, 1));
+#else
     Gtk::Adjustment *_pageNumberSpin_adj = Gtk::manage(
             new class Gtk::Adjustment(1, 1, _pdf_doc->getNumPages(), 1, 10, 0));
-    _pageNumberSpin = Gtk::manage(new class Gtk::SpinButton(*_pageNumberSpin_adj, 1, 1));
+    _pageNumberSpin = Gtk::manage(new class Inkscape::UI::Widget::SpinButton(*_pageNumberSpin_adj, 1, 1));
+#endif
     _labelTotalPages = Gtk::manage(new class Gtk::Label());
     hbox2 = Gtk::manage(new class Gtk::HBox(false, 0));
     // Disable the page selector when there's only one page
@@ -93,21 +113,27 @@ PdfImportDialog::PdfImportDialog(PDFDoc *doc, const gchar */*uri*/)
     _cropTypeCombo = Gtk::manage(new class Gtk::ComboBoxText());
     int num_crop_choices = sizeof(crop_setting_choices) / sizeof(crop_setting_choices[0]);
     for ( int i = 0 ; i < num_crop_choices ; i++ ) {
-        _cropTypeCombo->append_text(_(crop_setting_choices[i]));
+        _cropTypeCombo->append(_(crop_setting_choices[i]));
     }
     _cropTypeCombo->set_active_text(_(crop_setting_choices[0]));
     _cropTypeCombo->set_sensitive(false);
 
     hbox3 = Gtk::manage(new class Gtk::HBox(false, 4));
     vbox2 = Gtk::manage(new class Gtk::VBox(false, 4));
-    alignment3 = Gtk::manage(new class Gtk::Alignment(0.5, 0.5, 1, 1));
-    _labelPageSettings = Gtk::manage(new class Gtk::Label(_("Page settings")));
-    _pageSettingsFrame = Gtk::manage(new class Gtk::Frame());
+    _pageSettingsFrame = Gtk::manage(new class Inkscape::UI::Widget::Frame(_("Page settings")));
     _labelPrecision = Gtk::manage(new class Gtk::Label(_("Precision of approximating gradient meshes:")));
     _labelPrecisionWarning = Gtk::manage(new class Gtk::Label(_("<b>Note</b>: setting the precision too high may result in a large SVG file and slow performance.")));
 
+#ifdef HAVE_POPPLER_CAIRO
+    _importviaPopplerCheck = Gtk::manage(new class Gtk::CheckButton(_("import via Poppler")));
+#endif
+#if WITH_GTKMM_3_0
+    _fallbackPrecisionSlider_adj = Gtk::Adjustment::create(2, 1, 256, 1, 10, 10);
+    _fallbackPrecisionSlider = Gtk::manage(new class Gtk::Scale(_fallbackPrecisionSlider_adj));
+#else
     _fallbackPrecisionSlider_adj = Gtk::manage(new class Gtk::Adjustment(2, 1, 256, 1, 10, 10));
     _fallbackPrecisionSlider = Gtk::manage(new class Gtk::HScale(*_fallbackPrecisionSlider_adj));
+#endif
     _fallbackPrecisionSlider->set_value(2.0);
     _labelPrecisionComment = Gtk::manage(new class Gtk::Label(_("rough")));
     hbox6 = Gtk::manage(new class Gtk::HBox(false, 4));
@@ -115,24 +141,22 @@ PdfImportDialog::PdfImportDialog(PDFDoc *doc, const gchar */*uri*/)
     // Text options
     _labelText = Gtk::manage(new class Gtk::Label(_("Text handling:")));
     _textHandlingCombo = Gtk::manage(new class Gtk::ComboBoxText());
-    _textHandlingCombo->append_text(_("Import text as text"));
+    _textHandlingCombo->append(_("Import text as text"));
     _textHandlingCombo->set_active_text(_("Import text as text"));
     _localFontsCheck = Gtk::manage(new class Gtk::CheckButton(_("Replace PDF fonts by closest-named installed fonts")));
 
     hbox5 = Gtk::manage(new class Gtk::HBox(false, 4));
     _embedImagesCheck = Gtk::manage(new class Gtk::CheckButton(_("Embed images")));
     vbox3 = Gtk::manage(new class Gtk::VBox(false, 4));
-    alignment4 = Gtk::manage(new class Gtk::Alignment(0.5, 0.5, 1, 1));
-    _labelImportSettings = Gtk::manage(new class Gtk::Label(_("Import settings")));
-    _importSettingsFrame = Gtk::manage(new class Gtk::Frame());
+    _importSettingsFrame = Gtk::manage(new class Inkscape::UI::Widget::Frame(_("Import settings")));
     vbox1 = Gtk::manage(new class Gtk::VBox(false, 4));
     _previewArea = Gtk::manage(new class Gtk::DrawingArea());
     hbox1 = Gtk::manage(new class Gtk::HBox(false, 4));
-    cancelbutton->set_flags(Gtk::CAN_FOCUS);
-    cancelbutton->set_flags(Gtk::CAN_DEFAULT);
+    cancelbutton->set_can_focus();
+    cancelbutton->set_can_default();
     cancelbutton->set_relief(Gtk::RELIEF_NORMAL);
-    okbutton->set_flags(Gtk::CAN_FOCUS);
-    okbutton->set_flags(Gtk::CAN_DEFAULT);
+    okbutton->set_can_focus();
+    okbutton->set_can_default();
     okbutton->set_relief(Gtk::RELIEF_NORMAL);
     this->get_action_area()->property_layout_style().set_value(Gtk::BUTTONBOX_END);
     _labelSelect->set_alignment(0.5,0.5);
@@ -141,7 +165,7 @@ PdfImportDialog::PdfImportDialog(PDFDoc *doc, const gchar */*uri*/)
     _labelSelect->set_line_wrap(false);
     _labelSelect->set_use_markup(false);
     _labelSelect->set_selectable(false);
-    _pageNumberSpin->set_flags(Gtk::CAN_FOCUS);
+    _pageNumberSpin->set_can_focus();
     _pageNumberSpin->set_update_policy(Gtk::UPDATE_ALWAYS);
     _pageNumberSpin->set_numeric(true);
     _pageNumberSpin->set_digits(0);
@@ -155,7 +179,7 @@ PdfImportDialog::PdfImportDialog(PDFDoc *doc, const gchar */*uri*/)
     hbox2->pack_start(*_labelSelect, Gtk::PACK_SHRINK, 4);
     hbox2->pack_start(*_pageNumberSpin, Gtk::PACK_SHRINK, 4);
     hbox2->pack_start(*_labelTotalPages, Gtk::PACK_SHRINK, 4);
-    _cropCheck->set_flags(Gtk::CAN_FOCUS);
+    _cropCheck->set_can_focus();
     _cropCheck->set_relief(Gtk::RELIEF_NORMAL);
     _cropCheck->set_mode(true);
     _cropCheck->set_active(false);
@@ -164,18 +188,8 @@ PdfImportDialog::PdfImportDialog(PDFDoc *doc, const gchar */*uri*/)
     hbox3->pack_start(*_cropTypeCombo, Gtk::PACK_SHRINK, 0);
     vbox2->pack_start(*hbox2);
     vbox2->pack_start(*hbox3);
-    alignment3->add(*vbox2);
-    _labelPageSettings->set_alignment(0.5,0.5);
-    _labelPageSettings->set_padding(4,0);
-    _labelPageSettings->set_justify(Gtk::JUSTIFY_LEFT);
-    _labelPageSettings->set_line_wrap(false);
-    _labelPageSettings->set_use_markup(true);
-    _labelPageSettings->set_selectable(false);
+    _pageSettingsFrame->add(*vbox2);
     _pageSettingsFrame->set_border_width(4);
-    _pageSettingsFrame->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
-    _pageSettingsFrame->set_label_align(0,0.5);
-    _pageSettingsFrame->add(*alignment3);
-    _pageSettingsFrame->set_label_widget(*_labelPageSettings);
     _labelPrecision->set_alignment(0,0.5);
     _labelPrecision->set_padding(4,0);
     _labelPrecision->set_justify(Gtk::JUSTIFY_LEFT);
@@ -188,9 +202,14 @@ PdfImportDialog::PdfImportDialog(PDFDoc *doc, const gchar */*uri*/)
     _labelPrecisionWarning->set_line_wrap(true);
     _labelPrecisionWarning->set_use_markup(true);
     _labelPrecisionWarning->set_selectable(false);
+#ifdef HAVE_POPPLER_CAIRO
+    _importviaPopplerCheck->set_can_focus();
+    _importviaPopplerCheck->set_relief(Gtk::RELIEF_NORMAL);
+    _importviaPopplerCheck->set_mode(true);
+    _importviaPopplerCheck->set_active(false);
+#endif
     _fallbackPrecisionSlider->set_size_request(180,-1);
-    _fallbackPrecisionSlider->set_flags(Gtk::CAN_FOCUS);
-    _fallbackPrecisionSlider->set_update_policy(Gtk::UPDATE_CONTINUOUS);
+    _fallbackPrecisionSlider->set_can_focus();
     _fallbackPrecisionSlider->set_inverted(false);
     _fallbackPrecisionSlider->set_digits(1);
     _fallbackPrecisionSlider->set_draw_value(true);
@@ -212,46 +231,46 @@ PdfImportDialog::PdfImportDialog(PDFDoc *doc, const gchar */*uri*/)
     _labelText->set_selectable(false);
     hbox5->pack_start(*_labelText, Gtk::PACK_SHRINK, 0);
     hbox5->pack_start(*_textHandlingCombo, Gtk::PACK_SHRINK, 0);
-    _localFontsCheck->set_flags(Gtk::CAN_FOCUS);
+    _localFontsCheck->set_can_focus();
     _localFontsCheck->set_relief(Gtk::RELIEF_NORMAL);
     _localFontsCheck->set_mode(true);
     _localFontsCheck->set_active(true);
-    _embedImagesCheck->set_flags(Gtk::CAN_FOCUS);
+    _embedImagesCheck->set_can_focus();
     _embedImagesCheck->set_relief(Gtk::RELIEF_NORMAL);
     _embedImagesCheck->set_mode(true);
     _embedImagesCheck->set_active(true);
+#ifdef HAVE_POPPLER_CAIRO
+    vbox3->pack_start(*_importviaPopplerCheck, Gtk::PACK_SHRINK, 0);
+#endif    
     vbox3->pack_start(*_labelPrecision, Gtk::PACK_SHRINK, 0);
     vbox3->pack_start(*hbox6, Gtk::PACK_SHRINK, 0);
     vbox3->pack_start(*_labelPrecisionWarning, Gtk::PACK_SHRINK, 0);
     vbox3->pack_start(*hbox5, Gtk::PACK_SHRINK, 4);
     vbox3->pack_start(*_localFontsCheck, Gtk::PACK_SHRINK, 0);
     vbox3->pack_start(*_embedImagesCheck, Gtk::PACK_SHRINK, 0);
-    alignment4->add(*vbox3);
-    _labelImportSettings->set_alignment(0.5,0.5);
-    _labelImportSettings->set_padding(4,0);
-    _labelImportSettings->set_justify(Gtk::JUSTIFY_LEFT);
-    _labelImportSettings->set_line_wrap(false);
-    _labelImportSettings->set_use_markup(true);
-    _labelImportSettings->set_selectable(false);
+    _importSettingsFrame->add(*vbox3);
     _importSettingsFrame->set_border_width(4);
-    _importSettingsFrame->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
-    _importSettingsFrame->set_label_align(0,0.5);
-    _importSettingsFrame->add(*alignment4);
-    _importSettingsFrame->set_label_widget(*_labelImportSettings);
     vbox1->pack_start(*_pageSettingsFrame, Gtk::PACK_EXPAND_PADDING, 0);
     vbox1->pack_start(*_importSettingsFrame, Gtk::PACK_EXPAND_PADDING, 0);
     hbox1->pack_start(*vbox1);
     hbox1->pack_start(*_previewArea, Gtk::PACK_EXPAND_WIDGET, 4);
+
+#if WITH_GTKMM_3_0
+    get_content_area()->set_homogeneous(false);
+    get_content_area()->set_spacing(0);
+    get_content_area()->pack_start(*hbox1);
+#else
     this->get_vbox()->set_homogeneous(false);
     this->get_vbox()->set_spacing(0);
     this->get_vbox()->pack_start(*hbox1);
+#endif
+
     this->set_title(_("PDF Import Settings"));
     this->set_modal(true);
-    sp_transientize((GtkWidget *)this->gobj());  //Make transient
+    sp_transientize(GTK_WIDGET(this->gobj()));  //Make transient
     this->property_window_position().set_value(Gtk::WIN_POS_NONE);
     this->set_resizable(true);
     this->property_destroy_with_parent().set_value(false);
-    this->set_has_separator(true);
     this->add_action_widget(*cancelbutton, -6);
     this->add_action_widget(*okbutton, -5);
     cancelbutton->show();
@@ -264,11 +283,12 @@ PdfImportDialog::PdfImportDialog(PDFDoc *doc, const gchar */*uri*/)
     _cropTypeCombo->show();
     hbox3->show();
     vbox2->show();
-    alignment3->show();
-    _labelPageSettings->show();
     _pageSettingsFrame->show();
     _labelPrecision->show();
     _labelPrecisionWarning->show();
+#ifdef HAVE_POPPLER_CAIRO
+    _importviaPopplerCheck->show();
+#endif
     _fallbackPrecisionSlider->show();
     _labelPrecisionComment->show();
     hbox6->show();
@@ -278,15 +298,18 @@ PdfImportDialog::PdfImportDialog(PDFDoc *doc, const gchar */*uri*/)
     _localFontsCheck->show();
     _embedImagesCheck->show();
     vbox3->show();
-    alignment4->show();
-    _labelImportSettings->show();
     _importSettingsFrame->show();
     vbox1->show();
     _previewArea->show();
     hbox1->show();
 
     // Connect signals
+#if WITH_GTKMM_3_0
+    _previewArea->signal_draw().connect(sigc::mem_fun(*this, &PdfImportDialog::_onDraw));
+#else
     _previewArea->signal_expose_event().connect(sigc::mem_fun(*this, &PdfImportDialog::_onExposePreview));
+#endif
+
     _pageNumberSpin_adj->signal_value_changed().connect(sigc::mem_fun(*this, &PdfImportDialog::_onPageNumberChanged));
     _cropCheck->signal_toggled().connect(sigc::mem_fun(*this, &PdfImportDialog::_onToggleCropping));
     _fallbackPrecisionSlider_adj->signal_value_changed().connect(sigc::mem_fun(*this, &PdfImportDialog::_onPrecisionChanged));
@@ -350,6 +373,14 @@ int PdfImportDialog::getSelectedPage() {
     return _current_page;
 }
 
+int PdfImportDialog::getImportMethod() {
+#ifdef HAVE_POPPLER_CAIRO
+    return (_importviaPopplerCheck->get_active()) ? 1 : 0;
+#else
+    return 0;
+#endif
+}
+
 /**
  * \brief Retrieves the current settings into a repr which SvgBuilder will use
  *        for determining the behaviour desired by the user
@@ -381,6 +412,13 @@ void PdfImportDialog::getImportSettings(Inkscape::XML::Node *prefs) {
     } else {
         prefs->setAttribute("embedImages", "0");
     }
+#ifdef HAVE_POPPLER_CAIRO
+    if (_importviaPopplerCheck->get_active()) {
+        prefs->setAttribute("importviapoppler", "1");
+    } else {
+        prefs->setAttribute("importviapoppler", "0");
+    }
+#endif
 }
 
 /**
@@ -390,12 +428,10 @@ void PdfImportDialog::getImportSettings(Inkscape::XML::Node *prefs) {
 void PdfImportDialog::_onPrecisionChanged() {
 
     static Glib::ustring precision_comments[] = {
-        Glib::ustring(_("rough")),
-        //TRANSLATORS: only translate "string" in "context|string".
-        // For more details, see http://developer.gnome.org/doc/API/2.0/glib/glib-I18N.html#Q-:CAPS
-        Glib::ustring(Q_("pdfinput|medium")),
-        Glib::ustring(_("fine")),
-        Glib::ustring(_("very fine"))
+        Glib::ustring(C_("PDF input precision", "rough")),
+        Glib::ustring(C_("PDF input precision", "medium")),
+        Glib::ustring(C_("PDF input precision", "fine")),
+        Glib::ustring(C_("PDF input precision", "very fine"))
     };
 
     double min = _fallbackPrecisionSlider_adj->get_lower();
@@ -450,7 +486,7 @@ static void copy_cairo_surface_to_pixbuf (cairo_surface_t *surface,
         cairo_height = gdk_pixbuf_get_height (pixbuf);
     for (y = 0; y < cairo_height; y++)
     {
-        src = (unsigned int *) (cairo_data + y * cairo_rowstride);
+        src = reinterpret_cast<unsigned int *>(cairo_data + y * cairo_rowstride);
         dst = pixbuf_data + y * pixbuf_rowstride;
         for (x = 0; x < cairo_width; x++)
         {
@@ -470,8 +506,14 @@ static void copy_cairo_surface_to_pixbuf (cairo_surface_t *surface,
 /**
  * \brief Updates the preview area with the previously rendered thumbnail
  */
-bool PdfImportDialog::_onExposePreview(GdkEventExpose */*event*/) {
+#if !WITH_GTKMM_3_0
+bool PdfImportDialog::_onExposePreview(GdkEventExpose * /*event*/) {
+    Cairo::RefPtr<Cairo::Context> cr = _previewArea->get_window()->create_cairo_context();
+    return _onDraw(cr);
+}
+#endif
 
+bool PdfImportDialog::_onDraw(const Cairo::RefPtr<Cairo::Context>& cr) {
     // Check if we have a thumbnail at all
     if (!_thumb_data) {
         return true;
@@ -479,6 +521,7 @@ bool PdfImportDialog::_onExposePreview(GdkEventExpose */*event*/) {
 
     // Create the pixbuf for the thumbnail
     Glib::RefPtr<Gdk::Pixbuf> thumb;
+
     if (_render_thumb) {
         thumb = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, true,
                                     8, _thumb_width, _thumb_height);
@@ -493,16 +536,8 @@ bool PdfImportDialog::_onExposePreview(GdkEventExpose */*event*/) {
     // Set background to white
     if (_render_thumb) {
         thumb->fill(0xffffffff);
-        Glib::RefPtr<Gdk::Pixmap> back_pixmap = Gdk::Pixmap::create(
-                _previewArea->get_window(), _thumb_width, _thumb_height, -1);
-        if (!back_pixmap) {
-            return true;
-        }
-        back_pixmap->draw_pixbuf(Glib::RefPtr<Gdk::GC>(), thumb, 0, 0, 0, 0,
-                                 _thumb_width, _thumb_height,
-                                 Gdk::RGB_DITHER_NONE, 0, 0);
-        _previewArea->get_window()->set_back_pixmap(back_pixmap, false);
-        _previewArea->get_window()->clear();
+	Gdk::Cairo::set_source_pixbuf(cr, thumb, 0, 0);
+	cr->paint();
     }
 #ifdef HAVE_POPPLER_CAIRO
     // Copy the thumbnail image from the Cairo surface
@@ -510,10 +545,9 @@ bool PdfImportDialog::_onExposePreview(GdkEventExpose */*event*/) {
         copy_cairo_surface_to_pixbuf(_cairo_surface, _thumb_data, thumb->gobj());
     }
 #endif
-    _previewArea->get_window()->draw_pixbuf(Glib::RefPtr<Gdk::GC>(), thumb,
-                                            0, 0, 0, _render_thumb ? 0 : 20,
-                                            -1, -1, Gdk::RGB_DITHER_NONE, 0, 0);
 
+    Gdk::Cairo::set_source_pixbuf(cr, thumb, 0, _render_thumb ? 0 : 20);
+    cr->paint();
     return true;
 }
 
@@ -586,15 +620,58 @@ void PdfImportDialog::_setPreviewPage(int page) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
+bool
+PdfInput::wasCancelled () {
+    return _cancelled;
+}
+
+#ifdef HAVE_POPPLER_CAIRO
+/// helper method
+static cairo_status_t
+        _write_ustring_cb(void *closure, const unsigned char *data, unsigned int length)
+{
+    Glib::ustring* stream = static_cast<Glib::ustring*>(closure);
+    stream->append(reinterpret_cast<const char*>(data), length);
+
+    return CAIRO_STATUS_SUCCESS;
+}
+#endif
+
 /**
  * Parses the selected page of the given PDF document using PdfParser.
  */
 SPDocument *
 PdfInput::open(::Inkscape::Extension::Input * /*mod*/, const gchar * uri) {
 
+    _cancelled = false;
+
     // Initialize the globalParams variable for poppler
     if (!globalParams) {
+#ifdef ENABLE_OSX_APP_LOCATIONS
+        //
+        // data files for poppler are not relocatable (loaded from 
+        // path defined at build time). This fails to work with relocatable
+        // application bundles for OS X. 
+        //
+        // Workaround: 
+        // 1. define $POPPLER_DATADIR env variable in app launcher script
+        // 2. pass custom $POPPLER_DATADIR via poppler's GlobalParams()
+        //
+        // relevant poppler commit:
+        // <http://cgit.freedesktop.org/poppler/poppler/commit/?id=869584a84eed507775ff1c3183fe484c14b6f77b>
+        //
+        // FIXES: Inkscape bug #956282, #1264793
+        // TODO: report RFE upstream (full relocation support for OS X packaging)
+        //
+        gchar const *poppler_datadir = g_getenv("POPPLER_DATADIR");
+        if (poppler_datadir != NULL) {
+            globalParams = new GlobalParams(poppler_datadir);
+        } else {
+            globalParams = new GlobalParams();
+        }
+#else
         globalParams = new GlobalParams();
+#endif // ENABLE_OSX_APP_LOCATIONS
     }
     // poppler does not use glib g_open. So on win32 we must use unicode call. code was copied from glib gstdio.c
 #ifndef WIN32
@@ -602,7 +679,7 @@ PdfInput::open(::Inkscape::Extension::Input * /*mod*/, const gchar * uri) {
     PDFDoc *pdf_doc = new PDFDoc(filename_goo, NULL, NULL, NULL);   // TODO: Could ask for password
     //delete filename_goo;
 #else
-    wchar_t *wfilename = (wchar_t*)g_utf8_to_utf16 (uri, -1, NULL, NULL, NULL);
+    wchar_t *wfilename = reinterpret_cast<wchar_t*>(g_utf8_to_utf16 (uri, -1, NULL, NULL, NULL));
 
     if (wfilename == NULL) {
       return NULL;
@@ -643,9 +720,10 @@ PdfInput::open(::Inkscape::Extension::Input * /*mod*/, const gchar * uri) {
     }
 
     PdfImportDialog *dlg = NULL;
-    if (Inkscape::NSApplication::Application::getUseGui()) {
+    if (inkscape_use_gui()) {
         dlg = new PdfImportDialog(pdf_doc, uri);
         if (!dlg->showDialog()) {
+            _cancelled = true;
             delete dlg;
             delete pdf_doc;
             return NULL;
@@ -660,96 +738,165 @@ PdfInput::open(::Inkscape::Extension::Input * /*mod*/, const gchar * uri) {
         page_num = 1;
     Catalog *catalog = pdf_doc->getCatalog();
     Page *page = catalog->getPage(page_num);
-
-    SPDocument *doc = sp_document_new(NULL, TRUE, TRUE);
-    bool saved = sp_document_get_undo_sensitive(doc);
-    sp_document_set_undo_sensitive(doc, false); // No need to undo in this temporary document
-
-    // Create builder
-    gchar *docname = g_path_get_basename(uri);
-    gchar *dot = g_strrstr(docname, ".");
-    if (dot) {
-        *dot = 0;
+    
+    int is_importvia_poppler = 0;
+    if(dlg)
+    {
+#ifdef HAVE_POPPLER_CAIRO
+        is_importvia_poppler = dlg->getImportMethod();
+#endif
     }
-    SvgBuilder *builder = new SvgBuilder(doc, docname, pdf_doc->getXRef());
 
-    // Get preferences
-    Inkscape::XML::Node *prefs = builder->getPreferences();
-    if (dlg)
-        dlg->getImportSettings(prefs);
+    SPDocument *doc = NULL;
+    bool saved = false;
+    if(is_importvia_poppler == 0)
+    {
+        // native importer
+        doc = SPDocument::createNewDoc(NULL, TRUE, TRUE);
+        saved = DocumentUndo::getUndoSensitive(doc);
+        DocumentUndo::setUndoSensitive(doc, false); // No need to undo in this temporary document
 
-    // Apply crop settings
-    PDFRectangle *clipToBox = NULL;
-    double crop_setting;
-    sp_repr_get_double(prefs, "cropTo", &crop_setting);
-    if ( crop_setting >= 0.0 ) {    // Do page clipping
-        int crop_choice = (int)crop_setting;
-        switch (crop_choice) {
-            case 0: // Media box
-                clipToBox = page->getMediaBox();
-                break;
-            case 1: // Crop box
-                clipToBox = page->getCropBox();
-                break;
-            case 2: // Bleed box
-                clipToBox = page->getBleedBox();
-                break;
-            case 3: // Trim box
-                clipToBox = page->getTrimBox();
-                break;
-            case 4: // Art box
-                clipToBox = page->getArtBox();
-                break;
-            default:
-                break;
+        // Create builder
+        gchar *docname = g_path_get_basename(uri);
+        gchar *dot = g_strrstr(docname, ".");
+        if (dot) {
+            *dot = 0;
         }
-    }
+        SvgBuilder *builder = new SvgBuilder(doc, docname, pdf_doc->getXRef());
 
-    // Create parser
-    PdfParser *pdf_parser = new PdfParser(pdf_doc->getXRef(), builder, page_num-1, page->getRotate(),
-                                          page->getResourceDict(), page->getCropBox(), clipToBox);
+        // Get preferences
+        Inkscape::XML::Node *prefs = builder->getPreferences();
+        if (dlg)
+            dlg->getImportSettings(prefs);
 
-    // Set up approximation precision for parser
-    double color_delta;
-    sp_repr_get_double(prefs, "approximationPrecision", &color_delta);
-    if ( color_delta <= 0.0 ) {
-        color_delta = 1.0 / 2.0;
-    } else {
-        color_delta = 1.0 / color_delta;
-    }
-    for ( int i = 1 ; i <= pdfNumShadingTypes ; i++ ) {
-        pdf_parser->setApproximationPrecision(i, color_delta, 6);
-    }
+        //printf("pdf import via %s.", (is_importvia_poppler != 0) ? "poppler" : "native");
 
-    // Parse the document structure
-    Object obj;
-    page->getContents(&obj);
-    if (!obj.isNull()) {
-        pdf_parser->parse(&obj);
+        // Apply crop settings
+        PDFRectangle *clipToBox = NULL;
+        double crop_setting;
+        sp_repr_get_double(prefs, "cropTo", &crop_setting);
+        if ( crop_setting >= 0.0 ) {    // Do page clipping
+            int crop_choice = (int)crop_setting;
+            switch (crop_choice) {
+                case 0: // Media box
+                    clipToBox = page->getMediaBox();
+                    break;
+                case 1: // Crop box
+                    clipToBox = page->getCropBox();
+                    break;
+                case 2: // Bleed box
+                    clipToBox = page->getBleedBox();
+                    break;
+                case 3: // Trim box
+                    clipToBox = page->getTrimBox();
+                    break;
+                case 4: // Art box
+                    clipToBox = page->getArtBox();
+                    break;
+                default:
+                    break;
+            }
+        }
+
+        // Create parser
+        PdfParser *pdf_parser = new PdfParser(pdf_doc->getXRef(), builder, page_num-1, page->getRotate(),
+                                              page->getResourceDict(), page->getCropBox(), clipToBox);
+
+        // Set up approximation precision for parser
+        double color_delta;
+        sp_repr_get_double(prefs, "approximationPrecision", &color_delta);
+        if ( color_delta <= 0.0 ) {
+            color_delta = 1.0 / 2.0;
+        } else {
+            color_delta = 1.0 / color_delta;
+        }
+        for ( int i = 1 ; i <= pdfNumShadingTypes ; i++ ) {
+            pdf_parser->setApproximationPrecision(i, color_delta, 6);
+        }
+
+        // Parse the document structure
+        Object obj;
+        page->getContents(&obj);
+        if (!obj.isNull()) {
+            pdf_parser->parse(&obj);
+        }
+
+        // Cleanup
+        obj.free();
+        delete pdf_parser;
+        delete builder;
+        g_free(docname);
+    }
+    else
+    {
+#ifdef HAVE_POPPLER_CAIRO
+        // the poppler import
+        gchar* filename_uri = g_filename_to_uri(uri, NULL, NULL);
+        GError *error = NULL;
+        /// @todo handle passwort
+        /// @todo check if win32 unicode needs special attention
+        PopplerDocument* document = poppler_document_new_from_file(filename_uri, NULL, &error);
+
+        if(error != NULL) {
+            g_error_free (error);
+        }
+
+        if (document != NULL)
+        {
+            double width, height;
+            PopplerPage* page = poppler_document_get_page(document, page_num - 1);
+            poppler_page_get_size(page, &width, &height);
+
+            Glib::ustring output;
+            cairo_surface_t* surface = cairo_svg_surface_create_for_stream(Inkscape::Extension::Internal::_write_ustring_cb,
+                                                                           &output, width, height);
+            cairo_t* cr = cairo_create(surface);
+
+            poppler_page_render_for_printing(page, cr);
+            cairo_show_page(cr);
+
+            cairo_destroy(cr);
+            cairo_surface_destroy(surface);
+
+            doc = SPDocument::createNewDocFromMem(output.c_str(), output.length(), TRUE);
+            
+            // Cleanup
+            // delete output;
+            g_object_unref(G_OBJECT(page));
+            g_object_unref(G_OBJECT(document));
+        }
+        else
+        {
+            doc = SPDocument::createNewDoc(NULL, TRUE, TRUE);   // fallback create empthy document
+        }
+        saved = DocumentUndo::getUndoSensitive(doc);
+        DocumentUndo::setUndoSensitive(doc, false); // No need to undo in this temporary document
+
+        // Cleanup
+        g_free(filename_uri);
+#endif
     }
 
     // Cleanup
-    obj.free();
-    delete pdf_parser;
-    delete builder;
-    g_free(docname);
     delete pdf_doc;
     delete dlg;
 
+    // Set viewBox if it doesn't exist
+    if (!doc->getRoot()->viewBox_set) {
+        doc->setViewBox(Geom::Rect::from_xywh(0, 0, doc->getWidth().value(doc->getDefaultUnit()), doc->getHeight().value(doc->getDefaultUnit())));
+    }
+
     // Restore undo
-    sp_document_set_undo_sensitive(doc, saved);
+    DocumentUndo::setUndoSensitive(doc, saved);
 
     return doc;
 }
 
 #include "../clear-n_.h"
 
-void
-PdfInput::init(void) {
-    Inkscape::Extension::Extension * ext;
-
+void PdfInput::init(void) {
     /* PDF in */
-    ext = Inkscape::Extension::build_from_mem(
+    Inkscape::Extension::build_from_mem(
         "<inkscape-extension xmlns=\"" INKSCAPE_EXTENSION_URI "\">\n"
             "<name>" N_("PDF Input") "</name>\n"
             "<id>org.inkscape.input.pdf</id>\n"
@@ -762,7 +909,7 @@ PdfInput::init(void) {
         "</inkscape-extension>", new PdfInput());
 
     /* AI in */
-    ext = Inkscape::Extension::build_from_mem(
+    Inkscape::Extension::build_from_mem(
         "<inkscape-extension xmlns=\"" INKSCAPE_EXTENSION_URI "\">\n"
             "<name>" N_("AI Input") "</name>\n"
             "<id>org.inkscape.input.ai</id>\n"
