@@ -1,6 +1,4 @@
-/** \file
- * Inkscape::GC::Anchored - base class for anchored GC-managed objects
- *
+/*
  * Authors:
  *   MenTaLguY <mental@rydia.net>
  * * Copyright (C) 2004 MenTaLguY
@@ -63,7 +61,7 @@ protected:
 
 private:
     struct Anchor : public Managed<SCANNED, MANUAL> {
-        Anchor() : refcount(0) {}
+        Anchor() : refcount(0),base(NULL) {}
         Anchor(Anchored const *obj) : refcount(0) {
             base = Core::base(const_cast<Anchored *>(obj));
         }
@@ -176,4 +174,4 @@ static R *release(R *r) {
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
