@@ -17,6 +17,7 @@
 #define SEEN_INKSCAPE_GC_FINALIZED_H
 
 #include <new>
+#include <cstddef>
 #include "gc-core.h"
 
 namespace Inkscape {
@@ -142,4 +143,4 @@ private:
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
