@@ -45,22 +45,29 @@ struct _Ink_ComboBoxEntry_Action {
   GtkAction parent_instance;
 
   GtkTreeModel       *model;
-  GtkComboBoxEntry   *combobox;
+  GtkComboBox        *combobox;
   GtkEntry           *entry;
   GtkEntryCompletion *entry_completion;
-#if !GTK_CHECK_VERSION(2,16,0)
   GtkWidget          *indicator;
-#endif
 
   gpointer            cell_data_func; // drop-down menu format
+  gpointer            separator_func;
 
   gint                active;     // Index of active menu item (-1 if not in list).
   gchar              *text;       // Text of active menu item or entry box.
   gint                entry_width;// Width of GtkEntry in characters.
   gint                extra_width;// Extra Width of GtkComboBox.. to widen drop-down list in list mode.
   gboolean            popup;      // Do we pop-up an entry-completion dialog?
-  gchar              *warning;    // Text for warning that entry isn't in list.
+  gchar              *info;       // Text for tooltip info about entry.
+  gpointer            info_cb;    // Callback for clicking info icon.
+  gint                info_cb_id;
+  gboolean            info_cb_blocked;
+  gchar              *warning;    // Text for tooltip warning that entry isn't in list.
+  gpointer            warning_cb; // Callback for clicking warning icon.
+  gint                warning_cb_id;
+  gboolean            warning_cb_blocked;
   gchar              *altx_name;  // Target for Alt-X keyboard shortcut.
+  GtkWidget          *focusWidget;
 };
 
 
@@ -76,13 +83,15 @@ Ink_ComboBoxEntry_Action *ink_comboboxentry_action_new ( const gchar  *name,
 							 GtkTreeModel *model,
 							 gint          entry_width = -1,
 							 gint          extra_width = -1,
-							 gpointer cell_data_func = NULL );
+							 gpointer cell_data_func = NULL,
+							 gpointer separator_func = NULL,
+							 GtkWidget* focusWidget = NULL);
 
 GtkTreeModel     *ink_comboboxentry_action_get_model( Ink_ComboBoxEntry_Action* action );
-GtkComboBoxEntry *ink_comboboxentry_action_get_comboboxentry( Ink_ComboBoxEntry_Action* action );
+GtkComboBox      *ink_comboboxentry_action_get_comboboxentry( Ink_ComboBoxEntry_Action* action );
 
 gchar*   ink_comboboxentry_action_get_active_text( Ink_ComboBoxEntry_Action* action );
-gboolean ink_comboboxentry_action_set_active_text( Ink_ComboBoxEntry_Action* action, const gchar* text );
+gboolean ink_comboboxentry_action_set_active_text( Ink_ComboBoxEntry_Action* action, const gchar* text, int row=-1 );
 
 void     ink_comboboxentry_action_set_entry_width( Ink_ComboBoxEntry_Action* action, gint entry_width );
 void     ink_comboboxentry_action_set_extra_width( Ink_ComboBoxEntry_Action* action, gint extra_width );
@@ -90,7 +99,11 @@ void     ink_comboboxentry_action_set_extra_width( Ink_ComboBoxEntry_Action* act
 void     ink_comboboxentry_action_popup_enable(  Ink_ComboBoxEntry_Action* action );
 void     ink_comboboxentry_action_popup_disable( Ink_ComboBoxEntry_Action* action );
 
-void     ink_comboboxentry_action_set_warning( Ink_ComboBoxEntry_Action* action, const gchar* warning );
+void     ink_comboboxentry_action_set_info(      Ink_ComboBoxEntry_Action* action, const gchar* info );
+void     ink_comboboxentry_action_set_info_cb(   Ink_ComboBoxEntry_Action* action, gpointer info_cb );
+void     ink_comboboxentry_action_set_warning(   Ink_ComboBoxEntry_Action* action, const gchar* warning_cb );
+void     ink_comboboxentry_action_set_warning_cb(Ink_ComboBoxEntry_Action* action, gpointer warning );
+void     ink_comboboxentry_action_set_tooltip(   Ink_ComboBoxEntry_Action* action, const gchar* tooltip );
 
 void     ink_comboboxentry_action_set_altx_name( Ink_ComboBoxEntry_Action* action, const gchar* altx_name );
 
