@@ -1,5 +1,3 @@
-#define __SPSVGVIEW_C__
-
 /*
  * Inkscape - an ambitious vector drawing program
  *
@@ -14,6 +12,7 @@
  *   Chema Celorio <chema@celorio.com>
  *   Pawel Palucha
  * ... and various people who have worked with various projects
+ *   Abhishek Sharma
  *
  * Copyright (C) 1999-2002 authors
  * Copyright (C) 2001-2002 Ximian, Inc.
@@ -35,12 +34,12 @@
 #include <getopt.h>
 #endif
 
-#include <string.h>
+#include <cstring>
 #include <sys/stat.h>
 #include <locale.h>
 
+#include <gtkmm/main.h>
 #include <glib.h>
-#include <libnr/nr-macros.h>
 
 // #include <stropts.h>
 
@@ -48,8 +47,6 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
-#include <gtkmm/main.h>
-
 #include "gc-core.h"
 #include "preferences.h"
 
@@ -57,6 +54,7 @@
 #include "document.h"
 #include "svg-view.h"
 #include "svg-view-widget.h"
+#include "util/units.h"
 
 #ifdef WITH_INKJAR
 #include "io/inkjar.h"
@@ -72,6 +70,8 @@ Inkscape::Application *inkscape;
 #define bind_textdomain_codeset(p,c)
 #endif
 
+#include "ui/icon-names.h"
+
 extern char *optarg;
 extern int  optind, opterr;
 
@@ -104,7 +104,7 @@ static void usage();
 
 static GtkWidget *ctrlwin = NULL;
 
-/// Dummy functions to keep linker happy
+// Dummy functions to keep linker happy
 int sp_main_gui (int, char const**) { return 0; }
 int sp_main_console (int, char const**) { return 0; }
 
@@ -119,15 +119,15 @@ static int
 sp_svgview_main_key_press (GtkWidget */*widget*/, GdkEventKey *event, struct SPSlideShow *ss)
 {
     switch (event->keyval) {
-    case GDK_Up:
-    case GDK_Home:
+    case GDK_KEY_Up:
+    case GDK_KEY_Home:
 	sp_svgview_goto_first(ss);
 	break;
-    case GDK_Down:
-    case GDK_End:
+    case GDK_KEY_Down:
+    case GDK_KEY_End:
 	sp_svgview_goto_last(ss);
 	break;
-    case GDK_F11:
+    case GDK_KEY_F11:
 #ifdef HAVE_GTK_WINDOW_FULLSCREEN
 	if (ss->fullscreen) {
 	    gtk_window_unfullscreen (GTK_WINDOW(ss->window));
@@ -140,30 +140,30 @@ sp_svgview_main_key_press (GtkWidget */*widget*/, GdkEventKey *event, struct SPS
 	std::cout<<"Your GTK+ does not support fullscreen mode. Upgrade to 2.2."<<std::endl;
 #endif
 	break;
-    case GDK_Return:
+    case GDK_KEY_Return:
 	sp_svgview_control_show (ss);
 	break;
-    case GDK_KP_Page_Down:
-    case GDK_Page_Down:
-    case GDK_Right:
-    case GDK_space:
+    case GDK_KEY_KP_Page_Down:
+    case GDK_KEY_Page_Down:
+    case GDK_KEY_Right:
+    case GDK_KEY_space:
 	sp_svgview_show_next (ss);
 	break;
-    case GDK_KP_Page_Up:
-    case GDK_Page_Up:
-    case GDK_Left:
-    case GDK_BackSpace:
+    case GDK_KEY_KP_Page_Up:
+    case GDK_KEY_Page_Up:
+    case GDK_KEY_Left:
+    case GDK_KEY_BackSpace:
 	sp_svgview_show_prev (ss);
 	break;
-    case GDK_Escape:
-    case GDK_q:
-    case GDK_Q:
+    case GDK_KEY_Escape:
+    case GDK_KEY_q:
+    case GDK_KEY_Q:
 	gtk_main_quit();
 	break;
     default:
 	break;
     }
-    gtk_window_set_title(GTK_WINDOW(ss->window), SP_DOCUMENT_NAME(ss->doc));
+    gtk_window_set_title(GTK_WINDOW(ss->window), ss->doc->getName());
     return TRUE;
 }
 
@@ -269,13 +269,13 @@ main (int argc, const char **argv)
 			    ss.slides = g_renew (char *, ss.slides, ss.size);
 			}
 
-			ss.doc = sp_document_new_from_mem ((const gchar *)gba->data,
+			ss.doc = SPDocument::createNewDocFromMem ((const gchar *)gba->data,
 							   gba->len,
 							   TRUE);
 			gchar *last_filename = jar_file_reader.get_last_filename();
 			if (ss.doc) {
 			    ss.slides[ss.length++] = strdup (last_filename);
-			    sp_document_set_uri (ss.doc, strdup(last_filename));
+			    (ss.doc)->setUri (strdup(last_filename));
 			}
 			g_byte_array_free(gba, TRUE);
 			g_free(last_filename);
@@ -295,7 +295,7 @@ main (int argc, const char **argv)
 		ss.slides[ss.length++] = strdup (argv[i]);
 
                 if (!ss.doc) {
-                    ss.doc = sp_document_new (ss.slides[ss.current], TRUE, false);
+                    ss.doc = SPDocument::createNewDoc (ss.slides[ss.current], TRUE, false);
                     if (!ss.doc)
                         ++ss.current;
 		}
@@ -309,21 +309,19 @@ main (int argc, const char **argv)
        return 1; /* none of the slides loadable */
 
     w = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_title (GTK_WINDOW (w), SP_DOCUMENT_NAME (ss.doc));
+    gtk_window_set_title( GTK_WINDOW(w), ss.doc->getName() );
     gtk_window_set_default_size (GTK_WINDOW (w),
-				 MIN ((int)sp_document_width (ss.doc), (int)gdk_screen_width () - 64),
-				 MIN ((int)sp_document_height (ss.doc), (int)gdk_screen_height () - 64));
-    gtk_window_set_policy (GTK_WINDOW (w), TRUE, TRUE, FALSE);
+				 MIN ((int)(ss.doc)->getWidth().value("px"), (int)gdk_screen_width() - 64),
+				 MIN ((int)(ss.doc)->getHeight().value("px"), (int)gdk_screen_height() - 64));
     ss.window = w;
 
     g_signal_connect (G_OBJECT (w), "delete_event", (GCallback) sp_svgview_main_delete, &ss);
     g_signal_connect (G_OBJECT (w), "key_press_event", (GCallback) sp_svgview_main_key_press, &ss);
 
-    sp_document_ensure_up_to_date (ss.doc);
+    (ss.doc)->ensureUpToDate();
     ss.view = sp_svg_view_widget_new (ss.doc);
-    sp_document_unref (ss.doc);
-    sp_svg_view_widget_set_resize (SP_SVG_VIEW_WIDGET (ss.view), FALSE,
-                                   sp_document_width (ss.doc), sp_document_height (ss.doc));
+    (ss.doc)->doUnref ();
+    SP_SVG_VIEW_WIDGET(ss.view)->setResize( false, ss.doc->getWidth().value("px"), ss.doc->getHeight().value("px") );
     gtk_widget_show (ss.view);
     gtk_container_add (GTK_CONTAINER (w), ss.view);
 
@@ -341,44 +339,65 @@ sp_svgview_ctrlwin_delete (GtkWidget */*widget*/, GdkEvent */*event*/, void */*d
     return FALSE;
 }
 
-static GtkWidget *
-sp_svgview_control_show (struct SPSlideShow *ss)
+static GtkWidget* sp_svgview_control_show(struct SPSlideShow *ss)
 {
     if (!ctrlwin) {
-	GtkWidget *t, *b;
-	ctrlwin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_transient_for (GTK_WINDOW(ctrlwin), GTK_WINDOW(ss->window));
-    g_signal_connect (G_OBJECT (ctrlwin), "key_press_event", (GCallback) sp_svgview_main_key_press, ss);
-	g_signal_connect (G_OBJECT (ctrlwin), "delete_event", (GCallback) sp_svgview_ctrlwin_delete, NULL);
-	t = gtk_table_new (1, 4, TRUE);
-	gtk_container_add ((GtkContainer *) ctrlwin, t);
-	b = gtk_button_new_from_stock (GTK_STOCK_GOTO_FIRST);
-	gtk_table_attach ((GtkTable *) t, b, 0, 1, 0, 1,
-			  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
-			  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
-			  0, 0);
-	g_signal_connect ((GObject *) b, "clicked", (GCallback) sp_svgview_goto_first_cb, ss);
-	b = gtk_button_new_from_stock (GTK_STOCK_GO_BACK);
-	gtk_table_attach ((GtkTable *) t, b, 1, 2, 0, 1,
-			  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
-			  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
-			  0, 0);
-	g_signal_connect (G_OBJECT(b), "clicked", (GCallback) sp_svgview_show_prev_cb, ss);
-	b = gtk_button_new_from_stock (GTK_STOCK_GO_FORWARD);
-	gtk_table_attach ((GtkTable *) t, b, 2, 3, 0, 1,
-			  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
-			  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
-			  0, 0);
-	g_signal_connect (G_OBJECT(b), "clicked", (GCallback) sp_svgview_show_next_cb, ss);
-	b = gtk_button_new_from_stock (GTK_STOCK_GOTO_LAST);
-	gtk_table_attach ((GtkTable *) t, b, 3, 4, 0, 1,
-			  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
-			  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
-			  0, 0);
-	g_signal_connect (G_OBJECT(b), "clicked", (GCallback) sp_svgview_goto_last_cb, ss);
-	gtk_widget_show_all (ctrlwin);
+	ctrlwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+        gtk_window_set_resizable(GTK_WINDOW(ctrlwin), FALSE);
+        gtk_window_set_transient_for(GTK_WINDOW(ctrlwin), GTK_WINDOW(ss->window));
+        g_signal_connect(G_OBJECT (ctrlwin), "key_press_event", (GCallback) sp_svgview_main_key_press, ss);
+        g_signal_connect(G_OBJECT (ctrlwin), "delete_event", (GCallback) sp_svgview_ctrlwin_delete, NULL);
+
+#if GTK_CHECK_VERSION(3,0,0)
+	GtkWidget *t = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
+#else
+	GtkWidget *t = gtk_hbutton_box_new();
+#endif
+
+	gtk_container_add(GTK_CONTAINER(ctrlwin), t);
+
+#if GTK_CHECK_VERSION(3,10,0)
+        GtkWidget *b = gtk_button_new_from_icon_name(INKSCAPE_ICON("go-first"), GTK_ICON_SIZE_BUTTON);
+#else
+	GtkWidget *b   = gtk_button_new();
+        GtkWidget *img = gtk_image_new_from_icon_name(INKSCAPE_ICON("go-first"), GTK_ICON_SIZE_BUTTON);
+        gtk_button_set_image(GTK_BUTTON(b), img);
+#endif
+	gtk_container_add(GTK_CONTAINER(t), b);
+
+	g_signal_connect(G_OBJECT(b), "clicked", (GCallback) sp_svgview_goto_first_cb, ss);
+#if GTK_CHECK_VERSION(3,10,0)
+        b = gtk_button_new_from_icon_name(INKSCAPE_ICON("go-previous"), GTK_ICON_SIZE_BUTTON);
+#else
+	b   = gtk_button_new();
+        img = gtk_image_new_from_icon_name(INKSCAPE_ICON("go-previous"), GTK_ICON_SIZE_BUTTON);
+        gtk_button_set_image(GTK_BUTTON(b), img);
+#endif
+	gtk_container_add(GTK_CONTAINER(t), b);
+
+	g_signal_connect(G_OBJECT(b), "clicked", (GCallback) sp_svgview_show_prev_cb, ss);
+#if GTK_CHECK_VERSION(3,10,0)
+        b = gtk_button_new_from_icon_name(INKSCAPE_ICON("go-next"), GTK_ICON_SIZE_BUTTON);
+#else
+	b   = gtk_button_new();
+        img = gtk_image_new_from_icon_name(INKSCAPE_ICON("go-next"), GTK_ICON_SIZE_BUTTON);
+        gtk_button_set_image(GTK_BUTTON(b), img);
+#endif
+	gtk_container_add(GTK_CONTAINER(t), b);
+
+	g_signal_connect(G_OBJECT(b), "clicked", (GCallback) sp_svgview_show_next_cb, ss);
+#if GTK_CHECK_VERSION(3,10,0)
+        b = gtk_button_new_from_icon_name(INKSCAPE_ICON("go-last"), GTK_ICON_SIZE_BUTTON);
+#else
+	b   = gtk_button_new();
+        img = gtk_image_new_from_icon_name(INKSCAPE_ICON("go-last"), GTK_ICON_SIZE_BUTTON);
+        gtk_button_set_image(GTK_BUTTON(b), img);
+#endif
+	gtk_container_add(GTK_CONTAINER(t), b);
+	g_signal_connect(G_OBJECT(b), "clicked", (GCallback) sp_svgview_goto_last_cb, ss);
+	gtk_widget_show_all(ctrlwin);
     } else {
-	gtk_window_present ((GtkWindow *) ctrlwin);
+	gtk_window_present(GTK_WINDOW(ctrlwin));
     }
 
     return NULL;
@@ -416,12 +435,20 @@ static void
 sp_svgview_waiting_cursor(struct SPSlideShow *ss)
 {
     GdkCursor *waiting = gdk_cursor_new(GDK_WATCH);
-    gdk_window_set_cursor(GTK_WIDGET(ss->window)->window, waiting);
+    gdk_window_set_cursor(gtk_widget_get_window(GTK_WIDGET(ss->window)), waiting);
+#if GTK_CHECK_VERSION(3,0,0)
+    g_object_unref(waiting);
+#else
     gdk_cursor_unref(waiting);
+#endif
     if (ctrlwin) {
         GdkCursor *waiting = gdk_cursor_new(GDK_WATCH);
-        gdk_window_set_cursor(GTK_WIDGET(ctrlwin)->window, waiting);
+        gdk_window_set_cursor(gtk_widget_get_window(GTK_WIDGET(ctrlwin)), waiting);
+#if GTK_CHECK_VERSION(3,0,0)
+        g_object_unref(waiting);
+#else
         gdk_cursor_unref(waiting);
+#endif
     }
     while(gtk_events_pending())
        gtk_main_iteration();
@@ -430,9 +457,9 @@ sp_svgview_waiting_cursor(struct SPSlideShow *ss)
 static void
 sp_svgview_normal_cursor(struct SPSlideShow *ss)
 {
-   gdk_window_set_cursor(GTK_WIDGET(ss->window)->window, NULL);
+   gdk_window_set_cursor(gtk_widget_get_window(GTK_WIDGET(ss->window)), NULL);
     if (ctrlwin) {
-        gdk_window_set_cursor(GTK_WIDGET(ctrlwin)->window, NULL);
+        gdk_window_set_cursor(gtk_widget_get_window(GTK_WIDGET(ctrlwin)), NULL);
     }
 }
 
@@ -440,7 +467,7 @@ static void
 sp_svgview_set_document(struct SPSlideShow *ss, SPDocument *doc, int current)
 {
     if (doc && doc != ss->doc) {
-        sp_document_ensure_up_to_date (doc);
+        doc->ensureUpToDate();
         reinterpret_cast<SPSVGView*>(SP_VIEW_WIDGET_VIEW (ss->view))->setDocument (doc);
         ss->doc = doc;
         ss->current = current;
@@ -455,7 +482,7 @@ sp_svgview_show_next (struct SPSlideShow *ss)
     SPDocument *doc = NULL;
     int current = ss->current;
     while (!doc && (current < ss->length - 1)) {
-        doc = sp_document_new (ss->slides[++current], TRUE, false);
+        doc = SPDocument::createNewDoc (ss->slides[++current], TRUE, false);
     }
 
     sp_svgview_set_document(ss, doc, current);
@@ -471,7 +498,7 @@ sp_svgview_show_prev (struct SPSlideShow *ss)
     SPDocument *doc = NULL;
     int current = ss->current;
     while (!doc && (current > 0)) {
-        doc = sp_document_new (ss->slides[--current], TRUE, false);
+        doc = SPDocument::createNewDoc (ss->slides[--current], TRUE, false);
     }
 
     sp_svgview_set_document(ss, doc, current);
@@ -489,7 +516,7 @@ sp_svgview_goto_first (struct SPSlideShow *ss)
     while ( !doc && (current < ss->length - 1)) {
         if (current == ss->current)
             break;
-        doc = sp_document_new (ss->slides[current++], TRUE, false);
+        doc = SPDocument::createNewDoc (ss->slides[current++], TRUE, false);
     }
 
     sp_svgview_set_document(ss, doc, current - 1);
@@ -507,7 +534,7 @@ sp_svgview_goto_last (struct SPSlideShow *ss)
     while (!doc && (current >= 0)) {
         if (current == ss->current)
             break;
-        doc = sp_document_new (ss->slides[current--], TRUE, false);
+        doc = SPDocument::createNewDoc (ss->slides[current--], TRUE, false);
     }
 
     sp_svgview_set_document(ss, doc, current + 1);
