@@ -1,22 +1,39 @@
-/**
+/*
  * Base64-enabled input and output streams
  *
  * This class allows easy encoding and decoding
  * of Base64 data with a stream interface, hiding
  * the implementation from the user.
  *
+ * http://www.w3.org/TR/2004/REC-DOM-Level-3-Core-20040407/idl-definitions.html
+ *
  * Authors:
- *   Bob Jamison <rjamison@titan.com>
+
+ *   Bob Jamison
  *
- * Copyright (C) 2004 Inkscape.org
+ * Copyright (C) 2006 Bob Jamison
  *
- * Released under GNU GPL, read the file 'COPYING' for more information
+
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2.1 of the License, or (at your option) any later version.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "base64stream.h"
 
 
 
+
 namespace Inkscape
 {
 namespace IO
@@ -51,12 +68,15 @@ static int base64decode[] =
  *
  */ 
 Base64InputStream::Base64InputStream(InputStream &sourceStream)
-                    : BasicInputStream(sourceStream)
+                    : BasicInputStream(sourceStream),
+                      outCount(0),
+                      padCount(0),
+                      done(false)
 {
-    outCount = 0;
-    padCount = 0;
-    closed   = false;
-    done     = false;
+    for (int k=0;k<3;k++)
+    {
+        outBytes[k]=0;
+    }
 }
 
 /**
@@ -270,12 +290,12 @@ void Base64OutputStream::putCh(int ch)
 /**
  * Writes the specified byte to this output stream.
  */ 
-void Base64OutputStream::put(int ch)
+int Base64OutputStream::put(gunichar ch)
 {
     if (closed)
         {
         //probably throw an exception here
-        return;
+        return -1;
         }
 
     outBuf   <<=  8;
@@ -302,6 +322,7 @@ void Base64OutputStream::put(int ch)
         bitCount = 0;
         outBuf   = 0L;
         }
+    return 1;
 }
 
 
