@@ -1,5 +1,5 @@
-#ifndef __SP_KNOTHOLDER_H__
-#define __SP_KNOTHOLDER_H__
+#ifndef SEEN_SP_KNOTHOLDER_H
+#define SEEN_SP_KNOTHOLDER_H
 
 /*
  * KnotHolder - Hold SPKnot list and manage signals
@@ -18,22 +18,29 @@
  */
 
 #include <glib.h>
-#include "knot-enums.h"
-#include "forward.h"
-#include "libnr/nr-forward.h"
 #include <2geom/forward.h>
-#include "knot-holder-entity.h"
 #include <list>
+#include <sigc++/connection.h>
 
 namespace Inkscape {
 namespace XML {
 class Node;
 }
+namespace LivePathEffect {
+class PowerStrokePointArrayParamKnotHolderEntity;
 }
+}
+
+class KnotHolderEntity;
+class SPItem;
+class SPDesktop;
+class SPKnot;
+
+/* fixme: Think how to make callbacks most sensitive (Lauris) */
+typedef void (* SPKnotHolderReleasedFunc) (SPItem *item);
 
 class KnotHolder {
 public:
-    KnotHolder() {} // do nothing in the default constructor
     KnotHolder(SPDesktop *desktop, SPItem *item, SPKnotHolderReleasedFunc relhandler);
     virtual ~KnotHolder();
 
@@ -41,7 +48,7 @@ public:
 
     void knot_moved_handler(SPKnot *knot, Geom::Point const &p, guint state);
     void knot_clicked_handler(SPKnot *knot, guint state);
-    void knot_ungrabbed_handler(SPKnot *knot);
+    void knot_ungrabbed_handler(SPKnot *knot, guint);
 
     void add(KnotHolderEntity *e);
 
@@ -49,21 +56,30 @@ public:
 
     const SPItem *getItem() { return item; }
 
-    bool knot_mouseover();
+    bool knot_mouseover() const;
 
     friend class ShapeEditor;
+    friend class Inkscape::LivePathEffect::PowerStrokePointArrayParamKnotHolderEntity;
 
 protected:
+
+    void updateControlSizes();
+
     SPDesktop *desktop;
     SPItem *item; // TODO: Remove this and keep the actual item (e.g., SPRect etc.) in the item-specific knotholders
     Inkscape::XML::Node *repr; ///< repr of the item, for setting and releasing listeners.
     std::list<KnotHolderEntity *> entity;
 
+    sigc::connection sizeUpdatedConn;
+
     SPKnotHolderReleasedFunc released;
 
     gboolean local_change; ///< if true, no need to recreate knotholder if repr was changed.
 
     bool dragging;
+
+private:
+    KnotHolder(); // declared but not defined
 };
 
 /**
@@ -72,7 +88,7 @@ void knot_moved_handler(SPKnot *knot, Geom::Point const *p, guint state, gpointe
 void knot_ungrabbed_handler(SPKnot *knot, unsigned int state, KnotHolder *kh);
 **/
 
-#endif /* !__SP_KNOTHOLDER_H__ */
+#endif // SEEN_SP_KNOTHOLDER_H
 
 /*
   Local Variables:
@@ -83,4 +99,4 @@ void knot_ungrabbed_handler(SPKnot *knot, unsigned int state, KnotHolder *kh);
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
