@@ -204,12 +204,6 @@ class ShiftSegment
 };
 typedef std::list<ShiftSegment> ShiftSegmentList;
 
-bool cmpShiftSegment(const ShiftSegment& u, const ShiftSegment& v)
-{
-    return u < v;
-}
-
-
 struct Node;
 struct CmpNodePos { bool operator()(const Node* u, const Node* v) const; };
 
@@ -488,7 +482,7 @@ Event **events;
 
 
 // Used for quicksort.  Must return <0, 0, or >0.
-int compare_events(const void *a, const void *b)
+static int compare_events(const void *a, const void *b)
 {
 	Event *ea = *(Event**) a;
 	Event *eb = *(Event**) b;
@@ -1516,7 +1510,7 @@ extern void generateStaticOrthogonalVisGraph(Router *router)
         const int pass = 1;
         processEventVert(router, scanline, segments, events[i], pass);
     }
-    COLA_ASSERT(scanline.size() == 0);
+    COLA_ASSERT(scanline.empty());
     for (unsigned i = 0; i < totalEvents; ++i)
     {
         delete events[i];
@@ -1595,7 +1589,7 @@ extern void generateStaticOrthogonalVisGraph(Router *router)
         const int pass = 1;
         processEventHori(router, scanline, vertSegments, events[i], pass);
     }
-    COLA_ASSERT(scanline.size() == 0);
+    COLA_ASSERT(scanline.empty());
     for (unsigned i = 0; i < totalEvents; ++i)
     {
         delete events[i];
@@ -1885,7 +1879,7 @@ static void buildOrthogonalChannelInfo(Router *router,
         processShiftEvent(router, scanline, segmentList, events[i],
                 dim, pass);
     }
-    COLA_ASSERT(scanline.size() == 0);
+    COLA_ASSERT(scanline.empty());
     for (unsigned i = 0; i < totalEvents; ++i)
     {
         delete events[i];
