@@ -26,7 +26,8 @@
 #include "cr-node-iface.h"
 
 /**
- *@file:
+ *@CRSelEng:
+ *
  *The definition of the  #CRSelEng class.
  *The #CRSelEng is actually the "Selection Engine"
  *class. This is highly experimental for at the moment and
@@ -114,6 +115,10 @@ cr_sel_eng_set_node_iface (CRSelEng *const a_this, CRNodeIface const *const a_no
         PRIVATE(a_this)->node_iface = a_node_iface;
 }
 
+/* Quick strcmp.  Test only for == 0 or != 0, not < 0 or > 0.  */
+#define strqcmp(str,lit,lit_len) \
+  (strlen (str) != (lit_len) || memcmp (str, lit, lit_len))
+
 static gboolean
 lang_pseudo_class_handler (CRSelEng *const a_this,
                            CRAdditionalSel * a_sel, CRXMLNodePtr a_node)
@@ -131,8 +136,9 @@ lang_pseudo_class_handler (CRSelEng *const a_this,
 
         node_iface = PRIVATE(a_this)->node_iface;
 
-        if (strncmp (a_sel->content.pseudo->name->stryng->str, 
-                     "lang", 4)
+        /* "xml:lang" needed for SVG */
+        if ( (strqcmp (a_sel->content.pseudo->name->stryng->str, "lang", 4 ) &&
+              (strqcmp (a_sel->content.pseudo->name->stryng->str, "xml:lang", 8 ) ) )
             || !a_sel->content.pseudo->type == FUNCTION_PSEUDO) {
                 cr_utils_trace_info ("This handler is for :lang only");
                 return FALSE;
@@ -144,10 +150,9 @@ lang_pseudo_class_handler (CRSelEng *const a_this,
                 return FALSE;
         for (; node; node = get_next_parent_element_node (node_iface, node)) {
                 char *val = node_iface->getProp (node, "lang");
+                if (!val) val = node_iface->getProp (node, "xml:lang");
                 if (val) {
-                        if (!strncmp (val,
-                                      a_sel->content.pseudo->extra->stryng->str,
-                                      a_sel->content.pseudo->extra->stryng->len)) {
+                        if (!strcasecmp(val, a_sel->content.pseudo->extra->stryng->str)) {
                                 result = TRUE;
                                 break;
                         }
@@ -248,6 +253,10 @@ class_add_sel_matches_node (CRAdditionalSel * a_add_sel,
                                 if ((cur && !*cur)
                                     || cr_utils_is_white_space (*cur) == TRUE)
                                         result = TRUE;
+                        } else {  /* if it doesn't match,  */
+                                /*   then skip to next whitespace character to try again */
+                                while (cur && *cur && !(cr_utils_is_white_space(*cur) == TRUE)) 
+                                        cur++;
                         }
                         if (cur && !*cur)
                                 break ;
@@ -284,7 +293,7 @@ id_add_sel_matches_node (CRAdditionalSel * a_add_sel,
 
         id = a_node_iface->getProp (a_node, "id");
         if (id) {
-                if (!strncmp (id, a_add_sel->content.id_name->stryng->str,
+                if (!strqcmp (id, a_add_sel->content.id_name->stryng->str,
                               a_add_sel->content.id_name->stryng->len)) {
                         result = TRUE;
                 }
@@ -505,11 +514,11 @@ additional_selector_matches_node (CRSelEng * a_this,
                         continue ;
                 } else if (cur_add_sel->type == PSEUDO_CLASS_ADD_SELECTOR
                            && cur_add_sel->content.pseudo) {
-                        if (pseudo_class_add_sel_matches_node
+                        if (!pseudo_class_add_sel_matches_node
                             (a_this, cur_add_sel, a_node)) {
-                                return TRUE;
+                                return FALSE;
                         }
-                        return FALSE;
+                        continue ;
                 }
         }
         if (evaluated == TRUE)
@@ -839,7 +848,7 @@ cr_sel_eng_get_matched_rulesets_real (CRSelEng * a_this,
                 sel_list = NULL;
 
                 /*
-                 *get the the damn selector list in 
+                 *get the damn selector list in 
                  *which we have to look
                  */
                 switch (cur_stmt->type) {
@@ -1086,8 +1095,10 @@ set_style_from_props (CRStyle * a_style, CRPropList * a_props)
  ****************************************/
 
 /**
+ * cr_sel_eng_new:
  *Creates a new instance of #CRSelEng.
- *@return the newly built instance of #CRSelEng of
+ *
+ *Returns the newly built instance of #CRSelEng of
  *NULL if an error occurs.
  */
 CRSelEng *
@@ -1122,13 +1133,16 @@ cr_sel_eng_new (void)
 }
 
 /**
- *Adds a new handler entry in the handlers entry table.
- *@param a_this the current instance of #CRSelEng
- *@param a_pseudo_class_sel_name the name of the pseudo class selector.
- *@param a_pseudo_class_type the type of the pseudo class selector.
- *@param a_handler the actual handler or callback to be called during
+ * cr_sel_eng_register_pseudo_class_sel_handler:
+ *@a_this: the current instance of #CRSelEng
+ *@a_pseudo_class_sel_name: the name of the pseudo class selector.
+ *@a_pseudo_class_type: the type of the pseudo class selector.
+ *@a_handler: the actual handler or callback to be called during
  *the selector evaluation process.
- *@return CR_OK, upon successful completion, an error code otherwise.
+ *
+ *Adds a new handler entry in the handlers entry table.
+ *
+ *Returns CR_OK, upon successful completion, an error code otherwise.
  */
 enum CRStatus
 cr_sel_eng_register_pseudo_class_sel_handler (CRSelEng * a_this,
@@ -1198,10 +1212,13 @@ cr_sel_eng_unregister_pseudo_class_sel_handler (CRSelEng * a_this,
 }
 
 /**
+ * cr_sel_eng_unregister_all_pseudo_class_sel_handlers:
+ *@a_this: the current instance of #CRSelEng .
+ *
  *Unregisters all the pseudo class sel handlers
  *and frees all the associated allocated datastructures.
- *@param a_this the current instance of #CRSelEng .
- *@return CR_OK upon succesful completion, an error code
+ *
+ *Returns CR_OK upon succesful completion, an error code
  *otherwise.
  */
 enum CRStatus
@@ -1262,17 +1279,20 @@ cr_sel_eng_get_pseudo_class_selector_handler (CRSelEng * a_this,
 }
 
 /**
+ * cr_sel_eng_matches_node:
+ *@a_this: the selection engine.
+ *@a_sel: the simple selector against which the xml node 
+ *is going to be matched.
+ *@a_node: the node against which the selector is going to be matched.
+ *@a_result: out parameter. The result of the match. Is set to
+ *TRUE if the selector matches the node, FALSE otherwise. This value
+ *is considered if and only if this functions returns CR_OK.
+ *
  *Evaluates a chained list of simple selectors (known as a css2 selector).
  *Says wheter if this selector matches the xml node given in parameter or
  *not.
- *@param a_this the selection engine.
- *@param a_sel the simple selector against which the xml node 
- *is going to be matched.
- *@param a_node the node against which the selector is going to be matched.
- *@param a_result out parameter. The result of the match. Is set to
- *TRUE if the selector matches the node, FALSE otherwise. This value
- *is considered if and only if this functions returns CR_OK.
- *@return the CR_OK if the selection ran correctly, an error code otherwise.
+ *
+ *Returns the CR_OK if the selection ran correctly, an error code otherwise.
  */
 enum CRStatus
 cr_sel_eng_matches_node (CRSelEng * a_this, CRSimpleSel * a_sel,
@@ -1293,20 +1313,22 @@ cr_sel_eng_matches_node (CRSelEng * a_this, CRSimpleSel * a_sel,
 }
 
 /**
- *Returns an array of pointers to selectors that matches
- *the xml node given in parameter.
- *
- *@param a_this the current instance of the selection engine.
- *@param a_sheet the stylesheet that holds the selectors.
- *@param a_node the xml node to consider during the walk thru
+ * cr_sel_eng_get_matched_rulesets:
+ *@a_this: the current instance of the selection engine.
+ *@a_sheet: the stylesheet that holds the selectors.
+ *@a_node: the xml node to consider during the walk thru
  *the stylesheet.
- *@param a_rulesets out parameter. A pointer to an array of
+ *@a_rulesets: out parameter. A pointer to an array of
  *rulesets statement pointers. *a_rulesets is allocated by
  *this function and must be freed by the caller. However, the caller
  *must not alter the rulesets statements pointer because they
  *point to statements that are still in the css stylesheet.
- *@param a_len the length of *a_ruleset.
- *@return CR_OK upon sucessfull completion, an error code otherwise.
+ *@a_len: the length of *a_ruleset.
+ *
+ *Returns an array of pointers to selectors that matches
+ *the xml node given in parameter.
+ *
+ *Returns CR_OK upon sucessfull completion, an error code otherwise.
  */
 enum CRStatus
 cr_sel_eng_get_matched_rulesets (CRSelEng * a_this,
@@ -1517,8 +1539,10 @@ cr_sel_eng_get_matched_style (CRSelEng * a_this,
 }
 
 /**
+ * cr_sel_eng_destroy:
+ *@a_this: the current instance of the selection engine.
+ *
  *The destructor of #CRSelEng
- *@param a_this the current instance of the selection engine.
  */
 void
 cr_sel_eng_destroy (CRSelEng * a_this)
