@@ -4,20 +4,22 @@
  * Copyright (C) 2003  Ettore Perazzoli
  *               2007  Naba Kumar
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+* This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
+ *
  * Authors: Ettore Perazzoli <ettore@ximian.com>
  *          Naba Kumar  <naba@gnome.org>
  */
@@ -39,14 +41,6 @@ typedef struct _GdlSwitcher        GdlSwitcher;
 typedef struct _GdlSwitcherPrivate GdlSwitcherPrivate;
 typedef struct _GdlSwitcherClass   GdlSwitcherClass;
 
-typedef enum {
-    GDL_SWITCHER_STYLE_TEXT,
-    GDL_SWITCHER_STYLE_ICON,
-    GDL_SWITCHER_STYLE_BOTH,
-    GDL_SWITCHER_STYLE_TOOLBAR,
-    GDL_SWITCHER_STYLE_TABS
-} GdlSwitcherStyle;
-
 struct _GdlSwitcher {
     GtkNotebook parent;
 
@@ -66,7 +60,7 @@ gint       gdl_switcher_insert_page  (GdlSwitcher *switcher,
                                       const gchar *label,
                                       const gchar *tooltips,
                                       const gchar *stock_id,
-                                      const GdkPixbuf *pixbuf_icon,
+                                      GdkPixbuf *pixbuf_icon,
                                       gint position);
 G_END_DECLS
 
