@@ -29,7 +29,7 @@ public:
 
     virtual std::vector<Geom::Path> doEffect_path (std::vector<Geom::Path> const & path_in);
 
-    virtual void resetDefaults(SPItem * item);
+    virtual void resetDefaults(SPItem const* item);
 private:
     PathParam   trajectory_path;
     ScalarParam number_of_steps;
