@@ -10,18 +10,16 @@
  */
 
 #include <glibmm/ustring.h>
-#include <2geom/point.h>
-#include <2geom/path.h>
+#include <2geom/forward.h>
+#include <2geom/pathvector.h>
 
-#include "ui/widget/registry.h"
-#include "ui/widget/registered-widget.h"
-
-struct SPDesktop;
-struct SPItem;
+class KnotHolder;
+class SPLPEItem;
+class SPDesktop;
+class SPItem;
 
 namespace Gtk {
     class Widget;
-    class Tooltips;
 }
 
 namespace Inkscape {
@@ -30,6 +28,12 @@ namespace NodePath {
     class Path ;
 }
 
+namespace UI {
+namespace Widget {
+    class Registry;
+}
+}
+
 namespace LivePathEffect {
 
 class Effect;
@@ -44,25 +48,32 @@ public:
     virtual ~Parameter() {};
 
     virtual bool param_readSVGValue(const gchar * strvalue) = 0;   // returns true if new value is valid / accepted.
-    virtual gchar * param_writeSVGValue() const = 0;
-
+    virtual gchar * param_getSVGValue() const = 0;
+    void write_to_SVG();
+ 
     virtual void param_set_default() = 0;
 
     // This creates a new widget (newed with Gtk::manage(new ...);)
-    virtual Gtk::Widget * param_newWidget(Gtk::Tooltips * tooltips) = 0;
+    virtual Gtk::Widget * param_newWidget() = 0;
 
     virtual Glib::ustring * param_getTooltip() { return &param_tooltip; };
 
+    // overload these for your particular parameter to make it provide knotholder handles or canvas helperpaths
+    virtual bool providesKnotHolderEntities() const { return false; }
+    virtual void addKnotHolderEntities(KnotHolder */*knotholder*/, SPDesktop */*desktop*/, SPItem */*item*/) {};
+    virtual void addCanvasIndicators(SPLPEItem const*/*lpeitem*/, std::vector<Geom::PathVector> &/*hp_vec*/) {};
+
     virtual void param_editOncanvas(SPItem * /*item*/, SPDesktop * /*dt*/) {};
     virtual void param_setup_nodepath(Inkscape::NodePath::Path */*np*/) {};
 
-    virtual void param_transform_multiply(Geom::Matrix const& /*postmul*/, bool /*set*/) {};
+    virtual void param_transform_multiply(Geom::Affine const& /*postmul*/, bool /*set*/) {};
 
     Glib::ustring param_key;
     Inkscape::UI::Widget::Registry * param_wr;
     Glib::ustring param_label;
 
     bool oncanvas_editable;
+    bool widget_is_visible;
 
 protected:
     Glib::ustring param_tooltip;
@@ -88,7 +99,7 @@ public:
     virtual ~ScalarParam();
 
     virtual bool param_readSVGValue(const gchar * strvalue);
-    virtual gchar * param_writeSVGValue() const;
+    virtual gchar * param_getSVGValue() const;
 
     virtual void param_set_default();
     void param_set_value(gdouble val);
@@ -97,10 +108,11 @@ public:
     void param_set_digits(unsigned digits);
     void param_set_increments(double step, double page);
 
-    virtual Gtk::Widget * param_newWidget(Gtk::Tooltips * tooltips);
+    void addSlider(bool add_slider_widget) { add_slider = add_slider_widget; };
 
-    inline operator gdouble()
-        { return value; };
+    virtual Gtk::Widget * param_newWidget();
+
+    inline operator gdouble() const { return value; };
 
 protected:
     gdouble value;
@@ -111,6 +123,7 @@ protected:
     unsigned digits;
     double inc_step;
     double inc_page;
+    bool add_slider;
 
 private:
     ScalarParam(const ScalarParam&);
@@ -132,4 +145,4 @@ private:
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
