@@ -1,11 +1,13 @@
-#define INKSCAPE_LIVEPATHEFFECT_PARAMETER_PATH_CPP
-
 /*
  * Copyright (C) Johan Engelen 2007 <j.b.c.engelen@utwente.nl>
+ *   Abhishek Sharma
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#include "ui/widget/point.h"
+#include <glibmm/i18n.h>
+
 #include "live_effects/parameter/path.h"
 #include "live_effects/effect.h"
 #include "svg/svg.h"
@@ -14,9 +16,8 @@
 #include <2geom/pathvector.h>
 #include <2geom/d2.h>
 
-#include "ui/widget/point.h"
 #include "widgets/icon.h"
-#include <gtk/gtkstock.h>
+#include <gtk/gtk.h>
 #include "selection-chemistry.h"
 #include "xml/repr.h"
 #include "desktop.h"
@@ -24,6 +25,7 @@
 #include "message-stack.h"
 #include "verbs.h"
 #include "document.h"
+#include "document-undo.h"
 
 // needed for on-canvas editting:
 #include "tools-switch.h"
@@ -38,10 +40,13 @@
 #include "sp-text.h"
 #include "display/curve.h"
 
-#include "ui/tool/node-tool.h"
+#include "ui/tools/node-tool.h"
 #include "ui/tool/multi-path-manipulator.h"
 #include "ui/tool/shape-record.h"
 
+#include <gtkmm/button.h>
+#include <gtkmm/label.h>
+#include "ui/icon-names.h"
 
 namespace Inkscape {
 
@@ -73,7 +78,7 @@ PathParam::~PathParam()
 }
 
 std::vector<Geom::Path> const &
-PathParam::get_pathvector()
+PathParam::get_pathvector() const
 {
     return _pathvector;
 }
@@ -113,6 +118,11 @@ PathParam::param_readSVGValue(const gchar * strvalue)
             // Now do the attaching, which emits the changed signal.
             try {
                 ref.attach(Inkscape::URI(href));
+                //lp:1299948
+                SPItem* i = ref.getObject();
+                if (i) {
+                    linked_modified_callback(i, SP_OBJECT_MODIFIED_FLAG);
+                } // else: document still processing new events. Repr of the linked object not created yet.
             } catch (Inkscape::BadURIException &e) {
                 g_warning("%s", e.what());
                 ref.detach();
@@ -133,7 +143,7 @@ gchar *
 PathParam::param_getSVGValue() const
 {
     if (href) {
-        return href;
+        return g_strdup(href);
     } else {
         gchar * svgd = sp_svg_write_path( _pathvector );
         return svgd;
@@ -141,15 +151,15 @@ PathParam::param_getSVGValue() const
 }
 
 Gtk::Widget *
-PathParam::param_newWidget(Gtk::Tooltips * tooltips)
+PathParam::param_newWidget()
 {
     Gtk::HBox * _widget = Gtk::manage(new Gtk::HBox());
 
     Gtk::Label* pLabel = Gtk::manage(new Gtk::Label(param_label));
     static_cast<Gtk::HBox*>(_widget)->pack_start(*pLabel, true, true);
-    tooltips->set_tip(*pLabel, param_tooltip);
+    pLabel->set_tooltip_text(param_tooltip);
 
-    Gtk::Widget*  pIcon = Gtk::manage( sp_icon_get_icon( "tool-node-editor", Inkscape::ICON_SIZE_BUTTON) );
+    Gtk::Widget*  pIcon = Gtk::manage( sp_icon_get_icon( INKSCAPE_ICON("tool-node-editor"), Inkscape::ICON_SIZE_BUTTON) );
     Gtk::Button * pButton = Gtk::manage(new Gtk::Button());
     pButton->set_relief(Gtk::RELIEF_NONE);
     pIcon->show();
@@ -157,9 +167,9 @@ PathParam::param_newWidget(Gtk::Tooltips * tooltips)
     pButton->show();
     pButton->signal_clicked().connect(sigc::mem_fun(*this, &PathParam::on_edit_button_click));
     static_cast<Gtk::HBox*>(_widget)->pack_start(*pButton, true, true);
-    tooltips->set_tip(*pButton, _("Edit on-canvas"));
+    pButton->set_tooltip_text(_("Edit on-canvas"));
 
-    pIcon = Gtk::manage( sp_icon_get_icon( GTK_STOCK_COPY, Inkscape::ICON_SIZE_BUTTON) );
+    pIcon = Gtk::manage( sp_icon_get_icon( INKSCAPE_ICON("edit-copy"), Inkscape::ICON_SIZE_BUTTON) );
     pButton = Gtk::manage(new Gtk::Button());
     pButton->set_relief(Gtk::RELIEF_NONE);
     pIcon->show();
@@ -167,9 +177,9 @@ PathParam::param_newWidget(Gtk::Tooltips * tooltips)
     pButton->show();
     pButton->signal_clicked().connect(sigc::mem_fun(*this, &PathParam::on_copy_button_click));
     static_cast<Gtk::HBox*>(_widget)->pack_start(*pButton, true, true);
-    tooltips->set_tip(*pButton, _("Copy path"));
+    pButton->set_tooltip_text(_("Copy path"));
 
-    pIcon = Gtk::manage( sp_icon_get_icon( GTK_STOCK_PASTE, Inkscape::ICON_SIZE_BUTTON) );
+    pIcon = Gtk::manage( sp_icon_get_icon( INKSCAPE_ICON("edit-paste"), Inkscape::ICON_SIZE_BUTTON) );
     pButton = Gtk::manage(new Gtk::Button());
     pButton->set_relief(Gtk::RELIEF_NONE);
     pIcon->show();
@@ -177,9 +187,9 @@ PathParam::param_newWidget(Gtk::Tooltips * tooltips)
     pButton->show();
     pButton->signal_clicked().connect(sigc::mem_fun(*this, &PathParam::on_paste_button_click));
     static_cast<Gtk::HBox*>(_widget)->pack_start(*pButton, true, true);
-    tooltips->set_tip(*pButton, _("Paste path"));
+    pButton->set_tooltip_text(_("Paste path"));
 
-    pIcon = Gtk::manage( sp_icon_get_icon( "edit-clone", Inkscape::ICON_SIZE_BUTTON) );
+    pIcon = Gtk::manage( sp_icon_get_icon( INKSCAPE_ICON("edit-clone"), Inkscape::ICON_SIZE_BUTTON) );
     pButton = Gtk::manage(new Gtk::Button());
     pButton->set_relief(Gtk::RELIEF_NONE);
     pIcon->show();
@@ -187,7 +197,7 @@ PathParam::param_newWidget(Gtk::Tooltips * tooltips)
     pButton->show();
     pButton->signal_clicked().connect(sigc::mem_fun(*this, &PathParam::on_link_button_click));
     static_cast<Gtk::HBox*>(_widget)->pack_start(*pButton, true, true);
-    tooltips->set_tip(*pButton, _("Link to path"));
+    pButton->set_tooltip_text(_("Link to path on clipboard"));
 
     static_cast<Gtk::HBox*>(_widget)->show_all_children();
 
@@ -204,12 +214,12 @@ PathParam::param_editOncanvas(SPItem *item, SPDesktop * dt)
         tools_switch(dt, TOOLS_NODES);
     }
 
-    InkNodeTool *nt = static_cast<InkNodeTool*>(dt->event_context);
+    Inkscape::UI::Tools::NodeTool *nt = static_cast<Inkscape::UI::Tools::NodeTool*>(dt->event_context);
     std::set<ShapeRecord> shapes;
     ShapeRecord r;
 
     r.role = SHAPE_ROLE_LPE_PARAM;
-    r.edit_transform = sp_item_i2d_affine(item); // TODO is it right?
+    r.edit_transform = item->i2dt_affine(); // TODO is it right?
     if (!href) {
         r.item = reinterpret_cast<SPItem*>(param_effect->getLPEObj());
         r.lpe_key = param_key;
@@ -227,7 +237,7 @@ PathParam::param_setup_nodepath(Inkscape::NodePath::Path *)
 }
 
 void
-PathParam::addCanvasIndicators(SPLPEItem */*lpeitem*/, std::vector<Geom::PathVector> &hp_vec)
+PathParam::addCanvasIndicators(SPLPEItem const*/*lpeitem*/, std::vector<Geom::PathVector> &hp_vec)
 {
     hp_vec.push_back(_pathvector);
 }
@@ -236,7 +246,7 @@ PathParam::addCanvasIndicators(SPLPEItem */*lpeitem*/, std::vector<Geom::PathVec
  * Only applies transform when not referring to other path!
  */
 void
-PathParam::param_transform_multiply(Geom::Matrix const& postmul, bool /*set*/)
+PathParam::param_transform_multiply(Geom::Affine const& postmul, bool /*set*/)
 {
     // only apply transform when not referring to other path
     if (!href) {
@@ -324,6 +334,9 @@ PathParam::start_listening(SPObject * to)
     }
     linked_delete_connection = to->connectDelete(sigc::mem_fun(*this, &PathParam::linked_delete));
     linked_modified_connection = to->connectModified(sigc::mem_fun(*this, &PathParam::linked_modified));
+    if (SP_IS_ITEM(to)) {
+        linked_transformed_connection = SP_ITEM(to)->connectTransformed(sigc::mem_fun(*this, &PathParam::linked_transformed));
+    }
     linked_modified(to, SP_OBJECT_MODIFIED_FLAG); // simulate linked_modified signal, so that path data is updated
 }
 
@@ -332,6 +345,7 @@ PathParam::quit_listening(void)
 {
     linked_modified_connection.disconnect();
     linked_delete_connection.disconnect();
+    linked_transformed_connection.disconnect();
 }
 
 void
@@ -361,12 +375,22 @@ PathParam::linked_delete(SPObject */*deleted*/)
     set_new_value (_pathvector, true);
 }
 
+void PathParam::linked_modified(SPObject *linked_obj, guint flags)
+{
+    linked_modified_callback(linked_obj, flags);
+}
+
+void PathParam::linked_transformed(Geom::Affine const *rel_transf, SPItem *moved_item)
+{
+    linked_transformed_callback(rel_transf, moved_item);
+}
+
 void
-PathParam::linked_modified(SPObject *linked_obj, guint /*flags*/)
+PathParam::linked_modified_callback(SPObject *linked_obj, guint /*flags*/)
 {
     SPCurve *curve = NULL;
     if (SP_IS_SHAPE(linked_obj)) {
-        curve = sp_shape_get_curve(SP_SHAPE(linked_obj));
+        curve = SP_SHAPE(linked_obj)->getCurve();
     }
     if (SP_IS_TEXT(linked_obj)) {
         curve = SP_TEXT(linked_obj)->getNormalizedBpath();
@@ -385,6 +409,7 @@ PathParam::linked_modified(SPObject *linked_obj, guint /*flags*/)
     SP_OBJECT(param_effect->getLPEObj())->requestModified(SP_OBJECT_MODIFIED_FLAG);
 }
 
+
 /* CALLBACK FUNCTIONS FOR THE BUTTONS */
 void
 PathParam::on_edit_button_click()
@@ -414,8 +439,8 @@ PathParam::on_paste_button_click()
     Inkscape::UI::ClipboardManager *cm = Inkscape::UI::ClipboardManager::get();
     Glib::ustring svgd = cm->getPathParameter(SP_ACTIVE_DESKTOP);
     paste_param_path(svgd.data());
-    sp_document_done(param_effect->getSPDoc(), SP_VERB_DIALOG_LIVE_PATH_EFFECT,
-                     _("Paste path parameter"));
+    DocumentUndo::done(param_effect->getSPDoc(), SP_VERB_DIALOG_LIVE_PATH_EFFECT,
+                       _("Paste path parameter"));
 }
 
 void
@@ -446,8 +471,8 @@ PathParam::on_link_button_click()
         // check if linking to object to which LPE is applied (maybe delegated to PathReference
 
         param_write_to_repr(pathid.c_str());
-        sp_document_done(param_effect->getSPDoc(), SP_VERB_DIALOG_LIVE_PATH_EFFECT,
-                         _("Link path parameter to path"));
+        DocumentUndo::done(param_effect->getSPDoc(), SP_VERB_DIALOG_LIVE_PATH_EFFECT,
+                           _("Link path parameter to path"));
     }
 }
 
