@@ -14,10 +14,8 @@ static char const menus_skeleton[] =
 "  xmlns:inkscape=\"http://www.inkscape.org/namespaces/inkscape\">\n"
 "\n"
 "   <submenu name=\"" N_("_File") "\">\n"
-"       <submenu name=\"" N_("_New") "\">\n"
-"           <verb verb-id=\"FileNew\" />\n"
-"           <template-list/>\n"
-"       </submenu>\n"
+"       <verb verb-id=\"FileNew\" />\n"
+"       <verb verb-id=\"FileTemplates\" />\n"
 "       <verb verb-id=\"FileOpen\" />\n"
 "       <recent-file-list/>\n"
 "       <verb verb-id=\"FileRevert\" />\n"
@@ -26,8 +24,8 @@ static char const menus_skeleton[] =
 "       <verb verb-id=\"FileSaveACopy\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"FileImport\" />\n"
-"       <verb verb-id=\"FileExport\" />\n"
-#ifdef WITH_GNOME_VFS
+"       <verb verb-id=\"DialogExport\" />\n"
+#if !defined(ENABLE_OSX_APP_LOCATIONS)
 "       <verb verb-id=\"FileImportFromOCAL\" />\n"
 //"       <verb verb-id=\"FileExportToOCAL\" />\n"
 #endif
@@ -37,9 +35,7 @@ static char const menus_skeleton[] =
 "       <verb verb-id=\"FileVacuum\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"DialogDocumentProperties\" />\n"
-"       <verb verb-id=\"DialogMetadata\" />\n"
-"       <verb verb-id=\"DialogPreferences\" />\n"
-"       <verb verb-id=\"DialogInput\" />\n"
+//"       <verb verb-id=\"DialogMetadata\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"FileClose\" />\n"
 "       <verb verb-id=\"FileQuit\" />\n"
@@ -72,6 +68,7 @@ static char const menus_skeleton[] =
 "           <verb verb-id=\"EditUnlinkClone\" />\n"
 "           <verb verb-id=\"EditRelinkClone\" />\n"
 "           <verb verb-id=\"EditCloneSelectOriginal\" />\n"
+"           <verb verb-id=\"EditCloneOriginalPathLPE\" />\n"
 "       </submenu>\n"
 "       <verb verb-id=\"SelectionCreateBitmap\" />\n"
 "       <separator/>\n"
@@ -79,12 +76,23 @@ static char const menus_skeleton[] =
 "       <separator/>\n"
 "       <verb verb-id=\"EditSelectAll\" />\n"
 "       <verb verb-id=\"EditSelectAllInAllLayers\" />\n"
+"       <submenu name=\"" N_("Select Sa_me") "\">\n"
+"           <verb verb-id=\"EditSelectSameFillStroke\" />\n"
+"           <verb verb-id=\"EditSelectSameFillColor\" />\n"
+"           <verb verb-id=\"EditSelectSameStrokeColor\" />\n"
+"           <verb verb-id=\"EditSelectSameStrokeStyle\" />\n"
+"           <verb verb-id=\"EditSelectSameObjectType\" />\n"
+"       </submenu>\n"
 "       <verb verb-id=\"EditInvert\" />\n"
 "       <verb verb-id=\"EditDeselect\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"EditGuidesAroundPage\" />\n"
+"       <verb verb-id=\"EditRemoveAllGuides\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"DialogXMLEditor\" />\n"
+"       <separator/>\n"
+"       <verb verb-id=\"DialogInput\" />\n"
+"       <verb verb-id=\"DialogPreferences\" />\n"
 "   </submenu>\n"
 "   <submenu name=\"" N_("_View") "\">\n"
 "     <submenu name=\"" N_("_Zoom") "\">\n"
@@ -112,20 +120,26 @@ static char const menus_skeleton[] =
 //"         <verb verb-id=\"ViewModePrintColorsPreview\" radio=\"yes\"/>\n"
 //"         <verb verb-id=\"DialogPrintColorsPreview\" />\n"
 "       </submenu>\n"
+"       <submenu name=\"" N_("_Color display mode") "\">\n"
+"         <verb verb-id=\"ViewColorModeToggle\"/>\n"
+"         <verb verb-id=\"ViewColorModeNormal\" radio=\"yes\" default=\"yes\"/>\n"
+"         <verb verb-id=\"ViewColorModeGrayscale\" radio=\"yes\"/>\n"
+// Better location in menu needs to be found
+//"         <verb verb-id=\"ViewColorModePrintColorsPreview\" radio=\"yes\"/>\n"
+//"         <verb verb-id=\"DialogPrintColorsPreview\" />\n"
+"       </submenu>\n"
 "       <separator/>\n"
-"       <verb verb-id=\"ToggleGrid\" />\n"
-"       <verb verb-id=\"ToggleGuides\" />\n"
-"       <verb verb-id=\"ToggleSnapGlobal\" />\n"
-"       <verb verb-id=\"ViewCmsToggle\" />\n"
+"       <verb verb-id=\"ToggleGrid\" check=\"yes\" />\n"
+"       <verb verb-id=\"ToggleGuides\" check=\"yes\" />\n"
+"       <verb verb-id=\"ViewCmsToggle\" check=\"yes\" />\n"
 "       <separator/>\n"
-"       <submenu name=\"" N_("Show/Hide") "\">\n"
+"       <submenu name=\"" N_("Sh_ow/Hide") "\">\n"
 "           <objects-checkboxes/>\n"
 "       </submenu>\n"
 "       <verb verb-id=\"DialogsToggle\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"DialogSwatches\" />\n"
 "       <verb verb-id=\"DialogDebug\" />\n"
-//"       <verb verb-id=\"DialogScript\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"PrevWindow\" />\n"
 "       <verb verb-id=\"NextWindow\" />\n"
@@ -141,20 +155,24 @@ static char const menus_skeleton[] =
 "   </submenu>\n"
 "   <submenu name=\"" N_("_Layer") "\">\n"
 "       <verb verb-id=\"LayerNew\" />\n"
-"       <verb verb-id=\"LayerDuplicate\" />\n"
 "       <verb verb-id=\"LayerRename\" />\n"
 "       <separator/>\n"
+"       <verb verb-id=\"LayerToggleHide\" />\n"
+"       <verb verb-id=\"LayerToggleLock\" />\n"
+"       <separator/>\n"
 "       <verb verb-id=\"LayerNext\" />\n"
 "       <verb verb-id=\"LayerPrev\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"LayerMoveToNext\" />\n"
 "       <verb verb-id=\"LayerMoveToPrev\" />\n"
+"       <verb verb-id=\"LayerMoveTo\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"LayerRaise\" />\n"
 "       <verb verb-id=\"LayerLower\" />\n"
 "       <verb verb-id=\"LayerToTop\" />\n"
 "       <verb verb-id=\"LayerToBottom\" />\n"
 "       <separator/>\n"
+"       <verb verb-id=\"LayerDuplicate\" />\n"
 "       <verb verb-id=\"LayerDelete\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"DialogLayers\" />\n"
@@ -162,6 +180,7 @@ static char const menus_skeleton[] =
 "   <submenu name=\"" N_("_Object") "\">\n"
 "       <verb verb-id=\"DialogFillStroke\" />\n"
 "       <verb verb-id=\"DialogObjectProperties\" />\n"
+"       <verb verb-id=\"DialogSymbols\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"SelectionGroup\" />\n"
 "       <verb verb-id=\"SelectionUnGroup\" />\n"
@@ -196,12 +215,13 @@ static char const menus_skeleton[] =
 "       <separator/>\n"
 "       <verb verb-id=\"DialogTransform\" />\n"
 "       <verb verb-id=\"DialogAlignDistribute\" />\n"
-"       <verb verb-id=\"DialogGridArrange\" />\n"
+"       <verb verb-id=\"DialogArrange\" />\n"
 "   </submenu>\n"
 "   <submenu name=\"" N_("_Path") "\">\n"
 "       <verb verb-id=\"ObjectToPath\" />\n"
 "       <verb verb-id=\"StrokeToPath\" />\n"
 "       <verb verb-id=\"SelectionTrace\" />\n"
+"       <verb verb-id=\"SelectionPixelArt\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"SelectionUnion\" />\n"
 "       <verb verb-id=\"SelectionDiff\" />\n"
@@ -227,9 +247,7 @@ static char const menus_skeleton[] =
 "   </submenu>\n"
 "   <submenu name=\"" N_("_Text") "\">\n"
 "       <verb verb-id=\"DialogText\" />\n"
-#ifdef ENABLE_SVG_FONTS
 "       <verb verb-id=\"DialogSVGFonts\" />\n"
-#endif // ENABLE_SVG_FONTS
 "       <verb verb-id=\"DialogGlyphs\" />\n"
 "       <separator/>\n"
 "       <verb verb-id=\"SelectionTextToPath\" />\n"
@@ -257,11 +275,6 @@ static char const menus_skeleton[] =
 "       <separator/>\n"
 "       <effects-list/>\n"
 "   </submenu>\n"
-#ifdef WITH_INKBOARD
-"	<submenu name=\"" N_("Whiteboa_rd") "\">\n"
-"		<verb verb-id=\"DialogXmppClient\" />\n"
-"	</submenu>\n"
-#endif
 "   <submenu name=\"" N_("_Help") "\">\n"
 "       <verb verb-id=\"org.inkscape.help.manual\" />\n"
 "       <verb verb-id=\"org.inkscape.help.keys\" />\n"
@@ -271,6 +284,7 @@ static char const menus_skeleton[] =
 "           <verb verb-id=\"TutorialsShapes\" />\n"
 "           <verb verb-id=\"TutorialsAdvanced\" />\n"
 "           <verb verb-id=\"TutorialsTracing\" />\n"
+"           <verb verb-id=\"TutorialsTracingPixelArt\" />\n"
 "           <verb verb-id=\"TutorialsCalligraphy\" />\n"
 "           <verb verb-id=\"TutorialsInterpolate\" />\n"
 "           <verb verb-id=\"TutorialsDesign\" />\n"
@@ -303,4 +317,4 @@ static char const menus_skeleton[] =
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
