@@ -1,5 +1,5 @@
-#ifndef __PATH_CHEMISTRY_H__
-#define __PATH_CHEMISTRY_H__
+#ifndef SEEN_PATH_CHEMISTRY_H
+#define SEEN_PATH_CHEMISTRY_H
 
 /*
  * Here are handlers for modifying selections, specific to paths
@@ -13,17 +13,28 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include "forward.h"
+#include <glib.h>
+
+class SPDesktop;
+class SPItem;
+
+namespace Inkscape {
+class Selection;
+namespace XML {
+class Node;
+} // namespace XML
+} // namespace Inkscape
 
 void sp_selected_path_combine (SPDesktop *desktop);
 void sp_selected_path_break_apart (SPDesktop *desktop);
-void sp_selected_path_to_curves (SPDesktop *desktop, bool interactive = true);
+// interactive=true only has an effect if desktop != NULL, i.e. if a GUI is available
+void sp_selected_path_to_curves (Inkscape::Selection *selection, SPDesktop *desktop, bool interactive = true);
 void sp_selected_to_lpeitems(SPDesktop *desktop);
 Inkscape::XML::Node *sp_selected_item_to_curved_repr(SPItem *item, guint32 text_grouping_policy);
 void sp_selected_path_reverse (SPDesktop *desktop);
 bool sp_item_list_to_curves(const GSList *items, GSList **selected, GSList **to_select, bool skip_all_lpeitems = false);
 
-#endif
+#endif // SEEN_PATH_CHEMISTRY_H
 
 /*
   Local Variables:
@@ -34,4 +45,4 @@ bool sp_item_list_to_curves(const GSList *items, GSList **selected, GSList **to_
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
