@@ -11,6 +11,7 @@
  */
 
 #include "uri-references.h"
+#include <stddef.h>
 #include <sigc++/sigc++.h>
 #include "persp3d.h"
 
@@ -18,7 +19,7 @@ class SPObject;
 
 namespace Inkscape {
 namespace XML {
-struct Node;
+class Node;
 }
 }
 
@@ -63,4 +64,4 @@ protected:
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
