@@ -1,5 +1,5 @@
 /** @file
- * @brief  Singleton class to access the preferences file in a convenient way.
+ * Singleton class to access the preferences file in a convenient way.
  */
 /* Authors:
  *   Krzysztof Kosi_ski <tweenk.pl@gmail.com>
@@ -13,13 +13,11 @@
 #ifndef INKSCAPE_PREFSTORE_H
 #define INKSCAPE_PREFSTORE_H
 
-#include <string>
 #include <map>
 #include <vector>
 #include <climits>
 #include <cfloat>
 #include <glibmm/ustring.h>
-#include "xml/xml-forward.h"
 #include "xml/repr.h"
 
 class SPCSSAttr;
@@ -33,7 +31,7 @@ public:
 };
 
 /**
- * @brief Preference storage class.
+ * Preference storage class.
  *
  * This is a singleton that allows one to access the user preferences stored in
  * the preferences.xml file. The preferences are stored in a file system-like
@@ -54,6 +52,8 @@ public:
  * derive (e.g. GConf, flat XML file...)
  */
 class Preferences {
+    class _ObserverData;
+
 public:
     // #############################
     // ## inner class definitions ##
@@ -63,17 +63,18 @@ public:
     class Observer;
 
     /**
-     * @brief Base class for preference observers
+     * Base class for preference observers.
      *
      * If you want to watch for changes in the preferences, you'll have to
      * derive a class from this one and override the notify() method.
      */
     class Observer {
         friend class Preferences;
+
     public:
 
         /**
-         * @brief Constructor.
+         * Constructor.
          *
          * Since each Observer is assigned to a single path, the base
          * constructor takes this path as an argument. This prevents one from
@@ -88,26 +89,27 @@ public:
          * Watching the preference "/options/some_group/some_option" will only
          * generate notifications when this single preference changes.
          *
-         * @param path Preference path the observer should watch
+         * @param path Preference path the observer should watch.
          */
         Observer(Glib::ustring const &path);
         virtual ~Observer();
 
         /**
-         * @brief Notification about a preference change
+         * Notification about a preference change.
+         *
          * @param new_val  Entry object containing information about
-         *                 the modified preference
+         *                 the modified preference.
          */
         virtual void notify(Preferences::Entry const &new_val) = 0;
 
         Glib::ustring const observed_path; ///< Path which the observer watches
     private:
-        void *_data; ///< additional data used by the implementation while the observer is active
+        _ObserverData *_data; ///< additional data used by the implementation while the observer is active
     };
 
 
     /**
-     * @brief Data type representing a typeless value of a preference
+     * Data type representing a typeless value of a preference.
      *
      * This is passed to the observer in the notify() method.
      * To retrieve useful data from it, use its member functions. Setting
@@ -122,75 +124,87 @@ public:
         Entry(Entry const &other) : _pref_path(other._pref_path), _value(other._value) {}
 
         /**
-         * @brief Check whether the received entry is valid.
+         * Check whether the received entry is valid.
+         *
          * @return If false, the default value will be returned by the getters.
          */
         bool isValid() const { return _value != NULL; }
 
         /**
-         * @brief Interpret the preference as a Boolean value.
-         * @param def Default value if the preference is not set
+         * Interpret the preference as a Boolean value.
+         *
+         * @param def Default value if the preference is not set.
          */
         inline bool getBool(bool def=false) const;
 
         /**
-         * @brief Interpret the preference as an integer.
-         * @param def Default value if the preference is not set
+         * Interpret the preference as an integer.
+         *
+         * @param def Default value if the preference is not set.
          */
         inline int getInt(int def=0) const;
 
         /**
-         * @brief Interpret the preference as a limited integer.
+         * Interpret the preference as a limited integer.
          *
          * This method will return the default value if the interpreted value is
          * larger than @c max or smaller than @c min. Do not use to store
          * Boolean values as integers.
          *
-         * @param def Default value if the preference is not set
-         * @param min Minimum value allowed to return
-         * @param max Maximum value allowed to return
+         * @param def Default value if the preference is not set.
+         * @param min Minimum value allowed to return.
+         * @param max Maximum value allowed to return.
          */
         inline int getIntLimited(int def=0, int min=INT_MIN, int max=INT_MAX) const;
 
         /**
-         * @brief Interpret the preference as a floating point value.
-         * @param def Default value if the preference is not set
+         * Interpret the preference as a floating point value.
+         *
+         * @param def  Default value if the preference is not set.
+         * @param unit Specifies the unit of the returned result. Will be ignored when equal to "". If the preference has no unit set, the default unit will be assumed.
          */
-        inline double getDouble(double def=0.0) const;
+        inline double getDouble(double def=0.0, Glib::ustring const &unit = "") const;
 
         /**
-         * @brief Interpret the preference as a limited floating point value.
+         * Interpret the preference as a limited floating point value.
          *
          * This method will return the default value if the interpreted value is
          * larger than @c max or smaller than @c min.
          *
-         * @param def Default value if the preference is not set
-         * @param min Minimum value allowed to return
-         * @param max Maximum value allowed to return
+         * @param def Default value if the preference is not set.
+         * @param min Minimum value allowed to return.
+         * @param max Maximum value allowed to return.
+         * @param unit Specifies the unit of the returned result. Will be ignored when equal to "". If the preference has no unit set, the default unit will be assumed.
          */
-        inline double getDoubleLimited(double def=0.0, double min=DBL_MIN, double max=DBL_MAX) const;
+        inline double getDoubleLimited(double def=0.0, double min=DBL_MIN, double max=DBL_MAX, Glib::ustring const &unit = "") const;
 
         /**
-         * @brief Interpret the preference as an UTF-8 string.
+         * Interpret the preference as an UTF-8 string.
          *
          * To store a filename, convert it using Glib::filename_to_utf8().
          */
         inline Glib::ustring getString() const;
 
+       /**
+         * Interpret the preference as a number followed by a unit (without space), and return this unit string.
+         */
+        inline Glib::ustring getUnit() const;
+
         /**
-         * @brief Interpret the preference as an RGBA color value.
+         * Interpret the preference as an RGBA color value.
          */
         inline guint32 getColor(guint32 def) const;
 
         /**
-         * @brief Interpret the preference as a CSS style.
+         * Interpret the preference as a CSS style.
+         *
          * @return A CSS style that has to be unrefed when no longer necessary. Never NULL.
          */
         inline SPCSSAttr *getStyle() const;
 
         /**
-         * @brief Interpret the preference as a CSS style with directory-based
-         *        inheritance
+         * Interpret the preference as a CSS style with directory-based
+         * inheritance.
          *
          * This function will look up the preferences with the same entry name
          * in ancestor directories and return the inherited CSS style.
@@ -200,12 +214,12 @@ public:
         inline SPCSSAttr *getInheritedStyle() const;
 
         /**
-         * @brief Get the full path of the preference described by this Entry.
+         * Get the full path of the preference described by this Entry.
          */
         Glib::ustring const &getPath() const { return _pref_path; }
 
         /**
-         * @brief Get the last component of the preference's path
+         * Get the last component of the preference's path.
          *
          * E.g. For "/options/some_group/some_option" it will return "some_option".
          */
@@ -220,7 +234,7 @@ public:
     // utility methods
 
     /**
-     * @brief Save all preferences to the hard disk.
+     * Save all preferences to the hard disk.
      *
      * For some backends, the preferences may be saved as they are modified.
      * Not calling this method doesn't guarantee the preferences are unmodified
@@ -229,13 +243,13 @@ public:
     void save();
 
     /**
-     * @brief Check whether saving the preferences will have any effect.
+     * Check whether saving the preferences will have any effect.
      */
     bool isWritable() { return _writable; }
     /*@}*/
 
     /**
-     * @brief Return details of the last encountered error, if any.
+     * Return details of the last encountered error, if any.
      *
      * This method will return true if an error has been encountered, and fill
      * in the primary and secondary error strings of the last error. If an error
@@ -254,7 +268,7 @@ public:
      */
 
     /**
-     * @brief Get all entries from the specified directory
+     * Get all entries from the specified directory.
      *
      * This method will return a vector populated with preference entries
      * from the specified directory. Subdirectories will not be represented.
@@ -262,7 +276,7 @@ public:
     std::vector<Entry> getAllEntries(Glib::ustring const &path);
 
     /**
-     * @brief Get all subdirectories of the specified directory
+     * Get all subdirectories of the specified directory.
      *
      * This will return a vector populated with full paths to the subdirectories
      * present in the specified @c path.
@@ -276,71 +290,86 @@ public:
      */
 
     /**
-     * @brief Retrieve a Boolean value
-     * @param pref_path Path to the retrieved preference
-     * @param def The default value to return if the preference is not set
+     * Retrieve a Boolean value.
+     *
+     * @param pref_path Path to the retrieved preference.
+     * @param def The default value to return if the preference is not set.
      */
     bool getBool(Glib::ustring const &pref_path, bool def=false) {
         return getEntry(pref_path).getBool(def);
     }
 
     /**
-     * @brief Retrieve an integer
-     * @param pref_path Path to the retrieved preference
-     * @param def The default value to return if the preference is not set
+     * Retrieve an integer.
+     *
+     * @param pref_path Path to the retrieved preference.
+     * @param def The default value to return if the preference is not set.
      */
     int getInt(Glib::ustring const &pref_path, int def=0) {
         return getEntry(pref_path).getInt(def);
     }
 
     /**
-     * @brief Retrieve a limited integer
+     * Retrieve a limited integer.
      *
      * The default value is returned if the actual value is larger than @c max
      * or smaller than @c min. Do not use to store Boolean values.
      *
-     * @param pref_path Path to the retrieved preference
-     * @param def The default value to return if the preference is not set
-     * @param min Minimum value to return
-     * @param max Maximum value to return
+     * @param pref_path Path to the retrieved preference.
+     * @param def The default value to return if the preference is not set.
+     * @param min Minimum value to return.
+     * @param max Maximum value to return.
      */
     int getIntLimited(Glib::ustring const &pref_path, int def=0, int min=INT_MIN, int max=INT_MAX) {
         return getEntry(pref_path).getIntLimited(def, min, max);
     }
-    double getDouble(Glib::ustring const &pref_path, double def=0.0) {
-        return getEntry(pref_path).getDouble(def);
+
+    double getDouble(Glib::ustring const &pref_path, double def=0.0, Glib::ustring const &unit = "") {
+        return getEntry(pref_path).getDouble(def, unit);
     }
 
     /**
-     * @brief Retrieve a limited floating point value
+     * Retrieve a limited floating point value.
      *
      * The default value is returned if the actual value is larger than @c max
      * or smaller than @c min.
      *
-     * @param pref_path Path to the retrieved preference
-     * @param def The default value to return if the preference is not set
-     * @param min Minimum value to return
-     * @param max Maximum value to return
+     * @param pref_path Path to the retrieved preference.
+     * @param def The default value to return if the preference is not set.
+     * @param min Minimum value to return.
+     * @param max Maximum value to return.
+     * @param unit Specifies the unit of the returned result. Will be ignored when equal to "". If the preference has no unit set, the default unit will be assumed.
      */
-    double getDoubleLimited(Glib::ustring const &pref_path, double def=0.0, double min=DBL_MIN, double max=DBL_MAX) {
-        return getEntry(pref_path).getDoubleLimited(def, min, max);
+    double getDoubleLimited(Glib::ustring const &pref_path, double def=0.0, double min=DBL_MIN, double max=DBL_MAX, Glib::ustring const &unit = "") {
+        return getEntry(pref_path).getDoubleLimited(def, min, max, unit);
     }
 
     /**
-     * @brief Retrieve an UTF-8 string
-     * @param pref_path Path to the retrieved preference
+     * Retrieve an UTF-8 string.
+     *
+     * @param pref_path Path to the retrieved preference.
      */
     Glib::ustring getString(Glib::ustring const &pref_path) {
         return getEntry(pref_path).getString();
     }
 
+    /**
+     * Retrieve the unit string.
+     *
+     * @param pref_path Path to the retrieved preference.
+     */
+    Glib::ustring getUnit(Glib::ustring const &pref_path) {
+        return getEntry(pref_path).getUnit();
+    }
+
     guint32 getColor(Glib::ustring const &pref_path, guint32 def=0x000000ff) {
         return getEntry(pref_path).getColor(def);
     }
 
     /**
-     * @brief Retrieve a CSS style
-     * @param pref_path Path to the retrieved preference
+     * Retrieve a CSS style.
+     *
+     * @param pref_path Path to the retrieved preference.
      * @return A CSS style that has to be unrefed after use.
      */
     SPCSSAttr *getStyle(Glib::ustring const &pref_path) {
@@ -348,13 +377,13 @@ public:
     }
 
     /**
-     * @brief Retrieve an inherited CSS style
+     * Retrieve an inherited CSS style.
      *
      * This method will look up preferences with the same entry name in ancestor
      * directories and return a style obtained by inheriting properties from
      * ancestor styles.
      *
-     * @param pref_path Path to the retrieved preference
+     * @param pref_path Path to the retrieved preference.
      * @return An inherited CSS style that has to be unrefed after use.
      */
     SPCSSAttr *getInheritedStyle(Glib::ustring const &pref_path) {
@@ -362,48 +391,58 @@ public:
     }
 
     /**
-     * @brief Retrieve a preference entry without specifying its type
+     * Retrieve a preference entry without specifying its type.
      */
     Entry const getEntry(Glib::ustring const &pref_path);
     /*@}*/
 
+
+    Glib::ustring getPrefsFilename() {
+        return _prefs_filename;
+    }
+
     /**
      * @name Update preference values.
      * @{
      */
 
     /**
-     * @brief Set a Boolean value
+     * Set a Boolean value.
      */
     void setBool(Glib::ustring const &pref_path, bool value);
 
     /**
-     * @brief Set an integer value
+     * Set an integer value.
      */
     void setInt(Glib::ustring const &pref_path, int value);
 
     /**
-     * @brief Set a floating point value
+     * Set a floating point value.
      */
     void setDouble(Glib::ustring const &pref_path, double value);
 
     /**
-     * @brief Set an UTF-8 string value
+     * Set a floating point value with unit.
+     */
+    void setDoubleUnit(Glib::ustring const &pref_path, double value, Glib::ustring const &unit_abbr);
+
+    /**
+     * Set an UTF-8 string value.
      */
     void setString(Glib::ustring const &pref_path, Glib::ustring const &value);
 
     /**
-     * @brief Set an RGBA color value
+     * Set an RGBA color value.
      */
     void setColor(Glib::ustring const &pref_path, guint32 value);
 
     /**
-     * @brief Set a CSS style
+     * Set a CSS style.
      */
     void setStyle(Glib::ustring const &pref_path, SPCSSAttr *style);
 
     /**
-     * @brief Merge a CSS style with the current preference value
+     * Merge a CSS style with the current preference value.
      *
      * This method is similar to setStyle(), except that it merges the style
      * rather than replacing it. This means that if @c style doesn't have
@@ -419,12 +458,12 @@ public:
      */
 
     /**
-     * @brief Register a preference observer
+     * Register a preference observer.
      */
     void addObserver(Observer &);
 
     /**
-     * @brief Remove an observer an prevent further notifications to it.
+     * Remove an observer an prevent further notifications to it.
      */
     void removeObserver(Observer &);
     /*@}*/
@@ -441,7 +480,7 @@ public:
     static void migrate( std::string const& legacyDir, std::string const& prefdir );
 
     /**
-     * @brief Access the singleton Preferences object.
+     * Access the singleton Preferences object.
      */
     static Preferences *get() {
         if (!_instance) {
@@ -453,8 +492,9 @@ public:
     void setErrorHandler(ErrorReporter* handler);
 
     /**
-     * @brief Unload all preferences
-     * @param save Whether to save the preferences; defaults to true
+     * Unload all preferences.
+     *
+     * @param save Whether to save the preferences; defaults to true.
      *
      * This deletes the singleton object. Calling get() after this function
      * will reinstate it, so you shouldn't. Pass false as the parameter
@@ -463,6 +503,13 @@ public:
     static void unload(bool save=true);
     /*@}*/
 
+    /**
+     * Remove a node from prefs
+     * @param pref_path　Path to entry
+     *
+     */
+    void remove(Glib::ustring const &pref_path);
+
 protected:
     /* helper methods used by Entry
      * This will enable using the same Entry class with different backends.
@@ -472,7 +519,9 @@ protected:
     bool _extractBool(Entry const &v);
     int _extractInt(Entry const &v);
     double _extractDouble(Entry const &v);
+    double _extractDouble(Entry const &v, Glib::ustring const &requested_unit);
     Glib::ustring _extractString(Entry const &v);
+    Glib::ustring _extractUnit(Entry const &v);
     guint32 _extractColor(Entry const &v);
     SPCSSAttr *_extractStyle(Entry const &v);
     SPCSSAttr *_extractInheritedStyle(Entry const &v);
@@ -483,7 +532,7 @@ private:
     void _loadDefaults();
     void _load();
     void _getRawValue(Glib::ustring const &path, gchar const *&result);
-    void _setRawValue(Glib::ustring const &path, gchar const *value);
+    void _setRawValue(Glib::ustring const &path, Glib::ustring const &value);
     void _reportError(Glib::ustring const &, Glib::ustring const &);
     void _keySplit(Glib::ustring const &pref_path, Glib::ustring &node_key, Glib::ustring &attr_key);
     XML::Node *_getNode(Glib::ustring const &pref_path, bool create=false);
@@ -512,7 +561,7 @@ private:
 
     // privilege escalation methods for PrefNodeObserver
     static Entry const _create_pref_value(Glib::ustring const &, void const *ptr);
-    static void *_get_pref_observer_data(Observer &o) { return o._data; }
+    static _ObserverData *_get_pref_observer_data(Observer &o) { return o._data; }
 
     static Preferences *_instance;
 
@@ -554,21 +603,28 @@ inline int Preferences::Entry::getIntLimited(int def, int min, int max) const
     }
 }
 
-inline double Preferences::Entry::getDouble(double def) const
+inline double Preferences::Entry::getDouble(double def, Glib::ustring const &unit) const
 {
     if (!this->isValid()) {
         return def;
-    } else {
+    } else if (unit.length() == 0) {
         return Inkscape::Preferences::get()->_extractDouble(*this);
+    } else {
+        return Inkscape::Preferences::get()->_extractDouble(*this, unit);
     }
 }
 
-inline double Preferences::Entry::getDoubleLimited(double def, double min, double max) const
+inline double Preferences::Entry::getDoubleLimited(double def, double min, double max, Glib::ustring const &unit) const
 {
     if (!this->isValid()) {
         return def;
     } else {
-        double val = Inkscape::Preferences::get()->_extractDouble(*this);
+        double val = def;
+        if (unit.length() == 0) {
+            val = Inkscape::Preferences::get()->_extractDouble(*this);
+        } else {
+            val = Inkscape::Preferences::get()->_extractDouble(*this, unit);
+        }
         return ( val >= min && val <= max ? val : def );
     }
 }
@@ -582,6 +638,15 @@ inline Glib::ustring Preferences::Entry::getString() const
     }
 }
 
+inline Glib::ustring Preferences::Entry::getUnit() const
+{
+    if (!this->isValid()) {
+        return "";
+    } else {
+        return Inkscape::Preferences::get()->_extractUnit(*this);
+    }
+}
+
 inline guint32 Preferences::Entry::getColor(guint32 def) const
 {
     if (!this->isValid()) {
