@@ -6,6 +6,7 @@
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   Abhishek Sharma
  *
  * Copyright (C) 2000-2002 authors
  *
@@ -14,23 +15,22 @@
 
 #include "sp-object.h"
 
-#define SP_TYPE_DEFS            (sp_defs_get_type())
-#define SP_DEFS(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), SP_TYPE_DEFS, SPDefs))
-#define SP_DEFS_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), SP_TYPE_DEFS, SPDefsClass))
-#define SP_IS_DEFS(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), SP_TYPE_DEFS))
-#define SP_IS_DEFS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), SP_TYPE_DEFS))
+#define SP_DEFS(obj) (dynamic_cast<SPDefs*>((SPObject*)obj))
+#define SP_IS_DEFS(obj) (dynamic_cast<const SPDefs*>((SPObject*)obj) != NULL)
 
-struct SPDefs : public SPObject {
+class SPDefs : public SPObject {
+public:
+	SPDefs();
+	virtual ~SPDefs();
+
+protected:
+	virtual void release();
+	virtual void update(SPCtx* ctx, unsigned int flags);
+	virtual void modified(unsigned int flags);
+	virtual Inkscape::XML::Node* write(Inkscape::XML::Document *xml_doc, Inkscape::XML::Node *repr, guint flags);
 };
 
-struct SPDefsClass {
-    SPObjectClass parent_class;
-};
-
-GType sp_defs_get_type(void);
-
-
-#endif /* !SEEN_SP_DEFS_H */
+#endif // !SEEN_SP_DEFS_H
 
 /*
   Local Variables:
