@@ -16,17 +16,23 @@
 #include <map>
 #include "sp-lpe-item.h"
 
-#define SP_TYPE_GROUP            (sp_group_get_type ())
-#define SP_GROUP(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SP_TYPE_GROUP, SPGroup))
-#define SP_GROUP_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), SP_TYPE_GROUP, SPGroupClass))
-#define SP_IS_GROUP(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SP_TYPE_GROUP))
-#define SP_IS_GROUP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SP_TYPE_GROUP))
+#define SP_GROUP(obj) (dynamic_cast<SPGroup*>((SPObject*)obj))
+#define SP_IS_GROUP(obj) (dynamic_cast<const SPGroup*>((SPObject*)obj) != NULL)
 
 #define SP_IS_LAYER(obj)         (SP_IS_GROUP(obj) && SP_GROUP(obj)->layerMode() == SPGroup::LAYER)
 
-class CGroup;
+namespace Inkscape {
+
+class Drawing;
+class DrawingItem;
+
+} // namespace Inkscape
+
+class SPGroup : public SPLPEItem {
+public:
+	SPGroup();
+	virtual ~SPGroup();
 
-struct SPGroup : public SPLPEItem {
     enum LayerMode { GROUP, LAYER, MASK_HELPER };
 
     LayerMode _layer_mode;
@@ -46,46 +52,40 @@ struct SPGroup : public SPLPEItem {
     LayerMode layerDisplayMode(unsigned int display_key) const;
     void setLayerDisplayMode(unsigned int display_key, LayerMode mode);
     void translateChildItems(Geom::Translate const &tr);
+    void scaleChildItemsRec(Geom::Scale const &sc, Geom::Point const &p, bool noRecurse);
+
+    gint getItemCount() const;
+    virtual void _showChildren (Inkscape::Drawing &drawing, Inkscape::DrawingItem *ai, unsigned int key, unsigned int flags);
 
-    CGroup *group;
-    
 private:
     void _updateLayerMode(unsigned int display_key=0);
-};
 
-struct SPGroupClass {
-    SPLPEItemClass parent_class;
-};
-
-/*
- * Virtual methods of SPGroup
- */
-class CGroup {
 public:
-    CGroup(SPGroup *group);
-    virtual ~CGroup();
-    
-    virtual void onChildAdded(Inkscape::XML::Node *child);
-    virtual void onChildRemoved(Inkscape::XML::Node *child);
-    virtual void onUpdate(SPCtx *ctx, unsigned int flags);
-    virtual void onModified(guint flags);
-    virtual void calculateBBox(NRRect *bbox, Geom::Matrix const &transform, unsigned const flags);
-    virtual void onPrint(SPPrintContext *ctx);
-    virtual void onOrderChanged(Inkscape::XML::Node *child, Inkscape::XML::Node *old_ref, Inkscape::XML::Node *new_ref);
-    virtual gchar *getDescription();
-    virtual NRArenaItem *show (NRArena *arena, unsigned int key, unsigned int flags);
+    virtual void build(SPDocument *document, Inkscape::XML::Node *repr);
+   	virtual void release();
+
+    virtual void child_added(Inkscape::XML::Node* child, Inkscape::XML::Node* ref);
+    virtual void remove_child(Inkscape::XML::Node *child);
+    virtual void order_changed(Inkscape::XML::Node *child, Inkscape::XML::Node *old_ref, Inkscape::XML::Node *new_ref);
+
+    virtual void update(SPCtx *ctx, unsigned int flags);
+    virtual void modified(guint flags);
+    virtual void set(unsigned int key, gchar const* value);
+
+    virtual Inkscape::XML::Node* write(Inkscape::XML::Document *xml_doc, Inkscape::XML::Node *repr, guint flags);
+
+    virtual Geom::OptRect bbox(Geom::Affine const &transform, SPItem::BBoxType bboxtype) const;
+    virtual void print(SPPrintContext *ctx);
+    virtual const char* displayName() const;
+    virtual gchar *description() const;
+    virtual Inkscape::DrawingItem *show (Inkscape::Drawing &drawing, unsigned int key, unsigned int flags);
     virtual void hide (unsigned int key);
 
-    gint getItemCount();
+    virtual void snappoints(std::vector<Inkscape::SnapCandidatePoint> &p, Inkscape::SnapPreferences const *snapprefs) const;
 
-protected:
-    virtual void _showChildren (NRArena *arena, NRArenaItem *ai, unsigned int key, unsigned int flags);
-
-    SPGroup *_group;
+    virtual void update_patheffect(bool write);
 };
 
-GType sp_group_get_type (void);
-
 void sp_item_group_ungroup (SPGroup *group, GSList **children, bool do_done = true);
 
 
