@@ -6,6 +6,7 @@
 #include <2geom/transforms.h>
 #include <sp-guide.h>
 #include <sp-item-rm-unsatisfied-cns.h>
+#include <sp-item-notify-moveto.h>
 using std::vector;
 
 
@@ -25,7 +26,7 @@ void sp_item_notify_moveto(SPItem &item, SPGuide const &mv_g, int const snappoin
     g_return_if_fail( dir_lensq != 0 );
 
     std::vector<Inkscape::SnapCandidatePoint> snappoints;
-    sp_item_snappoints(&item, snappoints, NULL);
+    item.getSnappoints(snappoints, NULL);
     g_return_if_fail( snappoint_ix < int(snappoints.size()) );
 
     double const pos0 = dot(dir, snappoints[snappoint_ix].getPoint());
@@ -41,7 +42,7 @@ void sp_item_notify_moveto(SPItem &item, SPGuide const &mv_g, int const snappoin
        s = (position - pos0) / dot(dir, dir). */
     Geom::Translate const tr( ( position - pos0 )
                             * ( dir / dir_lensq ) );
-    sp_item_set_i2d_affine(&item, sp_item_i2d_affine(&item) * tr);
+    item.set_i2d_affine(item.i2dt_affine() * tr);
     /// \todo Reget snappoints, check satisfied.
 
     if (commit) {
@@ -49,7 +50,7 @@ void sp_item_notify_moveto(SPItem &item, SPGuide const &mv_g, int const snappoin
 
         /* Commit repr. */
         {
-            sp_item_write_transform(&item, SP_OBJECT_REPR(&item), item.transform);
+            item.doWriteTransform(item.getRepr(), item.transform);
         }
 
         sp_item_rm_unsatisfied_cns(item);
@@ -73,4 +74,4 @@ void sp_item_notify_moveto(SPItem &item, SPGuide const &mv_g, int const snappoin
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
