@@ -1,34 +1,41 @@
-#ifndef __SP_MASK_H__
-#define __SP_MASK_H__
+#ifndef SEEN_SP_MASK_H
+#define SEEN_SP_MASK_H
 
 /*
  * SVG <mask> implementation
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   Abhishek Sharma
  *
  * Copyright (C) 2003 authors
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#define SP_TYPE_MASK (sp_mask_get_type ())
-#define SP_MASK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), SP_TYPE_MASK, SPMask))
-#define SP_MASK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), SP_TYPE_MASK, SPMaskClass))
-#define SP_IS_MASK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SP_TYPE_MASK))
-#define SP_IS_MASK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SP_TYPE_MASK))
-
-class SPMask;
-class SPMaskClass;
-class SPMaskView;
-
-#include "display/nr-arena-forward.h"
-#include "libnr/nr-forward.h"
+#include <2geom/rect.h>
 #include "sp-object-group.h"
 #include "uri-references.h"
 #include "xml/node.h"
 
-struct SPMask : public SPObjectGroup {
+#define SP_MASK(obj) (dynamic_cast<SPMask*>((SPObject*)obj))
+#define SP_IS_MASK(obj) (dynamic_cast<const SPMask*>((SPObject*)obj) != NULL)
+
+struct SPMaskView;
+
+namespace Inkscape {
+
+class Drawing;
+class DrawingItem;
+
+} // namespace Inkscape
+
+
+class SPMask : public SPObjectGroup {
+public:
+	SPMask();
+	virtual ~SPMask();
+
 	unsigned int maskUnits_set : 1;
 	unsigned int maskUnits : 1;
 
@@ -36,19 +43,31 @@ struct SPMask : public SPObjectGroup {
 	unsigned int maskContentUnits : 1;
 
 	SPMaskView *display;
-};
 
-struct SPMaskClass {
-	SPObjectGroupClass parent_class;
-};
+	Inkscape::DrawingItem *sp_mask_show(Inkscape::Drawing &drawing, unsigned int key);
+	void sp_mask_hide(unsigned int key);
 
-GType sp_mask_get_type (void);
+	void sp_mask_set_bbox(unsigned int key, Geom::OptRect const &bbox);
+
+protected:
+	virtual void build(SPDocument* doc, Inkscape::XML::Node* repr);
+	virtual void release();
+
+	virtual void child_added(Inkscape::XML::Node* child, Inkscape::XML::Node* ref);
+
+	virtual void set(unsigned int key, const gchar* value);
+
+	virtual void update(SPCtx* ctx, unsigned int flags);
+	virtual void modified(unsigned int flags);
+
+	virtual Inkscape::XML::Node* write(Inkscape::XML::Document* doc, Inkscape::XML::Node* repr, guint flags);
+};
 
 class SPMaskReference : public Inkscape::URIReference {
 public:
 	SPMaskReference(SPObject *obj) : URIReference(obj) {}
 	SPMask *getObject() const {
-		return (SPMask *)URIReference::getObject();
+		return static_cast<SPMask *>(URIReference::getObject());
 	}
 protected:
     /**
@@ -63,8 +82,10 @@ protected:
 	    }
 	    SPObject * const owner = this->getOwner();
         if (obj->isAncestorOf(owner)) {
-            Inkscape::XML::Node * const owner_repr = owner->repr;
-            Inkscape::XML::Node * const obj_repr = obj->repr;
+	  //XML Tree being used directly here while it shouldn't be...
+	  Inkscape::XML::Node * const owner_repr = owner->getRepr();
+	  //XML Tree being used directly here while it shouldn't be...
+	  Inkscape::XML::Node * const obj_repr = obj->getRepr();
             gchar const * owner_name = NULL;
             gchar const * owner_mask = NULL;
             gchar const * obj_name = NULL;
@@ -87,11 +108,6 @@ protected:
 	}
 };
 
-NRArenaItem *sp_mask_show (SPMask *mask, NRArena *arena, unsigned int key);
-void sp_mask_hide (SPMask *mask, unsigned int key);
+const gchar *sp_mask_create (GSList *reprs, SPDocument *document, Geom::Affine const* applyTransform);
 
-void sp_mask_set_bbox (SPMask *mask, unsigned int key, NRRect *bbox);
-
-const gchar *sp_mask_create (GSList *reprs, SPDocument *document, Geom::Matrix const* applyTransform);
-
-#endif
+#endif // SEEN_SP_MASK_H
