@@ -6,6 +6,7 @@
  *
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   Abhishek Sharma
  *
  * Copyright (C) 2006 Johan Engelen <johan@shouraizou.nl>
  * Copyright (C) Lauris Kaplinski 2000-2002
@@ -13,19 +14,19 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#define SP_TYPE_NAMEDVIEW (sp_namedview_get_type())
-#define SP_NAMEDVIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), SP_TYPE_NAMEDVIEW, SPNamedView))
-#define SP_NAMEDVIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), SP_TYPE_NAMEDVIEW, SPNamedViewClass))
-#define SP_IS_NAMEDVIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), SP_TYPE_NAMEDVIEW))
-#define SP_IS_NAMEDVIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), SP_TYPE_NAMEDVIEW))
+#define SP_NAMEDVIEW(obj) (dynamic_cast<SPNamedView*>((SPObject*)obj))
+#define SP_IS_NAMEDVIEW(obj) (dynamic_cast<const SPNamedView*>((SPObject*)obj) != NULL)
 
-#include "helper/helper-forward.h"
 #include "sp-object-group.h"
-#include "sp-metric.h"
 #include "snap.h"
+#include "document.h"
+#include "util/units.h"
 
 namespace Inkscape {
-class CanvasGrid;
+    class CanvasGrid;
+    namespace Util {
+        class Unit;
+    }
 }
 
 enum {
@@ -33,7 +34,11 @@ enum {
     SP_BORDER_LAYER_TOP
 };
 
-struct SPNamedView : public SPObjectGroup {
+class SPNamedView : public SPObjectGroup {
+public:
+	SPNamedView();
+	virtual ~SPNamedView();
+
     unsigned int editable : 1;
     unsigned int showguides : 1;
     unsigned int showborder : 1;
@@ -53,8 +58,9 @@ struct SPNamedView : public SPObjectGroup {
     GSList * grids;
     bool grids_visible;
 
-    SPUnit const *doc_units;
-    SPUnit const *units;
+    Inkscape::Util::Unit const *svg_units;   // Units used for the values in SVG
+    Inkscape::Util::Unit const *display_units;   // Units used for the UI (*not* the same as units of SVG coordinates)
+    Inkscape::Util::Unit const *page_size_units; // Only used in "Custom size" part of Document Properties dialog 
     
     GQuark default_layer_id;
 
@@ -77,20 +83,36 @@ struct SPNamedView : public SPObjectGroup {
     gchar const *getName() const;
     guint getViewCount();
     GSList const *getViewList() const;
-    SPMetric getDefaultMetric() const;
+    Inkscape::Util::Unit const * getDefaultUnit() const;
+    Inkscape::Util::Unit const & getSVGUnit() const;
 
     void translateGuides(Geom::Translate const &translation);
     void translateGrids(Geom::Translate const &translation);
     void scrollAllDesktops(double dx, double dy, bool is_scrolling);
+    void writeNewGrid(SPDocument *document,int gridtype);
+    bool getSnapGlobal() const;
+    void setSnapGlobal(bool v);
+    void setGuides(bool v);
+    bool getGuides();
+
+private:
+    double getMarginLength(gchar const * const key,Inkscape::Util::Unit const * const margin_units,Inkscape::Util::Unit const * const return_units,double const width,double const height,bool const use_width);
+    friend class SPDocument;
+
+protected:
+	virtual void build(SPDocument *document, Inkscape::XML::Node *repr);
+	virtual void release();
+	virtual void set(unsigned int key, gchar const* value);
+
+	virtual void child_added(Inkscape::XML::Node* child, Inkscape::XML::Node* ref);
+	virtual void remove_child(Inkscape::XML::Node* child);
+
+	virtual Inkscape::XML::Node* write(Inkscape::XML::Document *xml_doc, Inkscape::XML::Node *repr, guint flags);
 };
 
-struct SPNamedViewClass {
-    SPObjectGroupClass parent_class;
-};
-
-GType sp_namedview_get_type();
 
 SPNamedView *sp_document_namedview(SPDocument *document, gchar const *name);
+SPNamedView const *sp_document_namedview(SPDocument const *document, gchar const *name);
 
 void sp_namedview_window_from_document(SPDesktop *desktop);
 void sp_namedview_document_from_window(SPDesktop *desktop);
@@ -100,6 +122,7 @@ void sp_namedview_toggle_guides(SPDocument *doc, Inkscape::XML::Node *repr);
 void sp_namedview_show_grids(SPNamedView *namedview, bool show, bool dirty_document);
 Inkscape::CanvasGrid * sp_namedview_get_first_enabled_grid(SPNamedView *namedview);
 
+
 #endif /* !INKSCAPE_SP_NAMEDVIEW_H */
 
 
@@ -112,4 +135,4 @@ Inkscape::CanvasGrid * sp_namedview_get_first_enabled_grid(SPNamedView *namedvie
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
