@@ -1,11 +1,11 @@
 #ifndef SP_OBJECT_H_SEEN
 #define SP_OBJECT_H_SEEN
 
-/** \file
- * Abstract base class for all nodes
- *
+/*
  * Authors:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   Jon A. Cruz <jon@joncruz.org>
+ *   Abhishek Sharma
  *
  * Copyright (C) 1999-2002 authors
  * Copyright (C) 2001-2002 Ximian, Inc.
@@ -15,6 +15,11 @@
 
 /* SPObject flags */
 
+class SPObject;
+
+#define SP_OBJECT(obj) (dynamic_cast<SPObject*>((SPObject*)obj))
+#define SP_IS_OBJECT(obj) (dynamic_cast<const SPObject*>((SPObject*)obj) != NULL)
+
 /* Async modification flags */
 #define SP_OBJECT_MODIFIED_FLAG (1 << 0)
 #define SP_OBJECT_CHILD_MODIFIED_FLAG (1 << 1)
@@ -36,39 +41,27 @@
 /* Parent, Style, Viewport, User */
 #define SP_OBJECT_MODIFIED_CASCADE (SP_OBJECT_FLAGS_ALL & ~(SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG))
 
-/* Generic */
-#define SP_OBJECT_IS_CLONED(o) (((SPObject *) (o))->cloned)
-
 /* Write flags */
 #define SP_OBJECT_WRITE_BUILD (1 << 0)
 #define SP_OBJECT_WRITE_EXT (1 << 1)
 #define SP_OBJECT_WRITE_ALL (1 << 2)
 
-/* Convenience stuff */
-#define SP_OBJECT_ID(o) (((SPObject *) (o))->getId())
-#define SP_OBJECT_REPR(o) (((SPObject *) (o))->repr)
-#define SP_OBJECT_DOCUMENT(o) (((SPObject *) (o))->document)
-#define SP_OBJECT_PARENT(o) (((SPObject *) (o))->parent)
-#define SP_OBJECT_NEXT(o) (((SPObject *) (o))->next)
-#define SP_OBJECT_PREV(o) (sp_object_prev((SPObject *) (o)))
-#define SP_OBJECT_HREFCOUNT(o) (((SPObject *) (o))->hrefcount)
-#define SP_OBJECT_STYLE(o) (((SPObject *) (o))->style)
-
-
 #include <glib-object.h>
 #include <stddef.h>
 #include <sigc++/connection.h>
 #include <sigc++/functors/slot.h>
 #include <sigc++/signal.h>
 
-#include "forward.h"
 #include "version.h"
 #include "util/forward-pointer-iterator.h"
 
+class SPCSSAttr;
+class SPStyle;
+
 namespace Inkscape {
 namespace XML {
 class Node;
-class Document;
+struct Document;
 }
 }
 
@@ -92,8 +85,6 @@ typedef enum {
     SP_INVALID_ACCESS_ERR
 } SPExceptionType;
 
-class SPException;
-
 /// An attempt to implement exceptions, unused?
 struct SPException {
     SPExceptionType code;
@@ -102,8 +93,6 @@ struct SPException {
 #define SP_EXCEPTION_INIT(ex) {(ex)->code = SP_NO_EXCEPTION;}
 #define SP_EXCEPTION_IS_OK(ex) (!(ex) || ((ex)->code == SP_NO_EXCEPTION))
 
-class SPCtx;
-
 /// Unused
 struct SPCtx {
     unsigned int flags;
@@ -114,16 +103,16 @@ enum {
     SP_XML_SPACE_PRESERVE
 };
 
-class SPIXmlSpace;
+class SPDocument;
 
 /// Internal class consisting of two bits.
-struct SPIXmlSpace {
+class SPIXmlSpace {
+public:    
+    SPIXmlSpace(): set(0), value(SP_XML_SPACE_DEFAULT) {};
     guint set : 1;
     guint value : 1;
 };
 
-class SPObject;
-
 /*
  * Refcounting
  *
@@ -131,19 +120,79 @@ class SPObject;
  * Ref should return object, NULL is error, unref return always NULL
  */
 
+/**
+ * Increase reference count of object, with possible debugging.
+ *
+ * @param owner If non-NULL, make debug log entry.
+ * @return object, NULL is error.
+ * \pre object points to real object
+ * @todo need to move this to be a member of SPObject.
+ */
 SPObject *sp_object_ref(SPObject *object, SPObject *owner=NULL);
+
+/**
+ * Decrease reference count of object, with possible debugging and
+ * finalization.
+ *
+ * @param owner If non-NULL, make debug log entry.
+ * @return always NULL
+ * \pre object points to real object
+ * @todo need to move this to be a member of SPObject.
+ */
 SPObject *sp_object_unref(SPObject *object, SPObject *owner=NULL);
 
+/**
+ * Increase weak refcount.
+ *
+ * Hrefcount is used for weak references, for example, to
+ * determine whether any graphical element references a certain gradient
+ * node.
+ * @param owner Ignored.
+ * @return object, NULL is error
+ * \pre object points to real object
+ * @todo need to move this to be a member of SPObject.
+ */
 SPObject *sp_object_href(SPObject *object, gpointer owner);
+
+/**
+ * Decrease weak refcount.
+ *
+ * Hrefcount is used for weak references, for example, to determine whether
+ * any graphical element references a certain gradient node.
+ * @param owner Ignored.
+ * @return always NULL
+ * \pre object points to real object and hrefcount>0
+ * @todo need to move this to be a member of SPObject.
+ */
 SPObject *sp_object_hunref(SPObject *object, gpointer owner);
 
-/// A refcounting tree node object.
-struct SPObject : public GObject {
+/**
+ * SPObject is an abstract base class of all of the document nodes at the
+ * SVG document level. Each SPObject subclass implements a certain SVG
+ * element node type, or is an abstract base class for different node
+ * types.  The SPObject layer is bound to the SPRepr layer, closely
+ * following the SPRepr mutations via callbacks.  During creation,
+ * SPObject parses and interprets all textual attributes and CSS style
+ * strings of the SPRepr, and later updates the internal state whenever
+ * it receives a signal about a change. The opposite is not true - there
+ * are methods manipulating SPObjects directly and such changes do not
+ * propagate to the SPRepr layer. This is important for implementation of
+ * the undo stack, animations and other features.
+ *
+ * SPObjects are bound to the higher-level container SPDocument, which
+ * provides document level functionality such as the undo stack,
+ * dictionary and so on. Source: doc/architecture.txt
+ */
+class SPObject { // : public GObject {
+public:
     enum CollectionPolicy {
         COLLECT_WITH_PARENT,
         ALWAYS_COLLECT
     };
 
+    SPObject();
+    virtual ~SPObject();
+
     unsigned int cloned : 1;
     unsigned int uflags : 8;
     unsigned int mflags : 8;
@@ -155,11 +204,15 @@ struct SPObject : public GObject {
     SPObject *children; /* Our children */
     SPObject *_last_child; /* Remembered last child */
     SPObject *next; /* Next object in linked list */
-    Inkscape::XML::Node *repr; /* Our xml representation */
 
 private:
+    SPObject(const SPObject&);
+    SPObject& operator=(const SPObject&);
+
     gchar *id; /* Our very own unique id */
+    Inkscape::XML::Node *repr; /* Our xml representation */
 public:
+    int refCount;
 
     /**
      * Returns the objects current ID string.
@@ -169,6 +222,7 @@ public:
     /**
      * Returns the XML representation of tree
      */
+//protected:
     Inkscape::XML::Node * getRepr();
 
     /**
@@ -176,16 +230,20 @@ public:
      */
     Inkscape::XML::Node const* getRepr() const;
 
-    /** @brief cleans up an SPObject, releasing its references and
-     *         requesting that references to it be released
+public:
+
+    /**
+     * Cleans up an SPObject, releasing its references and
+     * requesting that references to it be released
      */
     void releaseReferences();
 
-    /** @brief connects to the release request signal
+    /**
+     * Connects to the release request signal
      *
-     *  @param slot the slot to connect
+     * @param slot the slot to connect
      *
-     *  @returns the sigc::connection formed
+     * @return the sigc::connection formed
      */
     sigc::connection connectRelease(sigc::slot<void, SPObject *> slot) {
         return _release_signal.connect(slot);
@@ -195,7 +253,7 @@ public:
      * Represents the style properties, whether from presentation attributes, the <tt>style</tt>
      * attribute, or inherited.
      *
-     * sp_object_private_set doesn't handle SP_ATTR_STYLE or any presentation attributes at the
+     * private_set() doesn't handle SP_ATTR_STYLE or any presentation attributes at the
      * time of writing, so this is probably NULL for all SPObject's that aren't an SPItem.
      *
      * However, this gives rise to the bugs mentioned in sp_object_get_style_property.
@@ -226,52 +284,100 @@ public:
         g_return_val_if_fail(object != NULL, false);
         return this->parent && this->parent == object->parent;
     }
+
+    /**
+     * True if object is non-NULL and this is some in/direct parent of object.
+     */
     bool isAncestorOf(SPObject const *object) const;
 
+    /**
+     * Returns youngest object being parent to this and object.
+     */
     SPObject const *nearestCommonAncestor(SPObject const *object) const;
+
     /* A non-const version can be similarly constructed if you want one.
      * (Don't just cast away the constness, which would be ill-formed.) */
+    SPObject *getNext() {return next;}
 
-    SPObject *getNext();
+    SPObject const *getNext() const {return next;}
+
+    /**
+     * Returns previous object in sibling list or NULL.
+     */
     SPObject *getPrev();
 
     bool hasChildren() const { return ( children != NULL ); }
 
     SPObject *firstChild() { return children; }
     SPObject const *firstChild() const { return children; }
+
     SPObject *lastChild() { return _last_child; }
     SPObject const *lastChild() const { return _last_child; }
 
     enum Action { ActionGeneral, ActionBBox, ActionUpdate, ActionShow };
-    /** @brief Retrieves children as a GSList */
+
+    /**
+     * Retrieves the children as a GSList object, optionally ref'ing the children
+     * in the process, if add_ref is specified.
+     */
     GSList *childList(bool add_ref, Action action = ActionGeneral);
 
+    /**
+     * Append repr as child of this object.
+     * \pre this is not a cloned object
+     */
     SPObject *appendChildRepr(Inkscape::XML::Node *repr);
 
-    /** @brief Gets the author-visible label for this object. */
+    /**
+     * Gets the author-visible label property for the object or a default if
+     * no label is defined.
+     */
     gchar const *label() const;
-    /** @brief Returns a default label for this object. */
+
+    /**
+     * Returns a default label property for this object.
+     */
     gchar const *defaultLabel() const;
-    /** @brief Sets the author-visible label for this object.
+
+    /**
+     * Sets the author-visible label for this object.
      *
-     * Sets the author-visible label for the object.
-     *
-     * @param label the new label
+     * @param label the new label.
      */
     void setLabel(gchar const *label);
 
-    /** Retrieves the title of this object */
+    /**
+     * Returns the title of this object, or NULL if there is none.
+     * The caller must free the returned string using g_free() - see comment
+     * for getTitleOrDesc() below.
+     */
     gchar *title() const;
-    /** Sets the title of this object */
-    bool setTitle(gchar const *title, bool verbatim=false);
 
-    /** Retrieves the description of this object */
+    /**
+     * Sets the title of this object.
+     * A NULL first argument is interpreted as meaning that the existing title
+     * (if any) should be deleted.
+     * The second argument is optional - @see setTitleOrDesc() below for details.
+     */
+    bool setTitle(gchar const *title, bool verbatim = false);
+
+    /**
+     * Returns the description of this object, or NULL if there is none.
+     * The caller must free the returned string using g_free() - see comment
+     * for getTitleOrDesc() below.
+     */
     gchar *desc() const;
-    /** Sets the description of this object */
+
+    /**
+     * Sets the description of this object.
+     * A NULL first argument is interpreted as meaning that the existing
+     * description (if any) should be deleted.
+     * The second argument is optional - @see setTitleOrDesc() below for details.
+     */
     bool setDesc(gchar const *desc, bool verbatim=false);
 
-    /** @brief Set the policy under which this object will be
-     *         orphan-collected.
+    /**
+     * Set the policy under which this object will be orphan-collected.
      *
      * Orphan-collection is the process of deleting all objects which no longer have
      * hyper-references pointing to them.  The policy determines when this happens.  Many objects
@@ -288,21 +394,23 @@ public:
      *  COLLECT_ALWAYS - always collect the object as soon as its
      *                   hrefcount reaches zero
      *
-     * @returns the current collection policy in effect for this object
+     * @return the current collection policy in effect for this object
      */
     CollectionPolicy collectionPolicy() const { return _collection_policy; }
 
-    /** @brief Sets the orphan-collection policy in effect for this object.
-     *
-     * @see SPObject::collectionPolicy
+    /**
+     * Sets the orphan-collection policy in effect for this object.
      *
      * @param policy the new policy to adopt
+     *
+     * @see SPObject::collectionPolicy
      */
     void setCollectionPolicy(CollectionPolicy policy) {
         _collection_policy = policy;
     }
 
-    /** @brief Requests a later automatic call to collectOrphan().
+    /**
+     * Requests a later automatic call to collectOrphan().
      *
      * This method requests that collectOrphan() be called during the document update cycle,
      * deleting the object if it is no longer used.
@@ -313,7 +421,8 @@ public:
      */
     void requestOrphanCollection();
 
-    /** @brief Unconditionally delete the object if it is not referenced.
+    /**
+     * Unconditionally delete the object if it is not referenced.
      *
      * Unconditionally delete the object if there are no outstanding hyper-references to it.
      * Observers are not notified of the object's deletion (at the SPObject level; XML tree
@@ -327,31 +436,41 @@ public:
         }
     }
 
-    /** @brief Check if object is referenced by any other object.
+    /**
+     * Check if object is referenced by any other object.
      */
     bool isReferenced() { return ( _total_hrefcount > 0 ); }
 
-    /** @brief Deletes an object.
+    /**
+     * Deletes an object, unparenting it from its parent.
      *
      * Detaches the object's repr, and optionally sends notification that the object has been
      * deleted.
      *
-     * @param propagate notify observers that the object has been deleted?
+     * @param propagate If it is set to true, it emits a delete signal.
      *
-     * @param propagate_descendants notify observers of children that they have been deleted?
+     * @param propagate_descendants If it is is true, it recursively sends the delete signal to children.
      */
     void deleteObject(bool propagate, bool propagate_descendants);
 
-    /** @brief Deletes on object.
+    /**
+     * Deletes on object.
      *
      * @param propagate Notify observers of this object and its children that they have been
      *                  deleted?
      */
-    void deleteObject(bool propagate=true) {
+    void deleteObject(bool propagate = true)
+    {
         deleteObject(propagate, propagate);
     }
 
-    /** @brief Connects a slot to be called when an object is deleted.
+    /**
+     * Removes all children except for the given object, it's children and it's ancesstors.
+     */
+     void cropToObject(SPObject *except);
+
+    /**
+     * Connects a slot to be called when an object is deleted.
      *
      * This connects a slot to an object's internal delete signal, which is invoked when the object
      * is deleted
@@ -370,14 +489,17 @@ public:
         return _position_changed_signal.connect(slot);
     }
 
-    /** @brief Returns the object which supercedes this one (if any).
+    /**
+     * Returns the object which supercedes this one (if any).
      *
      * This is mainly useful for ensuring we can correctly perform a series of moves or deletes,
      * even if the objects in question have been replaced in the middle of the sequence.
      */
     SPObject *successor() { return _successor; }
 
-    /** @brief Indicates that another object supercedes this one. */
+    /**
+     * Indicates that another object supercedes this one.
+     */
     void setSuccessor(SPObject *successor) {
         g_assert(successor != NULL);
         g_assert(_successor == NULL);
@@ -394,18 +516,23 @@ public:
      * essentially just flushes any changes back to the backing store (the repr layer); maybe it
      * should be called something else and made public at that point. */
 
-    /** @brief Updates the object's repr based on the object's state.
+    /**
+     * Updates the object's repr based on the object's state.
      *
-     *  This method updates the the repr attached to the object to reflect the object's current
+     *  This method updates the repr attached to the object to reflect the object's current
      *  state; see the three-argument version for details.
      *
-     *  @param flags object write flags that apply to this update
+     * @param flags object write flags that apply to this update
      *
-     *  @return the updated repr
+     * @return the updated repr
      */
-    Inkscape::XML::Node *updateRepr(unsigned int flags=SP_OBJECT_WRITE_EXT);
+    Inkscape::XML::Node *updateRepr(unsigned int flags = SP_OBJECT_WRITE_EXT);
 
-    /** @brief Updates the given repr based on the object's state.
+    /**
+     * Updates the given repr based on the object's state.
+     *
+     * Used both to create reprs in the original document, and to create reprs
+     * in another document (e.g. a temporary document used when saving as "Plain SVG".
      *
      *  This method updates the given repr to reflect the object's current state.  There are
      *  several flags that affect this:
@@ -420,14 +547,15 @@ public:
      *   SP_OBJECT_WRITE_ALL   - create all nodes and attributes,
      *                           even those which might be redundant
      *
-     *  @param repr the repr to update
-     *  @param flags object write flags that apply to this update
+     * @param repr the repr to update
+     * @param flags object write flags that apply to this update
      *
-     *  @return the updated repr
+     * @return the updated repr
      */
     Inkscape::XML::Node *updateRepr(Inkscape::XML::Document *doc, Inkscape::XML::Node *repr, unsigned int flags);
 
-    /** @brief Queues an deferred update of this object's display.
+    /**
+     * Queues an deferred update of this object's display.
      *
      *  This method sets flags to indicate updates to be performed later, during the idle loop.
      *
@@ -445,11 +573,12 @@ public:
      *
      *  One of either MODIFIED or CHILD_MODIFIED is required.
      *
-     *  @param flags flags indicating what to update
+     * @param flags flags indicating what to update
      */
     void requestDisplayUpdate(unsigned int flags);
 
-    /** @brief Updates the object's display immediately
+    /**
+     * Updates the object's display immediately
      *
      *  This method is called during the idle loop by SPDocument in order to update the object's
      *  display.
@@ -459,33 +588,43 @@ public:
      *   SP_OBJECT_PARENT_MODIFIED_FLAG - the parent has been
      *                                    modified
      *
-     *  @param ctx an SPCtx which accumulates various state
+     * @param ctx an SPCtx which accumulates various state
      *             during the recursive update -- beware! some
      *             subclasses try to cast this to an SPItemCtx *
      *
-     *  @param flags flags indicating what to update (in addition
+     * @param flags flags indicating what to update (in addition
      *               to any already set flags)
      */
     void updateDisplay(SPCtx *ctx, unsigned int flags);
 
-    /** @brief Requests that a modification notification signal
-     *         be emitted later (e.g. during the idle loop)
+    /**
+     * Requests that a modification notification signal
+     * be emitted later (e.g. during the idle loop)
      *
-     *  @param flags flags indicating what has been modified
+     * Request modified always bubbles *up* the tree, as opposed to
+     * request display update, which trickles down and relies on the
+     * flags set during this pass...
+     *
+     * @param flags flags indicating what has been modified
      */
     void requestModified(unsigned int flags);
 
-    /** @brief Emits a modification notification signal
+    /**
+     *  Emits the MODIFIED signal with the object's flags.
+     *  The object's mflags are the original set aside during the update pass for
+     *  later delivery here.  Once emitModified() is called, those flags don't
+     *  need to be stored any longer.
      *
-     *  @param flags indicating what has been modified
+     * @param flags indicating what has been modified.
      */
     void emitModified(unsigned int flags);
 
-    /** @brief Connects to the modification notification signal
+    /**
+     * Connects to the modification notification signal
      *
-     *  @param slot the slot to connect
+     * @param slot the slot to connect
      *
-     *  @returns the connection formed thereby
+     * @return the connection formed thereby
      */
     sigc::connection connectModified(
       sigc::slot<void, SPObject *, unsigned int> slot
@@ -493,12 +632,21 @@ public:
         return _modified_signal.connect(slot);
     }
 
+    /** Sends the delete signal to all children of this object recursively */
     void _sendDeleteSignalRecursive();
+
+    /**
+     * Adds increment to _total_hrefcount of object and its parents.
+     */
     void _updateTotalHRefCount(int increment);
 
     void _requireSVGVersion(unsigned major, unsigned minor) {
         _requireSVGVersion(Inkscape::Version(major, minor));
     }
+
+    /**
+     * Lifts SVG version of all root objects to version.
+     */
     void _requireSVGVersion(Inkscape::Version version);
 
     sigc::signal<void, SPObject *> _release_signal;
@@ -510,78 +658,209 @@ public:
     gchar *_label;
     mutable gchar *_default_label;
 
+    // WARNING:
+    // Methods below should not be used outside of the SP tree,
+    // as they operate directly on the XML representation.
+    // In future, they will be made protected.
+
+    /**
+     * Put object into object tree, under parent, and behind prev;
+     * also update object's XML space.
+     */
+    void attach(SPObject *object, SPObject *prev);
+
+    /**
+     * In list of object's siblings, move object behind prev.
+     */
+    void reorder(SPObject *prev);
+
+    /**
+     * Remove object from parent's children, release and unref it.
+     */
+    void detach(SPObject *object);
+
+    /**
+     * Return object's child whose node pointer equals repr.
+     */
+    SPObject *get_child_by_repr(Inkscape::XML::Node *repr);
+
+    void invoke_build(SPDocument *document, Inkscape::XML::Node *repr, unsigned int cloned);
+
+    int getIntAttribute(char const *key, int def);
+
+    unsigned getPosition();
+
+    gchar const * getAttribute(gchar const *name,SPException *ex=NULL) const;
+
+    void appendChild(Inkscape::XML::Node *child);
+
+    void addChild(Inkscape::XML::Node *child,Inkscape::XML::Node *prev=NULL);
+
+    /**
+     * Call virtual set() function of object.
+     */
+    void setKeyValue(unsigned int key, gchar const *value);
+
+    void setAttribute(gchar const *key, gchar const *value, SPException *ex=NULL);
+
+    /**
+     * Read value of key attribute from XML node into object.
+     */
+    void readAttr(gchar const *key);
+
+    gchar const *getTagName(SPException *ex) const;
+
+    void removeAttribute(gchar const *key, SPException *ex=NULL);
+
+    /**
+     * Returns an object style property.
+     *
+     * \todo
+     * fixme: Use proper CSS parsing.  The current version is buggy
+     * in a number of situations where key is a substring of the
+     * style string other than as a property name (including
+     * where key is a substring of a property name), and is also
+     * buggy in its handling of inheritance for properties that
+     * aren't inherited by default.  It also doesn't allow for
+     * the case where the property is specified but with an invalid
+     * value (in which case I believe the CSS2 error-handling
+     * behaviour applies, viz. behave as if the property hadn't
+     * been specified).  Also, the current code doesn't use CRSelEng
+     * stuff to take a value from stylesheets.  Also, we aren't
+     * setting any hooks to force an update for changes in any of
+     * the inputs (i.e., in any of the elements that this function
+     * queries).
+     *
+     * \par
+     * Given that the default value for a property depends on what
+     * property it is (e.g., whether to inherit or not), and given
+     * the above comment about ignoring invalid values, and that the
+     * repr parent isn't necessarily the right element to inherit
+     * from (e.g., maybe we need to inherit from the referencing
+     * <use> element instead), we should probably make the caller
+     * responsible for ascending the repr tree as necessary.
+     */
+    gchar const *getStyleProperty(gchar const *key, gchar const *def) const;
+
+    void setCSS(SPCSSAttr *css, gchar const *attr);
+
+    void changeCSS(SPCSSAttr *css, gchar const *attr);
+
+    bool storeAsDouble( gchar const *key, double *val ) const;
+
 private:
     // Private member functions used in the definitions of setTitle(),
     // setDesc(), title() and desc().
+
+    /**
+     * Sets or deletes the title or description of this object.
+     * A NULL 'value' argument causes the title or description to be deleted.
+     *
+     * 'verbatim' parameter:
+     * If verbatim==true, then the title or description is set to exactly the
+     * specified value.  If verbatim==false then two exceptions are made:
+     *   (1) If the specified value is just whitespace, then the title/description
+     *       is deleted.
+     *   (2) If the specified value is the same as the current value except for
+     *       mark-up, then the current value is left unchanged.
+     * This is usually the desired behaviour, so 'verbatim' defaults to false for
+     * setTitle() and setDesc().
+     *
+     * The return value is true if a change was made to the title/description,
+     * and usually false otherwise.
+     */
     bool setTitleOrDesc(gchar const *value, gchar const *svg_tagname, bool verbatim);
+
+    /**
+     * Returns the title or description of this object, or NULL if there is none.
+     *
+     * The SVG spec allows 'title' and 'desc' elements to contain text marked up
+     * using elements from other namespaces.  Therefore, this function cannot
+     * in general just return a pointer to an existing string - it must instead
+     * construct a string containing the title or description without the mark-up.
+     * Consequently, the return value is a newly allocated string (or NULL), and
+     * must be freed (using g_free()) by the caller.
+     */
     gchar * getTitleOrDesc(gchar const *svg_tagname) const;
+
+    /**
+     * Find the first child of this object with a given tag name,
+     * and return it.  Returns NULL if there is no matching child.
+     */
     SPObject * findFirstChild(gchar const *tagname) const;
+
+    /**
+     * Return the full textual content of an element (typically all the
+     * content except the tags).
+     * Must not be used on anything except elements.
+     */
     GString * textualContent() const;
 
+    /* Real handlers of repr signals */
+
+public:
+    /**
+     * Callback for attr_changed node event.
+     */
+    static void repr_attr_changed(Inkscape::XML::Node *repr, gchar const *key, gchar const *oldval, gchar const *newval, bool is_interactive, gpointer data);
+
+    /**
+     * Callback for content_changed node event.
+     */
+    static void repr_content_changed(Inkscape::XML::Node *repr, gchar const *oldcontent, gchar const *newcontent, gpointer data);
+
+    /**
+     * Callback for child_added node event.
+     */
+    static void repr_child_added(Inkscape::XML::Node *repr, Inkscape::XML::Node *child, Inkscape::XML::Node *ref, gpointer data);
+
+    /**
+     * Callback for remove_child node event.
+     */
+    static void repr_child_removed(Inkscape::XML::Node *repr, Inkscape::XML::Node *child, Inkscape::XML::Node *ref, void *data);
+
+    /**
+     * Callback for order_changed node event.
+     *
+     * \todo fixme:
+     */
+    static void repr_order_changed(Inkscape::XML::Node *repr, Inkscape::XML::Node *child, Inkscape::XML::Node *old, Inkscape::XML::Node *newer, gpointer data);
+
+
     friend class SPObjectImpl;
-};
 
-/// The SPObject vtable.
-struct SPObjectClass {
-    GObjectClass parent_class;
+protected:
+	virtual void build(SPDocument* doc, Inkscape::XML::Node* repr);
+	virtual void release();
 
-    void (* build) (SPObject *object, SPDocument *doc, Inkscape::XML::Node *repr);
-    void (* release) (SPObject *object);
+	virtual void child_added(Inkscape::XML::Node* child, Inkscape::XML::Node* ref);
+	virtual void remove_child(Inkscape::XML::Node* child);
 
-    /* Virtual handlers of repr signals */
-    void (* child_added) (SPObject *object, Inkscape::XML::Node *child, Inkscape::XML::Node *ref);
-    void (* remove_child) (SPObject *object, Inkscape::XML::Node *child);
+	virtual void order_changed(Inkscape::XML::Node* child, Inkscape::XML::Node* old_repr, Inkscape::XML::Node* new_repr);
 
-    void (* order_changed) (SPObject *object, Inkscape::XML::Node *child, Inkscape::XML::Node *old, Inkscape::XML::Node *new_repr);
+	virtual void set(unsigned int key, const gchar* value);
 
-    void (* set) (SPObject *object, unsigned int key, gchar const *value);
+	virtual void update(SPCtx* ctx, unsigned int flags);
+	virtual void modified(unsigned int flags);
 
-    void (* read_content) (SPObject *object);
+	virtual Inkscape::XML::Node* write(Inkscape::XML::Document* doc, Inkscape::XML::Node* repr, guint flags);
 
-    /* Update handler */
-    void (* update) (SPObject *object, SPCtx *ctx, unsigned int flags);
-    /* Modification handler */
-    void (* modified) (SPObject *object, unsigned int flags);
-
-    Inkscape::XML::Node * (* write) (SPObject *object, Inkscape::XML::Document *doc, Inkscape::XML::Node *repr, unsigned int flags);
+public:
+	virtual void read_content();
 };
 
 
-/*
- * Attaching/detaching
+/**
+ * Compares height of objects in tree.
+ *
+ * Works for different-parent objects, so long as they have a common ancestor.
+ * \return \verbatim
+ *    0    positions are equivalent
+ *    1    first object's position is greater than the second
+ *   -1    first object's position is less than the second   \endverbatim
  */
-
-void sp_object_attach(SPObject *parent, SPObject *object, SPObject *prev);
-void sp_object_reorder(SPObject *object, SPObject *prev);
-void sp_object_detach(SPObject *parent, SPObject *object);
-
-inline SPObject *sp_object_first_child(SPObject *parent) {
-    return parent->firstChild();
-}
-SPObject *sp_object_get_child_by_repr(SPObject *object, Inkscape::XML::Node *repr);
-
-void sp_object_invoke_build(SPObject *object, SPDocument *document, Inkscape::XML::Node *repr, unsigned int cloned);
-
-void sp_object_set(SPObject *object, unsigned int key, gchar const *value);
-
-void sp_object_read_attr(SPObject *object, gchar const *key);
-
-/* Public */
-
-gchar const *sp_object_tagName_get(SPObject const *object, SPException *ex);
-gchar const *sp_object_getAttribute(SPObject const *object, gchar const *key, SPException *ex);
-void sp_object_setAttribute(SPObject *object, gchar const *key, gchar const *value, SPException *ex);
-void sp_object_removeAttribute(SPObject *object, gchar const *key, SPException *ex);
-
-/* Style */
-
-gchar const *sp_object_get_style_property(SPObject const *object,
-                                          gchar const *key, gchar const *def);
-
 int sp_object_compare_position(SPObject const *first, SPObject const *second);
 
-SPObject *sp_object_prev(SPObject *child);
-
 
 #endif // SP_OBJECT_H_SEEN
 
@@ -595,4 +874,4 @@ SPObject *sp_object_prev(SPObject *child);
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
