@@ -18,7 +18,7 @@
 #include "sp-object.h"
 #include "uri-references.h"
 
-struct SPPaintServer;
+class SPPaintServer;
 
 class SPPaintServerReference : public Inkscape::URIReference {
 public:
@@ -40,4 +40,4 @@ protected:
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
