@@ -1,8 +1,7 @@
-/**
- * Inkscape::SVG::PathString - builder for SVG path strings
- *
+/*
  * Copyright 2007 MenTaLguY <mental@rydia.net>
  * Copyright 2008 Jasper van de Gronde <th.v.d.gronde@hccnet.nl>
+ * Copyright 2013 Tavmjong Bah <tavmjong@free.fr>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,6 +24,17 @@ namespace Inkscape {
 
 namespace SVG {
 
+// Relative vs. absolute coordinates
+enum PATHSTRING_FORMAT {
+    PATHSTRING_ABSOLUTE,  // Use only absolute coordinates
+    PATHSTRING_RELATIVE,  // Use only relative coordinates
+    PATHSTRING_OPTIMIZE,  // Optimize for path string length
+    PATHSTRING_FORMAT_SIZE
+};
+
+/**
+ * Builder for SVG path strings.
+ */
 class PathString {
 public:
     PathString();
@@ -37,6 +47,7 @@ public:
         final.reserve(commonbase.size()+t.size());
         final = commonbase;
         final += tail();
+        // std::cout << " final: " << final << std::endl;
         return final;
     }
 
@@ -129,8 +140,10 @@ public:
     }
 
     PathString &closePath() {
-        _abs_state.appendOp('z');
+
+        _abs_state.appendOp('Z');
         _rel_state.appendOp('z');
+
         _current_point = _initial_point;
         return *this;
     }
@@ -224,9 +237,13 @@ private:
     // to cause a quadratic time complexity (in the number of characters/operators)
     std::string commonbase;
     std::string final;
-    std::string const &tail() const { return ((_abs_state <= _rel_state || !allow_relative_coordinates) ? _abs_state.str : _rel_state.str); }
+    std::string const &tail() const {
+        return ( (format == PATHSTRING_ABSOLUTE) ||
+                 (format == PATHSTRING_OPTIMIZE && _abs_state <= _rel_state ) ?
+                 _abs_state.str : _rel_state.str );
+    }
 
-    bool const allow_relative_coordinates;
+    static PATHSTRING_FORMAT format;
     bool const force_repeat_commands;
     static int numericprecision;
     static int minimumexponent;
@@ -246,4 +263,4 @@ private:
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
