@@ -1,4 +1,3 @@
-
 #ifndef SEEN_TEST_HELPERS_H
 #define SEEN_TEST_HELPERS_H
 
@@ -9,7 +8,7 @@
 #include "inkscape-private.h"
 
 
-/// Dummy functions to keep linker happy
+// Dummy functions to keep linker happy
 #if !defined(DUMMY_MAIN_TEST_CALLS_SEEN)
 #define DUMMY_MAIN_TEST_CALLS_SEEN
 int sp_main_gui (int, char const**) { return 0; }
@@ -24,7 +23,10 @@ T* createSuiteAndDocument( void (*fun)(T*&) )
 {
     T* suite = 0;
 
+#if !GLIB_CHECK_VERSION(2,36,0)
     g_type_init();
+#endif
+
     Inkscape::GC::init();
     if ( !inkscape_get_instance() )
     {
@@ -32,7 +34,7 @@ T* createSuiteAndDocument( void (*fun)(T*&) )
         static_cast<void>(g_object_new(inkscape_get_type(), NULL));
     }
 
-    SPDocument* tmp = sp_document_new( NULL, TRUE, true );
+    SPDocument* tmp = SPDocument::createNewDoc( NULL, TRUE, true );
     if ( tmp ) {
         fun( suite );
         if ( suite )
@@ -41,7 +43,7 @@ T* createSuiteAndDocument( void (*fun)(T*&) )
         }
         else
         {
-            sp_document_unref( tmp );
+            tmp->doUnref();
         }
     }
 
@@ -61,4 +63,4 @@ T* createSuiteAndDocument( void (*fun)(T*&) )
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
