@@ -1,10 +1,10 @@
-#define __SP_TEXT_CHEMISTRY_C__
-
 /*
  * Text commands
  *
  * Authors:
  *   bulia byak
+ *   Jon A. Cruz <jon@joncruz.org>
+ *   Abhishek Sharma
  *
  * Copyright (C) 2004 authors
  *
@@ -19,13 +19,13 @@
 #include <string>
 #include <glibmm/i18n.h>
 
-#include "libnr/nr-matrix-fns.h"
 #include "xml/repr.h"
 #include "sp-rect.h"
 #include "sp-textpath.h"
 #include "inkscape.h"
 #include "desktop.h"
 #include "document.h"
+#include "document-undo.h"
 #include "message-stack.h"
 #include "selection.h"
 #include "style.h"
@@ -36,21 +36,11 @@
 #include "sp-flowregion.h"
 #include "sp-flowdiv.h"
 #include "sp-tspan.h"
+#include "verbs.h"
 
+using Inkscape::DocumentUndo;
 
-SPItem *
-text_in_selection(Inkscape::Selection *selection)
-{
-    for (GSList *items = (GSList *) selection->itemList();
-         items != NULL;
-         items = items->next) {
-        if (SP_IS_TEXT(items->data))
-            return ((SPItem *) items->data);
-    }
-    return NULL;
-}
-
-SPItem *
+static SPItem *
 flowtext_in_selection(Inkscape::Selection *selection)
 {
     for (GSList *items = (GSList *) selection->itemList();
@@ -62,7 +52,7 @@ flowtext_in_selection(Inkscape::Selection *selection)
     return NULL;
 }
 
-SPItem *
+static SPItem *
 text_or_flowtext_in_selection(Inkscape::Selection *selection)
 {
     for (GSList *items = (GSList *) selection->itemList();
@@ -74,7 +64,7 @@ text_or_flowtext_in_selection(Inkscape::Selection *selection)
     return NULL;
 }
 
-SPItem *
+static SPItem *
 shape_in_selection(Inkscape::Selection *selection)
 {
     for (GSList *items = (GSList *) selection->itemList();
@@ -98,7 +88,7 @@ text_put_on_path()
     SPItem *text = text_or_flowtext_in_selection(selection);
     SPItem *shape = shape_in_selection(selection);
 
-    Inkscape::XML::Document *xml_doc = sp_document_repr_doc(desktop->doc());
+    Inkscape::XML::Document *xml_doc = desktop->doc()->getReprDoc();
 
     if (!text || !shape || g_slist_length((GSList *) selection->itemList()) != 2) {
         sp_desktop_message_stack(desktop)->flash(Inkscape::WARNING_MESSAGE, _("Select <b>a text and a path</b> to put text on path."));
@@ -129,17 +119,17 @@ text_put_on_path()
 
         if (!repr) return;
 
-        Inkscape::XML::Node *parent = SP_OBJECT_REPR(text)->parent();
+        Inkscape::XML::Node *parent = text->getRepr()->parent();
         parent->appendChild(repr);
 
         SPItem *new_item = (SPItem *) sp_desktop_document(desktop)->getObjectByRepr(repr);
-        sp_item_write_transform(new_item, repr, text->transform);
-        SP_OBJECT(new_item)->updateRepr();
+        new_item->doWriteTransform(repr, text->transform);
+        new_item->updateRepr();
 
         Inkscape::GC::release(repr);
         text->deleteObject(); // delete the orignal flowtext
 
-        sp_document_ensure_up_to_date(sp_desktop_document(desktop));
+        sp_desktop_document(desktop)->ensureUpToDate();
 
         selection->clear();
 
@@ -150,24 +140,27 @@ text_put_on_path()
     Inkscape::Text::Layout::Alignment text_alignment = layout->paragraphAlignment(layout->begin());
 
     // remove transform from text, but recursively scale text's fontsize by the expansion
-    SP_TEXT(text)->_adjustFontsizeRecursive (text, NR::expansion(SP_ITEM(text)->transform));
-    SP_OBJECT_REPR(text)->setAttribute("transform", NULL);
+    SP_TEXT(text)->_adjustFontsizeRecursive (text, text->transform.descrim());
+    text->getRepr()->setAttribute("transform", NULL);
 
     // make a list of text children
     GSList *text_reprs = NULL;
-    for (SPObject *o = SP_OBJECT(text)->children; o != NULL; o = o->next) {
-        text_reprs = g_slist_prepend(text_reprs, SP_OBJECT_REPR(o));
+    for (SPObject *o = text->children; o != NULL; o = o->next) {
+        text_reprs = g_slist_prepend(text_reprs, o->getRepr());
     }
 
     // create textPath and put it into the text
     Inkscape::XML::Node *textpath = xml_doc->createElement("svg:textPath");
     // reference the shape
-    textpath->setAttribute("xlink:href", g_strdup_printf("#%s", SP_OBJECT_REPR(shape)->attribute("id")));
-    if (text_alignment == Inkscape::Text::Layout::RIGHT)
+    gchar *href_str = g_strdup_printf("#%s", shape->getRepr()->attribute("id"));
+    textpath->setAttribute("xlink:href", href_str);
+    g_free(href_str);
+    if (text_alignment == Inkscape::Text::Layout::RIGHT) {
         textpath->setAttribute("startOffset", "100%");
-    else if (text_alignment == Inkscape::Text::Layout::CENTER)
+    } else if (text_alignment == Inkscape::Text::Layout::CENTER) {
         textpath->setAttribute("startOffset", "50%");
-    SP_OBJECT_REPR(text)->addChild(textpath, NULL);
+    }
+    text->getRepr()->addChild(textpath, NULL);
 
     for ( GSList *i = text_reprs ; i ; i = i->next ) {
         // Make a copy of each text child
@@ -179,17 +172,17 @@ text_put_on_path()
             copy->setAttribute("y", NULL);
         }
         // remove the old repr from under text
-        SP_OBJECT_REPR(text)->removeChild((Inkscape::XML::Node *) i->data);
+        text->getRepr()->removeChild(reinterpret_cast<Inkscape::XML::Node *>(i->data));
         // put its copy into under textPath
         textpath->addChild(copy, NULL); // fixme: copy id
     }
 
     // x/y are useless with textpath, and confuse Batik 1.5
-    SP_OBJECT_REPR(text)->setAttribute("x", NULL);
-    SP_OBJECT_REPR(text)->setAttribute("y", NULL);
+    text->getRepr()->setAttribute("x", NULL);
+    text->getRepr()->setAttribute("y", NULL);
 
-    sp_document_done(sp_desktop_document(desktop), SP_VERB_CONTEXT_TEXT, 
-                     _("Put text on path"));
+    DocumentUndo::done(sp_desktop_document(desktop), SP_VERB_CONTEXT_TEXT, 
+                       _("Put text on path"));
     g_slist_free(text_reprs);
 }
 
@@ -210,50 +203,50 @@ text_remove_from_path()
     for (GSList *items = g_slist_copy((GSList *) selection->itemList());
          items != NULL;
          items = items->next) {
+        SPObject *obj = SP_OBJECT(items->data);
 
-        if (!SP_IS_TEXT_TEXTPATH(SP_OBJECT(items->data))) {
-            continue;
+        if (SP_IS_TEXT_TEXTPATH(obj)) {
+            SPObject *tp = obj->firstChild();
+
+            did = true;
+
+            sp_textpath_to_text(tp);
         }
-
-        SPObject *tp = sp_object_first_child(SP_OBJECT(items->data));
-
-        did = true;
-
-        sp_textpath_to_text(tp);
     }
 
     if (!did) {
         sp_desktop_message_stack(desktop)->flash(Inkscape::ERROR_MESSAGE, _("<b>No texts-on-paths</b> in the selection."));
     } else {
-        sp_document_done(sp_desktop_document(desktop), SP_VERB_CONTEXT_TEXT, 
-                         _("Remove text from path"));
+        DocumentUndo::done(sp_desktop_document(desktop), SP_VERB_CONTEXT_TEXT, 
+                           _("Remove text from path"));
         selection->setList(g_slist_copy((GSList *) selection->itemList())); // reselect to update statusbar description
     }
 }
 
-void
+static void
 text_remove_all_kerns_recursively(SPObject *o)
 {
-    SP_OBJECT_REPR(o)->setAttribute("dx", NULL);
-    SP_OBJECT_REPR(o)->setAttribute("dy", NULL);
-    SP_OBJECT_REPR(o)->setAttribute("rotate", NULL);
+    o->getRepr()->setAttribute("dx", NULL);
+    o->getRepr()->setAttribute("dy", NULL);
+    o->getRepr()->setAttribute("rotate", NULL);
 
     // if x contains a list, leave only the first value
-    gchar *x = (gchar *) SP_OBJECT_REPR(o)->attribute("x");
+    gchar const *x = o->getRepr()->attribute("x");
     if (x) {
         gchar **xa_space = g_strsplit(x, " ", 0);
         gchar **xa_comma = g_strsplit(x, ",", 0);
         if (xa_space && *xa_space && *(xa_space + 1)) {
-            SP_OBJECT_REPR(o)->setAttribute("x", g_strdup(*xa_space));
+            o->getRepr()->setAttribute("x", *xa_space);
         } else if (xa_comma && *xa_comma && *(xa_comma + 1)) {
-            SP_OBJECT_REPR(o)->setAttribute("x", g_strdup(*xa_comma));
+            o->getRepr()->setAttribute("x", *xa_comma);
         }
         g_strfreev(xa_space);
         g_strfreev(xa_comma);
     }
 
-    for (SPObject *i = sp_object_first_child(o); i != NULL; i = SP_OBJECT_NEXT(i)) {
+    for (SPObject *i = o->firstChild(); i != NULL; i = i->getNext()) {
         text_remove_all_kerns_recursively(i);
+        i->requestDisplayUpdate(SP_OBJECT_MODIFIED_FLAG | SP_TEXT_LAYOUT_MODIFIED_FLAG);
     }
 }
 
@@ -289,8 +282,8 @@ text_remove_all_kerns()
     if (!did) {
         sp_desktop_message_stack(desktop)->flash(Inkscape::ERROR_MESSAGE, _("Select <b>text(s)</b> to remove kerns from."));
     } else {
-        sp_document_done(sp_desktop_document(desktop), SP_VERB_CONTEXT_TEXT, 
-                         _("Remove manual kerns"));
+        DocumentUndo::done(sp_desktop_document(desktop), SP_VERB_CONTEXT_TEXT, 
+                           _("Remove manual kerns"));
     }
 }
 
@@ -302,7 +295,7 @@ text_flow_into_shape()
         return;
 
     SPDocument *doc = sp_desktop_document (desktop);
-    Inkscape::XML::Document *xml_doc = sp_document_repr_doc(doc);
+    Inkscape::XML::Document *xml_doc = doc->getReprDoc();
 
     Inkscape::Selection *selection = sp_desktop_selection(desktop);
 
@@ -316,14 +309,14 @@ text_flow_into_shape()
 
     if (SP_IS_TEXT(text)) {
       // remove transform from text, but recursively scale text's fontsize by the expansion
-      SP_TEXT(text)->_adjustFontsizeRecursive(text, NR::expansion(SP_ITEM(text)->transform));
-      SP_OBJECT_REPR(text)->setAttribute("transform", NULL);
+      SP_TEXT(text)->_adjustFontsizeRecursive(text, text->transform.descrim());
+      text->getRepr()->setAttribute("transform", NULL);
     }
 
     Inkscape::XML::Node *root_repr = xml_doc->createElement("svg:flowRoot");
     root_repr->setAttribute("xml:space", "preserve"); // we preserve spaces in the text objects we create
-    root_repr->setAttribute("style", SP_OBJECT_REPR(text)->attribute("style")); // fixme: transfer style attrs too
-    SP_OBJECT_REPR(SP_OBJECT_PARENT(shape))->appendChild(root_repr);
+    root_repr->setAttribute("style", text->getRepr()->attribute("style")); // fixme: transfer style attrs too
+    shape->parent->getRepr()->appendChild(root_repr);
     SPObject *root_object = doc->getObjectByRepr(root_repr);
     g_return_if_fail(SP_IS_FLOWTEXT(root_object));
 
@@ -341,7 +334,9 @@ text_flow_into_shape()
             Inkscape::XML::Node *clone = xml_doc->createElement("svg:use");
             clone->setAttribute("x", "0");
             clone->setAttribute("y", "0");
-            clone->setAttribute("xlink:href", g_strdup_printf("#%s", SP_OBJECT_REPR(item)->attribute("id")));
+            gchar *href_str = g_strdup_printf("#%s", item->getRepr()->attribute("id"));
+            clone->setAttribute("xlink:href", href_str);
+            g_free(href_str);
 
             // add the new clone to the region
             region_repr->appendChild(clone);
@@ -364,9 +359,9 @@ text_flow_into_shape()
         Inkscape::GC::release(text_repr);
 
     } else { // reflow an already flowed text, preserving paras
-        for (SPObject *o = SP_OBJECT(text)->children; o != NULL; o = o->next) {
+        for (SPObject *o = text->children; o != NULL; o = o->next) {
             if (SP_IS_FLOWPARA(o)) {
-                Inkscape::XML::Node *para_repr = SP_OBJECT_REPR(o)->duplicate(xml_doc);
+                Inkscape::XML::Node *para_repr = o->getRepr()->duplicate(xml_doc);
                 root_repr->appendChild(para_repr);
                 object = doc->getObjectByRepr(para_repr);
                 g_return_if_fail(SP_IS_FLOWPARA(object));
@@ -375,10 +370,10 @@ text_flow_into_shape()
         }
     }
 
-    SP_OBJECT(text)->deleteObject (true);
+    text->deleteObject(true);
 
-    sp_document_done(doc, SP_VERB_CONTEXT_TEXT,
-                     _("Flow text into shape"));
+    DocumentUndo::done(doc, SP_VERB_CONTEXT_TEXT,
+                       _("Flow text into shape"));
 
     sp_desktop_selection(desktop)->set(SP_ITEM(root_object));
 
@@ -394,7 +389,7 @@ text_unflow ()
         return;
 
     SPDocument *doc = sp_desktop_document (desktop);
-    Inkscape::XML::Document *xml_doc = sp_document_repr_doc(doc);
+    Inkscape::XML::Document *xml_doc = doc->getReprDoc();
 
     Inkscape::Selection *selection = sp_desktop_selection(desktop);
 
@@ -430,12 +425,11 @@ text_unflow ()
         rtext->setAttribute("xml:space", "preserve"); // we preserve spaces in the text objects we create
 
         /* Set style */
-        rtext->setAttribute("style", SP_OBJECT_REPR(flowtext)->attribute("style")); // fixme: transfer style attrs too; and from descendants
+        rtext->setAttribute("style", flowtext->getRepr()->attribute("style")); // fixme: transfer style attrs too; and from descendants
 
-        NRRect bbox;
-        sp_item_invoke_bbox(SP_ITEM(flowtext), &bbox, sp_item_i2doc_affine(SP_ITEM(flowtext)), TRUE);
-        Geom::Point xy(bbox.x0, bbox.y0);
-        if (xy[Geom::X] != 1e18 && xy[Geom::Y] != 1e18) {
+        Geom::OptRect bbox = flowtext->geometricBounds(flowtext->i2doc_affine());
+        if (bbox) {
+            Geom::Point xy = bbox->min();
             sp_repr_set_svg_double(rtext, "x", xy[Geom::X]);
             sp_repr_set_svg_double(rtext, "y", xy[Geom::Y]);
         }
@@ -450,11 +444,12 @@ text_unflow ()
         free(text_string);
         rtspan->appendChild(text_repr);
 
-        SP_OBJECT_REPR(SP_OBJECT_PARENT(flowtext))->appendChild(rtext);
+        flowtext->parent->getRepr()->appendChild(rtext);
         SPObject *text_object = doc->getObjectByRepr(rtext);
 
         // restore the font size multiplier from the flowtext's transform
-        SP_TEXT(text_object)->_adjustFontsizeRecursive(SP_ITEM(text_object), ex);
+        SPText *text = SP_TEXT(text_object);
+        text->_adjustFontsizeRecursive(text, ex);
 
         new_objs = g_slist_prepend (new_objs, text_object);
         old_objs = g_slist_prepend (old_objs, flowtext);
@@ -473,8 +468,8 @@ text_unflow ()
     g_slist_free (old_objs);
     g_slist_free (new_objs);
 
-    sp_document_done(doc, SP_VERB_CONTEXT_TEXT, 
-                     _("Unflow flowed text"));
+    DocumentUndo::done(doc, SP_VERB_CONTEXT_TEXT, 
+                       _("Unflow flowed text"));
 }
 
 void
@@ -514,12 +509,12 @@ flowtext_to_text()
 
         did = true;
 
-        Inkscape::XML::Node *parent = SP_OBJECT_REPR(item)->parent();
-        parent->addChild(repr, SP_OBJECT_REPR(item));
+        Inkscape::XML::Node *parent = item->getRepr()->parent();
+        parent->addChild(repr, item->getRepr());
 
-        SPItem *new_item = (SPItem *) sp_desktop_document(desktop)->getObjectByRepr(repr);
-        sp_item_write_transform(new_item, repr, item->transform);
-        SP_OBJECT(new_item)->updateRepr();
+        SPItem *new_item = reinterpret_cast<SPItem *>(sp_desktop_document(desktop)->getObjectByRepr(repr));
+        new_item->doWriteTransform(repr, item->transform);
+        new_item->updateRepr();
     
         Inkscape::GC::release(repr);
         item->deleteObject();
@@ -530,9 +525,9 @@ flowtext_to_text()
     g_slist_free(items);
 
     if (did) {
-        sp_document_done(sp_desktop_document(desktop), 
-                         SP_VERB_OBJECT_FLOWTEXT_TO_TEXT,
-                         _("Convert flowed text to text"));
+        DocumentUndo::done(sp_desktop_document(desktop), 
+                           SP_VERB_OBJECT_FLOWTEXT_TO_TEXT,
+                           _("Convert flowed text to text"));
         selection->setReprList(reprs);        
     } else {
         sp_desktop_message_stack(desktop)->
@@ -553,4 +548,4 @@ flowtext_to_text()
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
