@@ -1,5 +1,5 @@
-#ifndef __TEXT_CHEMISTRY_H__
-#define __TEXT_CHEMISTRY_H__
+#ifndef SEEN_TEXT_CHEMISTRY_H
+#define SEEN_TEXT_CHEMISTRY_H
 
 /*
  * Text commands
@@ -12,16 +12,14 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include "forward.h"
-
-void text_put_on_path (void);
-void text_remove_from_path (void);
-void text_remove_all_kerns (void);
+void text_put_on_path(void);
+void text_remove_from_path(void);
+void text_remove_all_kerns(void);
 void text_flow_into_shape();
 void text_unflow();
 void flowtext_to_text();
 
-#endif
+#endif // SEEN_TEXT_CHEMISTRY_H
 
 /*
   Local Variables:
@@ -32,4 +30,4 @@ void flowtext_to_text();
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
