@@ -1,5 +1,5 @@
-#ifndef __SP_TEXT_EDITING_H__
-#define __SP_TEXT_EDITING_H__
+#ifndef SEEN_SP_TEXT_EDITING_H
+#define SEEN_SP_TEXT_EDITING_H
 
 /*
  * Text editing functions common for for text and flowtext
@@ -16,24 +16,36 @@
 #include <glib.h>
 #include <utility>   // std::pair
 #include "libnrtype/Layout-TNG.h"
-#include <libnr/nr-forward.h>
 #include "text-tag-attributes.h"
 
 class SPCSSAttr;
-struct SPItem;
-struct SPObject;
-struct SPStyle;
+class SPDesktop;
+class SPItem;
+class SPObject;
+class SPStyle;
 
 typedef std::pair<Inkscape::Text::Layout::iterator, Inkscape::Text::Layout::iterator> iterator_pair; 
 
 
 Inkscape::Text::Layout const * te_get_layout (SPItem const *item);
-bool sp_te_output_is_empty (SPItem const *item);
-bool sp_te_input_is_empty (SPObject const *item);
 
+void te_update_layout_now_recursive(SPItem *item);
+
+/** Returns true if there are no visible characters on the canvas. */
+bool sp_te_output_is_empty(SPItem const *item);
+
+/** Returns true if the user has typed nothing in the text box. */
+bool sp_te_input_is_empty(SPObject const *item);
+
+/** Recursively gets the length of all the SPStrings at or below the given
+\a item. Also adds 1 for each line break encountered. */
 unsigned sp_text_get_length(SPObject const *item);
+
+/** Recursively gets the length of all the SPStrings at or below the given
+\a item, before and not including \a upto. Also adds 1 for each line break encountered. */
 unsigned sp_text_get_length_upto(SPObject const *item, SPObject const *upto);
-std::vector<Geom::Point> sp_te_create_selection_quads(SPItem const *item, Inkscape::Text::Layout::iterator const &start, Inkscape::Text::Layout::iterator const &end, Geom::Matrix const &transform);
+
+std::vector<Geom::Point> sp_te_create_selection_quads(SPItem const *item, Inkscape::Text::Layout::iterator const &start, Inkscape::Text::Layout::iterator const &end, Geom::Affine const &transform);
 
 Inkscape::Text::Layout::iterator sp_te_get_position_by_coords (SPItem const *item, Geom::Point const &i_p);
 void sp_te_get_cursor_coords (SPItem const *item, Inkscape::Text::Layout::iterator const &position, Geom::Point &p0, Geom::Point &p1);
@@ -70,4 +82,4 @@ bool is_part_of_text_subtree (SPObject *obj);
 bool is_top_level_text_object (SPObject *obj);
 bool has_visible_text (SPObject *obj);
 
-#endif
+#endif // SEEN_SP_TEXT_EDITING_H
