@@ -12,7 +12,12 @@
 #ifndef SEEN_TOOLS_SWITCH_H
 #define SEEN_TOOLS_SWITCH_H
 
-#include <forward.h>
+class SPDesktop;
+class SPItem;
+namespace Geom {
+class Point;
+}
+
 
 enum {
   TOOLS_INVALID,
@@ -30,7 +35,9 @@ enum {
   TOOLS_CALLIGRAPHIC,
   TOOLS_TEXT,
   TOOLS_GRADIENT,
+  TOOLS_MESH,
   TOOLS_ZOOM,
+  TOOLS_MEASURE,
   TOOLS_DROPPER,
   TOOLS_CONNECTOR,
   TOOLS_PAINTBUCKET,
@@ -43,7 +50,7 @@ int tools_active(SPDesktop *dt);
 void tools_switch(SPDesktop *dt, int num);
 void tools_switch_by_item (SPDesktop *dt, SPItem *item, Geom::Point const p);
 
-#endif /* !SEEN_TOOLS_SWITCH_H */
+#endif // !SEEN_TOOLS_SWITCH_H
 
 /*
   Local Variables:
