@@ -17,7 +17,8 @@
 #include "inkscape-potrace.h"
 
 #include <glibmm/i18n.h>
-#include <gtkmm.h>
+#include <gtkmm/main.h>
+#include <iomanip>
 
 #include "trace/filterset.h"
 #include "trace/quantize.h"
@@ -31,7 +32,7 @@
 #include "curve.h"
 #include "bitmap.h"
 
-
+using Glib::ustring;
 
 static void updateGui()
 {
@@ -57,6 +58,12 @@ static void potraceStatusCallback(double /*progress*/, void *userData) /* callba
 }
 
 
+namespace {
+ustring twohex( int value )
+{
+    return ustring::format(std::hex, std::setfill(L'0'), std::setw(2), value);
+}
+} // namespace
 
 
 //required by potrace
@@ -70,20 +77,23 @@ namespace Potrace {
 /**
  *
  */
-PotraceTracingEngine::PotraceTracingEngine()
+PotraceTracingEngine::PotraceTracingEngine() :
+    keepGoing(1),
+    traceType(TRACE_BRIGHTNESS),
+    invert(false),
+    quantizationNrColors(8),
+    brightnessThreshold(0.45),
+    brightnessFloor(0),
+    cannyHighThreshold(0.65),
+    multiScanNrColors(8),
+    multiScanStack(true),
+    multiScanSmooth(false),
+    multiScanRemoveBackground(false)
 {
     /* get default parameters */
     potraceParams = potrace_param_default();
     potraceParams->progress.callback = potraceStatusCallback;
     potraceParams->progress.data = (void *)this;
-
-    //##### Our defaults
-    invert    = false;
-    traceType = TRACE_BRIGHTNESS;
-    quantizationNrColors = 8;
-    brightnessThreshold  = 0.45;
-    cannyHighThreshold   = 0.65;
-
 }
 
 PotraceTracingEngine::~PotraceTracingEngine()
@@ -105,8 +115,7 @@ typedef struct
  * Check a point against a list of points to see if it
  * has already occurred.
  */
-static bool
-hasPoint(std::vector<Point> &points, double x, double y)
+static bool hasPoint(std::vector<Point> &points, double x, double y)
 {
     for (unsigned int i=0; i<points.size() ; i++)
         {
@@ -123,8 +132,7 @@ hasPoint(std::vector<Point> &points, double x, double y)
  *  format into the output stream.  The Point vector is used to prevent
  *  redundant paths.  Returns number of paths processed.
  */
-static long
-writePaths(PotraceTracingEngine *engine, potrace_path_t *plist,
+static long writePaths(PotraceTracingEngine *engine, potrace_path_t *plist,
            Inkscape::SVG::PathString& data, std::vector<Point> &points)
 {
     long nodeCount = 0L;
@@ -195,11 +203,7 @@ writePaths(PotraceTracingEngine *engine, potrace_path_t *plist,
 }
 
 
-
-
-
-static GrayMap *
-filter(PotraceTracingEngine &engine, GdkPixbuf * pixbuf)
+static GrayMap *filter(PotraceTracingEngine &engine, GdkPixbuf * pixbuf)
 {
     if (!pixbuf)
         return NULL;
@@ -273,8 +277,7 @@ filter(PotraceTracingEngine &engine, GdkPixbuf * pixbuf)
 }
 
 
-static IndexedMap *
-filterIndexed(PotraceTracingEngine &engine, GdkPixbuf * pixbuf)
+static IndexedMap *filterIndexed(PotraceTracingEngine &engine, GdkPixbuf * pixbuf)
 {
     if (!pixbuf)
         return NULL;
@@ -312,7 +315,7 @@ filterIndexed(PotraceTracingEngine &engine, GdkPixbuf * pixbuf)
 
 
 
-Glib::RefPtr<Gdk::Pixbuf>
+Glib::RefPtr<Gdk::Pixbuf> 
 PotraceTracingEngine::preview(Glib::RefPtr<Gdk::Pixbuf> thePixbuf)
 {
     GdkPixbuf *pixbuf = thePixbuf->gobj();
@@ -348,9 +351,14 @@ PotraceTracingEngine::preview(Glib::RefPtr<Gdk::Pixbuf> thePixbuf)
 
 
 //*This is the core inkscape-to-potrace binding
-std::string
-PotraceTracingEngine::grayMapToPath(GrayMap *grayMap, long *nodeCount)
+std::string PotraceTracingEngine::grayMapToPath(GrayMap *grayMap, long *nodeCount)
 {
+    if (!keepGoing)
+    {
+        g_warning("aborted");
+        return "";
+    }
+
     potrace_bitmap_t *potraceBitmap = bm_new(grayMap->width, grayMap->height);
     bm_clear(potraceBitmap, 0);
 
@@ -371,12 +379,6 @@ PotraceTracingEngine::grayMapToPath(GrayMap *grayMap, long *nodeCount)
     fclose(f);
     */
 
-    if (!keepGoing)
-        {
-        g_warning("aborted");
-        return "";
-        }
-
     /* trace a bitmap*/
     potrace_state_t *potraceState = potrace_trace(potraceParams,
                                                   potraceBitmap);
@@ -414,8 +416,7 @@ PotraceTracingEngine::grayMapToPath(GrayMap *grayMap, long *nodeCount)
 /**
  *  This is called for a single scan
  */
-std::vector<TracingEngineResult>
-PotraceTracingEngine::traceSingle(GdkPixbuf * thePixbuf)
+std::vector<TracingEngineResult> PotraceTracingEngine::traceSingle(GdkPixbuf * thePixbuf)
 {
 
     std::vector<TracingEngineResult> results;
@@ -445,11 +446,10 @@ PotraceTracingEngine::traceSingle(GdkPixbuf * thePixbuf)
 
 
 /**
- *  This allow routines that already generate GrayMaps to skip image filtering,
+ *  This allows routines that already generate GrayMaps to skip image filtering,
  *  increasing performance.
  */
-std::vector<TracingEngineResult>
-PotraceTracingEngine::traceGrayMap(GrayMap *grayMap)
+std::vector<TracingEngineResult> PotraceTracingEngine::traceGrayMap(GrayMap *grayMap)
 {
 
     std::vector<TracingEngineResult> results;
@@ -471,67 +471,56 @@ PotraceTracingEngine::traceGrayMap(GrayMap *grayMap)
 /**
  *  Called for multiple-scanning algorithms
  */
-std::vector<TracingEngineResult>
-PotraceTracingEngine::traceBrightnessMulti(GdkPixbuf * thePixbuf)
+std::vector<TracingEngineResult> PotraceTracingEngine::traceBrightnessMulti(GdkPixbuf * thePixbuf)
 {
-
     std::vector<TracingEngineResult> results;
 
-    if (!thePixbuf)
-        return results;
+    if ( thePixbuf ) {
+        double low     = 0.2; //bottom of range
+        double high    = 0.9; //top of range
+        double delta   = (high - low ) / ((double)multiScanNrColors);
 
-    double low     = 0.2; //bottom of range
-    double high    = 0.9; //top of range
-    double delta   = (high - low ) / ((double)multiScanNrColors);
+        brightnessFloor = 0.0; //Set bottom to black
 
-    brightnessFloor = 0.0; //Set bottom to black
+        int traceCount = 0;
 
-    int traceCount = 0;
+        for ( brightnessThreshold = low ;
+              brightnessThreshold <= high ;
+              brightnessThreshold += delta) {
+            GrayMap *grayMap = filter(*this, thePixbuf);
+            if ( grayMap ) {
+                long nodeCount;
+                std::string d = grayMapToPath(grayMap, &nodeCount);
 
-    for ( brightnessThreshold = low ;
-          brightnessThreshold <= high ;
-          brightnessThreshold += delta)
+                grayMap->destroy(grayMap);
 
-        {
+                if ( !d.empty() ) {
+                    //### get style info
+                    int grayVal = (int)(256.0 * brightnessThreshold);
+                    ustring style = ustring::compose("fill-opacity:1.0;fill:%1%2%3", twohex(grayVal), twohex(grayVal), twohex(grayVal) );
 
-        GrayMap *grayMap = filter(*this, thePixbuf);
-        if (!grayMap)
-            return results;
+                    //g_message("### GOT '%s' \n", style.c_str());
+                    TracingEngineResult result(style, d, nodeCount);
+                    results.push_back(result);
 
-        long nodeCount;
-        std::string d = grayMapToPath(grayMap, &nodeCount);
+                    if (!multiScanStack) {
+                        brightnessFloor = brightnessThreshold;
+                    }
 
-        grayMap->destroy(grayMap);
-
-        if (d.size() == 0)
-            return results;
-
-        int grayVal = (int)(256.0 * brightnessThreshold);
-        char style[31];
-        sprintf(style, "fill-opacity:1.0;fill:#%02x%02x%02x",
-                    grayVal, grayVal, grayVal);
-
-        //g_message("### GOT '%s' \n", d);
-        TracingEngineResult result(style, d, nodeCount);
-        results.push_back(result);
-
-        if (!multiScanStack)
-            brightnessFloor = brightnessThreshold;
-
-        SPDesktop *desktop = SP_ACTIVE_DESKTOP;
-        if (desktop)
-            {
-            gchar *msg = g_strdup_printf(_("Trace: %d.  %ld nodes"), traceCount++, nodeCount);
-            sp_desktop_message_stack(desktop)->flash(Inkscape::NORMAL_MESSAGE, msg);
-            g_free(msg);
+                    SPDesktop *desktop = SP_ACTIVE_DESKTOP;
+                    if (desktop) {
+                        ustring msg = ustring::compose(_("Trace: %1.  %2 nodes"), traceCount++, nodeCount);
+                        sp_desktop_message_stack(desktop)->flash(Inkscape::NORMAL_MESSAGE, msg);
+                    }
+                }
             }
         }
 
-    //# Remove the bottom-most scan, if requested
-    if (results.size() > 1 && multiScanRemoveBackground)
-        {
-        results.erase(results.end() - 1);
+        //# Remove the bottom-most scan, if requested
+        if (results.size() > 1 && multiScanRemoveBackground) {
+            results.erase(results.end() - 1);
         }
+    }
 
     return results;
 }
@@ -540,78 +529,65 @@ PotraceTracingEngine::traceBrightnessMulti(GdkPixbuf * thePixbuf)
 /**
  *  Quantization
  */
-std::vector<TracingEngineResult>
-PotraceTracingEngine::traceQuant(GdkPixbuf * thePixbuf)
+std::vector<TracingEngineResult> PotraceTracingEngine::traceQuant(GdkPixbuf * thePixbuf)
 {
-
     std::vector<TracingEngineResult> results;
 
-    if (!thePixbuf)
-        return results;
-
-    IndexedMap *iMap = filterIndexed(*this, thePixbuf);
-    if (!iMap)
-        return results;
-
-    //Create and clear a gray map
-    GrayMap *gm = GrayMapCreate(iMap->width, iMap->height);
-    for (int row=0 ; row<gm->height ; row++)
-        for (int col=0 ; col<gm->width ; col++)
-            gm->setPixel(gm, col, row, GRAYMAP_WHITE);
-
-
-    for (int colorIndex=0 ; colorIndex<iMap->nrColors ; colorIndex++)
-        {
-
-        /*Make a gray map for each color index */
-        for (int row=0 ; row<iMap->height ; row++)
-            {
-            for (int col=0 ; col<iMap->width ; col++)
-                {
-                int indx = (int) iMap->getPixel(iMap, col, row);
-                if (indx == colorIndex)
-                    gm->setPixel(gm, col, row, GRAYMAP_BLACK); //black
-                else if (!multiScanStack)
-                    gm->setPixel(gm, col, row, GRAYMAP_WHITE); //white
+    if (thePixbuf) {
+        IndexedMap *iMap = filterIndexed(*this, thePixbuf);
+        if ( iMap ) {
+            //Create and clear a gray map
+            GrayMap *gm = GrayMapCreate(iMap->width, iMap->height);
+            for (int row=0 ; row<gm->height ; row++) {
+                for (int col=0 ; col<gm->width ; col++) {
+                    gm->setPixel(gm, col, row, GRAYMAP_WHITE);
                 }
             }
 
-        //## Now we have a traceable graymap
-        long nodeCount;
-        std::string d = grayMapToPath(gm, &nodeCount);
+            for (int colorIndex=0 ; colorIndex<iMap->nrColors ; colorIndex++) {
+                // Make a gray map for each color index
+                for (int row=0 ; row<iMap->height ; row++) {
+                    for (int col=0 ; col<iMap->width ; col++) {
+                        int indx = (int) iMap->getPixel(iMap, col, row);
+                        if (indx == colorIndex) {
+                            gm->setPixel(gm, col, row, GRAYMAP_BLACK); //black
+                        } else if (!multiScanStack) {
+                            gm->setPixel(gm, col, row, GRAYMAP_WHITE); //white
+                        }
+                    }
+                }
 
-        if (d.size() == 0)
-            return results;
+                //## Now we have a traceable graymap
+                long nodeCount;
+                std::string d = grayMapToPath(gm, &nodeCount);
 
-        //### get style info
-        char style[13];
-        RGB rgb = iMap->clut[colorIndex];
-        sprintf(style, "fill:#%02x%02x%02x", rgb.r, rgb.g, rgb.b);
+                if ( !d.empty() ) {
+                    //### get style info
+                    RGB rgb = iMap->clut[colorIndex];
+                    ustring style = ustring::compose("fill:#%1%2%3", twohex(rgb.r), twohex(rgb.g), twohex(rgb.b) );
 
-        //g_message("### GOT '%s' \n", d);
-        TracingEngineResult result(style, d, nodeCount);
-        results.push_back(result);
+                    //g_message("### GOT '%s' \n", style.c_str());
+                    TracingEngineResult result(style, d, nodeCount);
+                    results.push_back(result);
 
-        SPDesktop *desktop = SP_ACTIVE_DESKTOP;
-        if (desktop)
-            {
-            gchar *msg = g_strdup_printf(_("Trace: %d.  %ld nodes"), colorIndex, nodeCount);
-            sp_desktop_message_stack(desktop)->flash(Inkscape::NORMAL_MESSAGE, msg);
-            g_free(msg);
-            }
+                    SPDesktop *desktop = SP_ACTIVE_DESKTOP;
+                    if (desktop) {
+                        ustring msg = ustring::compose(_("Trace: %1.  %2 nodes"), colorIndex, nodeCount);
+                        sp_desktop_message_stack(desktop)->flash(Inkscape::NORMAL_MESSAGE, msg);
+                    }
+                }
+            }// for colorIndex
 
-
-        }// for colorIndex
-
-    gm->destroy(gm);
-    iMap->destroy(iMap);
-
-    //# Remove the bottom-most scan, if requested
-    if (results.size() > 1 && multiScanRemoveBackground)
-        {
-        results.erase(results.end() - 1);
+            gm->destroy(gm);
+            iMap->destroy(iMap);
         }
 
+        //# Remove the bottom-most scan, if requested
+        if (results.size() > 1 && multiScanRemoveBackground) {
+            results.erase(results.end() - 1);
+        }
+    }
+
     return results;
 }
 
@@ -647,14 +623,10 @@ PotraceTracingEngine::trace(Glib::RefPtr<Gdk::Pixbuf> pixbuf)
 }
 
 
-
-
-
 /**
  *  Abort the thread that is executing getPathDataFromPixbuf()
  */
-void
-PotraceTracingEngine::abort()
+void PotraceTracingEngine::abort()
 {
     //g_message("PotraceTracingEngine::abort()\n");
     keepGoing = 0;
@@ -667,3 +639,13 @@ PotraceTracingEngine::abort()
 }  // namespace Trace
 }  // namespace Inkscape
 
+/*
+  Local Variables:
+  mode:c++
+  c-file-style:"stroustrup"
+  c-file-offsets:((innamespace . 0)(inline-open . 0))
+  indent-tabs-mode:nil
+  fill-column:99
+  End:
+*/
+// vim: expandtab:shiftwidth=4:tabstop=8:softtabstop=4 :
