@@ -1,11 +1,13 @@
-/** @file
- * @brief Inkscape About box - implementation
+/**
+ * @file
+ * Inkscape About box - implementation.
  */
 /* Authors:
  *   Derek P. Moore <derekm@hackunix.org>
  *   MenTaLguY <mental@rydia.net>
  *   Kees Cook <kees@outflux.net>
  *   Jon Phillips <jon@rejon.org>
+ *   Abhishek Sharma
  *
  * Copyright (C) 2004 Derek P. Moore
  * Copyright 2004 Kees Cook
@@ -19,6 +21,7 @@
 # include <config.h>
 #endif
 
+#include "ui/dialog/aboutbox.h"
 #include <glibmm/i18n.h>
 #include <gtkmm/notebook.h>
 #include <gtkmm/scrolledwindow.h>
@@ -31,7 +34,7 @@
 #include "svg-view-widget.h"
 #include "sp-text.h"
 #include "text-editing.h"
-#include "ui/dialog/aboutbox.h"
+#include "util/units.h"
 
 #include "inkscape-version.h"
 
@@ -95,7 +98,12 @@ AboutBox::AboutBox() : Gtk::Dialog(_("About Inkscape")) {
     tabs->append_page(*manage(
         make_scrolled_text(license_text)), _("_License"), true);
 
+#if WITH_GTKMM_3_0
+    get_content_area()->pack_end(*manage(tabs), true, true);
+#else
     get_vbox()->pack_end(*manage(tabs), true, true);
+#endif
+
     tabs->show_all();
 
     add_button(Gtk::Stock::CLOSE, Gtk::RESPONSE_CLOSE);
@@ -106,16 +114,24 @@ AboutBox::AboutBox() : Gtk::Dialog(_("About Inkscape")) {
         g_strdup_printf("<small>Inkscape %s</small>",
               Inkscape::version_string);
     label->set_markup(label_text);
-    label->set_alignment(Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER);
+    label->set_alignment(Gtk::ALIGN_END, Gtk::ALIGN_CENTER);
     label->set_padding(5,0);
     g_free(label_text);
     label->set_selectable(true);
     label->show();
 
+#if WITH_GTKMM_3_0
+    get_content_area()->pack_start(*manage(label), false, false);
+#else
     get_vbox()->pack_start(*manage(label), false, false);
+#endif
 
     Gtk::Requisition requisition;
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_widget_get_preferred_size(reinterpret_cast<GtkWidget*>(gobj()), &requisition, NULL);
+#else
     gtk_widget_size_request (reinterpret_cast<GtkWidget*>(gobj()), &requisition);
+#endif
     // allow window to shrink
     set_size_request(0, 0);
     set_default_size(requisition.width, requisition.height);
@@ -148,7 +164,7 @@ Gtk::Widget *build_splash_widget() {
     // should be in UTF-*8..
 
     char *about=g_build_filename(INKSCAPE_SCREENSDIR, _("about.svg"), NULL);
-    SPDocument *doc=sp_document_new (about, TRUE);
+    SPDocument *doc=SPDocument::createNewDoc (about, TRUE);
     g_free(about);
     g_return_val_if_fail(doc != NULL, NULL);
 
@@ -156,16 +172,16 @@ Gtk::Widget *build_splash_widget() {
     if ( version && SP_IS_TEXT(version) ) {
         sp_te_set_repr_text_multiline (SP_TEXT (version), Inkscape::version_string);
     }
-    sp_document_ensure_up_to_date(doc);
+    doc->ensureUpToDate();
 
     GtkWidget *v=sp_svg_view_widget_new(doc);
 
-    double width=sp_document_width(doc);
-    double height=sp_document_height(doc);
+    double width=doc->getWidth().value("px");
+    double height=doc->getHeight().value("px");
     
-    sp_document_unref(doc);
+    doc->doUnref();
 
-    sp_svg_view_widget_set_resize(SP_SVG_VIEW_WIDGET(v), FALSE, (int)width, (int)height);
+    SP_SVG_VIEW_WIDGET(v)->setResize(false, static_cast<int>(width), static_cast<int>(height));
 
     Gtk::AspectFrame *frame=new Gtk::AspectFrame();
     frame->unset_label();
@@ -225,156 +241,178 @@ void AboutBox::initStrings() {
      * and paste the result from the combo box here.
      */ 
     authors_text =
-    "Maximilian Albert\n"
-    "Josh Andler\n"
-    "Tavmjong Bah\n"
-    "Pierre Barbry-Blot\n"
-    "Jean-François Barraud\n"
-    "Bill Baxter\n"
-    "John Beard\n"
-    "John Bintz\n"
-    "Arpad Biro\n"
-    "Nicholas Bishop\n"
-    "Joshua L. Blocher\n"
-    "Hanno Böck\n"
-    "Henrik Bohre\n"
-    "Boldewyn\n"
-    "Daniel Borgmann\n"
-    "Bastien Bouclet\n"
-    "Hans Breuer\n"
-    "Gustav Broberg\n"
-    "Christopher Brown\n"
-    "Marcus Brubaker\n"
-    "Luca Bruno\n"
-    "Nicu Buculei\n"
-    "Bulia Byak\n"
-    "Pierre Caclin\n"
-    "Ian Caldwell\n"
-    "Gail Carmichael\n"
-    "Ed Catmur\n"
-    "Chema Celorio\n"
-    "Johan Ceuppens\n"
-    "Zbigniew Chyla\n"
-    "Alexander Clausen\n"
-    "John Cliff\n"
-    "Kees Cook\n"
-    "Ben Cromwell\n"
-    "Robert Crosbie\n"
-    "Jon Cruz\n"
-    "Aurélie De-Cooman\n"
-    "Milosz Derezynski\n"
-    "Daniel Díaz\n"
-    "Bruno Dilly\n"
-    "Larry Doolittle\n"
-    "Tim Dwyer\n"
-    "Maxim V. Dziumanenko\n"
-    "Johan Engelen\n"
-    "Miklos Erdelyi\n"
-    "Ulf Erikson\n"
-    "Noé Falzon\n"
-    "Frank Felfe\n"
-    "Andrew Fitzsimon\n"
-    "Edward Flick\n"
-    "Marcin Floryan\n"
-    "Ben Fowler\n"
-    "Fred\n"
-    "Cedric Gemy\n"
-    "Steren Giannini\n"
-    "Olivier Gondouin\n"
-    "Ted Gould\n"
-    "Toine de Greef\n"
-    "Michael Grosberg\n"
-    "Bryce Harrington\n"
-    "Dale Harvey\n"
-    "Aurélio A. Heckert\n"
-    "Carl Hetherington\n"
-    "Jos Hirth\n"
-    "Hannes Hochreiner\n"
-    "Thomas Holder\n"
-    "Joel Holdsworth\n"
-    "Alan Horkan\n"
-    "Karl Ove Hufthammer\n"
-    "Richard Hughes\n"
-    "Nathan Hurst\n"
-    "inductiveload\n"
-    "Thomas Ingham\n"
-    "Jean-Olivier Irisson\n"
-    "Bob Jamison\n"
-    "jEsuSdA\n"
-    "Lauris Kaplinski\n"
-    "Lynn Kerby\n"
-    "Niko Kiirala\n"
-    "James Kilfiger\n"
-    "Jason Kivlighn\n"
-    "Adrian Knoth\n"
-    "Krzysztof Kosiński\n"
-    "Petr Kovar\n"
-    "Benoît Lavorata\n"
-    "Alex Leone\n"
-    "Julien Leray\n"
-    "Raph Levien\n"
-    "Diederik van Lierop\n"
-    "Nicklas Lindgren\n"
-    "Vitaly Lipatov\n"
-    "Ivan Louette\n"
-    "Pierre-Antoine Marc\n"
-    "Aurel-Aimé Marmion\n"
-    "Colin Marquardt\n"
-    "Dmitry G. Mastrukov\n"
-    "Matiphas\n"
-    "Michael Meeks\n"
-    "Federico Mena\n"
-    "MenTaLguY\n"
-    "Aubanel Monnier\n"
-    "Vincent Montagne\n"
-    "Tim Mooney\n"
-    "Derek P. Moore\n"
-    "Peter Moulder\n"
-    "Jörg Müller\n"
-    "Yukihiro Nakai\n"
-    "Victor Navez\n"
-    "Christian Neumair\n"
-    "Andreas Nilsson\n"
-    "Mitsuru Oka\n"
-    "Marten Owens\n"
-    "Alvin Penner\n"
-    "Jon Phillips\n"
-    "Zdenko Podobny\n"
-    "Alexandre Prokoudine\n"
-    "Jean-René Reinhard\n"
-    "Alexey Remizov\n"
-    "Frederic Rodrigo\n"
-    "Hugo Rodrigues\n"
-    "Juarez Rudsatz\n"
-    "Xavier Conde Rueda\n"
-    "Felipe Corrêa da Silva Sanches\n"
-    "Christian Schaller\n"
-    "Marco Scholten\n"
-    "Tom von Schwerdtner\n"
-    "Danilo Šegan\n"
-    "Shivaken\n"
-    "Michael Sloan\n"
-    "John Smith\n"
-    "Boštjan Špetič\n"
-    "Aaron Spike\n"
-    "Kaushik Sridharan\n"
-    "Ralf Stephan\n"
-    "Dariusz Stojek\n"
-    "~suv\n"
-    "Martin Sucha\n"
-    "Pat Suwalski\n"
-    "Adib Taraben\n"
-    "Hugh Tebby\n"
-    "Jonas Termeau\n"
-    "David Turner\n"
-    "Andre Twupack\n"
-    "Aleksandar Urošević\n"
-    "Alex Valavanis\n"
-    "Lucas Vieites\n"
-    "Michael Wybrow\n"
-    "Daniel Yacob\n"
-    "Masatake Yamato\n"
-    "David Yip";
+"Maximilian Albert\n"
+"Joshua A. Andler\n"
+"Tavmjong Bah\n"
+"Pierre Barbry-Blot\n"
+"Jean-François Barraud\n"
+"Campbell Barton\n"
+"Bill Baxter\n"
+"John Beard\n"
+"John Bintz\n"
+"Arpad Biro\n"
+"Nicholas Bishop\n"
+"Joshua L. Blocher\n"
+"Hanno Böck\n"
+"Tomasz Boczkowski\n"
+"Henrik Bohre\n"
+"Boldewyn\n"
+"Daniel Borgmann\n"
+"Bastien Bouclet\n"
+"Hans Breuer\n"
+"Gustav Broberg\n"
+"Christopher Brown\n"
+"Marcus Brubaker\n"
+"Luca Bruno\n"
+"Nicu Buculei\n"
+"Bulia Byak\n"
+"Pierre Caclin\n"
+"Ian Caldwell\n"
+"Gail Carmichael\n"
+"Ed Catmur\n"
+"Chema Celorio\n"
+"Jabiertxo Arraiza Cenoz\n"
+"Johan Ceuppens\n"
+"Zbigniew Chyla\n"
+"Alexander Clausen\n"
+"John Cliff\n"
+"Kees Cook\n"
+"Ben Cromwell\n"
+"Robert Crosbie\n"
+"Jon Cruz\n"
+"Aurélie De-Cooman\n"
+"Milosz Derezynski\n"
+"Daniel Díaz\n"
+"Bruno Dilly\n"
+"Larry Doolittle\n"
+"Nicolas Dufour\n"
+"Tim Dwyer\n"
+"Maxim V. Dziumanenko\n"
+"Johan Engelen\n"
+"Miklos Erdelyi\n"
+"Ulf Erikson\n"
+"Noé Falzon\n"
+"Frank Felfe\n"
+"Andrew Fitzsimon\n"
+"Edward Flick\n"
+"Marcin Floryan\n"
+"Ben Fowler\n"
+"Fred\n"
+"Cedric Gemy\n"
+"Steren Giannini\n"
+"Olivier Gondouin\n"
+"Ted Gould\n"
+"Toine de Greef\n"
+"Michael Grosberg\n"
+"Kris De Gussem\n"
+"Bryce Harrington\n"
+"Dale Harvey\n"
+"Aurélio Adnauer Heckert\n"
+"Carl Hetherington\n"
+"Jos Hirth\n"
+"Hannes Hochreiner\n"
+"Thomas Holder\n"
+"Joel Holdsworth\n"
+"Christoffer Holmstedt\n"
+"Alan Horkan\n"
+"Karl Ove Hufthammer\n"
+"Richard Hughes\n"
+"Nathan Hurst\n"
+"inductiveload\n"
+"Thomas Ingham\n"
+"Jean-Olivier Irisson\n"
+"Bob Jamison\n"
+"Ted Janeczko\n"
+"jEsuSdA\n"
+"Fernando Lucchesi Bastos Jurema\n"
+"Lauris Kaplinski\n"
+"Lynn Kerby\n"
+"Niko Kiirala\n"
+"James Kilfiger\n"
+"Nikita Kitaev\n"
+"Jason Kivlighn\n"
+"Adrian Knoth\n"
+"Krzysztof Kosiński\n"
+"Petr Kovar\n"
+"Benoît Lavorata\n"
+"Alex Leone\n"
+"Julien Leray\n"
+"Raph Levien\n"
+"Diederik van Lierop\n"
+"Nicklas Lindgren\n"
+"Vitaly Lipatov\n"
+"Ivan Louette\n"
+"Pierre-Antoine Marc\n"
+"Aurel-Aimé Marmion\n"
+"Colin Marquardt\n"
+"Craig Marshall\n"
+"Ivan Masár\n"
+"Dmitry G. Mastrukov\n"
+"David Mathog\n"
+"Matiphas\n"
+"Michael Meeks\n"
+"Federico Mena\n"
+"MenTaLguY\n"
+"Aubanel Monnier\n"
+"Vincent Montagne\n"
+"Tim Mooney\n"
+"Derek P. Moore\n"
+"Chris Morgan\n"
+"Peter Moulder\n"
+"Jörg Müller\n"
+"Yukihiro Nakai\n"
+"Victor Navez\n"
+"Christian Neumair\n"
+"Nick\n"
+"Andreas Nilsson\n"
+"Mitsuru Oka\n"
+"Vinícius dos Santos Oliveira\n"
+"Martin Owens\n"
+"Alvin Penner\n"
+"Matthew Petroff\n"
+"Jon Phillips\n"
+"Zdenko Podobny\n"
+"Alexandre Prokoudine\n"
+"Jean-René Reinhard\n"
+"Alexey Remizov\n"
+"Frederic Rodrigo\n"
+"Hugo Rodrigues\n"
+"Juarez Rudsatz\n"
+"Xavier Conde Rueda\n"
+"Felipe Corrêa da Silva Sanches\n"
+"Christian Schaller\n"
+"Marco Scholten\n"
+"Tom von Schwerdtner\n"
+"Danilo Šegan\n"
+"Abhishek Sharma\n"
+"Shivaken\n"
+"Michael Sloan\n"
+"John Smith\n"
+"Boštjan Špetič\n"
+"Aaron Spike\n"
+"Kaushik Sridharan\n"
+"Ralf Stephan\n"
+"Dariusz Stojek\n"
+"Martin Sucha\n"
+"~suv\n"
+"Pat Suwalski\n"
+"Adib Taraben\n"
+"Hugh Tebby\n"
+"Jonas Termeau\n"
+"David Turner\n"
+"Andre Twupack\n"
+"Aleksandar Urošević\n"
+"Alex Valavanis\n"
+"Joakim Verona\n"
+"Lucas Vieites\n"
+"Daniel Wagenaar\n"
+"Liam P. White\n"
+"Sebastian Wüst\n"
+"Michael Wybrow\n"
+"Gellule Xg\n"
+"Daniel Yacob\n"
+"Masatake Yamato\n"
+"David Yip";
 
     //##############################
     //# T R A N S L A T O R S
@@ -413,133 +451,147 @@ void AboutBox::initStrings() {
      * and paste the result from the combo box here.
      */          
     gchar const *allTranslators =
-    "3ARRANO.com <3arrano@3arrano.com>, 2005.\n"
-    "Adib Taraben <theadib@googlemail.com>, 2004.\n"
-    "Alan Monfort <alan.monfort@free.fr>, 2009-2010.\n"
-    "Alastair McKinstry <mckinstry@computer.org>, 2000.\n"
-    "Aleksandar Urošević <urke@users.sourceforge.net>, 2004, 2005, 2006.\n"
-    "Alessio Frusciante <algol@firenze.linux.it>, 2002, 2003.\n"
-    "Alexander Shopov <ash@contact.bg>, 2006.\n"
-    "Alexandre Prokoudine <alexandre.prokoudine@gmail.com>, 2005.\n"
-    "Alexey Remizov <alexey@remizov.pp.ru>, 2004.\n"
-    "Ali Ghanavatian <ghanvatian.ali@gmail.com>, 2010.\n"
-    "Álvaro Lopes <alvieboy@alvie.com>, 2001, 2002.\n"
-    "Andreas Hyden <a.hyden@cyberpoint.se>, 2000.\n"
-    "Andrius Ramanauskas <knutux@gmail.com>, 2006.\n"
-    "Antonio Codazzi <f_sophia@libero.it>, 2006, 2007.\n"
-    "Antônio Cláudio (LedStyle) <ledstyle@gmail.com>, 2006.\n"
-    "Amanpreet Singh Brar Alamwalia <apbrar@gmail.com>, 2005.\n"
-    "Arman Aksoy <armish@linux-sevenler.de>, 2003.\n"
-    "Arpad Biro <biro_arpad@yahoo.com>, 2004, 2005.\n"
-    "Benedikt Roth <Benedikt.Roth@gmx.net>, 2000.\n"
-    "Benno Schulenberg <benno@vertaalt.nl>,  2008.\n"
-    "Boštjan Špetič <igzebedze@cyberpipe.org>, 2004, 2005.\n"
-    "Brisa Francesco <fbrisa@yahoo.it>, 2000.\n"
-    "bulia byak <buliabyak@users.sf.net>, 2004.\n"
-    "Chris jia <Chrisjiasl@gmail.com>, 2006.\n"
-    "Christian Meyer <chrisime@gnome.org>, 2000-2002.\n"
-    "Christian Neumair <chris@gnome-de.org>, 2002, 2003.\n"
-    "Christian Rose <menthos@menthos.com>, 2000, 2001, 2002, 2003.\n"
-    "Cristian Secară <cristi@secarica.ro>, 2010.\n"
-    "Christophe Merlet (RedFox) <redfox@redfoxcenter.org>, 2000-2002.\n"
-    "Clytie Siddall <clytie@riverland.net.au>, 2004-2008.\n"
-    "Colin Marquardt <colin@marquardt-home.de>, 2004-2006.\n"
-    "Cédric Gemy <radar.map35@free.fr>, 2006.\n"
-    "Daniel Díaz <yosoy@danieldiaz.org>, 2004.\n"
-    "Didier Conchaudron <conchaudron@free.fr>, 2003.\n"
-    "Dorji Tashi <dorjee_doss@hotmail.com>, 2006.\n"
-    "Duarte Loreto <happyguy_pt@hotmail.com> 2002, 2003 (Maintainer).\n"
-    "Equipe de Tradução Inkscape Brasil <www.inkscapebrasil.org>, 2007.\n"
-    "Fatih Demir <kabalak@gtranslator.org>, 2000.\n"
-    "Foppe Benedictus <foppe.benedictus@gmail.com>, 2007-2009.\n"
-    "Francesc Dorca <f.dorca@filnet.es>, 2003. Traducció sodipodi.\n"
-    "Francisco Javier F. Serrador <serrador@arrakis.es>, 2003.\n"
-    "Francisco Xosé Vázquez Grandal <fxvazquez@arrakis.es>, 2001.\n"
-    "Frederic Rodrigo <f.rodrigo free.fr>, 2004-2005.\n"
-    "Ge'ez Frontier Foundation <locales@geez.org>, 2002.\n"
-    "Hleb Valoshka <375gnu@gmail.com>, 2008-2009.\n"
-    "Hizkuntza Politikarako Sailburuordetza <hizkpol@ej-gv.es>, 2005.\n"
-    "Ilia Penev <lichopicho@gmail.com>, 2006.\n"
-    "Ivan Masár <helix84@centrum.sk>, 2006, 2007, 2008, 2009, 2010. \n"
-    "Iñaki Larrañaga <dooteo@euskalgnu.org>, 2006.\n"
-    "Jeffrey Steve Borbón Sanabria <jeff_kerokid@yahoo.com>, 2005.\n"
-    "Joaquim Perez i Noguer <noguer@gmail.com>, 2008-2009.\n"
-    "Jörg Müller <jfm@ram-brand.de>, 2005.\n"
-    "Jeroen van der Vegt <jvdvegt@gmail.com>, 2003, 2005, 2008.\n"
-    "Jin-Hwan Jeong <yongdoria@gmail.com>, 2009.\n"
-    "Jonathan Ernst <jernst@users.sourceforge.net>, 2006.\n"
-    "Jose Antonio Salgueiro Aquino <developer@telefonica.net>, 2003.\n"
-    "Josef Vybiral <josef.vybiral@gmail.com>, 2005-2006.\n"
-    "Juarez Rudsatz <juarez@correio.com>, 2004.\n"
-    "Junichi Uekawa <dancer@debian.org>, 2002.\n"
-    "Jurmey Rabgay <jur_gay@yahoo.com>, 2006.\n"
-    "Kai Lahmann <kailahmann@01019freenet.de>, 2000.\n"
-    "Karl Ove Hufthammer <karl@huftis.org>, 2004, 2005.\n"
-    "KATSURAGAWA Naoki <naopon@private.email.ne.jp>, 2006.\n"
-    "Keld Simonsen <keld@dkuug.dk>, 2000, 2001.\n"
-    "Kenji Inoue <kenz@oct.zaq.ne.jp>, 2006-2007.\n"
-    "Khandakar Mujahidul Islam <suzan@bengalinux.org>, 2006.\n"
-    "Kitae <bluetux@gmail.com>, 2006.\n"
-    "Kjartan Maraas <kmaraas@gnome.org>, 2000-2002.\n"
-    "Kris De Gussem <kris.DeGussem@gmail.com>, 2008-2010.\n"
-    "Lauris Kaplinski <lauris@ariman.ee>, 2000.\n"
-    "Leandro Regueiro <leandro.regueiro@gmail.com>, 2006-2008,2010.\n"
-    "Liu Xiaoqin <liuxqsmile@gmail.com>, 2008.\n"
-    "Luca Bruno <luca.br@uno.it>, 2005.\n"
-    "Lucas Vieites Fariña<lucas@codexion.com>, 2003-2010.\n"
-    "Mahesh subedi <submanesh@hotmail.com>, 2006.\n"
-    "Martin Srebotnjak, <miles@filmsi.net>, 2005, 2010.\n"
-    "Masatake YAMATO <jet@gyve.org>, 2002.\n"
-    "Masato Hashimoto <cabezon.hashimoto@gmail.com>, 2009-2010.\n"
-    "Matiphas <matiphas _a_ free _point_ fr>, 2004-2006.\n"
-    "Mattias Hultgren <mattias_hultgren@tele2.se>, 2005.\n"
-    "Maxim Dziumanenko <mvd@mylinux.com.ua>, 2004.\n"
-    "Mətin Əmirov <metin@karegen.com>, 2003.\n"
-    "Mitsuru Oka <oka326@parkcity.ne.jp>, 2002.\n"
-    "Morphix User <pchitrakar@gmail.com>, 2006.\n"
-    "Mufit Eribol <meribol@ere.com.tr>, 2000.\n"
-    "Muhammad Bashir Al-Noimi <mhdbnoimi@gmail.com>, 2008.\n"
-    "Myckel Habets <myckel@sdf.lonestar.org>, 2008.\n"
-    "Nguyen Dinh Trung <nguyendinhtrung141@gmail.com>, 2007, 2008.\n"
-    "Nicolas Dufour <nicoduf@yahoo.fr>, 2008-2010.\n"
-    "Pawan Chitrakar <pchitrakar@gmail.com>, 2006.\n"
-    "Przemysław Loesch <p_loesch@poczta.onet.pl>, 2005.\n"
-    "Quico Llach <quico@softcatala.org>, 2000. Traducció sodipodi.\n"
-    "Raymond Ostertag <raymond@linuxgraphic.org>, 2002, 2003.\n"
-    "Riku Leino <tsoots@gmail.com>, 2006.\n"
-    "Rune Rønde Laursen <runerl@skjoldhoej.dk>, 2006.\n"
-    "Ruud Steltenpool <svg@steltenpower.com>, 2006.\n"
-    "Serdar Soytetir <sendirom@gmail.com>, 2005.\n"
-    "shivaken <shivaken@owls-nest.net>, 2004.\n"
-    "Shyam Krishna Bal <shyamkrishna_bal@yahoo.com>, 2006.\n"
-    "Simos Xenitellis <simos@hellug.gr>, 2001.\n"
-    "Spyros Blanas <cid_e@users.sourceforge.net>, 2006.\n"
-    "Stefan Graubner <pflaumenmus92@gmx.net>, 2005.\n"
-    "Supranee Thirawatthanasuk <supranee@opentle.org>, 2006.\n"
-    "Takeshi Aihana <aihana@muc.biglobe.ne.jp>, 2000, 2001.\n"
-    "Tim Sheridan <tim.sheridan@gmail.com>, 2007-2010.\n"
-    "Theppitak Karoonboonyanan <thep@linux.thai.net>, 2006.\n"
-    "Thiago Pimentel <thiago.merces@gmail.com>, 2006.\n"
-    "Toshifumi Sato <sato@centrosystem.com>, 2005.\n"
-    "Jon South <striker@lunar-linux.org>, 2006. \n"
-    "Uwe Schöler <oss@oss-marketplace.com>, 2006.\n"
-    "Valek Filippov <frob@df.ru>, 2000, 2003.\n"
-    "Victor Dachev <vdachev@gmail.com>, 2006.\n"
-    "Vincent van Adrighem <V.vanAdrighem@dirck.mine.nu>, 2003.\n"
-    "Vital Khilko <dojlid@mova.org>, 2003.\n"
-    "Vitaly Lipatov <lav@altlinux.ru>, 2002, 2004.\n"
-    "vonHalenbach <vonHalenbach@users.sourceforge.net>, 2005.\n"
-    "Wang Li <charlesw1234@163.com>, 2002.\n"
-    "Wei-Lun Chao <william.chao@ossii.com.tw>, 2006.\n"
-    "Wolfram Strempfer <wolfram@strempfer.de>, 2006.\n"
-    "Xavier Conde Rueda <xavi.conde@gmail.com>, 2004-2008.\n"
-    "Yaron Shahrabani <sh.yaron@gmail.com>, 2009.\n"
-    "Yukihiro Nakai <nakai@gnome.gr.jp>, 2000, 2003.\n"
-    "Yuri Beznos <zhiz0id@gmail.com>, 2006.\n"
-    "Yuri Chornoivan <yurchor@ukr.net>, 2007-2010.\n"
-    "Yuri Syrota <rasta@renome.rovno.ua>, 2000.\n"
-    "Yves Guillou <yvesguillou@users.sourceforge.net>, 2004.\n"
-    "Zdenko Podobný <zdpo@mailbox.sk>, 2003, 2004."
+"3ARRANO.com <3arrano@3arrano.com>, 2005.\n"
+"Adib Taraben <theadib@gmail.com>, 2004-2014.\n"
+"Alan Monfort <alan.monfort@free.fr>, 2009-2010.\n"
+"Alastair McKinstry <mckinstry@computer.org>, 2000.\n"
+"Aleksandar Urošević <urke@users.sourceforge.net>, 2004-2006.\n"
+"Alessio Frusciante <algol@firenze.linux.it>, 2002, 2003.\n"
+"Alexander Shopov <ash@contact.bg>, 2006.\n"
+"Alexandre Prokoudine <alexandre.prokoudine@gmail.com>, 2005, 2010-2014.\n"
+"Alexey Remizov <alexey@remizov.pp.ru>, 2004.\n"
+"Ali Ghanavatian <ghanvatian.ali@gmail.com>, 2010.\n"
+"Álvaro Lopes <alvieboy@alvie.com>, 2001, 2002.\n"
+"Andreas Hyden <a.hyden@cyberpoint.se>, 2000.\n"
+"Andrius Ramanauskas <knutux@gmail.com>, 2006.\n"
+"Antonio Codazzi <f_sophia@libero.it>, 2006, 2007.\n"
+"Antônio Cláudio (LedStyle) <ledstyle@gmail.com>, 2006.\n"
+"Amanpreet Singh Brar Alamwalia <apbrar@gmail.com>, 2005.\n"
+"Arman Aksoy <armish@linux-sevenler.de>, 2003.\n"
+"Arpad Biro <biro_arpad@yahoo.com>, 2004, 2005.\n"
+"Benedikt Roth <Benedikt.Roth@gmx.net>, 2000.\n"
+"Benjamin Weis <benjamin.weis@gmx.com>, 2014.\n"
+"Benno Schulenberg <benno@vertaalt.nl>,  2008.\n"
+"Boštjan Špetič <igzebedze@cyberpipe.org>, 2004, 2005.\n"
+"Brisa Francesco <fbrisa@yahoo.it>, 2000.\n"
+"Bruce Cowan <bruce@bcowan.me.uk>, 2010.\n"
+"bulia byak <buliabyak@users.sf.net>, 2004.\n"
+"Chris jia <Chrisjiasl@gmail.com>, 2006.\n"
+"Christian Meyer <chrisime@gnome.org>, 2000-2002.\n"
+"Christian Neumair <chris@gnome-de.org>, 2002, 2003.\n"
+"Christian Rose <menthos@menthos.com>, 2000-2003.\n"
+"Cristian Secară <cristi@secarica.ro>, 2010-2013.\n"
+"Christophe Merlet (RedFox) <redfox@redfoxcenter.org>, 2000-2002.\n"
+"Clytie Siddall <clytie@riverland.net.au>, 2004-2008.\n"
+"Colin Marquardt <colin@marquardt-home.de>, 2004-2006.\n"
+"Cédric Gemy <radar.map35@free.fr>, 2006.\n"
+"Daniel Díaz <yosoy@danieldiaz.org>, 2004.\n"
+"Didier Conchaudron <conchaudron@free.fr>, 2003.\n"
+"Dimitris Spingos <dmtrs32@gmail.com>, 2011-2013.\n"
+"Dorji Tashi <dorjee_doss@hotmail.com>, 2006.\n"
+"Duarte Loreto <happyguy_pt@hotmail.com> 2002, 2003 (Maintainer).\n"
+"Elias Norberg <elno0959 at student.su.se>, 2009.\n"
+"Equipe de Tradução Inkscape Brasil <www.inkscapebrasil.org>, 2007.\n"
+"Fatih Demir <kabalak@gtranslator.org>, 2000.\n"
+"Firas Hanife <FirasHanife@gmail.com>, 2014-2015.\n"
+"Foppe Benedictus <foppe.benedictus@gmail.com>, 2007-2009.\n"
+"Francesc Dorca <f.dorca@filnet.es>, 2003. Traducció sodipodi.\n"
+"Francisco Javier F. Serrador <serrador@arrakis.es>, 2003.\n"
+"Francisco Xosé Vázquez Grandal <fxvazquez@arrakis.es>, 2001.\n"
+"Frederic Rodrigo <f.rodrigo free.fr>, 2004-2005.\n"
+"Ge'ez Frontier Foundation <locales@geez.org>, 2002.\n"
+"George Boukeas <boukeas@gmail.com>, 2011.\n"
+"Heiko Wöhrle <mail@heikowoehrle.de>, 2014.\n"
+"Hleb Valoshka <375gnu@gmail.com>, 2008-2009.\n"
+"Hizkuntza Politikarako Sailburuordetza <hizkpol@ej-gv.es>, 2005.\n"
+"Ilia Penev <lichopicho@gmail.com>, 2006.\n"
+"Ivan Masár <helix84@centrum.sk>, 2006-2014. \n"
+"Ivan Řihošek <irihosek@seznam.cz>, 2014.\n"
+"Iñaki Larrañaga <dooteo@euskalgnu.org>, 2006.\n"
+"Jānis Eisaks <jancs@dv.lv>, 2012-2014.\n"
+"Jeffrey Steve Borbón Sanabria <jeff_kerokid@yahoo.com>, 2005.\n"
+"Jesper Öqvist <jesper@llbit.se>, 2010, 2011.\n"
+"Joaquim Perez i Noguer <noguer@gmail.com>, 2008-2009.\n"
+"Jörg Müller <jfm@ram-brand.de>, 2005.\n"
+"Jeroen van der Vegt <jvdvegt@gmail.com>, 2003, 2005, 2008.\n"
+"Jin-Hwan Jeong <yongdoria@gmail.com>, 2009.\n"
+"Jonathan Ernst <jernst@users.sourceforge.net>, 2006.\n"
+"Jose Antonio Salgueiro Aquino <developer@telefonica.net>, 2003.\n"
+"Josef Vybiral <josef.vybiral@gmail.com>, 2005-2006.\n"
+"Juarez Rudsatz <juarez@correio.com>, 2004.\n"
+"Junichi Uekawa <dancer@debian.org>, 2002.\n"
+"Jurmey Rabgay <jur_gay@yahoo.com>, 2006.\n"
+"Kai Lahmann <kailahmann@01019freenet.de>, 2000.\n"
+"Karl Ove Hufthammer <karl@huftis.org>, 2004, 2005.\n"
+"KATSURAGAWA Naoki <naopon@private.email.ne.jp>, 2006.\n"
+"Keld Simonsen <keld@dkuug.dk>, 2000, 2001.\n"
+"Kenji Inoue <kenz@oct.zaq.ne.jp>, 2006-2007.\n"
+"Khandakar Mujahidul Islam <suzan@bengalinux.org>, 2006.\n"
+"Kingsley Turner <kingsley@maddogsbreakfast.com.au>, 2006.\n"
+"Kitae <bluetux@gmail.com>, 2006.\n"
+"Kjartan Maraas <kmaraas@gnome.org>, 2000-2002.\n"
+"Kris De Gussem <Kris.DeGussem@gmail.com>, 2008-2014.\n"
+"Lauris Kaplinski <lauris@ariman.ee>, 2000.\n"
+"Leandro Regueiro <leandro.regueiro@gmail.com>, 2006-2008, 2010.\n"
+"Liu Xiaoqin <liuxqsmile@gmail.com>, 2008.\n"
+"Luca Bruno <luca.br@uno.it>, 2005.\n"
+"Lucas Vieites Fariña<lucas@codexion.com>, 2003-2013.\n"
+"Mahesh subedi <submanesh@hotmail.com>, 2006.\n"
+"Martin Srebotnjak, <miles@filmsi.net>, 2005, 2010.\n"
+"Masatake YAMATO <jet@gyve.org>, 2002.\n"
+"Masato Hashimoto <cabezon.hashimoto@gmail.com>, 2009-2014.\n"
+"Matiphas <matiphas _a_ free _point_ fr>, 2004-2006.\n"
+"Mattias Hultgren <mattias_hultgren@tele2.se>, 2005, 2006.\n"
+"Maxim Dziumanenko <mvd@mylinux.com.ua>, 2004.\n"
+"Mətin Əmirov <metin@karegen.com>, 2003.\n"
+"Mitsuru Oka <oka326@parkcity.ne.jp>, 2002.\n"
+"Morphix User <pchitrakar@gmail.com>, 2006.\n"
+"Mufit Eribol <meribol@ere.com.tr>, 2000.\n"
+"Muhammad Bashir Al-Noimi <mhdbnoimi@gmail.com>, 2008.\n"
+"Myckel Habets <myckel@sdf.lonestar.org>, 2008.\n"
+"Nguyen Dinh Trung <nguyendinhtrung141@gmail.com>, 2007, 2008.\n"
+"Nicolas Dufour <nicoduf@yahoo.fr>, 2008-2014.\n"
+"Pawan Chitrakar <pchitrakar@gmail.com>, 2006.\n"
+"Przemysław Loesch <p_loesch@poczta.onet.pl>, 2005.\n"
+"Quico Llach <quico@softcatala.org>, 2000. Traducció sodipodi.\n"
+"Raymond Ostertag <raymond@linuxgraphic.org>, 2002, 2003.\n"
+"Riku Leino <tsoots@gmail.com>, 2006-2011.\n"
+"Rune Rønde Laursen <runerl@skjoldhoej.dk>, 2006.\n"
+"Ruud Steltenpool <svg@steltenpower.com>, 2006.\n"
+"Serdar Soytetir <sendirom@gmail.com>, 2005.\n"
+"shivaken <shivaken@owls-nest.net>, 2004.\n"
+"Shyam Krishna Bal <shyamkrishna_bal@yahoo.com>, 2006.\n"
+"Simos Xenitellis <simos@hellug.gr>, 2001, 2011.\n"
+"Spyros Blanas <cid_e@users.sourceforge.net>, 2006, 2011.\n"
+"Stefan Graubner <pflaumenmus92@gmx.net>, 2005.\n"
+"Supranee Thirawatthanasuk <supranee@opentle.org>, 2006.\n"
+"Takeshi Aihana <aihana@muc.biglobe.ne.jp>, 2000, 2001.\n"
+"Tim Sheridan <tghs@tghs.net>, 2007-2014.\n"
+"Theppitak Karoonboonyanan <thep@linux.thai.net>, 2006.\n"
+"Thiago Pimentel <thiago.merces@gmail.com>, 2006.\n"
+"Toshifumi Sato <sato@centrosystem.com>, 2005.\n"
+"Jon South <striker@lunar-linux.org>, 2006. \n"
+"Uwe Schöler <oss@oss-marketplace.com>, 2006-2014.\n"
+"Valek Filippov <frob@df.ru>, 2000, 2003.\n"
+"Victor Dachev <vdachev@gmail.com>, 2006.\n"
+"Victor Westmann <victor.westmann@gmail.com>, 2011, 2014.\n"
+"Ville Pätsi, 2013.\n"
+"Vincent van Adrighem <V.vanAdrighem@dirck.mine.nu>, 2003.\n"
+"Vital Khilko <dojlid@mova.org>, 2003.\n"
+"Vitaly Lipatov <lav@altlinux.ru>, 2002, 2004.\n"
+"vonHalenbach <vonHalenbach@users.sourceforge.net>, 2005.\n"
+"Waluyo Adi Siswanto <was.uthm@gmail.com>, 2011.\n"
+"Wang Li <charlesw1234@163.com>, 2002.\n"
+"Wei-Lun Chao <william.chao@ossii.com.tw>, 2006.\n"
+"Wolfram Strempfer <wolfram@strempfer.de>, 2006.\n"
+"Xavier Conde Rueda <xavi.conde@gmail.com>, 2004-2008.\n"
+"Yaron Shahrabani <sh.yaron@gmail.com>, 2009.\n"
+"Yukihiro Nakai <nakai@gnome.gr.jp>, 2000, 2003.\n"
+"Yuri Beznos <zhiz0id@gmail.com>, 2006.\n"
+"Yuri Chornoivan <yurchor@ukr.net>, 2007-2014.\n"
+"Yuri Syrota <rasta@renome.rovno.ua>, 2000.\n"
+"Yves Guillou <yvesguillou@users.sourceforge.net>, 2004.\n"
+"Zdenko Podobný <zdpo@mailbox.sk>, 2003, 2004."
     ;
     
     translators_text.append(allTranslators);
