@@ -16,13 +16,13 @@
 #ifndef __FILE_DIALOG_H__
 #define __FILE_DIALOG_H__
 
-#include <glibmm.h>
 #include <vector>
 #include <set>
-#include <gtkmm.h>
 
 #include "extension/system.h"
 
+#include <glibmm/ustring.h>
+
 class SPDocument;
 
 namespace Inkscape {
@@ -46,7 +46,9 @@ namespace Dialog
 typedef enum {
     SVG_TYPES,
     IMPORT_TYPES,
-    EXPORT_TYPES
+    EXPORT_TYPES,
+    EXE_TYPES,
+    CUSTOM_TYPE
     } FileDialogType;
 
 /**
@@ -124,6 +126,8 @@ public:
 
     virtual Glib::ustring getCurrentDirectory() = 0;
 
+    virtual void addFilterMenu(Glib::ustring name, Glib::ustring pattern) = 0;
+
 protected:
     /**
      * Filename that was given
@@ -205,6 +209,8 @@ public:
 
     virtual Glib::ustring getCurrentDirectory() = 0;
 
+    virtual void addFileType(Glib::ustring name, Glib::ustring pattern) = 0;
+
 protected:
 
     /**
@@ -386,4 +392,4 @@ public:
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
