@@ -1,5 +1,6 @@
-/** @file
- * @brief Fill and Stroke dialog - implementation
+/**
+ * @file
+ * Fill and Stroke dialog - implementation.
  *
  * Based on the old sp_object_properties_dialog.
  */
@@ -14,6 +15,7 @@
  * Released under GNU GPL.  Read the file 'COPYING' for more information.
  */
 
+#include "ui/widget/notebook-page.h"
 #include "desktop-handles.h"
 #include "desktop-style.h"
 #include "document.h"
@@ -21,6 +23,7 @@
 #include "filter-chemistry.h"
 #include "inkscape.h"
 #include "selection.h"
+#include "preferences.h"
 #include "style.h"
 #include "svg/css-ostringstream.h"
 #include "ui/icon-names.h"
@@ -33,15 +36,17 @@
 
 #include "ui/view/view-widget.h"
 
+#include <gtkmm/table.h>
+
 namespace Inkscape {
 namespace UI {
 namespace Dialog {
 
 FillAndStroke::FillAndStroke()
     : UI::Widget::Panel ("", "/dialogs/fillstroke", SP_VERB_DIALOG_FILL_STROKE),
-      _page_fill(1, 1, true, true),
-      _page_stroke_paint(1, 1, true, true),
-      _page_stroke_style(1, 1, true, true),
+      _page_fill(Gtk::manage(new UI::Widget::NotebookPage(1, 1, true, true))),
+      _page_stroke_paint(Gtk::manage(new UI::Widget::NotebookPage(1, 1, true, true))),
+      _page_stroke_style(Gtk::manage(new UI::Widget::NotebookPage(1, 1, true, true))),
       _composite_settings(SP_VERB_DIALOG_FILL_STROKE, "fillstroke", UI::Widget::SimpleFilterModifier::BLUR),
       deskTrack(),
       targetDesktop(0),
@@ -50,19 +55,21 @@ FillAndStroke::FillAndStroke()
       desktopChangeConn()
 {
     Gtk::Box *contents = _getContents();
-    contents->set_spacing(0);
+    contents->set_spacing(2);
 
     contents->pack_start(_notebook, true, true);
 
-    _notebook.append_page(_page_fill, _createPageTabLabel(_("Fill"), INKSCAPE_ICON_OBJECT_FILL));
-    _notebook.append_page(_page_stroke_paint, _createPageTabLabel(_("Stroke _paint"), INKSCAPE_ICON_OBJECT_STROKE));
-    _notebook.append_page(_page_stroke_style, _createPageTabLabel(_("Stroke st_yle"), INKSCAPE_ICON_OBJECT_STROKE_STYLE));
+    _notebook.append_page(*_page_fill, _createPageTabLabel(_("_Fill"), INKSCAPE_ICON("object-fill")));
+    _notebook.append_page(*_page_stroke_paint, _createPageTabLabel(_("Stroke _paint"), INKSCAPE_ICON("object-stroke")));
+    _notebook.append_page(*_page_stroke_style, _createPageTabLabel(_("Stroke st_yle"), INKSCAPE_ICON("object-stroke-style")));
+
+    _notebook.signal_switch_page().connect(sigc::mem_fun(this, &FillAndStroke::_onSwitchPage));
 
     _layoutPageFill();
     _layoutPageStrokePaint();
     _layoutPageStrokeStyle();
 
-    contents->pack_start(_composite_settings, false, false, 0);
+    contents->pack_start(_composite_settings, true, true, 0);
 
     show_all_children();
 
@@ -95,32 +102,68 @@ void FillAndStroke::setTargetDesktop(SPDesktop *desktop)
             sp_fill_style_widget_set_desktop(fillWdgt, desktop);
         }
         if (strokeWdgt) {
-            sp_stroke_style_widget_set_desktop(strokeWdgt, desktop);
+            sp_fill_style_widget_set_desktop(strokeWdgt, desktop);
         }
+        if (strokeStyleWdgt) {
+            sp_stroke_style_widget_set_desktop(strokeStyleWdgt, desktop);
+        }
+        _composite_settings.setSubject(&_subject);
     }
 }
 
+#if WITH_GTKMM_3_0
+void FillAndStroke::_onSwitchPage(Gtk::Widget * /*page*/, guint pagenum)
+#else
+void FillAndStroke::_onSwitchPage(GtkNotebookPage * /*page*/, guint pagenum)
+#endif
+{
+    _savePagePref(pagenum);
+}
+
+void
+FillAndStroke::_savePagePref(guint page_num)
+{
+    // remember the current page
+    Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+    prefs->setInt("/dialogs/fillstroke/page", page_num);
+}
+
 void
 FillAndStroke::_layoutPageFill()
 {
-    fillWdgt = manage(sp_fill_style_widget_new());
-    _page_fill.table().attach(*fillWdgt, 0, 1, 0, 1);
+    fillWdgt = Gtk::manage(sp_fill_style_widget_new());
+
+#if WITH_GTKMM_3_0
+    _page_fill->table().attach(*fillWdgt, 0, 0, 1, 1);
+#else
+    _page_fill->table().attach(*fillWdgt, 0, 1, 0, 1);
+#endif
 }
 
 void
 FillAndStroke::_layoutPageStrokePaint()
 {
-    strokeWdgt = manage(sp_stroke_style_paint_widget_new());
-    _page_stroke_paint.table().attach(*strokeWdgt, 0, 1, 0, 1);
+    strokeWdgt = Gtk::manage(sp_stroke_style_paint_widget_new());
+
+#if WITH_GTKMM_3_0
+    _page_stroke_paint->table().attach(*strokeWdgt, 0, 0, 1, 1);
+#else
+    _page_stroke_paint->table().attach(*strokeWdgt, 0, 1, 0, 1);
+#endif
 }
 
 void
 FillAndStroke::_layoutPageStrokeStyle()
 {
-    //Gtk::Widget *ssl = manage(Glib::wrap(sp_stroke_style_line_widget_new()));
-    //Gtk::Widget *ssl = static_cast<Gtk::Widget *>(sp_stroke_style_line_widget_new());
-    Gtk::Widget *ssl = sp_stroke_style_line_widget_new();
-    _page_stroke_style.table().attach(*ssl, 0, 1, 0, 1);
+    //Gtk::Widget *strokeStyleWdgt = manage(Glib::wrap(sp_stroke_style_line_widget_new()));
+    //Gtk::Widget *strokeStyleWdgt = static_cast<Gtk::Widget *>(sp_stroke_style_line_widget_new());
+    strokeStyleWdgt = sp_stroke_style_line_widget_new();
+
+#if WITH_GTKMM_3_0
+    _page_stroke_style->table().attach(*strokeStyleWdgt, 0, 0, 1, 1);
+#else
+    _page_stroke_style->table().attach(*strokeStyleWdgt, 0, 1, 0, 1);
+#endif
 }
 
 void
@@ -128,6 +171,8 @@ FillAndStroke::showPageFill()
 {
     present();
     _notebook.set_current_page(0);
+    _savePagePref(0);
+
 }
 
 void
@@ -135,6 +180,7 @@ FillAndStroke::showPageStrokePaint()
 {
     present();
     _notebook.set_current_page(1);
+    _savePagePref(1);
 }
 
 void
@@ -142,16 +188,18 @@ FillAndStroke::showPageStrokeStyle()
 {
     present();
     _notebook.set_current_page(2);
+    _savePagePref(2);
+
 }
 
 Gtk::HBox&
 FillAndStroke::_createPageTabLabel(const Glib::ustring& label, const char *label_image)
 {
-    Gtk::HBox *_tab_label_box = manage(new Gtk::HBox(false, 0));
+    Gtk::HBox *_tab_label_box = Gtk::manage(new Gtk::HBox(false, 4));
     _tab_label_box->pack_start(*Glib::wrap(sp_icon_new(Inkscape::ICON_SIZE_DECORATION,
                                                        label_image)));
 
-    Gtk::Label *_tab_label = manage(new Gtk::Label(label, true));
+    Gtk::Label *_tab_label = Gtk::manage(new Gtk::Label(label, true));
     _tab_label_box->pack_start(*_tab_label);
     _tab_label_box->show_all();
 
@@ -171,4 +219,4 @@ FillAndStroke::_createPageTabLabel(const Glib::ustring& label, const char *label
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
