@@ -12,20 +12,23 @@
 #ifndef SEEN_LAYERS_PANEL_H
 #define SEEN_LAYERS_PANEL_H
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#if GLIBMM_DISABLE_DEPRECATED && HAVE_GLIBMM_THREADS_H
+#include <glibmm/threads.h>
+#endif
+
+#include <gtkmm/box.h>
 #include <gtkmm/treeview.h>
 #include <gtkmm/treestore.h>
-#include <gtkmm/tooltips.h>
-#include <gtkmm/scale.h>
 #include <gtkmm/scrolledwindow.h>
-#include <gtkmm/box.h>
-#include <gtkmm/buttonbox.h>
-#include <gtkmm/spinbutton.h>
-#include <gtkmm/notebook.h>
-
-//#include "ui/previewholder.h"
+#include "ui/widget/spinbutton.h"
 #include "ui/widget/panel.h"
 #include "ui/widget/object-composite-settings.h"
 #include "desktop-tracker.h"
+#include "ui/widget/style-subject.h"
 
 class SPObject;
 
@@ -69,8 +72,14 @@ private:
     void _preToggle( GdkEvent const *event );
     void _toggled( Glib::ustring const& str, int targetCol );
 
-    void _handleButtonEvent(GdkEventButton* evt);
-    void _handleRowChange( Gtk::TreeModel::Path const& path, Gtk::TreeModel::iterator const& iter );
+    bool _handleButtonEvent(GdkEventButton *event);
+    bool _handleKeyEvent(GdkEventKey *event);
+    bool _handleDragDrop(const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, guint time);
+    void _handleEdited(const Glib::ustring& path, const Glib::ustring& new_text);
+    void _handleEditingCancelled();
+
+    void _doTreeMove();
+    void _renameLayer(Gtk::TreeModel::Row row, const Glib::ustring& name);
 
     void _pushTreeSelectionToCurrent();
     void _checkTreeSelection();
@@ -106,18 +115,31 @@ private:
     SPDesktop* _desktop;
     ModelColumns* _model;
     InternalUIBounce* _pending;
+    gboolean _dnd_into;
+    SPItem* _dnd_source;
+    SPItem* _dnd_target;
     GdkEvent* _toggleEvent;
+
     Glib::RefPtr<Gtk::TreeStore> _store;
     std::vector<Gtk::Widget*> _watching;
     std::vector<Gtk::Widget*> _watchingNonTop;
     std::vector<Gtk::Widget*> _watchingNonBottom;
 
-    Gtk::Tooltips _tips;
     Gtk::TreeView _tree;
-    Gtk::HButtonBox _buttonsRow;
+    Gtk::CellRendererText *_text_renderer;
+    Gtk::TreeView::Column *_name_column;
+#if WITH_GTKMM_3_0
+    Gtk::Box _buttonsRow;
+    Gtk::Box _buttonsPrimary;
+    Gtk::Box _buttonsSecondary;
+#else
+    Gtk::HBox _buttonsRow;
+    Gtk::HBox _buttonsPrimary;
+    Gtk::HBox _buttonsSecondary;
+#endif
     Gtk::ScrolledWindow _scroller;
     Gtk::Menu _popupMenu;
-    Gtk::SpinButton _spinBtn;
+    Inkscape::UI::Widget::SpinButton _spinBtn;
     Gtk::VBox _layersPage;
 
     UI::Widget::StyleSubject::CurrentLayer _subject;
@@ -144,4 +166,4 @@ private:
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
