@@ -1,10 +1,11 @@
-
-/** @file
- * @brief Color swatches dialog
+/**
+ * @file
+ * Color swatches dialog.
  */
 /* Authors:
  *   Jon A. Cruz
  *   John Bintz
+ *   Abhishek Sharma
  *
  * Copyright (C) 2005 Jon A. Cruz
  * Copyright (C) 2008 John Bintz
@@ -15,10 +16,14 @@
 #include <errno.h>
 #include <map>
 #include <algorithm>
+#include <set>
+
+#include "swatches.h"
+#include <gtkmm/radiomenuitem.h>
 
-#include <gtk/gtk.h> //for GTK_RESPONSE* types
 #include <glibmm/i18n.h>
 #include <glibmm/main.h>
+#include <glibmm/timer.h>
 #include <gdkmm/pixbuf.h>
 
 #include "color-item.h"
@@ -27,6 +32,7 @@
 #include "desktop-style.h"
 #include "document.h"
 #include "document-private.h"
+#include "document-undo.h"
 #include "extension/db.h"
 #include "inkscape.h"
 #include "inkscape.h"
@@ -36,18 +42,21 @@
 #include "path-prefix.h"
 #include "preferences.h"
 #include "sp-item.h"
-#include "sp-gradient-fns.h"
 #include "sp-gradient.h"
 #include "sp-gradient-vector.h"
-#include "swatches.h"
 #include "style.h"
 #include "ui/previewholder.h"
 #include "widgets/desktop-widget.h"
 #include "widgets/gradient-vector.h"
 #include "widgets/eek-preview.h"
-#include "display/nr-plain-stuff.h"
+#include "display/cairo-utils.h"
 #include "sp-gradient-reference.h"
 #include "dialog-manager.h"
+#include "selection.h"
+#include "verbs.h"
+#include "gradient-chemistry.h"
+#include "helper/action.h"
+#include "helper/action-context.h"
 
 namespace Inkscape {
 namespace UI {
@@ -56,10 +65,10 @@ namespace Dialogs {
 #define VBLOCK 16
 #define PREVIEW_PIXBUF_WIDTH 128
 
-void _loadPaletteFile( gchar const *filename );
+void _loadPaletteFile( gchar const *filename, gboolean user=FALSE );
 
-
-std::vector<SwatchPage*> possible;
+std::list<SwatchPage*> userSwatchPages;
+std::list<SwatchPage*> systemSwatchPages;
 static std::map<SPDocument*, SwatchPage*> docPalettes;
 static std::vector<DocTrack*> docTrackings;
 static std::map<SwatchesPanel*, SPDocument*> docPerPanel;
@@ -136,8 +145,21 @@ static void editGradientImpl( SPDesktop* desktop, SPGradient* gr )
         }
 
         if (!shown) {
-            GtkWidget *dialog = sp_gradient_vector_editor_new( gr );
-            gtk_widget_show( dialog );
+            Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+            if (prefs->getBool("/dialogs/gradienteditor/showlegacy", false)) {
+                // Legacy gradient dialog
+                GtkWidget *dialog = sp_gradient_vector_editor_new( gr );
+                gtk_widget_show( dialog );
+            } else {
+                // Invoke the gradient tool
+                Inkscape::Verb *verb = Inkscape::Verb::get( SP_VERB_CONTEXT_GRADIENT );
+                if ( verb ) {
+                    SPAction *action = verb->get_action( Inkscape::ActionContext( ( Inkscape::UI::View::View * ) SP_ACTIVE_DESKTOP ) );
+                    if ( action ) {
+                        sp_action_perform( action, NULL );
+                    }
+                }
+            }
         }
     }
 }
@@ -150,7 +172,7 @@ static void editGradient( GtkMenuItem */*menuitem*/, gpointer /*user_data*/ )
         SPDocument *doc = desktop ? desktop->doc() : 0;
         if (doc) {
             std::string targetName(bounceTarget->def.descr);
-            const GSList *gradients = sp_document_get_resource_list(doc, "gradient");
+            const GSList *gradients = doc->getResourceList("gradient");
             for (const GSList *item = gradients; item; item = item->next) {
                 SPGradient* grad = SP_GRADIENT(item->data);
                 if ( targetName == grad->getId() ) {
@@ -172,13 +194,13 @@ void SwatchesPanelHook::convertGradient( GtkMenuItem * /*menuitem*/, gpointer us
         if ( doc && (index >= 0) && (static_cast<guint>(index) < popupItems.size()) ) {
             Glib::ustring targetName = popupItems[index];
 
-            const GSList *gradients = sp_document_get_resource_list(doc, "gradient");
+            const GSList *gradients = doc->getResourceList("gradient");
             for (const GSList *item = gradients; item; item = item->next) {
                 SPGradient* grad = SP_GRADIENT(item->data);
                 if ( targetName == grad->getId() ) {
                     grad->setSwatch();
-                    sp_document_done(doc, SP_VERB_CONTEXT_GRADIENT,
-                                     _("Add gradient stop"));
+                    DocumentUndo::done(doc, SP_VERB_CONTEXT_GRADIENT,
+                                       _("Add gradient stop"));
                     break;
                 }
             }
@@ -191,20 +213,7 @@ void SwatchesPanelHook::deleteGradient( GtkMenuItem */*menuitem*/, gpointer /*us
     if ( bounceTarget ) {
         SwatchesPanel* swp = bouncePanel;
         SPDesktop* desktop = swp ? swp->getDesktop() : 0;
-        SPDocument *doc = desktop ? desktop->doc() : 0;
-        if (doc) {
-            std::string targetName(bounceTarget->def.descr);
-            const GSList *gradients = sp_document_get_resource_list(doc, "gradient");
-            for (const GSList *item = gradients; item; item = item->next) {
-                SPGradient* grad = SP_GRADIENT(item->data);
-                if ( targetName == grad->getId() ) {
-                    grad->setSwatch(false);
-                    sp_document_done(doc, SP_VERB_CONTEXT_GRADIENT,
-                                     _("Delete"));
-                    break;
-                }
-            }
-        }
+        sp_gradient_unset_swatch(desktop, bounceTarget->def.descr);
     }
 }
 
@@ -231,11 +240,14 @@ static void removeit( GtkWidget *widget, gpointer data )
     gtk_container_remove( GTK_CONTAINER(data), widget );
 }
 
+/* extern'ed from colot-item.cpp */
+gboolean colorItemHandleButtonPress( GtkWidget* widget, GdkEventButton* event, gpointer user_data );
+
 gboolean colorItemHandleButtonPress( GtkWidget* widget, GdkEventButton* event, gpointer user_data )
 {
     gboolean handled = FALSE;
 
-    if ( (event->button == 3) && (event->type == GDK_BUTTON_PRESS) ) {
+    if ( event && (event->button == 3) && (event->type == GDK_BUTTON_PRESS) ) {
         SwatchesPanel* swp = findContainingPanel( widget );
 
         if ( !popupMenu ) {
@@ -297,8 +309,8 @@ gboolean colorItemHandleButtonPress( GtkWidget* widget, GdkEventButton* event, g
             gtk_widget_show_all(popupMenu);
         }
 
-        ColorItem* item = reinterpret_cast<ColorItem*>(user_data);
-        if ( item ) {
+        if ( user_data ) {
+            ColorItem* item = reinterpret_cast<ColorItem*>(user_data);
             bool show = swp && (swp->getSelectedIndex() == 0);
             for ( std::vector<GtkWidget*>::iterator it = popupExtras.begin(); it != popupExtras.end(); ++ it) {
                 gtk_widget_set_sensitive(*it, show);
@@ -315,7 +327,7 @@ gboolean colorItemHandleButtonPress( GtkWidget* widget, GdkEventButton* event, g
                     SPDesktopWidget *dtw = SP_DESKTOP_WIDGET(wdgt);
                     if ( dtw && dtw->desktop ) {
                         // Pick up all gradients with vectors
-                        const GSList *gradients = sp_document_get_resource_list(dtw->desktop->doc(), "gradient");
+                        const GSList *gradients = (dtw->desktop->doc())->getResourceList("gradient");
                         gint index = 0;
                         for (const GSList *curr = gradients; curr; curr = curr->next) {
                             SPGradient* grad = SP_GRADIENT(curr->data);
@@ -365,13 +377,13 @@ static char* trim( char* str ) {
     return ret;
 }
 
-void skipWhitespace( char*& str ) {
+static void skipWhitespace( char*& str ) {
     while ( *str == ' ' || *str == '\t' ) {
         str++;
     }
 }
 
-bool parseNum( char*& str, int& val ) {
+static bool parseNum( char*& str, int& val ) {
     val = 0;
     while ( '0' <= *str && *str <= '9' ) {
         val = val * 10 + (*str - '0');
@@ -382,7 +394,7 @@ bool parseNum( char*& str, int& val ) {
 }
 
 
-void _loadPaletteFile( gchar const *filename )
+void _loadPaletteFile( gchar const *filename, gboolean user/*=FALSE*/ )
 {
     char block[1024];
     FILE *f = Inkscape::IO::fopen_utf8name( filename, "r" );
@@ -414,11 +426,11 @@ void _loadPaletteFile( gchar const *filename )
                                 inHeader = false;
                                 ptr = block;
                                 Glib::ustring name("");
-                                int r = 0;
-                                int g = 0;
-                                int b = 0;
                                 skipWhitespace(ptr);
                                 if ( *ptr ) {
+                                    int r = 0;
+                                    int g = 0;
+                                    int b = 0;
                                     hasErr = parseNum(ptr, r);
                                     if ( !hasErr ) {
                                         skipWhitespace(ptr);
@@ -431,7 +443,7 @@ void _loadPaletteFile( gchar const *filename )
                                     if ( !hasErr && *ptr ) {
                                         char* n = trim(ptr);
                                         if (n != NULL) {
-                                            name = n;
+                                            name = g_dpgettext2(NULL, "Palette", n);
                                         }
                                     }
                                     if ( !hasErr ) {
@@ -484,7 +496,10 @@ void _loadPaletteFile( gchar const *filename )
                     }
                 } while ( result && !hasErr );
                 if ( !hasErr ) {
-                    possible.push_back(onceMore);
+                    if (user)
+                        userSwatchPages.push_back(onceMore);
+                    else
+                        systemSwatchPages.push_back(onceMore);
 #if ENABLE_MAGIC_COLORS
                     ColorItem::_wireMagicColors( onceMore );
 #endif // ENABLE_MAGIC_COLORS
@@ -498,9 +513,16 @@ void _loadPaletteFile( gchar const *filename )
     }
 }
 
+static bool
+compare_swatch_names(SwatchPage const *a, SwatchPage const *b) {
+
+    return g_utf8_collate(a->_name.c_str(), b->_name.c_str()) < 0;
+}
+
 static void loadEmUp()
 {
     static bool beenHere = false;
+    gboolean userPalete = true;
     if ( !beenHere ) {
         beenHere = true;
 
@@ -512,7 +534,6 @@ static void loadEmUp()
         // Use this loop to iterate through a list of possible document locations.
         while (!sources.empty()) {
             gchar *dirname = sources.front();
-
             if ( Inkscape::IO::file_test( dirname, G_FILE_TEST_EXISTS )
                 && Inkscape::IO::file_test( dirname, G_FILE_TEST_IS_DIR )) {
                 GError *err = 0;
@@ -526,11 +547,13 @@ static void loadEmUp()
                     while ((filename = (gchar *)g_dir_read_name(directory)) != NULL) {
                         gchar* lower = g_ascii_strdown( filename, -1 );
 //                        if ( g_str_has_suffix(lower, ".gpl") ) {
+                          if ( !g_str_has_suffix(lower, "~") ) {
                             gchar* full = g_build_filename(dirname, filename, NULL);
                             if ( !Inkscape::IO::file_test( full, G_FILE_TEST_IS_DIR ) ) {
-                                _loadPaletteFile(full);
+                                _loadPaletteFile(full, userPalete);
                             }
                             g_free(full);
+                          }
 //                      }
                         g_free(lower);
                     }
@@ -541,18 +564,18 @@ static void loadEmUp()
             // toss the dirname
             g_free(dirname);
             sources.pop_front();
+            userPalete = false;
         }
     }
+
+   // Sort the list of swatches by name, grouped by user/system
+   userSwatchPages.sort(compare_swatch_names);
+   systemSwatchPages.sort(compare_swatch_names);
+
 }
 
 
 
-
-
-
-
-
-
 SwatchesPanel& SwatchesPanel::getInstance()
 {
     return *new SwatchesPanel();
@@ -583,7 +606,7 @@ SwatchesPanel::SwatchesPanel(gchar const* prefsPath) :
     }
 
     loadEmUp();
-    if ( !possible.empty() ) {
+    if ( !systemSwatchPages.empty() ) {
         SwatchPage* first = 0;
         int index = 0;
         Glib::ustring targetName;
@@ -594,8 +617,9 @@ SwatchesPanel::SwatchesPanel(gchar const* prefsPath) :
                 if (targetName == "Auto") {
                     first = docPalettes[0];
                 } else {
-                    index++;
-                    for ( std::vector<SwatchPage*>::iterator iter = possible.begin(); iter != possible.end(); ++iter ) {
+                    //index++;
+                    std::vector<SwatchPage*> pages = _getSwatchSets();
+                    for ( std::vector<SwatchPage*>::iterator iter = pages.begin(); iter != pages.end(); ++iter ) {
                         if ( (*iter)->_name == targetName ) {
                             first = *iter;
                             break;
@@ -619,17 +643,31 @@ SwatchesPanel::SwatchesPanel(gchar const* prefsPath) :
 
         int i = 0;
         std::vector<SwatchPage*> swatchSets = _getSwatchSets();
-        for ( std::vector<SwatchPage*>::iterator it = swatchSets.begin(); it != swatchSets.end(); it++ ) {
+        for ( std::vector<SwatchPage*>::iterator it = swatchSets.begin(); it != swatchSets.end(); ++it) {
             SwatchPage* curr = *it;
-            Gtk::RadioMenuItem* single = manage(new Gtk::RadioMenuItem(groupOne, curr->_name));
+            Gtk::RadioMenuItem* single = Gtk::manage(new Gtk::RadioMenuItem(groupOne, curr->_name));
             if ( curr == first ) {
                 hotItem = single;
             }
             _regItem( single, 3, i );
+
             i++;
         }
     }
 
+    if (Glib::ustring(prefsPath) == "/dialogs/swatches") {
+        Gtk::Requisition sreq;
+#if WITH_GTKMM_3_0
+        Gtk::Requisition sreq_natural;
+        get_preferred_size(sreq_natural, sreq);
+#else
+        sreq = size_request();
+#endif
+        int minHeight = 60;
+        if (sreq.height < minHeight) {
+            set_size_request(70, minHeight);
+        }
+    }
 
     _getContents()->pack_start(*_holder, Gtk::PACK_EXPAND_WIDGET);
     _setTargetFillable(_holder);
@@ -660,14 +698,14 @@ SwatchesPanel::~SwatchesPanel()
     }
 }
 
-void SwatchesPanel::setOrientation( Gtk::AnchorType how )
+void SwatchesPanel::setOrientation(SPAnchorType how)
 {
     // Must call the parent class or bad things might happen
     Inkscape::UI::Widget::Panel::setOrientation( how );
 
     if ( _holder )
     {
-        _holder->setOrientation( Gtk::ANCHOR_SOUTH );
+        _holder->setOrientation(SP_ANCHOR_SOUTH);
     }
 }
 
@@ -707,10 +745,11 @@ void SwatchesPanel::setDesktop( SPDesktop* desktop )
 class DocTrack
 {
 public:
-    DocTrack(SPDocument *doc, sigc::connection &gradientRsrcChanged, sigc::connection &defsChanged, sigc::connection &defsModified) :
+    DocTrack(SPDocument *doc, sigc::connection &docDestroy, sigc::connection &gradientRsrcChanged, sigc::connection &defsChanged, sigc::connection &defsModified) :
         doc(doc),
         updatePending(false),
         lastGradientUpdate(0.0),
+        docDestroy(docDestroy),
         gradientRsrcChanged(gradientRsrcChanged),
         defsChanged(defsChanged),
         defsModified(defsModified)
@@ -735,9 +774,11 @@ public:
             }
         }
         if (doc) {
+            docDestroy.disconnect();
             gradientRsrcChanged.disconnect();
             defsChanged.disconnect();
             defsModified.disconnect();
+            doc = NULL;
         }
     }
 
@@ -757,6 +798,7 @@ public:
     SPDocument *doc;
     bool updatePending;
     double lastGradientUpdate;
+    sigc::connection docDestroy;
     sigc::connection gradientRsrcChanged;
     sigc::connection defsChanged;
     sigc::connection defsModified;
@@ -820,7 +862,7 @@ bool DocTrack::queueUpdateIfNeeded( SPDocument *doc )
 
 void SwatchesPanel::_trackDocument( SwatchesPanel *panel, SPDocument *document )
 {
-    SPDocument *oldDoc = 0;
+    SPDocument *oldDoc = NULL;
     if (docPerPanel.find(panel) != docPerPanel.end()) {
         oldDoc = docPerPanel[panel];
         if (!oldDoc) {
@@ -829,7 +871,7 @@ void SwatchesPanel::_trackDocument( SwatchesPanel *panel, SPDocument *document )
     }
     if (oldDoc != document) {
         if (oldDoc) {
-            docPerPanel[panel] = 0;
+            docPerPanel[panel] = NULL;
             bool found = false;
             for (std::map<SwatchesPanel*, SPDocument*>::iterator it = docPerPanel.begin(); (it != docPerPanel.end()) && !found; ++it) {
                 found = (it->second == document);
@@ -852,11 +894,12 @@ void SwatchesPanel::_trackDocument( SwatchesPanel *panel, SPDocument *document )
             }
             docPerPanel[panel] = document;
             if (!found) {
-                sigc::connection conn1 = sp_document_resources_changed_connect( document, "gradient", sigc::bind(sigc::ptr_fun(&SwatchesPanel::handleGradientsChange), document) );
-                sigc::connection conn2 = SP_DOCUMENT_DEFS(document)->connectRelease( sigc::hide(sigc::bind(sigc::ptr_fun(&SwatchesPanel::handleDefsModified), document)) );
-                sigc::connection conn3 = SP_DOCUMENT_DEFS(document)->connectModified( sigc::hide(sigc::hide(sigc::bind(sigc::ptr_fun(&SwatchesPanel::handleDefsModified), document))) );
+                sigc::connection conn0 = document->connectDestroy(sigc::bind(sigc::ptr_fun(&SwatchesPanel::handleDocumentDestroy), document));
+                sigc::connection conn1 = document->connectResourcesChanged( "gradient", sigc::bind(sigc::ptr_fun(&SwatchesPanel::handleGradientsChange), document) );
+                sigc::connection conn2 = document->getDefs()->connectRelease( sigc::hide(sigc::bind(sigc::ptr_fun(&SwatchesPanel::handleDefsModified), document)) );
+                sigc::connection conn3 = document->getDefs()->connectModified( sigc::hide(sigc::hide(sigc::bind(sigc::ptr_fun(&SwatchesPanel::handleDefsModified), document))) );
 
-                DocTrack *dt = new DocTrack(document, conn1, conn2, conn3);
+                DocTrack *dt = new DocTrack(document, conn0, conn1, conn2, conn3);
                 docTrackings.push_back(dt);
 
                 if (docPalettes.find(document) == docPalettes.end()) {
@@ -867,11 +910,6 @@ void SwatchesPanel::_trackDocument( SwatchesPanel *panel, SPDocument *document )
             }
         }
     }
-
-    std::set<SPDocument*> docs;
-    for (std::map<SwatchesPanel*, SPDocument*>::iterator it = docPerPanel.begin(); it != docPerPanel.end(); ++it) {
-        docs.insert(it->second);
-    }
 }
 
 void SwatchesPanel::_setDocument( SPDocument *document )
@@ -884,17 +922,19 @@ void SwatchesPanel::_setDocument( SPDocument *document )
 }
 
 static void recalcSwatchContents(SPDocument* doc,
-                std::vector<ColorItem*> &tmpColors,
-                std::map<ColorItem*, guchar*> &previewMappings,
+                boost::ptr_vector<ColorItem> &tmpColors,
+                std::map<ColorItem*, cairo_pattern_t*> &previewMappings,
                 std::map<ColorItem*, SPGradient*> &gradMappings)
 {
     std::vector<SPGradient*> newList;
 
-    const GSList *gradients = sp_document_get_resource_list(doc, "gradient");
-    for (const GSList *item = gradients; item; item = item->next) {
-        SPGradient* grad = SP_GRADIENT(item->data);
-        if ( grad->isSwatch() ) {
-            newList.push_back(SP_GRADIENT(item->data));
+    if (doc) {
+        const GSList *gradients = doc->getResourceList("gradient");
+        for (const GSList *item = gradients; item; item = item->next) {
+            SPGradient* grad = SP_GRADIENT(item->data);
+            if ( grad->isSwatch() ) {
+                newList.push_back(SP_GRADIENT(item->data));
+            }
         }
     }
 
@@ -903,30 +943,29 @@ static void recalcSwatchContents(SPDocument* doc,
         for ( std::vector<SPGradient*>::iterator it = newList.begin(); it != newList.end(); ++it )
         {
             SPGradient* grad = *it;
-            grad->ensureVector();
-            SPGradientStop first = grad->vector.stops[0];
-            SPColor color = first.color;
-            guint32 together = color.toRGBA32(first.opacity);
 
-            SPGradientStop second = (*it)->vector.stops[1];
-            SPColor color2 = second.color;
+            cairo_surface_t *preview = cairo_image_surface_create(CAIRO_FORMAT_ARGB32,
+                PREVIEW_PIXBUF_WIDTH, VBLOCK);
+            cairo_t *ct = cairo_create(preview);
 
             Glib::ustring name( grad->getId() );
-            unsigned int r = SP_RGBA32_R_U(together);
-            unsigned int g = SP_RGBA32_G_U(together);
-            unsigned int b = SP_RGBA32_B_U(together);
-            ColorItem* item = new ColorItem( r, g, b, name );
+            ColorItem* item = new ColorItem( 0, 0, 0, name );
 
-            gint width = PREVIEW_PIXBUF_WIDTH;
-            gint height = VBLOCK;
-            guchar* px = g_new( guchar, 3 * height * width );
-            nr_render_checkerboard_rgb( px, width, height, 3 * width, 0, 0 );
+            cairo_pattern_t *check = ink_cairo_pattern_create_checkerboard();
+            cairo_pattern_t *gradient = sp_gradient_create_preview_pattern(grad, PREVIEW_PIXBUF_WIDTH);
+            cairo_set_source(ct, check);
+            cairo_paint(ct);
+            cairo_set_source(ct, gradient);
+            cairo_paint(ct);
 
-            sp_gradient_render_vector_block_rgb( grad,
-                                                 px, width, height, 3 * width,
-                                                 0, width, TRUE );
+            cairo_destroy(ct);
+            cairo_pattern_destroy(gradient);
+            cairo_pattern_destroy(check);
 
-            previewMappings[item] = px;
+            cairo_pattern_t *prevpat = cairo_pattern_create_for_surface(preview);
+            cairo_surface_destroy(preview);
+
+            previewMappings[item] = prevpat;
 
             tmpColors.push_back(item);
             gradMappings[item] = grad;
@@ -934,17 +973,49 @@ static void recalcSwatchContents(SPDocument* doc,
     }
 }
 
+void SwatchesPanel::handleDocumentDestroy(SPDocument *document)
+{
+    if (document) {
+        for (std::vector<DocTrack*>::iterator it = docTrackings.begin(); it != docTrackings.end(); ++it){
+            if ((*it)->doc == document) {
+                delete *it;
+                docTrackings.erase(it);
+                break;
+            }
+        }
+
+        if (docPalettes.find(document) != docPalettes.end()) {
+            docPalettes.erase(document);
+        }
+
+        for (std::map<SwatchesPanel*, SPDocument*>::iterator it = docPerPanel.begin(); it != docPerPanel.end(); ++it) {
+            if (it->second == document) {
+                SwatchesPanel* swp = it->first;
+                std::vector<SwatchPage*> pages = swp->_getSwatchSets();
+                if ((swp->_currentIndex >= static_cast<int>(pages.size())) && (pages.size() > 0))
+                {
+                    swp->_setSelectedIndex(swp->_getSwatchSets().size() - 1);
+                }
+                swp->_rebuild();
+                docPerPanel.erase(it);
+                break;
+            }
+        }
+    }
+}
+
 void SwatchesPanel::handleGradientsChange(SPDocument *document)
 {
     SwatchPage *docPalette = (docPalettes.find(document) != docPalettes.end()) ? docPalettes[document] : 0;
     if (docPalette) {
-        std::vector<ColorItem*> tmpColors;
-        std::map<ColorItem*, guchar*> tmpPrevs;
+        boost::ptr_vector<ColorItem> tmpColors;
+        std::map<ColorItem*, cairo_pattern_t*> tmpPrevs;
         std::map<ColorItem*, SPGradient*> tmpGrads;
         recalcSwatchContents(document, tmpColors, tmpPrevs, tmpGrads);
 
-        for (std::map<ColorItem*, guchar*>::iterator it = tmpPrevs.begin(); it != tmpPrevs.end(); ++it) {
-            it->first->setPixData(it->second, PREVIEW_PIXBUF_WIDTH, VBLOCK);
+        for (std::map<ColorItem*, cairo_pattern_t*>::iterator it = tmpPrevs.begin(); it != tmpPrevs.end(); ++it) {
+            it->first->setPattern(it->second);
+            cairo_pattern_destroy(it->second);
         }
 
         for (std::map<ColorItem*, SPGradient*>::iterator it = tmpGrads.begin(); it != tmpGrads.end(); ++it) {
@@ -952,10 +1023,6 @@ void SwatchesPanel::handleGradientsChange(SPDocument *document)
         }
 
         docPalette->_colors.swap(tmpColors);
-        for (std::vector<ColorItem*>::iterator it = tmpColors.begin(); it != tmpColors.end(); ++it) {
-            delete *it;
-        }
-
 
         // Figure out which SwatchesPanel instances are affected and update them.
 
@@ -976,8 +1043,8 @@ void SwatchesPanel::handleDefsModified(SPDocument *document)
 {
     SwatchPage *docPalette = (docPalettes.find(document) != docPalettes.end()) ? docPalettes[document] : 0;
     if (docPalette && !DocTrack::queueUpdateIfNeeded(document) ) {
-        std::vector<ColorItem*> tmpColors;
-        std::map<ColorItem*, guchar*> tmpPrevs;
+        boost::ptr_vector<ColorItem> tmpColors;
+        std::map<ColorItem*, cairo_pattern_t*> tmpPrevs;
         std::map<ColorItem*, SPGradient*> tmpGrads;
         recalcSwatchContents(document, tmpColors, tmpPrevs, tmpGrads);
 
@@ -986,8 +1053,8 @@ void SwatchesPanel::handleDefsModified(SPDocument *document)
         } else {
             int cap = std::min(docPalette->_colors.size(), tmpColors.size());
             for (int i = 0; i < cap; i++) {
-                ColorItem* newColor = tmpColors[i];
-                ColorItem* oldColor = docPalette->_colors[i];
+                ColorItem *newColor = &tmpColors[i];
+                ColorItem *oldColor = &docPalette->_colors[i];
                 if ( (newColor->def.getType() != oldColor->def.getType()) ||
                      (newColor->def.getR() != oldColor->def.getR()) ||
                      (newColor->def.getG() != oldColor->def.getG()) ||
@@ -998,13 +1065,18 @@ void SwatchesPanel::handleDefsModified(SPDocument *document)
                     oldColor->setGradient(tmpGrads[newColor]);
                 }
                 if ( tmpPrevs.find(newColor) != tmpPrevs.end() ) {
-                    oldColor->setPixData(tmpPrevs[newColor], PREVIEW_PIXBUF_WIDTH, VBLOCK);
+                    oldColor->setPattern(tmpPrevs[newColor]);
                 }
             }
         }
+
+        for (std::map<ColorItem*, cairo_pattern_t*>::iterator it = tmpPrevs.begin(); it != tmpPrevs.end(); ++it) {
+            cairo_pattern_destroy(it->second);
+        }
     }
 }
 
+
 std::vector<SwatchPage*> SwatchesPanel::_getSwatchSets() const
 {
     std::vector<SwatchPage*> tmp;
@@ -1012,7 +1084,8 @@ std::vector<SwatchPage*> SwatchesPanel::_getSwatchSets() const
         tmp.push_back(docPalettes[_currentDocument]);
     }
 
-    tmp.insert(tmp.end(), possible.begin(), possible.end());
+    tmp.insert(tmp.end(), userSwatchPages.begin(), userSwatchPages.end());
+    tmp.insert(tmp.end(), systemSwatchPages.begin(), systemSwatchPages.end());
 
     return tmp;
 }
@@ -1046,7 +1119,8 @@ void SwatchesPanel::_updateFromSelection()
                             }
                         }
                         if ( target ) {
-                            gchar const* id = target->repr->attribute("id");
+                            //XML Tree being used directly here while it shouldn't be
+                            gchar const* id = target->getRepr()->attribute("id");
                             if ( id ) {
                                 fillId = id;
                             }
@@ -1077,7 +1151,8 @@ void SwatchesPanel::_updateFromSelection()
                             }
                         }
                         if ( target ) {
-                            gchar const* id = target->repr->attribute("id");
+                            //XML Tree being used directly here while it shouldn't be
+                            gchar const* id = target->getRepr()->attribute("id");
                             if ( id ) {
                                 strokeId = id;
                             }
@@ -1089,8 +1164,8 @@ void SwatchesPanel::_updateFromSelection()
         }
         sp_style_unref(tmpStyle);
 
-        for ( std::vector<ColorItem*>::iterator it = docPalette->_colors.begin(); it != docPalette->_colors.end(); ++it ) {
-            ColorItem* item = *it;
+        for ( boost::ptr_vector<ColorItem>::iterator it = docPalette->_colors.begin(); it != docPalette->_colors.end(); ++it ) {
+            ColorItem* item = &*it;
             bool isFill = (fillId == item->def.descr);
             bool isStroke = (strokeId == item->def.descr);
             item->setState( isFill, isStroke );
@@ -1103,43 +1178,47 @@ void SwatchesPanel::_handleAction( int setId, int itemId )
     switch( setId ) {
         case 3:
         {
-            std::vector<SwatchPage*> pages = _getSwatchSets();
-            if ( itemId >= 0 && itemId < static_cast<int>(pages.size()) ) {
-                _currentIndex = itemId;
-
-                if ( !_prefs_path.empty() ) {
-                    Inkscape::Preferences *prefs = Inkscape::Preferences::get();
-                    prefs->setString(_prefs_path + "/palette", pages[_currentIndex]->_name);
-                }
-
-                _rebuild();
-            }
+            _setSelectedIndex(itemId);
         }
         break;
     }
 }
 
+void SwatchesPanel::_setSelectedIndex( int index )
+{
+    std::vector<SwatchPage*> pages = _getSwatchSets();
+    if ( index >= 0 && index < static_cast<int>(pages.size()) ) {
+        _currentIndex = index;
+
+        if ( !_prefs_path.empty() ) {
+            Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+            prefs->setString(_prefs_path + "/palette", pages[_currentIndex]->_name);
+        }
+
+        _rebuild();
+    }
+}
+
 void SwatchesPanel::_rebuild()
 {
     std::vector<SwatchPage*> pages = _getSwatchSets();
-    SwatchPage* curr = pages[_currentIndex];
-    _holder->clear();
+    if (_currentIndex < static_cast<int>(pages.size())) {
+        SwatchPage* curr = pages[_currentIndex];
+        _holder->clear();
 
-    if ( curr->_prefWidth > 0 ) {
-        _holder->setColumnPref( curr->_prefWidth );
+        if ( curr->_prefWidth > 0 ) {
+            _holder->setColumnPref( curr->_prefWidth );
+        }
+        _holder->freezeUpdates();
+        // TODO restore once 'clear' works _holder->addPreview(_clear);
+        _holder->addPreview(_remove);
+        for ( boost::ptr_vector<ColorItem>::iterator it = curr->_colors.begin(); it != curr->_colors.end(); ++it) {
+            _holder->addPreview(&*it);
+        }
+        _holder->thawUpdates();
     }
-    _holder->freezeUpdates();
-    // TODO restore once 'clear' works _holder->addPreview(_clear);
-    _holder->addPreview(_remove);
-    for ( std::vector<ColorItem*>::iterator it = curr->_colors.begin(); it != curr->_colors.end(); it++ ) {
-        _holder->addPreview(*it);
-    }
-    _holder->thawUpdates();
 }
 
-
-
-
 } //namespace Dialogs
 } //namespace UI
 } //namespace Inkscape
@@ -1154,4 +1233,4 @@ void SwatchesPanel::_rebuild()
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
