@@ -9,12 +9,12 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#include "filter-effect-chooser.h"
 #include <glibmm/i18n.h>
 
 #include "desktop.h"
 #include "desktop-handles.h"
 #include "document.h"
-#include "filter-effect-chooser.h"
 #include "inkscape.h"
 
 namespace Inkscape {
@@ -22,22 +22,19 @@ namespace UI {
 namespace Widget {
 
 SimpleFilterModifier::SimpleFilterModifier(int flags)
-    : _lb_blend(_("_Blend mode:")),
-      _lb_blur(_("B_lur:"), Gtk::ALIGN_LEFT),
+    : _lb_blend(_("Blend mode:")),
       _blend(BlendModeConverter, SP_ATTR_INVALID, false),
-      _blur(0, 0, 100, 1, 0.01, 1)
+      _blur(_("Blur (%)"), 0, 0, 100, 1, 0.01, 1)
 {
     _flags = flags;
 
     if (flags & BLEND) {
         add(_hb_blend);
-        _hb_blend.pack_start(_lb_blend, false, false);
+        _hb_blend.pack_start(_lb_blend, false, false, 0);
         _hb_blend.pack_start(_blend);
     }
     if (flags & BLUR) {
-        add(_vb_blur);
-        _vb_blur.add(_lb_blur);
-        _vb_blur.add(_blur);
+        add(_blur);
     }
 
     show_all_children();
@@ -45,10 +42,7 @@ SimpleFilterModifier::SimpleFilterModifier(int flags)
     _hb_blend.set_spacing(12);
     _lb_blend.set_use_underline();
     _lb_blend.set_mnemonic_widget(_blend);
-    _lb_blur.set_use_underline();
-    _lb_blur.set_mnemonic_widget(_blur.get_scale());
     _blend.signal_changed().connect(signal_blend_blur_changed());
-    _blur.set_update_policy(Gtk::UPDATE_DELAYED);
     _blur.signal_value_changed().connect(signal_blend_blur_changed());
 }
 
@@ -62,15 +56,12 @@ const Glib::ustring SimpleFilterModifier::get_blend_mode()
     if (!(_flags & BLEND)) {
         return "normal";
     }
-    if (_blend.get_active_row_number() == 5) {
+
+    const Util::EnumData<Inkscape::Filters::FilterBlendMode> *d = _blend.get_active_data();
+    if (d) {
+        return _blend.get_active_data()->key;
+    } else
         return "normal";
-    } else {
-        const Util::EnumData<Inkscape::Filters::FilterBlendMode> *d = _blend.get_active_data();
-        if (d) {
-            return _blend.get_active_data()->key;
-        } else
-            return "normal";
-    }
 }
 
 void SimpleFilterModifier::set_blend_mode(const int val)
@@ -106,4 +97,4 @@ void SimpleFilterModifier::set_blur_sensitive(const bool s)
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
