@@ -1,7 +1,4 @@
-/**
- * \brief Labelled Widget - Adds a label with optional icon or suffix to
- *        another widget.
- *
+/*
  * Authors:
  *   Carl Hetherington <inkscape@carlh.net>
  *   Derek P. Moore <derekm@hackunix.org>
@@ -14,18 +11,43 @@
 #ifndef INKSCAPE_UI_WIDGET_LABELLED_H
 #define INKSCAPE_UI_WIDGET_LABELLED_H
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#if GLIBMM_DISABLE_DEPRECATED && HAVE_GLIBMM_THREADS_H
+#include <glibmm/threads.h>
+#endif
+
 #include <gtkmm/box.h>
-#include <gtkmm/label.h>
-#include <gtkmm/image.h>
-#include <gtkmm/tooltips.h>
+
+namespace Gtk {
+class Label;
+}
 
 namespace Inkscape {
 namespace UI {
 namespace Widget {
 
+/**
+ * Adds a label with optional icon or suffix to another widget.
+ */
 class Labelled : public Gtk::HBox
 {
 public:
+
+    /**
+     * Construct a Labelled Widget.
+     *
+     * @param label     Label.
+     * @param widget    Widget to label; should be allocated with new, as it will
+     *                  be passed to Gtk::manage().
+     * @param suffix    Suffix, placed after the widget (defaults to "").
+     * @param icon      Icon filename, placed before the label (defaults to "").
+     * @param mnemonic  Mnemonic toggle; if true, an underscore (_) in the text
+     *                  indicates the next character should be used for the
+     *                  mnemonic accelerator key (defaults to true).
+     */
     Labelled(Glib::ustring const &label, Glib::ustring const &tooltip,
              Gtk::Widget *widget,
              Glib::ustring const &suffix = "",
@@ -39,14 +61,18 @@ public:
     Gtk::Widget const *getWidget() const;
     Gtk::Label const *getLabel() const;
 
-    void setLabelText(const Glib::ustring &str) { _label->set_text(str); };
+    void setLabelText(const Glib::ustring &str);
+    void setTooltipText(const Glib::ustring &tooltip);
+
+private:
+    virtual bool on_mnemonic_activate( bool group_cycling );
 
 protected:
+
     Gtk::Widget  *_widget;
     Gtk::Label   *_label;
     Gtk::Label   *_suffix;
     Gtk::Widget  *_icon;
-    Gtk::Tooltips _tooltips;
 };
 
 } // namespace Widget
@@ -64,4 +90,4 @@ protected:
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
