@@ -1,11 +1,11 @@
-/** \file
- *
+/*
  * Authors:
  *   bulia byak <buliabyak@users.sf.net>
  *   Bryce W. Harrington <bryce@bryceharrington.org>
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *   Jon Phillips <jon@rejon.org>
  *   Ralf Stephan <ralf@ark.in-berlin.de> (Gtkmm)
+ *   Abhishek Sharma
  *
  * Copyright (C) 2000 - 2005 Authors
  *
@@ -16,14 +16,19 @@
 # include <config.h>
 #endif
 
+#include "licensor.h"
+
 #include <gtkmm/entry.h>
 
 #include "ui/widget/entity-entry.h"
 #include "ui/widget/registry.h"
 #include "rdf.h"
 #include "inkscape.h"
+#include "document-undo.h"
+#include "document-private.h"
+#include "verbs.h"
+#include <gtkmm/radiobutton.h>
 
-#include "licensor.h"
 
 namespace Inkscape {
 namespace UI {
@@ -39,7 +44,7 @@ const struct rdf_license_t _other_license =
 
 class LicenseItem : public Gtk::RadioButton {
 public:
-    LicenseItem (struct rdf_license_t const* license, EntityEntry* entity, Registry &wr);
+    LicenseItem (struct rdf_license_t const* license, EntityEntry* entity, Registry &wr, Gtk::RadioButtonGroup *group);
 protected:
     void on_toggled();
     struct rdf_license_t const *_lic;
@@ -47,25 +52,25 @@ protected:
     Registry                   &_wr;
 };
 
-LicenseItem::LicenseItem (struct rdf_license_t const* license, EntityEntry* entity, Registry &wr)
+LicenseItem::LicenseItem (struct rdf_license_t const* license, EntityEntry* entity, Registry &wr, Gtk::RadioButtonGroup *group)
 : Gtk::RadioButton(_(license->name)), _lic(license), _eep(entity), _wr(wr)
 {
-    static Gtk::RadioButtonGroup group = get_group();
-    static bool first = true;
-    if (first) first = false;
-    else       set_group (group);
+    if (group) {
+        set_group (*group);
+    }
 }
 
 /// \pre it is assumed that the license URI entry is a Gtk::Entry
-void
-LicenseItem::on_toggled()
+void LicenseItem::on_toggled()
 {
     if (_wr.isUpdating()) return;
 
     _wr.setUpdating (true);
-    rdf_set_license (SP_ACTIVE_DOCUMENT, _lic->details ? _lic : 0);
-    sp_document_done (SP_ACTIVE_DOCUMENT, SP_VERB_NONE, 
-                      /* TODO: annotate */ "licensor.cpp:65");
+    SPDocument *doc = SP_ACTIVE_DOCUMENT;
+    rdf_set_license (doc, _lic->details ? _lic : 0);
+    if (doc->priv->sensitive) {
+        DocumentUndo::done(doc, SP_VERB_NONE, "Document license updated");
+    }
     _wr.setUpdating (false);
     static_cast<Gtk::Entry*>(_eep->_packable)->set_text (_lic->uri);
     _eep->on_changed();
@@ -74,7 +79,8 @@ LicenseItem::on_toggled()
 //---------------------------------------------------
 
 Licensor::Licensor()
-: Gtk::VBox(false,4)
+: Gtk::VBox(false,4),
+  _eentry (NULL)
 {
 }
 
@@ -83,33 +89,33 @@ Licensor::~Licensor()
     if (_eentry) delete _eentry;
 }
 
-void
-Licensor::init (Gtk::Tooltips& tt, Registry& wr)
+void Licensor::init (Registry& wr)
 {
     /* add license-specific metadata entry areas */
     rdf_work_entity_t* entity = rdf_find_entity ( "license_uri" );
-    _eentry = EntityEntry::create (entity, tt, wr);
+    _eentry = EntityEntry::create (entity, wr);
 
     LicenseItem *i;
     wr.setUpdating (true);
-    i = manage (new LicenseItem (&_proprietary_license, _eentry, wr));
+    i = Gtk::manage (new LicenseItem (&_proprietary_license, _eentry, wr, NULL));
+    Gtk::RadioButtonGroup group = i->get_group();
     add (*i);
     LicenseItem *pd = i;
 
     for (struct rdf_license_t * license = rdf_licenses;
              license && license->name;
              license++) {
-        i = manage (new LicenseItem (license, _eentry, wr));
+        i = Gtk::manage (new LicenseItem (license, _eentry, wr, &group));
         add(*i);
     }
     // add Other at the end before the URI field for the confused ppl.
-    LicenseItem *io = manage (new LicenseItem (&_other_license, _eentry, wr));
+    LicenseItem *io = Gtk::manage (new LicenseItem (&_other_license, _eentry, wr, &group));
     add (*io);
 
     pd->set_active();
     wr.setUpdating (false);
 
-    Gtk::HBox *box = manage (new Gtk::HBox);
+    Gtk::HBox *box = Gtk::manage (new Gtk::HBox);
     pack_start (*box, true, true, 0);
 
     box->pack_start (_eentry->_label, false, false, 5);
@@ -118,8 +124,7 @@ Licensor::init (Gtk::Tooltips& tt, Registry& wr)
     show_all_children();
 }
 
-void 
-Licensor::update (SPDocument *doc)
+void Licensor::update (SPDocument *doc)
 {
     /* identify the license info */
     struct rdf_license_t * license = rdf_get_license (doc);
@@ -129,10 +134,18 @@ Licensor::update (SPDocument *doc)
         for (i=0; rdf_licenses[i].name; i++) 
             if (license == &rdf_licenses[i]) 
                 break;
+#if WITH_GTKMM_3_0
+        static_cast<LicenseItem*>(get_children()[i+1])->set_active();
+#else
         static_cast<LicenseItem*>(children()[i+1].get_widget())->set_active();
+#endif
     }
     else {
+#if WITH_GTKMM_3_0
+        static_cast<LicenseItem*>(get_children()[0])->set_active();
+#else
         static_cast<LicenseItem*>(children()[0].get_widget())->set_active();
+#endif
     }
     
     /* update the URI */
