@@ -1,5 +1,5 @@
-/**
- * \brief Notebook page widget
+/*
+ * Notebook page widget.
  *
  * Author:
  *   Bryce Harrington <bryce@bryceharrington.org>
@@ -9,28 +9,35 @@
  * Released under GNU GPL.  Read the file 'COPYING' for more information
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
-
 #include "notebook-page.h"
 
+#if WITH_GTKMM_3_0
+# include <gtkmm/grid.h>
+#else
+# include <gtkmm/table.h>
+#endif
+
 namespace Inkscape {
 namespace UI {
 namespace Widget {
 
-/**
- *    Construct a NotebookPage
- *
- *    \param label Label.
- */
-  
 NotebookPage::NotebookPage(int n_rows, int n_columns, bool expand, bool fill, guint padding)
-    :_table(n_rows, n_columns)
+#if WITH_GTKMM_3_0
+    :_table(Gtk::manage(new Gtk::Grid()))
+#else
+    :_table(Gtk::manage(new Gtk::Table(n_rows, n_columns)))
+#endif
 {
     set_border_width(2);
-    _table.set_spacings(2);
-    pack_start(_table, expand, fill, padding);
+
+#if WITH_GTKMM_3_0
+    _table->set_row_spacing(2);
+    _table->set_column_spacing(2);
+#else
+    _table->set_spacings(2);
+#endif
+
+    pack_start(*_table, expand, fill, padding);
 }
 
 } // namespace Widget
