@@ -1,6 +1,4 @@
-/**
- * \brief Notebook Page Widget - A tabbed notebook page for dialogs.
- *
+/*
  * Author:
  *   Bryce Harrington <bryce@bryceharrington.org>
  *
@@ -12,24 +10,55 @@
 #ifndef INKSCAPE_UI_WIDGET_NOTEBOOK_PAGE_H
 #define INKSCAPE_UI_WIDGET_NOTEBOOK_PAGE_H
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#if GLIBMM_DISABLE_DEPRECATED && HAVE_GLIBMM_THREADS_H
+#include <glibmm/threads.h>
+#endif
+
 #include <gtkmm/box.h>
-#include <gtkmm/frame.h>
-#include <gtkmm/table.h>
+
+namespace Gtk {
+#if WITH_GTKMM_3_0
+class Grid;
+#else
+class Table;
+#endif
+}
 
 namespace Inkscape {
 namespace UI {
 namespace Widget {
 
+/**
+ * A tabbed notebook page for dialogs.
+ */
 class NotebookPage : public Gtk::VBox
 {
 public:
+
     NotebookPage();
+
+    /**
+     * Construct a NotebookPage.
+     */
     NotebookPage(int n_rows, int n_columns, bool expand=false, bool fill=false, guint padding=0);
 
-    Gtk::Table& table() { return _table; }
+#if WITH_GTKMM_3_0
+    Gtk::Grid& table() { return *_table; }
+#else
+    Gtk::Table& table() { return *_table; }
+#endif
 
 protected:
-    Gtk::Table _table;
+
+#if WITH_GTKMM_3_0
+    Gtk::Grid *_table;
+#else
+    Gtk::Table *_table;
+#endif
 };
 
 } // namespace Widget
