@@ -2,8 +2,6 @@
 #define SEEN_UI_WIDGET_OBJECT_COMPOSITE_SETTINGS_H
 
 /*
- * A widget for controlling object compositing (filter, opacity, etc.)
- *
  * Authors:
  *   Bryce W. Harrington <bryce@bryceharrington.org>
  *   Gustav Broberg <broberg@kth.se>
@@ -13,22 +11,37 @@
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#if GLIBMM_DISABLE_DEPRECATED && HAVE_GLIBMM_THREADS_H
+#include <glibmm/threads.h>
+#endif
+
 #include <gtkmm/box.h>
 #include <gtkmm/alignment.h>
 #include <gtkmm/adjustment.h>
 #include <gtkmm/label.h>
-#include <gtkmm/spinbutton.h>
 #include <gtkmm/scale.h>
-
 #include <glibmm/ustring.h>
 
 #include "ui/widget/filter-effect-chooser.h"
-#include "ui/widget/style-subject.h"
+#include "ui/widget/spinbutton.h"
+
+class SPDesktop;
 
 namespace Inkscape {
+struct Application;
+
 namespace UI {
 namespace Widget {
 
+class StyleSubject;
+
+/*
+ * A widget for controlling object compositing (filter, opacity, etc.)
+ */
 class ObjectCompositeSettings : public Gtk::VBox {
 public:
     ObjectCompositeSettings(unsigned int verb_code, char const *history_prefix, int flags);
@@ -42,30 +55,23 @@ private:
     Glib::ustring   _opacity_tag;
 
     Gtk::VBox       _opacity_vbox;
-    Gtk::HBox       _opacity_label_box;
-    Gtk::HBox       _opacity_hbox;
-    Gtk::Label      _opacity_label;
-    Gtk::Adjustment _opacity_adjustment;
-    Gtk::HScale     _opacity_hscale;
-    Gtk::SpinButton _opacity_spin_button;
+    Inkscape::UI::Widget::SpinScale _opacity_scale;
 
     StyleSubject *_subject;
 
     SimpleFilterModifier _fe_cb;
     Gtk::VBox       _fe_vbox;
-    Gtk::Alignment  _fe_alignment;
 
+    bool _blocked;
+    gulong _desktop_activated;
+    sigc::connection _subject_changed;
+    
     static void _on_desktop_activate(Inkscape::Application *application, SPDesktop *desktop, ObjectCompositeSettings *w);
     static void _on_desktop_deactivate(Inkscape::Application *application, SPDesktop *desktop, ObjectCompositeSettings *w);
     void _subjectChanged();
-    sigc::connection _subject_changed;
-
     void _blendBlurValueChanged();
     void _opacityValueChanged();
 
-    bool _blocked;
-
-    gulong _desktop_activated;
 };
 
 }
@@ -83,4 +89,4 @@ private:
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
