@@ -1,6 +1,4 @@
-/** \file
- * \brief Widget for specifying page size; part of Document Preferences dialog.
- *
+/*
  * Author:
  *   Ralf Stephan <ralf@ark.in-berlin.de>
  *
@@ -9,22 +7,43 @@
  * Released under GNU GPL.  Read the file 'COPYING' for more information.
  */
 
-#ifndef INKSCAPE_UI_WIDGET_PAGE_SIZER__H
-#define INKSCAPE_UI_WIDGET_PAGE_SIZER__H
+#ifndef INKSCAPE_UI_WIDGET_PAGE_SIZER_H
+#define INKSCAPE_UI_WIDGET_PAGE_SIZER_H
+
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 
-#include <gtkmm.h>
 #include <stddef.h>
+#include "ui/widget/registered-widget.h"
 #include <sigc++/sigc++.h>
 
-#include "helper/units.h"
-#include "ui/widget/registry.h"
-#include "ui/widget/registered-widget.h"
-#include "xml/node.h"
+#include "util/units.h"
+
+#include <gtkmm/alignment.h>
+#include <gtkmm/expander.h>
+#include <gtkmm/frame.h>
+#include <gtkmm/liststore.h>
+#include <gtkmm/scrolledwindow.h>
+
+#if WITH_GTKMM_3_0
+# include <gtkmm/grid.h>
+#else
+# include <gtkmm/table.h>
+#endif
+
+#include <gtkmm/radiobutton.h>
 
 namespace Inkscape {    
+namespace XML {
+class Node;
+}
+
 namespace UI {
 namespace Widget {
 
+class Registry;
+
 /**
  * Data class used to store common paper dimensions.  Used to make
  * PageSizer's _paperSizeTable. 
@@ -45,7 +64,7 @@ public:
     PaperSize(const Glib::ustring &nameArg,
 	          double smallerArg,
 	          double largerArg,
-			  SPUnitId unitArg)
+			  Inkscape::Util::Unit const *unitArg)
 	    {
 	    name    = nameArg;
 	    smaller = smallerArg;
@@ -89,7 +108,7 @@ public:
     /**
      * The units (px, pt, mm, etc) of this specification
      */	     
-    SPUnitId unit;
+    Inkscape::Util::Unit const *unit; /// pointer to object in UnitTable, do not delete
 
 private:
 
@@ -98,7 +117,7 @@ private:
 	    name    = "";
 	    smaller = 0.0;
 	    larger  = 0.0;
-	    unit    = SP_UNIT_PX;
+	    unit    = unit_table.getUnit("px");
 	    }
 
 	void assign(const PaperSize &other)
@@ -142,7 +161,7 @@ public:
      * Set the page size to the given dimensions.  If 'changeList' is
      * true, then reset the paper size list to the closest match
      */
-    void setDim (double w, double h, bool changeList=true);
+    void setDim (Inkscape::Util::Quantity w, Inkscape::Util::Quantity h, bool changeList=true, bool changeSize=true);
     
     /**
      * Updates the scalar widgets for the fit margins.  (Just changes the value
@@ -160,12 +179,10 @@ protected:
     /**
      *	Find the closest standard paper size in the table, to the
      */
-    Gtk::ListStore::iterator find_paper_size (double w, double h) const;
+    Gtk::ListStore::iterator find_paper_size (Inkscape::Util::Quantity w, Inkscape::Util::Quantity h) const;
  
     void fire_fit_canvas_to_selection_or_drawing();
     
-    Gtk::Tooltips _tips;
-    
     //### The Paper Size selection list
     Gtk::HBox _paperSizeListBox;
     Gtk::Label _paperSizeListLabel;
@@ -200,7 +217,13 @@ protected:
 
     //### Custom size frame
     Gtk::Frame           _customFrame;
+
+#if WITH_GTKMM_3_0
+    Gtk::Grid            _customDimTable;
+#else
     Gtk::Table           _customDimTable;
+#endif
+
     RegisteredUnitMenu   _dimensionUnits;
     RegisteredScalarUnit _dimensionWidth;
     RegisteredScalarUnit _dimensionHeight;
@@ -208,7 +231,13 @@ protected:
 
     //### Fit Page options
     Gtk::Expander        _fitPageMarginExpander;
+
+#if WITH_GTKMM_3_0
+    Gtk::Grid            _marginTable;
+#else
     Gtk::Table           _marginTable;
+#endif
+
     Gtk::Alignment       _marginTopAlign;
     Gtk::Alignment       _marginLeftAlign;
     Gtk::Alignment       _marginRightAlign;
@@ -223,13 +252,17 @@ protected:
 
     //callback
     void on_value_changed();
+    void on_units_changed();
     sigc::connection    _changedw_connection;
     sigc::connection    _changedh_connection;
+    sigc::connection    _changedu_connection;
 
     Registry            *_widgetRegistry;
 
     //### state - whether we are currently landscape or portrait
     bool                 _landscape;
+    
+    Glib::ustring       _unit;
 
 };
 
@@ -238,7 +271,7 @@ protected:
 } // namespace Inkscape
 
 
-#endif /* INKSCAPE_UI_WIDGET_PAGE_SIZER__H */
+#endif // INKSCAPE_UI_WIDGET_PAGE_SIZER_H
 
 /*
   Local Variables:
