@@ -1,8 +1,11 @@
-/** @file
- * @brief Unclumping objects
+/**
+ * @file
+ * Unclumping objects.
  */
 /* Authors:
  *   bulia byak
+ *   Jon A. Cruz <jon@joncruz.org>
+ *   Abhishek Sharma
  *
  * Copyright (C) 2005 Authors
  * Released under GNU GPL, read the file 'COPYING' for more information
@@ -10,7 +13,9 @@
 
 #include <algorithm>
 #include <map>
+#include <2geom/transforms.h>
 #include "sp-item.h"
+#include "unclump.h"
 
 
 // Taking bbox of an item is an expensive operation, and we need to do it many times, so here we
@@ -23,7 +28,7 @@ std::map<const gchar *, Geom::Point> wh_cache;
 /**
 Center of bbox of item
 */
-Geom::Point
+static Geom::Point
 unclump_center (SPItem *item)
 {
     std::map<const gchar *, Geom::Point>::iterator i = c_cache.find(item->getId());
@@ -31,7 +36,7 @@ unclump_center (SPItem *item)
         return i->second;
     }
 
-    Geom::OptRect r = item->getBounds(sp_item_i2d_affine(item));
+    Geom::OptRect r = item->desktopVisualBounds();
     if (r) {
         Geom::Point const c = r->midpoint();
         c_cache[item->getId()] = c;
@@ -42,7 +47,7 @@ unclump_center (SPItem *item)
     }
 }
 
-Geom::Point
+static Geom::Point
 unclump_wh (SPItem *item)
 {
     Geom::Point wh;
@@ -50,7 +55,7 @@ unclump_wh (SPItem *item)
     if ( i != wh_cache.end() ) {
         wh = i->second;
     } else {
-        Geom::OptRect r = item->getBounds(sp_item_i2d_affine(item));
+        Geom::OptRect r = item->desktopVisualBounds();
         if (r) {
             wh = r->dimensions();
             wh_cache[item->getId()] = wh;
@@ -67,7 +72,7 @@ Distance between "edges" of item1 and item2. An item is considered to be an elli
 so its radius (distance from center to edge) depends on the w/h and the angle towards the other item.
 May be negative if the edge of item1 is between the center and the edge of item2.
 */
-double
+static double
 unclump_dist (SPItem *item1, SPItem *item2)
 {
 	Geom::Point c1 = unclump_center (item1);
@@ -162,7 +167,7 @@ unclump_dist (SPItem *item1, SPItem *item2)
 /**
 Average unclump_dist from item to others
 */
-double unclump_average (SPItem *item, GSList *others)
+static double unclump_average (SPItem *item, GSList *others)
 {
     int n = 0;
     double sum = 0;
@@ -186,7 +191,7 @@ double unclump_average (SPItem *item, GSList *others)
 /**
 Closest to item among others
  */
-SPItem *unclump_closest (SPItem *item, GSList *others)
+static SPItem *unclump_closest (SPItem *item, GSList *others)
 {
     double min = HUGE_VAL;
     SPItem *closest = NULL;
@@ -210,7 +215,7 @@ SPItem *unclump_closest (SPItem *item, GSList *others)
 /**
 Most distant from item among others
  */
-SPItem *unclump_farest (SPItem *item, GSList *others)
+static SPItem *unclump_farest (SPItem *item, GSList *others)
 {
     double max = -HUGE_VAL;
     SPItem *farest = NULL;
@@ -236,7 +241,7 @@ Removes from the \a rest list those items that are "behind" \a closest as seen f
 i.e. those on the other side of the line through \a closest perpendicular to the direction from \a
 item to \a closest. Returns a newly created list which must be freed.
  */
-GSList *
+static GSList *
 unclump_remove_behind (SPItem *item, SPItem *closest, GSList *rest)
 {
     Geom::Point it = unclump_center (item);
@@ -278,47 +283,47 @@ unclump_remove_behind (SPItem *item, SPItem *closest, GSList *rest)
 /**
 Moves \a what away from \a from by \a dist
  */
-void
+static void
 unclump_push (SPItem *from, SPItem *what, double dist)
 {
     Geom::Point it = unclump_center (what);
     Geom::Point p = unclump_center (from);
     Geom::Point by = dist * Geom::unit_vector (- (p - it));
 
-    Geom::Matrix move = Geom::Translate (by);
+    Geom::Affine move = Geom::Translate (by);
 
     std::map<const gchar *, Geom::Point>::iterator i = c_cache.find(what->getId());
     if ( i != c_cache.end() ) {
         i->second *= move;
     }
 
-    //g_print ("push %s at %g,%g from %g,%g by %g,%g, dist %g\n", SP_OBJECT_ID(what), it[Geom::X],it[Geom::Y], p[Geom::X],p[Geom::Y], by[Geom::X],by[Geom::Y], dist);
+    //g_print ("push %s at %g,%g from %g,%g by %g,%g, dist %g\n", what->getId(), it[Geom::X],it[Geom::Y], p[Geom::X],p[Geom::Y], by[Geom::X],by[Geom::Y], dist);
 
-    sp_item_set_i2d_affine(what, sp_item_i2d_affine(what) * move);
-    sp_item_write_transform(what, SP_OBJECT_REPR(what), what->transform, NULL);
+    what->set_i2d_affine(what->i2dt_affine() * move);
+    what->doWriteTransform(what->getRepr(), what->transform, NULL);
 }
 
 /**
 Moves \a what towards \a to by \a dist
  */
-void
+static void
 unclump_pull (SPItem *to, SPItem *what, double dist)
 {
     Geom::Point it = unclump_center (what);
     Geom::Point p = unclump_center (to);
     Geom::Point by = dist * Geom::unit_vector (p - it);
 
-    Geom::Matrix move = Geom::Translate (by);
+    Geom::Affine move = Geom::Translate (by);
 
     std::map<const gchar *, Geom::Point>::iterator i = c_cache.find(what->getId());
     if ( i != c_cache.end() ) {
         i->second *= move;
     }
 
-    //g_print ("pull %s at %g,%g to %g,%g by %g,%g, dist %g\n", SP_OBJECT_ID(what), it[Geom::X],it[Geom::Y], p[Geom::X],p[Geom::Y], by[Geom::X],by[Geom::Y], dist);
+    //g_print ("pull %s at %g,%g to %g,%g by %g,%g, dist %g\n", what->getId(), it[Geom::X],it[Geom::Y], p[Geom::X],p[Geom::Y], by[Geom::X],by[Geom::Y], dist);
 
-    sp_item_set_i2d_affine(what, sp_item_i2d_affine(what) * move);
-    sp_item_write_transform(what, SP_OBJECT_REPR(what), what->transform, NULL);
+    what->set_i2d_affine(what->i2dt_affine() * move);
+    what->doWriteTransform(what->getRepr(), what->transform, NULL);
 }
 
 
@@ -364,7 +369,7 @@ unclump (GSList *items)
             double dist_closest = unclump_dist (closest, item);
             double dist_farest = unclump_dist (farest, item);
 
-            //g_print ("NEI %d for item %s    closest %s at %g  farest %s at %g  ave %g\n", g_slist_length(nei), SP_OBJECT_ID(item), SP_OBJECT_ID(closest), dist_closest, SP_OBJECT_ID(farest), dist_farest, ave);
+            //g_print ("NEI %d for item %s    closest %s at %g  farest %s at %g  ave %g\n", g_slist_length(nei), item->getId(), closest->getId(), dist_closest, farest->getId(), dist_farest, ave);
 
             if (fabs (ave) < 1e6 && fabs (dist_closest) < 1e6 && fabs (dist_farest) < 1e6) { // otherwise the items are bogus
                 // increase these coefficients to make unclumping more aggressive and less stable
@@ -385,4 +390,4 @@ unclump (GSList *items)
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
