@@ -16,11 +16,9 @@
 #include <2geom/point.h>
 #include "knot.h"
 #include "selection.h"
-#include "axis-manip.h"
-#include "inkscape.h"
 #include "persp3d.h"
 #include "box3d.h"
-#include "persp3d-reference.h"
+#include "ui/control-manager.h" // TODO break enums out separately
 
 class SPBox3D;
 
@@ -67,7 +65,7 @@ public:
         return persp3d_get_VP (_persp, _axis).is_finite();
     }
     inline Geom::Point get_pos() const {
-        g_return_val_if_fail (_persp, Geom::Point (NR_HUGE, NR_HUGE));
+        g_return_val_if_fail (_persp, Geom::Point (Geom::infinity(), Geom::infinity()));
         return persp3d_get_VP (_persp,_axis).affine();
     }
     inline Persp3D * get_perspective() const {
@@ -112,7 +110,7 @@ private:
     Proj::Axis _axis;
 };
 
-class VPDrag;
+struct VPDrag;
 
 struct less_ptr : public std::binary_function<VanishingPoint *, VanishingPoint *, bool> {
     bool operator()(VanishingPoint *vp1, VanishingPoint *vp2) {
@@ -154,6 +152,11 @@ public:
     void updateZOrders();
 
     void printVPs();
+
+private:
+    sigc::connection _moved_connection;
+    sigc::connection _grabbed_connection;
+    sigc::connection _ungrabbed_connection;
 };
 
 struct VPDrag {
@@ -199,7 +202,10 @@ public:
 private:
     //void deselect_all();
 
-    void addLine (Geom::Point p1, Geom::Point p2, guint32 rgba);
+    /**
+     * Create a line from p1 to p2 and add it to the lines list.
+     */
+    void addLine(Geom::Point const &p1, Geom::Point const &p2, Inkscape::CtrlLineType type);
 
     Inkscape::Selection *selection;
     sigc::connection sel_changed_connection;
