@@ -1,9 +1,6 @@
 #ifndef SEEN_SP_VERBS_H
 #define SEEN_SP_VERBS_H
-
-/** \file
- * \brief Frontend to actions
- *
+/*
  * Author:
  *   Lauris Kaplinski <lauris@kaplinski.com>
  *   Ted Gould <ted@gould.cx>
@@ -18,16 +15,29 @@
 
 #include <cstring>
 #include <string>
-#include <string.h>
 #include "config.h"
 #include "require-config.h"   /* HAVE_GTK_WINDOW_FULLSCREEN */
-#include "helper/helper-forward.h"
-#include "forward.h"
 #include <glibmm/ustring.h>
 
-/** \brief This anonymous enum is used to provide a list of the Verbs
-           which are defined staticly in the verb files.  There may be
-           other verbs which are defined dynamically also. */
+struct SPAction;
+class SPDocument;
+
+namespace Inkscape {
+
+class ActionContext;
+
+namespace UI {
+namespace View {
+class View;
+} // namespace View
+} // namespace UI
+} // namespace Inkscape
+
+/**
+ * This anonymous enum is used to provide a list of the Verbs
+ * which are defined staticly in the verb files.  There may be
+ * other verbs which are defined dynamically also.
+ */
 enum {
     /* Header */
     SP_VERB_INVALID,               /**< A dummy verb to represent doing something wrong. */
@@ -41,15 +51,15 @@ enum {
     SP_VERB_FILE_SAVE_A_COPY,      /**< Save a copy of the current file */
     SP_VERB_FILE_PRINT,
     SP_VERB_FILE_VACUUM,
-    SP_VERB_FILE_PRINT_PREVIEW,
     SP_VERB_FILE_IMPORT,
-    SP_VERB_FILE_EXPORT,
+//    SP_VERB_FILE_EXPORT,
     SP_VERB_FILE_IMPORT_FROM_OCAL, /**< Import the file from Open Clip Art Library */
 //    SP_VERB_FILE_EXPORT_TO_OCAL, /**< Export the file to Open Clip Art  Library */
     SP_VERB_FILE_NEXT_DESKTOP,
     SP_VERB_FILE_PREV_DESKTOP,
     SP_VERB_FILE_CLOSE_VIEW,
     SP_VERB_FILE_QUIT,
+    SP_VERB_FILE_TEMPLATES,
     /* Edit */
     SP_VERB_EDIT_UNDO,
     SP_VERB_EDIT_REDO,
@@ -73,18 +83,27 @@ enum {
     SP_VERB_EDIT_UNLINK_CLONE,
     SP_VERB_EDIT_RELINK_CLONE,
     SP_VERB_EDIT_CLONE_SELECT_ORIGINAL,
+    SP_VERB_EDIT_CLONE_ORIGINAL_PATH_LPE,
     SP_VERB_EDIT_SELECTION_2_MARKER,
     SP_VERB_EDIT_SELECTION_2_GUIDES,
     SP_VERB_EDIT_TILE,
     SP_VERB_EDIT_UNTILE,
+    SP_VERB_EDIT_SYMBOL,
+    SP_VERB_EDIT_UNSYMBOL,
     SP_VERB_EDIT_CLEAR_ALL,
     SP_VERB_EDIT_SELECT_ALL,
     SP_VERB_EDIT_SELECT_ALL_IN_ALL_LAYERS,
+    SP_VERB_EDIT_SELECT_SAME_FILL_STROKE,
+    SP_VERB_EDIT_SELECT_SAME_FILL_COLOR,
+    SP_VERB_EDIT_SELECT_SAME_STROKE_COLOR,
+    SP_VERB_EDIT_SELECT_SAME_STROKE_STYLE,
+    SP_VERB_EDIT_SELECT_SAME_OBJECT_TYPE,
     SP_VERB_EDIT_INVERT,
     SP_VERB_EDIT_INVERT_IN_ALL_LAYERS,
     SP_VERB_EDIT_SELECT_NEXT,
     SP_VERB_EDIT_SELECT_PREV,
     SP_VERB_EDIT_DESELECT,
+    SP_VERB_EDIT_DELETE_ALL_GUIDES,
     SP_VERB_EDIT_GUIDES_AROUND_PAGE,
     SP_VERB_EDIT_NEXT_PATHEFFECT_PARAMETER,
     /* Selection */
@@ -115,10 +134,11 @@ enum {
     SP_VERB_SELECTION_SIMPLIFY,
     SP_VERB_SELECTION_REVERSE,
     SP_VERB_SELECTION_TRACE,
+    SP_VERB_SELECTION_PIXEL_ART,
     SP_VERB_SELECTION_CREATE_BITMAP,
     SP_VERB_SELECTION_COMBINE,
     SP_VERB_SELECTION_BREAK_APART,
-    SP_VERB_SELECTION_GRIDTILE,
+    SP_VERB_SELECTION_ARRANGE, // Former SP_VERB_SELECTION_GRIDTILE
     /* Layer */
     SP_VERB_LAYER_NEW,
     SP_VERB_LAYER_RENAME,
@@ -126,6 +146,7 @@ enum {
     SP_VERB_LAYER_PREV,
     SP_VERB_LAYER_MOVE_TO_NEXT,
     SP_VERB_LAYER_MOVE_TO_PREV,
+    SP_VERB_LAYER_MOVE_TO,
     SP_VERB_LAYER_TO_TOP,
     SP_VERB_LAYER_TO_BOTTOM,
     SP_VERB_LAYER_RAISE,
@@ -133,6 +154,13 @@ enum {
     SP_VERB_LAYER_DUPLICATE,
     SP_VERB_LAYER_DELETE,
     SP_VERB_LAYER_SOLO,
+    SP_VERB_LAYER_SHOW_ALL,
+    SP_VERB_LAYER_HIDE_ALL,
+    SP_VERB_LAYER_LOCK_ALL,
+    SP_VERB_LAYER_LOCK_OTHERS,
+    SP_VERB_LAYER_UNLOCK_ALL,
+    SP_VERB_LAYER_TOGGLE_LOCK,
+    SP_VERB_LAYER_TOGGLE_HIDE,
     /* Object */
     SP_VERB_OBJECT_ROTATE_90_CW,
     SP_VERB_OBJECT_ROTATE_90_CCW,
@@ -164,7 +192,9 @@ enum {
     SP_VERB_CONTEXT_CALLIGRAPHIC,
     SP_VERB_CONTEXT_TEXT,
     SP_VERB_CONTEXT_GRADIENT,
+    SP_VERB_CONTEXT_MESH,
     SP_VERB_CONTEXT_ZOOM,
+    SP_VERB_CONTEXT_MEASURE,
     SP_VERB_CONTEXT_DROPPER,
     SP_VERB_CONTEXT_CONNECTOR,
     SP_VERB_CONTEXT_PAINTBUCKET,
@@ -186,7 +216,9 @@ enum {
     SP_VERB_CONTEXT_CALLIGRAPHIC_PREFS,
     SP_VERB_CONTEXT_TEXT_PREFS,
     SP_VERB_CONTEXT_GRADIENT_PREFS,
+    SP_VERB_CONTEXT_MESH_PREFS,
     SP_VERB_CONTEXT_ZOOM_PREFS,
+    SP_VERB_CONTEXT_MEASURE_PREFS,
     SP_VERB_CONTEXT_DROPPER_PREFS,
     SP_VERB_CONTEXT_CONNECTOR_PREFS,
     SP_VERB_CONTEXT_PAINTBUCKET_PREFS,
@@ -200,6 +232,12 @@ enum {
     SP_VERB_TOGGLE_GRID,
     SP_VERB_TOGGLE_GUIDES,
     SP_VERB_TOGGLE_SNAPPING,
+    SP_VERB_TOGGLE_COMMANDS_TOOLBAR,
+    SP_VERB_TOGGLE_SNAP_TOOLBAR,
+    SP_VERB_TOGGLE_TOOL_TOOLBAR,
+    SP_VERB_TOGGLE_TOOLBOX,
+    SP_VERB_TOGGLE_PALETTE,
+    SP_VERB_TOGGLE_STATUSBAR,
     SP_VERB_ZOOM_NEXT,
     SP_VERB_ZOOM_PREV,
     SP_VERB_ZOOM_1_1,
@@ -207,6 +245,7 @@ enum {
     SP_VERB_ZOOM_2_1,
 #ifdef HAVE_GTK_WINDOW_FULLSCREEN
     SP_VERB_FULLSCREEN,
+    SP_VERB_FULLSCREENFOCUS,
 #endif /* HAVE_GTK_WINDOW_FULLSCREEN */
     SP_VERB_FOCUSTOGGLE,
     SP_VERB_VIEW_NEW,
@@ -214,8 +253,11 @@ enum {
     SP_VERB_VIEW_MODE_NORMAL,
     SP_VERB_VIEW_MODE_NO_FILTERS,
     SP_VERB_VIEW_MODE_OUTLINE,
-//    SP_VERB_VIEW_MODE_PRINT_COLORS_PREVIEW,
     SP_VERB_VIEW_MODE_TOGGLE,
+    SP_VERB_VIEW_COLOR_MODE_NORMAL,
+    SP_VERB_VIEW_COLOR_MODE_GRAYSCALE,
+//    SP_VERB_VIEW_COLOR_MODE_PRINT_COLORS_PREVIEW,
+    SP_VERB_VIEW_COLOR_MODE_TOGGLE,
     SP_VERB_VIEW_CMS_TOGGLE,
     SP_VERB_VIEW_ICON_PREVIEW,
     SP_VERB_ZOOM_PAGE,
@@ -229,6 +271,7 @@ enum {
     SP_VERB_DIALOG_FILL_STROKE,
     SP_VERB_DIALOG_GLYPHS,
     SP_VERB_DIALOG_SWATCHES,
+    SP_VERB_DIALOG_SYMBOLS,
     SP_VERB_DIALOG_TRANSFORM,
     SP_VERB_DIALOG_ALIGN_DISTRIBUTE,
     SP_VERB_DIALOG_SPRAY_OPTION,
@@ -239,13 +282,10 @@ enum {
     SP_VERB_DIALOG_FINDREPLACE,
     SP_VERB_DIALOG_SPELLCHECK,
     SP_VERB_DIALOG_DEBUG,
-    SP_VERB_DIALOG_SCRIPT,
     SP_VERB_DIALOG_TOGGLE,
     SP_VERB_DIALOG_CLONETILER,
+    SP_VERB_DIALOG_ATTR,
     SP_VERB_DIALOG_ITEM,
-/*#ifdef WITH_INKBOARD
-    SP_VERB_XMPP_CLIENT,
-#endif*/
     SP_VERB_DIALOG_INPUT,
     SP_VERB_DIALOG_EXTENSIONEDITOR,
     SP_VERB_DIALOG_LAYERS,
@@ -253,6 +293,7 @@ enum {
     SP_VERB_DIALOG_FILTER_EFFECTS,
     SP_VERB_DIALOG_SVG_FONTS,
     SP_VERB_DIALOG_PRINT_COLORS_PREVIEW,
+    SP_VERB_DIALOG_EXPORT,
     /* Help */
     SP_VERB_HELP_ABOUT_EXTENSIONS,
     SP_VERB_HELP_MEMORY,
@@ -263,6 +304,7 @@ enum {
     SP_VERB_TUTORIAL_SHAPES,
     SP_VERB_TUTORIAL_ADVANCED,
     SP_VERB_TUTORIAL_TRACING,
+    SP_VERB_TUTORIAL_TRACING_PIXELART,
     SP_VERB_TUTORIAL_CALLIGRAPHY,
     SP_VERB_TUTORIAL_INTERPOLATE,
     SP_VERB_TUTORIAL_DESIGN,
@@ -284,7 +326,23 @@ enum {
     SP_VERB_EDIT_REMOVE_COLOR_PROFILE,
     /*Scripting*/
     SP_VERB_EDIT_ADD_EXTERNAL_SCRIPT,
+    SP_VERB_EDIT_ADD_EMBEDDED_SCRIPT,
+    SP_VERB_EDIT_EMBEDDED_SCRIPT,
     SP_VERB_EDIT_REMOVE_EXTERNAL_SCRIPT,
+    SP_VERB_EDIT_REMOVE_EMBEDDED_SCRIPT,
+    /* Alignment */
+    SP_VERB_ALIGN_HORIZONTAL_RIGHT_TO_ANCHOR,
+    SP_VERB_ALIGN_HORIZONTAL_LEFT,
+    SP_VERB_ALIGN_HORIZONTAL_CENTER,
+    SP_VERB_ALIGN_HORIZONTAL_RIGHT,
+    SP_VERB_ALIGN_HORIZONTAL_LEFT_TO_ANCHOR,
+    SP_VERB_ALIGN_VERTICAL_BOTTOM_TO_ANCHOR,
+    SP_VERB_ALIGN_VERTICAL_TOP,
+    SP_VERB_ALIGN_VERTICAL_CENTER,
+    SP_VERB_ALIGN_VERTICAL_BOTTOM,
+    SP_VERB_ALIGN_VERTICAL_TOP_TO_ANCHOR,
+    SP_VERB_ALIGN_VERTICAL_HORIZONTAL_CENTER,
+
     /* Footer */
     SP_VERB_LAST
 };
@@ -292,22 +350,24 @@ enum {
 gchar *sp_action_get_title (const SPAction *action);
 
 #include <map>
+#include <vector>
 
 namespace Inkscape {
 
-/** \brief A class to represent things the user can do.  In many ways
-           these are 'action factories' as they are used to create
-           individual actions that are based on a given view.
-*/
+/**
+ * A class to represent things the user can do.  In many ways
+ * these are 'action factories' as they are used to create
+ * individual actions that are based on a given view.
+ */
 class Verb {
 private:
-    /** \brief An easy to use defition of the table of verbs by code. */
+    /** An easy to use defition of the table of verbs by code. */
     typedef std::map<unsigned int, Inkscape::Verb *> VerbTable;
 
-    /** \brief A table of all the dynamically created verbs. */
+    /** A table of all the dynamically created verbs. */
     static VerbTable _verbs;
 
-    /** \brief The table of statically created verbs which are mostly
+    /** The table of statically created verbs which are mostly
                'base verbs'. */
     static Verb * _base_verbs[SP_VERB_LAST + 1];
     /* Plus one because there is an entry for SP_VERB_LAST */
@@ -326,131 +386,164 @@ private:
         }
     };
 
-    /** \brief An easy to use definition of the table of verbs by ID. */
+    /** An easy to use definition of the table of verbs by ID. */
     typedef std::map<gchar const *, Verb *, ltstr> VerbIDTable;
 
-    /** \brief Quick lookup of verbs by ID */
+    /** Quick lookup of verbs by ID */
     static VerbIDTable _verb_ids;
 
-    /** \brief A simple typedef to make using the action table easier. */
+    /** A simple typedef to make using the action table easier. */
     typedef std::map<Inkscape::UI::View::View *, SPAction *> ActionTable;
-    /** \brief A list of all the actions that have been created for this
+    /** A list of all the actions that have been created for this
                verb.  It is referenced by the view that they are created for. */
     ActionTable * _actions;
 
-    /** \brief A unique textual ID for the verb. */
+    /** A unique textual ID for the verb. */
     gchar const * _id;
 
-    /** \brief The full name of the verb.  (shown on menu entries) */
+    /** The full name of the verb.  (shown on menu entries) */
     gchar const * _name;
 
-    /** \brief Tooltip for the verb. */
+    /** Tooltip for the verb. */
     gchar const * _tip;
+
     gchar * _full_tip; // includes shortcut
+
     unsigned int _shortcut;
 
-    /** \brief Name of the image that represents the verb. */
+    /** Name of the image that represents the verb. */
     gchar const * _image;
 
-    /** \brief Unique numerical representation of the verb.  In most cases
-               it is a value from the anonymous enum at the top of this
-               file. */
+    /**
+     * Unique numerical representation of the verb.  In most cases
+     * it is a value from the anonymous enum at the top of this
+     * file.
+     */
     unsigned int  _code;
 
-    /** \brief Whether this verb is set to default to sensitive or
-               insensitive when new actions are created. */
+    /** Name of the group the verb belongs to. */
+    gchar const * _group;
+
+    /**
+     * Whether this verb is set to default to sensitive or
+     * insensitive when new actions are created.
+     */
     bool _default_sensitive;
 
 protected:
-    /** \brief Allows for preliminary setting of the \c _default_sensitive
-               value without effecting existing actions
-        \param in_val New value
 
-        This function is mostly used at initialization where there are
-        not actions to effect.  I can't think of another case where it
-        should be used.
-    */
+    /**
+     * Allows for preliminary setting of the \c _default_sensitive
+     * value without effecting existing actions.
+     * This function is mostly used at initialization where there are
+     * not actions to effect.  I can't think of another case where it
+     * should be used.
+     *
+     * @param in_val New value.
+     */
     bool set_default_sensitive (bool in_val) { return _default_sensitive = in_val; }
+
 public:
-    /** \brief Accessor to get the \c _default_sensitive value */
+
+    /** Accessor to get the \c _default_sensitive value. */
     bool get_default_sensitive (void) { return _default_sensitive; }
 
-public:
-    /** \brief Accessor to get the internal variable. */
+    /** Accessor to get the internal variable. */
     unsigned int get_code (void) { return _code; }
-    /** \brief Accessor to get the internal variable. */
+
+    /** Accessor to get the internal variable. */
     gchar const * get_id (void) { return _id; }
-    /** \brief Accessor to get the internal variable. */
+
+    /** Accessor to get the internal variable. */
     gchar const * get_name (void) { return _name; }
-    /** \brief Accessor to get the internal variable. */
+
+    /** Accessor to get the internal variable. */
+    gchar const * get_short_tip (void) { return _tip; };
+
+    /** Accessor to get the internal variable. */
     gchar const * get_tip (void) ;
-    /** \brief Accessor to get the internal variable. */
+
+    /** Accessor to get the internal variable. */
     gchar const * get_image (void) { return _image; }
 
-    /** \brief Set the name after initialization. */
+    /** Get the verbs group */
+    gchar const * get_group (void) { return _group; }
+
+    /** Set the name after initialization. */
     gchar const * set_name (gchar const * name) { _name = name; return _name; }
-    /** \brief Set the tooltip after initialization. */
+
+    /** Set the tooltip after initialization. */
     gchar const * set_tip (gchar const * tip) { _tip = tip; return _tip; }
 
+
 protected:
-    SPAction * make_action_helper (Inkscape::UI::View::View * view, SPActionEventVector * vector, void * in_pntr = NULL);
-    virtual SPAction * make_action (Inkscape::UI::View::View * view);
+    SPAction *make_action_helper (Inkscape::ActionContext const & context, void (*perform_fun)(SPAction *, void *), void *in_pntr = NULL);
+    virtual SPAction *make_action (Inkscape::ActionContext const & context);
 
 public:
-    /** \brief Inititalizes the Verb with the parameters
-        \param code  Goes to \c _code
-        \param id    Goes to \c _id
-        \param name  Goes to \c _name
-        \param tip   Goes to \c _tip
-        \param image Goes to \c _image
 
-        This function also sets \c _actions to NULL.
-
-        \warning NO DATA IS COPIED BY CALLING THIS FUNCTION.
-
-        In many respects this is very bad object oriented design, but it
-        is done for a reason.  All verbs today are of two types: 1) static
-        or 2) created for extension.  In the static case all of the
-        strings are constants in the code, and thus don't really need to
-        be copied.  In the extensions case the strings are identical to
-        the ones already created in the extension object, copying them
-        would be a waste of memory.
-    */
+    /**
+     * Inititalizes the Verb with the parameters.
+     *
+     * This function also sets \c _actions to NULL.
+     *
+     * @warning NO DATA IS COPIED BY CALLING THIS FUNCTION.
+     *
+     * In many respects this is very bad object oriented design, but it
+     * is done for a reason.  All verbs today are of two types: 1) static
+     * or 2) created for extension.  In the static case all of the
+     * strings are constants in the code, and thus don't really need to
+     * be copied.  In the extensions case the strings are identical to
+     * the ones already created in the extension object, copying them
+     * would be a waste of memory.
+     *
+     * @param code  Goes to \c _code.
+     * @param id    Goes to \c _id.
+     * @param name  Goes to \c _name.
+     * @param tip   Goes to \c _tip.
+     * @param image Goes to \c _image.
+     */
     Verb(const unsigned int code,
          gchar const * id,
          gchar const * name,
          gchar const * tip,
-         gchar const * image) :
+         gchar const * image,
+         gchar const * group) :
         _actions(0),
         _id(id),
         _name(name),
         _tip(tip),
         _full_tip(0),
+        _shortcut(0),
         _image(image),
         _code(code),
+        _group(group),
         _default_sensitive(true)
     {
         _verbs.insert(VerbTable::value_type(_code, this));
         _verb_ids.insert(VerbIDTable::value_type(_id, this));
     }
-    Verb (gchar const * id, gchar const * name, gchar const * tip, gchar const * image);
+    Verb (gchar const * id, gchar const * name, gchar const * tip, gchar const * image, gchar const * group);
     virtual ~Verb (void);
 
-    SPAction * get_action(Inkscape::UI::View::View * view);
+    SPAction * get_action(Inkscape::ActionContext const & context);
 
 private:
     static Verb * get_search (unsigned int code);
 public:
-    /** \brief A function to turn a code into a verb.
-        \param  code  The code to be translated
-        \return A pointer to a verb object or a NULL if not found.
 
-        This is an inline function to translate the codes which are
-        static quickly.  This should optimize into very quick code
-        everywhere which hard coded \c codes are used.  In the case
-        where the \c code is not static the \c get_search function
-        is used.
-    */
+    /**
+     * A function to turn a code into a verb.
+     *
+     * This is an inline function to translate the codes which are
+     * static quickly.  This should optimize into very quick code
+     * everywhere which hard coded \c codes are used.  In the case
+     * where the \c code is not static the \c get_search function
+     * is used.
+     *
+     * @param  code  The code to be translated
+     * @return A pointer to a verb object or a NULL if not found.
+     */
     static Verb * get (unsigned int code) {
         if (code <= SP_VERB_LAST) {
             return _base_verbs[code];
@@ -459,6 +552,11 @@ public:
         }
     }
     static Verb * getbyid (gchar const * id);
+    
+    /**
+     * Print a message to stderr indicating that this verb needs a GUI to run
+     */
+    static bool ensure_desktop_valid(SPAction *action);
 
     static void delete_all_view (Inkscape::UI::View::View * view);
     void delete_view (Inkscape::UI::View::View * view);
@@ -468,17 +566,22 @@ public:
 
 // Yes, multiple public, protected and private sections are bad. We'll clean that up later
 protected:
-    /** \brief Returns the size of the internal base verb array.
-        \return The size in elements of the internal base array.
 
-        This is an inline function intended for testing. This should normally not be used.
-        For testing, a subclass that returns this value can be created to verify that the
-        length matches the enum values, etc.
-    */
+    /**
+     * Returns the size of the internal base verb array.
+     *
+     * This is an inline function intended for testing. This should normally not be used.
+     * For testing, a subclass that returns this value can be created to verify that the
+     * length matches the enum values, etc.
+     *
+     * @return The size in elements of the internal base array.
+     */
     static int _getBaseListSize(void) {return G_N_ELEMENTS(_base_verbs);}
 
 public:
     static void list (void);
+    static std::vector<Inkscape::Verb *>getList (void);
+
 }; /* Verb class */
 
 
