@@ -18,6 +18,7 @@ namespace Inkscape {
 namespace Widgets {
 
 Gtk::Widget *createStyleWidget( FillOrStroke kind );
+Gtk::Widget *createStrokeStyleWidget( );
 
 } // namespace Widgets
 } // namespace Inkscape
@@ -33,4 +34,4 @@ Gtk::Widget *createStyleWidget( FillOrStroke kind );
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
