@@ -14,7 +14,7 @@
  */
 
 #include <gtk/gtk.h>
-#include "../libnr/nr-matrix.h"
+
 class SPGradient;
 
 #include <glib.h>
@@ -23,15 +23,14 @@ class SPGradient;
 #include <sigc++/connection.h>
 
 #define SP_TYPE_GRADIENT_IMAGE (sp_gradient_image_get_type ())
-#define SP_GRADIENT_IMAGE(o) (GTK_CHECK_CAST ((o), SP_TYPE_GRADIENT_IMAGE, SPGradientImage))
-#define SP_GRADIENT_IMAGE_CLASS(k) (GTK_CHECK_CLASS_CAST ((k), SP_TYPE_GRADIENT_IMAGE, SPGradientImageClass))
-#define SP_IS_GRADIENT_IMAGE(o) (GTK_CHECK_TYPE ((o), SP_TYPE_GRADIENT_IMAGE))
-#define SP_IS_GRADIENT_IMAGE_CLASS(k) (GTK_CHECK_CLASS_TYPE ((k), SP_TYPE_GRADIENT_IMAGE))
+#define SP_GRADIENT_IMAGE(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SP_TYPE_GRADIENT_IMAGE, SPGradientImage))
+#define SP_GRADIENT_IMAGE_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), SP_TYPE_GRADIENT_IMAGE, SPGradientImageClass))
+#define SP_IS_GRADIENT_IMAGE(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SP_TYPE_GRADIENT_IMAGE))
+#define SP_IS_GRADIENT_IMAGE_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), SP_TYPE_GRADIENT_IMAGE))
 
 struct SPGradientImage {
 	GtkWidget widget;
 	SPGradient *gradient;
-	guchar *px;
 
 	sigc::connection release_connection;
 	sigc::connection modified_connection;
@@ -41,11 +40,10 @@ struct SPGradientImageClass {
 	GtkWidgetClass parent_class;
 };
 
-GtkType sp_gradient_image_get_type (void);
+GType sp_gradient_image_get_type (void);
 
 GtkWidget *sp_gradient_image_new (SPGradient *gradient);
+GdkPixbuf *sp_gradient_to_pixbuf (SPGradient *gr, int width, int height);
 void sp_gradient_image_set_gradient (SPGradientImage *gi, SPGradient *gr);
 
-
-
 #endif
