@@ -6,21 +6,38 @@
  *
  * Author:
  *   Lauris Kaplinski <lauris@kaplinski.com>
+ *   Jon A. Cruz <jon@joncruz.org>
  *
  * Copyright (C) 2002 Lauris Kaplinski
+ * Copyright (C) 2010 Authors
  *
  * Released under GNU GPL, read the file 'COPYING' for more information
  */
 
-#include <glib.h>
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
+#if GLIBMM_DISABLE_DEPRECATED && HAVE_GLIBMM_THREADS_H
+#include <glibmm/threads.h>
+#endif
+
+#include <gtkmm/widget.h>
 #include "icon-size.h"
 
-#define SP_TYPE_ICON (sp_icon_get_type ())
-#define SP_ICON(o) (GTK_CHECK_CAST ((o), SP_TYPE_ICON, SPIcon))
-#define SP_IS_ICON(o) (GTK_CHECK_TYPE ((o), SP_TYPE_ICON))
+#define SP_TYPE_ICON sp_icon_get_type()
+#define SP_ICON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SP_TYPE_ICON, SPIcon))
+#define SP_IS_ICON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SP_TYPE_ICON))
 
-#include <gtk/gtk.h>
+namespace Glib {
+class ustring;
+}
+
+struct SPIconClass {
+    GtkWidgetClass parent_class;
+};
+
+GType sp_icon_get_type() G_GNUC_CONST;
 
 struct SPIcon {
     GtkWidget widget;
@@ -30,18 +47,13 @@ struct SPIcon {
     gchar *name;
 
     GdkPixbuf *pb;
+
+    friend class SPIconImpl;
 };
 
-struct SPIconClass {
-    GtkWidgetClass parent_class;
-};
-
-GType sp_icon_get_type (void);
 
 GtkWidget *sp_icon_new( Inkscape::IconSize size, const gchar *name );
-
-#include <glibmm/ustring.h>
-#include <gtkmm/widget.h>
+GdkPixbuf *sp_pixbuf_new( Inkscape::IconSize size, const gchar *name );
 
 // Might return a wrapped SPIcon, or Gtk::Image
 Gtk::Widget *sp_icon_get_icon( const Glib::ustring &oid, Inkscape::IconSize size = Inkscape::ICON_SIZE_BUTTON );
@@ -54,3 +66,14 @@ namespace Inkscape {
 }
 
 #endif // SEEN_SP_ICON_H
+
+/*
+  Local Variables:
+  mode:c++
+  c-file-style:"stroustrup"
+  c-file-offsets:((innamespace . 0)(inline-open . 0)(case-label . +))
+  indent-tabs-mode:nil
+  fill-column:99
+  End:
+*/
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
