@@ -9,15 +9,16 @@
 #include "sp-color-scales.h"
 #include "sp-color-icc-selector.h"
 #include "../svg/svg-icc-color.h"
+#include "ui/widget/gimpcolorwheel.h"
 
 G_BEGIN_DECLS
 
 static void sp_color_wheel_selector_class_init (SPColorWheelSelectorClass *klass);
 static void sp_color_wheel_selector_init (SPColorWheelSelector *cs);
-static void sp_color_wheel_selector_destroy (GtkObject *object);
+static void sp_color_wheel_selector_dispose(GObject *object);
 
 static void sp_color_wheel_selector_show_all (GtkWidget *widget);
-static void sp_color_wheel_selector_hide_all (GtkWidget *widget);
+static void sp_color_wheel_selector_hide(GtkWidget *widget);
 
 
 G_END_DECLS
@@ -53,27 +54,22 @@ sp_color_wheel_selector_get_type (void)
     return type;
 }
 
-static void
-sp_color_wheel_selector_class_init (SPColorWheelSelectorClass *klass)
+static void sp_color_wheel_selector_class_init(SPColorWheelSelectorClass *klass)
 {
     static const gchar* nameset[] = {N_("Wheel"), 0};
-    GtkObjectClass *object_class;
-    GtkWidgetClass *widget_class;
-    SPColorSelectorClass *selector_class;
-
-    object_class = (GtkObjectClass *) klass;
-    widget_class = (GtkWidgetClass *) klass;
-    selector_class = SP_COLOR_SELECTOR_CLASS (klass);
+    GObjectClass   *object_class = G_OBJECT_CLASS(klass);
+    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);
+    SPColorSelectorClass *selector_class = SP_COLOR_SELECTOR_CLASS (klass);
 
     parent_class = SP_COLOR_SELECTOR_CLASS (g_type_class_peek_parent (klass));
 
     selector_class->name = nameset;
     selector_class->submode_count = 1;
 
-    object_class->destroy = sp_color_wheel_selector_destroy;
+    object_class->dispose = sp_color_wheel_selector_dispose;
 
     widget_class->show_all = sp_color_wheel_selector_show_all;
-    widget_class->hide_all = sp_color_wheel_selector_hide_all;
+    widget_class->hide = sp_color_wheel_selector_hide;
 }
 
 ColorWheelSelector::ColorWheelSelector( SPColorSelector* csel )
@@ -82,6 +78,7 @@ ColorWheelSelector::ColorWheelSelector( SPColorSelector* csel )
       _dragging( FALSE ),
       _adj(0),
       _wheel(0),
+      _slider(0),
       _sbtn(0),
       _label(0)
 {
@@ -107,41 +104,75 @@ void sp_color_wheel_selector_init (SPColorWheelSelector *cs)
 
 void ColorWheelSelector::init()
 {
-    GtkWidget *t;
     gint row = 0;
 
     _updating = FALSE;
     _dragging = FALSE;
 
-    _tt = gtk_tooltips_new();
+#if GTK_CHECK_VERSION(3,0,0)
+    GtkWidget *t = gtk_grid_new();
+#else
+    GtkWidget *t = gtk_table_new (5, 3, FALSE);
+#endif
 
-    t = gtk_table_new (5, 3, FALSE);
     gtk_widget_show (t);
     gtk_box_pack_start (GTK_BOX (_csel), t, TRUE, TRUE, 0);
 
     /* Create components */
     row = 0;
 
-    _wheel = sp_color_wheel_new ();
-    gtk_widget_show (_wheel);
-    gtk_table_attach (GTK_TABLE (t), _wheel, 0, 3, row, row + 1, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), XPAD, YPAD);
+    _wheel = gimp_color_wheel_new();
+    gtk_widget_show( _wheel );
+
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_widget_set_halign(_wheel, GTK_ALIGN_FILL);
+    gtk_widget_set_valign(_wheel, GTK_ALIGN_FILL);
+    gtk_widget_set_hexpand(_wheel, TRUE);
+    gtk_widget_set_vexpand(_wheel, TRUE);
+    gtk_grid_attach(GTK_GRID(t), _wheel, 0, row, 3, 1);
+#else
+    gtk_table_attach(GTK_TABLE(t), _wheel, 0, 3, row, row + 1, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
+#endif
 
     row++;
 
     /* Label */
-    _label = gtk_label_new_with_mnemonic (_("_A"));
+    _label = gtk_label_new_with_mnemonic (_("_A:"));
     gtk_misc_set_alignment (GTK_MISC (_label), 1.0, 0.5);
     gtk_widget_show (_label);
+
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_widget_set_margin_left(_label, XPAD);
+    gtk_widget_set_margin_right(_label, XPAD);
+    gtk_widget_set_margin_top(_label, YPAD);
+    gtk_widget_set_margin_bottom(_label, YPAD);
+    gtk_widget_set_halign(_label, GTK_ALIGN_FILL);
+    gtk_widget_set_valign(_label, GTK_ALIGN_FILL);
+    gtk_grid_attach(GTK_GRID(t), _label, 0, row, 1, 1);
+#else
     gtk_table_attach (GTK_TABLE (t), _label, 0, 1, row, row + 1, GTK_FILL, GTK_FILL, XPAD, YPAD);
+#endif
 
     /* Adjustment */
-    _adj = (GtkAdjustment *) gtk_adjustment_new (0.0, 0.0, 255.0, 1.0, 10.0, 10.0);
+    _adj = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 255.0, 1.0, 10.0, 10.0));
 
     /* Slider */
     _slider = sp_color_slider_new (_adj);
-    gtk_tooltips_set_tip (_tt, _slider, _("Alpha (opacity)"), NULL);
+    gtk_widget_set_tooltip_text (_slider, _("Alpha (opacity)"));
     gtk_widget_show (_slider);
-    gtk_table_attach (GTK_TABLE (t), _slider, 1, 2, row, row + 1, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)GTK_FILL, XPAD, YPAD);
+
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_widget_set_margin_left(_slider, XPAD);
+    gtk_widget_set_margin_right(_slider, XPAD);
+    gtk_widget_set_margin_top(_slider, YPAD);
+    gtk_widget_set_margin_bottom(_slider, YPAD);
+    gtk_widget_set_hexpand(_slider, TRUE);
+    gtk_widget_set_halign(_slider, GTK_ALIGN_FILL);
+    gtk_widget_set_valign(_slider, GTK_ALIGN_FILL);
+    gtk_grid_attach(GTK_GRID(t), _slider, 1, row, 1, 1);
+#else
+    gtk_table_attach(GTK_TABLE (t), _slider, 1, 2, row, row + 1, (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), GTK_FILL, XPAD, YPAD);
+#endif
 
     sp_color_slider_set_colors (SP_COLOR_SLIDER (_slider),
                                 SP_RGBA32_F_COMPOSE (1.0, 1.0, 1.0, 0.0),
@@ -151,32 +182,42 @@ void ColorWheelSelector::init()
 
     /* Spinbutton */
     _sbtn = gtk_spin_button_new (GTK_ADJUSTMENT (_adj), 1.0, 0);
-    gtk_tooltips_set_tip (_tt, _sbtn, _("Alpha (opacity)"), NULL);
+    gtk_widget_set_tooltip_text (_sbtn, _("Alpha (opacity)"));
     sp_dialog_defocus_on_enter (_sbtn);
     gtk_label_set_mnemonic_widget (GTK_LABEL(_label), _sbtn);
     gtk_widget_show (_sbtn);
+
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_widget_set_margin_left(_sbtn, XPAD);
+    gtk_widget_set_margin_right(_sbtn, XPAD);
+    gtk_widget_set_margin_top(_sbtn, YPAD);
+    gtk_widget_set_margin_bottom(_sbtn, YPAD);
+    gtk_widget_set_halign(_sbtn, GTK_ALIGN_CENTER);
+    gtk_widget_set_valign(_sbtn, GTK_ALIGN_CENTER);
+    gtk_grid_attach(GTK_GRID(t), _sbtn, 2, row, 1, 1);
+#else
     gtk_table_attach (GTK_TABLE (t), _sbtn, 2, 3, row, row + 1, (GtkAttachOptions)0, (GtkAttachOptions)0, XPAD, YPAD);
+#endif
 
     /* Signals */
-    gtk_signal_connect (GTK_OBJECT (_adj), "value_changed",
-                        GTK_SIGNAL_FUNC (_adjustmentChanged), _csel);
+    g_signal_connect (G_OBJECT (_adj), "value_changed",
+                        G_CALLBACK (_adjustmentChanged), _csel);
 
-    gtk_signal_connect (GTK_OBJECT (_slider), "grabbed",
-                        GTK_SIGNAL_FUNC (_sliderGrabbed), _csel);
-    gtk_signal_connect (GTK_OBJECT (_slider), "released",
-                        GTK_SIGNAL_FUNC (_sliderReleased), _csel);
-    gtk_signal_connect (GTK_OBJECT (_slider), "changed",
-                        GTK_SIGNAL_FUNC (_sliderChanged), _csel);
+    g_signal_connect (G_OBJECT (_slider), "grabbed",
+                        G_CALLBACK (_sliderGrabbed), _csel);
+    g_signal_connect (G_OBJECT (_slider), "released",
+                        G_CALLBACK (_sliderReleased), _csel);
+    g_signal_connect (G_OBJECT (_slider), "changed",
+                        G_CALLBACK (_sliderChanged), _csel);
 
-    gtk_signal_connect (GTK_OBJECT(_wheel), "changed",
-                        GTK_SIGNAL_FUNC (_wheelChanged), _csel);
+    g_signal_connect( G_OBJECT(_wheel), "changed",
+                        G_CALLBACK (_wheelChanged), _csel );
 }
 
-static void
-sp_color_wheel_selector_destroy (GtkObject *object)
+static void sp_color_wheel_selector_dispose(GObject *object)
 {
-    if (((GtkObjectClass *) (parent_class))->destroy)
-        (* ((GtkObjectClass *) (parent_class))->destroy) (object);
+    if ((G_OBJECT_CLASS(parent_class))->dispose)
+        (* (G_OBJECT_CLASS(parent_class))->dispose) (object);
 }
 
 static void
@@ -185,18 +226,14 @@ sp_color_wheel_selector_show_all (GtkWidget *widget)
     gtk_widget_show (widget);
 }
 
-static void
-sp_color_wheel_selector_hide_all (GtkWidget *widget)
+static void sp_color_wheel_selector_hide(GtkWidget *widget)
 {
-    gtk_widget_hide (widget);
+    gtk_widget_hide(widget);
 }
 
-GtkWidget *
-sp_color_wheel_selector_new (void)
+GtkWidget *sp_color_wheel_selector_new()
 {
-    SPColorWheelSelector *csel;
-
-    csel = (SPColorWheelSelector*)gtk_type_new (SP_TYPE_COLOR_WHEEL_SELECTOR);
+    SPColorWheelSelector *csel = SP_COLOR_WHEEL_SELECTOR(g_object_new (SP_TYPE_COLOR_WHEEL_SELECTOR, NULL));
 
     return GTK_WIDGET (csel);
 }
@@ -204,7 +241,7 @@ sp_color_wheel_selector_new (void)
 /* Helpers for setting color value */
 
 static void preserve_icc(SPColor *color, SPColorWheelSelector *cs){
-    ColorSelector* selector = (ColorSelector*)(SP_COLOR_SELECTOR(cs)->base);
+    ColorSelector* selector = static_cast<ColorSelector*>(SP_COLOR_SELECTOR(cs)->base);
     color->icc = selector->getColor().icc ? new SVGICCColor(*selector->getColor().icc) : 0;
 }
 
@@ -214,7 +251,11 @@ void ColorWheelSelector::_colorChanged()
     g_message("ColorWheelSelector::_colorChanged( this=%p, %f, %f, %f,   %f)", this, color.v.c[0], color.v.c[1], color.v.c[2], alpha );
 #endif
     _updating = TRUE;
-    sp_color_wheel_set_color( SP_COLOR_WHEEL( _wheel ), &_color );
+    {
+        float hsv[3] = {0,0,0};
+        sp_color_rgb_to_hsv_floatv(hsv, _color.v.c[0], _color.v.c[1], _color.v.c[2]);
+        gimp_color_wheel_set_color( GIMP_COLOR_WHEEL(_wheel), hsv[0], hsv[1], hsv[2] );
+    }
 
     guint32 start = _color.toRGBA32( 0x00 );
     guint32 mid = _color.toRGBA32( 0x7f );
@@ -231,11 +272,14 @@ void ColorWheelSelector::_adjustmentChanged( GtkAdjustment *adjustment, SPColorW
 {
 // TODO check this. It looks questionable:
     // if a value is entered between 0 and 1 exclusive, normalize it to (int) 0..255  or 0..100
-    if (adjustment->value > 0.0 && adjustment->value < 1.0) {
-        gtk_adjustment_set_value( adjustment, floor ((adjustment->value) * adjustment->upper + 0.5) );
+    gdouble value = gtk_adjustment_get_value (adjustment);
+    gdouble upper = gtk_adjustment_get_upper (adjustment);
+    
+    if (value > 0.0 && value < 1.0) {
+        gtk_adjustment_set_value( adjustment, floor (value * upper + 0.5) );
     }
 
-    ColorWheelSelector* wheelSelector = (ColorWheelSelector*)(SP_COLOR_SELECTOR(cs)->base);
+    ColorWheelSelector* wheelSelector = static_cast<ColorWheelSelector*>(SP_COLOR_SELECTOR(cs)->base);
     if (wheelSelector->_updating) return;
 
     wheelSelector->_updating = TRUE;
@@ -249,7 +293,7 @@ void ColorWheelSelector::_adjustmentChanged( GtkAdjustment *adjustment, SPColorW
 void ColorWheelSelector::_sliderGrabbed( SPColorSlider *slider, SPColorWheelSelector *cs )
 {
     (void)slider;
-    ColorWheelSelector* wheelSelector = (ColorWheelSelector*)(SP_COLOR_SELECTOR(cs)->base);
+    ColorWheelSelector* wheelSelector = static_cast<ColorWheelSelector*>(SP_COLOR_SELECTOR(cs)->base);
     if (!wheelSelector->_dragging) {
         wheelSelector->_dragging = TRUE;
         wheelSelector->_grabbed();
@@ -262,7 +306,7 @@ void ColorWheelSelector::_sliderGrabbed( SPColorSlider *slider, SPColorWheelSele
 void ColorWheelSelector::_sliderReleased( SPColorSlider *slider, SPColorWheelSelector *cs )
 {
     (void)slider;
-    ColorWheelSelector* wheelSelector = (ColorWheelSelector*)(SP_COLOR_SELECTOR(cs)->base);
+    ColorWheelSelector* wheelSelector = static_cast<ColorWheelSelector*>(SP_COLOR_SELECTOR(cs)->base);
     if (wheelSelector->_dragging) {
         wheelSelector->_dragging = FALSE;
         wheelSelector->_released();
@@ -275,18 +319,25 @@ void ColorWheelSelector::_sliderReleased( SPColorSlider *slider, SPColorWheelSel
 void ColorWheelSelector::_sliderChanged( SPColorSlider *slider, SPColorWheelSelector *cs )
 {
     (void)slider;
-    ColorWheelSelector* wheelSelector = (ColorWheelSelector*)(SP_COLOR_SELECTOR(cs)->base);
+    ColorWheelSelector* wheelSelector = static_cast<ColorWheelSelector*>(SP_COLOR_SELECTOR(cs)->base);
 
     preserve_icc(&wheelSelector->_color, cs);
     wheelSelector->_updateInternals( wheelSelector->_color, ColorScales::getScaled( wheelSelector->_adj ), wheelSelector->_dragging );
 }
 
-void ColorWheelSelector::_wheelChanged( SPColorWheel *wheel, SPColorWheelSelector *cs )
+void ColorWheelSelector::_wheelChanged( GimpColorWheel *wheel, SPColorWheelSelector *cs )
 {
-    ColorWheelSelector* wheelSelector = (ColorWheelSelector*)(SP_COLOR_SELECTOR(cs)->base);
-    SPColor color;
+    ColorWheelSelector* wheelSelector = static_cast<ColorWheelSelector*>(SP_COLOR_SELECTOR(cs)->base);
 
-    sp_color_wheel_get_color( wheel, &color );
+    gdouble h = 0;
+    gdouble s = 0;
+    gdouble v = 0;
+    gimp_color_wheel_get_color( wheel, &h, &s, &v );
+    
+    float rgb[3] = {0,0,0};
+    sp_color_hsv_to_rgb_floatv (rgb, h, s, v); 
+
+    SPColor color(rgb[0], rgb[1], rgb[2]);
 
     guint32 start = color.toRGBA32( 0x00 );
     guint32 mid = color.toRGBA32( 0x7f );
@@ -295,7 +346,7 @@ void ColorWheelSelector::_wheelChanged( SPColorWheel *wheel, SPColorWheelSelecto
     sp_color_slider_set_colors (SP_COLOR_SLIDER(wheelSelector->_slider), start, mid, end);
 
     preserve_icc(&color, cs);
-    wheelSelector->_updateInternals( color, wheelSelector->_alpha, sp_color_wheel_is_adjusting( wheel ) );
+    wheelSelector->_updateInternals( color, wheelSelector->_alpha, gimp_color_wheel_is_adjusting(wheel) );
 }
 
 
@@ -308,4 +359,4 @@ void ColorWheelSelector::_wheelChanged( SPColorWheel *wheel, SPColorWheelSelecto
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
