@@ -4,17 +4,15 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 
-#include "../color.h"
-#include "sp-color-wheel.h"
 #include "sp-color-slider.h"
 #include "sp-color-selector.h"
 
 
 
+typedef struct _GimpColorWheel GimpColorWheel;
 struct SPColorWheelSelector;
 struct SPColorWheelSelectorClass;
 
-
 class ColorWheelSelector: public ColorSelector
 {
 public:
@@ -31,7 +29,7 @@ protected:
     static void _sliderGrabbed( SPColorSlider *slider, SPColorWheelSelector *cs );
     static void _sliderReleased( SPColorSlider *slider, SPColorWheelSelector *cs );
     static void _sliderChanged( SPColorSlider *slider, SPColorWheelSelector *cs );
-    static void _wheelChanged( SPColorWheel* wheel, SPColorWheelSelector *cs );
+    static void _wheelChanged( GimpColorWheel *wheel, SPColorWheelSelector *cs );
 
     static void _fooChanged( GtkWidget foo, SPColorWheelSelector *cs );
 
@@ -39,13 +37,11 @@ protected:
 
     gboolean _updating : 1;
     gboolean _dragging : 1;
-    GtkAdjustment* _adj; /* Channel adjustment */
+    GtkAdjustment* _adj; // Channel adjustment
     GtkWidget* _wheel;
     GtkWidget* _slider;
-    GtkWidget* _sbtn; /* Spinbutton */
-    GtkWidget* _label; /* Label */
-
-    GtkTooltips* _tt; /* tooltip container */
+    GtkWidget* _sbtn; // Spinbutton
+    GtkWidget* _label; // Label
 
 private:
     // By default, disallow copy constructor and assignment operator
@@ -56,10 +52,10 @@ private:
 
 
 #define SP_TYPE_COLOR_WHEEL_SELECTOR (sp_color_wheel_selector_get_type ())
-#define SP_COLOR_WHEEL_SELECTOR(o) (GTK_CHECK_CAST ((o), SP_TYPE_COLOR_WHEEL_SELECTOR, SPColorWheelSelector))
-#define SP_COLOR_WHEEL_SELECTOR_CLASS(k) (GTK_CHECK_CLASS_CAST ((k), SP_TYPE_COLOR_WHEEL_SELECTOR, SPColorWheelSelectorClass))
-#define SP_IS_COLOR_WHEEL_SELECTOR(o) (GTK_CHECK_TYPE ((o), SP_TYPE_COLOR_WHEEL_SELECTOR))
-#define SP_IS_COLOR_WHEEL_SELECTOR_CLASS(k) (GTK_CHECK_CLASS_TYPE ((k), SP_TYPE_COLOR_WHEEL_SELECTOR))
+#define SP_COLOR_WHEEL_SELECTOR(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SP_TYPE_COLOR_WHEEL_SELECTOR, SPColorWheelSelector))
+#define SP_COLOR_WHEEL_SELECTOR_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), SP_TYPE_COLOR_WHEEL_SELECTOR, SPColorWheelSelectorClass))
+#define SP_IS_COLOR_WHEEL_SELECTOR(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SP_TYPE_COLOR_WHEEL_SELECTOR))
+#define SP_IS_COLOR_WHEEL_SELECTOR_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), SP_TYPE_COLOR_WHEEL_SELECTOR))
 
 struct SPColorWheelSelector {
     SPColorSelector parent;
@@ -86,4 +82,4 @@ GtkWidget *sp_color_wheel_selector_new (void);
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
