@@ -13,20 +13,15 @@
  */
 
 #include <config.h>
-
 #include <stdio.h>
-
 #include <gtk/gtk.h>
-#include "../xml/repr.h"
-
 #include <glib.h>
 
 
-
 #define SP_TYPE_XMLVIEW_CONTENT (sp_xmlview_content_get_type ())
-#define SP_XMLVIEW_CONTENT(o) (GTK_CHECK_CAST ((o), SP_TYPE_XMLVIEW_CONTENT, SPXMLViewContent))
-#define SP_IS_XMLVIEW_CONTENT(o) (GTK_CHECK_TYPE ((o), SP_TYPE_XMLVIEW_CONTENT))
-#define SP_XMLVIEW_CONTENT_CLASS(k) (GTK_CHECK_CLASS_TYPE ((k), SP_TYPE_XMLVIEW_CONTENT))
+#define SP_XMLVIEW_CONTENT(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), SP_TYPE_XMLVIEW_CONTENT, SPXMLViewContent))
+#define SP_IS_XMLVIEW_CONTENT(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), SP_TYPE_XMLVIEW_CONTENT))
+#define SP_XMLVIEW_CONTENT_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), SP_TYPE_XMLVIEW_CONTENT))
 
 struct SPXMLViewContent
 {
@@ -41,7 +36,7 @@ struct SPXMLViewContentClass
 	GtkTextViewClass parent_class;
 };
 
-GtkType sp_xmlview_content_get_type (void);
+GType sp_xmlview_content_get_type (void);
 GtkWidget * sp_xmlview_content_new (Inkscape::XML::Node * repr);
 
 #define SP_XMLVIEW_CONTENT_GET_REPR(text) (SP_XMLVIEW_CONTENT (text)->repr)
