@@ -1,6 +1,9 @@
 #ifndef SEEN_XML_QUOTE_H
 #define SEEN_XML_QUOTE_H
 
+#include <stddef.h>
+
+size_t xml_quoted_strlen(char const *val);
 char *xml_quote_strdup(char const *src);
 
 
