@@ -1,5 +1,3 @@
-#define __SP_REPR_IO_C__
-
 /*
  * Dirty DOM-like  tree
  *
@@ -20,10 +18,13 @@
 #include <string>
 #include <stdexcept>
 
+#include <libxml/parser.h>
+
 #include "xml/repr.h"
 #include "xml/attribute-record.h"
 #include "xml/rebase-hrefs.h"
 #include "xml/simple-document.h"
+#include "xml/text-node.h"
 
 #include "io/sys.h"
 #include "io/uristream.h"
@@ -32,8 +33,12 @@
 
 #include "extension/extension.h"
 
+#include "attribute-rel-util.h"
+
 #include "preferences.h"
 
+#include <glibmm/miscutils.h>
+
 using Inkscape::IO::Writer;
 using Inkscape::Util::List;
 using Inkscape::Util::cons;
@@ -52,6 +57,7 @@ static void sp_repr_write_stream_root_element(Node *repr, Writer &out,
                                               int inlineattrs, int indent,
                                               gchar const *old_href_abs_base,
                                               gchar const *new_href_abs_base);
+
 static void sp_repr_write_stream_element(Node *repr, Writer &out,
                                          gint indent_level, bool add_whitespace,
                                          Glib::QueryQuark elide_prefix,
@@ -72,12 +78,19 @@ public:
     XmlSource()
         : filename(0),
           encoding(0),
-          fp(0),
+          fp(NULL),
           firstFewLen(0),
+          LoadEntities(false),
+          cachedData(),
+          cachedPos(0),
           dummy("x"),
-          instr(0),
-          gzin(0)
+          instr(NULL),
+          gzin(NULL)
     {
+        for (int k=0;k<4;k++)
+        {
+            firstFew[k]=0;
+        }
     }
     virtual ~XmlSource()
     {
@@ -169,11 +182,10 @@ int XmlSource::setFile(char const *filename, bool load_entities=false)
             retVal = 0; // no error
         }
     }
-
     if(load_entities) {
         this->cachedData = std::string("");
         this->cachedPos = 0;
- 
+
         // First get data from file in typical way (cache it all)
         char *buffer = new char [4096];
         while(true) {
@@ -182,18 +194,18 @@ int XmlSource::setFile(char const *filename, bool load_entities=false)
             buffer[len] = 0;
             this->cachedData += buffer;
         }
-        free(buffer);
- 
+        delete[] buffer;
+
         // Check for SYSTEM or PUBLIC entities and remove them from the cache
         GMatchInfo *info;
         gint start, end;
- 
+
         GRegex *regex = g_regex_new(
             "<!ENTITY\\s+[^>\\s]+\\s+(SYSTEM|PUBLIC\\s+\"[^>\"]+\")\\s+\"[^>\"]+\"\\s*>",
             G_REGEX_CASELESS, G_REGEX_MATCH_NEWLINE_ANY, NULL);
- 
+
         g_regex_match (regex, this->cachedData.c_str(), G_REGEX_MATCH_NEWLINE_ANY, &info);
- 
+
         while (g_match_info_matches (info)) {
             if (g_match_info_fetch_pos (info, 1, &start, &end))
                 this->cachedData.erase(start, end - start);
@@ -210,13 +222,13 @@ int XmlSource::setFile(char const *filename, bool load_entities=false)
 xmlDocPtr XmlSource::readXml()
 {
     int parse_options = XML_PARSE_HUGE | XML_PARSE_RECOVER;
- 
+
     Inkscape::Preferences *prefs = Inkscape::Preferences::get();
     bool allowNetAccess = prefs->getBool("/options/externalresources/xml/allow_net_access", false);
     if (!allowNetAccess) parse_options |= XML_PARSE_NONET;
 
     // Allow NOENT only if we're filtering out SYSTEM and PUBLIC entities
-    if (LoadEntities)    parse_options |= XML_PARSE_NOENT;
+    if (LoadEntities)     parse_options |= XML_PARSE_NOENT;
 
     return xmlReadIO( readCb, closeCb, this,
                       filename, getEncoding(), parse_options);
@@ -230,7 +242,6 @@ int XmlSource::readCb( void * context, char * buffer, int len )
         XmlSource* self = static_cast<XmlSource*>(context);
         retVal = self->read( buffer, len );
     }
-
     return retVal;
 }
 
@@ -314,16 +325,19 @@ int XmlSource::close()
  * Reads XML from a file, including WMF files, and returns the Document.
  * The default namespace can also be specified, if desired.
  */
-Document *
-sp_repr_read_file (const gchar * filename, const gchar *default_ns)
+Document *sp_repr_read_file (const gchar * filename, const gchar *default_ns)
 {
+    // g_warning( "Reading file: %s", filename );
     xmlDocPtr doc = 0;
     Document * rdoc = 0;
 
     xmlSubstituteEntitiesDefault(1);
 
     g_return_val_if_fail (filename != NULL, NULL);
-    g_return_val_if_fail (Inkscape::IO::file_test( filename, G_FILE_TEST_EXISTS ), NULL);
+    if (!Inkscape::IO::file_test( filename, G_FILE_TEST_EXISTS )) {
+        g_warning("Can't open file: %s (doesn't exist)", filename);
+        return NULL;
+    }
     /* fixme: A file can disappear at any time, including between now and when we actually try to
      * open it.  Get rid of the above test once we're sure that we correctly handle
      * non-existence. */
@@ -365,6 +379,7 @@ sp_repr_read_file (const gchar * filename, const gchar *default_ns)
         }
     }
 
+
     if ( doc ) {
         xmlFreeDoc( doc );
     }
@@ -379,8 +394,7 @@ sp_repr_read_file (const gchar * filename, const gchar *default_ns)
 /**
  * Reads and parses XML from a buffer, returning it as an Document
  */
-Document *
-sp_repr_read_mem (const gchar * buffer, gint length, const gchar *default_ns)
+Document *sp_repr_read_mem (const gchar * buffer, gint length, const gchar *default_ns)
 {
     xmlDocPtr doc;
     Document * rdoc;
@@ -401,8 +415,7 @@ sp_repr_read_mem (const gchar * buffer, gint length, const gchar *default_ns)
 /**
  * Reads and parses XML from a buffer, returning it as an Document
  */
-Document *
-sp_repr_read_buf (const Glib::ustring &buf, const gchar *default_ns)
+Document *sp_repr_read_buf (const Glib::ustring &buf, const gchar *default_ns)
 {
     return sp_repr_read_mem(buf.c_str(), buf.size(), default_ns);
 }
@@ -452,7 +465,7 @@ void promote_to_namespace(Node *repr, const gchar *prefix) {
             repr->setCodeUnsafe(g_quark_from_string(svg_name));
             g_free(svg_name);
         }
-        for ( Node *child = sp_repr_children(repr) ; child ; child = sp_repr_next(child) ) {
+        for ( Node *child = repr->firstChild() ; child ; child = child->next() ) {
             promote_to_namespace(child, prefix);
         }
     }
@@ -463,8 +476,7 @@ void promote_to_namespace(Node *repr, const gchar *prefix) {
 /**
  * Reads in a XML file to create a Document
  */
-Document *
-sp_repr_do_read (xmlDocPtr doc, const gchar *default_ns)
+Document *sp_repr_do_read (xmlDocPtr doc, const gchar *default_ns)
 {
     if (doc == NULL) {
         return NULL;
@@ -510,6 +522,18 @@ sp_repr_do_read (xmlDocPtr doc, const gchar *default_ns)
                 promote_to_namespace(root, INKSCAPE_EXTENSION_NS_NC);
             }
         }
+
+
+        // Clean unnecessary attributes and style properties from SVG documents. (Controlled by
+        // preferences.)  Note: internal Inkscape svg files will also be cleaned (filters.svg,
+        // icons.svg). How can one tell if a file is internal?
+        if ( !strcmp(root->name(), "svg:svg" ) ) {
+            Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+            bool clean = prefs->getBool("/options/svgoutput/check_on_reading");
+            if( clean ) {
+                sp_attribute_clean_tree( root );
+            }
+        }
     }
 
     g_hash_table_destroy (prefix_map);
@@ -517,17 +541,22 @@ sp_repr_do_read (xmlDocPtr doc, const gchar *default_ns)
     return rdoc;
 }
 
-gint
-sp_repr_qualified_name (gchar *p, gint len, xmlNsPtr ns, const xmlChar *name, const gchar */*default_ns*/, GHashTable *prefix_map)
+gint sp_repr_qualified_name (gchar *p, gint len, xmlNsPtr ns, const xmlChar *name, const gchar */*default_ns*/, GHashTable *prefix_map)
 {
     const xmlChar *prefix;
-    if ( ns && ns->href ) {
-        prefix = reinterpret_cast<const xmlChar*>( sp_xml_ns_uri_prefix(reinterpret_cast<const gchar*>(ns->href),
-                                                                        reinterpret_cast<const char*>(ns->prefix)) );
-        void* p0 = reinterpret_cast<gpointer>(const_cast<xmlChar *>(prefix));
-        void* p1 = reinterpret_cast<gpointer>(const_cast<xmlChar *>(ns->href));
-        g_hash_table_insert( prefix_map, p0, p1 );
-    } else {
+    if (ns){
+        if (ns->href ) {
+            prefix = reinterpret_cast<const xmlChar*>( sp_xml_ns_uri_prefix(reinterpret_cast<const gchar*>(ns->href),
+                                                                            reinterpret_cast<const char*>(ns->prefix)) );
+            void* p0 = reinterpret_cast<gpointer>(const_cast<xmlChar *>(prefix));
+            void* p1 = reinterpret_cast<gpointer>(const_cast<xmlChar *>(ns->href));
+            g_hash_table_insert( prefix_map, p0, p1 );
+        }
+        else {
+            prefix = NULL;
+        }
+    }
+    else {
         prefix = NULL;
     }
 
@@ -538,10 +567,8 @@ sp_repr_qualified_name (gchar *p, gint len, xmlNsPtr ns, const xmlChar *name, co
     }
 }
 
-static Node *
-sp_repr_svg_read_node (Document *xml_doc, xmlNodePtr node, const gchar *default_ns, GHashTable *prefix_map)
+static Node *sp_repr_svg_read_node (Document *xml_doc, xmlNodePtr node, const gchar *default_ns, GHashTable *prefix_map)
 {
-    Node *repr, *crepr;
     xmlAttrPtr prop;
     xmlNodePtr child;
     gchar c[256];
@@ -562,7 +589,9 @@ sp_repr_svg_read_node (Document *xml_doc, xmlNodePtr node, const gchar *default_
             return NULL; // we do not preserve all-whitespace nodes unless we are asked to
         }
 
-        return xml_doc->createTextNode(reinterpret_cast<gchar *>(node->content));
+        // We keep track of original node type so that CDATA sections are preserved on output.
+        return xml_doc->createTextNode(reinterpret_cast<gchar *>(node->content),
+                                       node->type == XML_CDATA_SECTION_NODE );
     }
 
     if (node->type == XML_COMMENT_NODE) {
@@ -579,7 +608,7 @@ sp_repr_svg_read_node (Document *xml_doc, xmlNodePtr node, const gchar *default_
     }
 
     sp_repr_qualified_name (c, 256, node->ns, node->name, default_ns, prefix_map);
-    repr = xml_doc->createElement(c);
+    Node *repr = xml_doc->createElement(c);
     /* TODO remember node->ns->prefix if node->ns != NULL */
 
     for (prop = node->properties; prop != NULL; prop = prop->next) {
@@ -594,9 +623,8 @@ sp_repr_svg_read_node (Document *xml_doc, xmlNodePtr node, const gchar *default_
         repr->setContent(reinterpret_cast<gchar*>(node->content));
     }
 
-    child = node->xmlChildrenNode;
     for (child = node->xmlChildrenNode; child != NULL; child = child->next) {
-        crepr = sp_repr_svg_read_node (xml_doc, child, default_ns, prefix_map);
+        Node *crepr = sp_repr_svg_read_node (xml_doc, child, default_ns, prefix_map);
         if (crepr) {
             repr->appendChild(crepr);
             Inkscape::GC::release(crepr);
@@ -607,8 +635,7 @@ sp_repr_svg_read_node (Document *xml_doc, xmlNodePtr node, const gchar *default_
 }
 
 
-static void
-sp_repr_save_writer(Document *doc, Inkscape::IO::Writer *out,
+static void sp_repr_save_writer(Document *doc, Inkscape::IO::Writer *out,
                     gchar const *default_ns,
                     gchar const *old_href_abs_base,
                     gchar const *new_href_abs_base)
@@ -626,7 +653,7 @@ sp_repr_save_writer(Document *doc, Inkscape::IO::Writer *out,
     }
 
     for (Node *repr = sp_repr_document_first_child(doc);
-         repr; repr = sp_repr_next(repr))
+         repr; repr = repr->next())
     {
         Inkscape::XML::NodeType const node_type = repr->type();
         if ( node_type == Inkscape::XML::ELEMENT_NODE ) {
@@ -643,28 +670,21 @@ sp_repr_save_writer(Document *doc, Inkscape::IO::Writer *out,
 }
 
 
-
-
-Glib::ustring
-sp_repr_save_buf(Document *doc)
+Glib::ustring sp_repr_save_buf(Document *doc)
 {   
     Inkscape::IO::StringOutputStream souts;
     Inkscape::IO::OutputStreamWriter outs(souts);
 
     sp_repr_save_writer(doc, &outs, SP_INKSCAPE_NS_URI, 0, 0);
 
-	outs.close();
-	Glib::ustring buf = souts.getString();
+    outs.close();
+    Glib::ustring buf = souts.getString();
 
-	return buf;
+    return buf;
 }
 
 
-
-
-
-void
-sp_repr_save_stream(Document *doc, FILE *fp, gchar const *default_ns, bool compress,
+void sp_repr_save_stream(Document *doc, FILE *fp, gchar const *default_ns, bool compress,
                     gchar const *const old_href_abs_base,
                     gchar const *const new_href_abs_base)
 {
@@ -682,15 +702,14 @@ sp_repr_save_stream(Document *doc, FILE *fp, gchar const *default_ns, bool compr
 
 
 /**
- * Returns true iff file successfully saved.
+ * Returns true if file successfully saved.
  *
  * \param filename The actual file to do I/O to, which might be a temp file.
  *
  * \param for_filename The base URI [actually filename] to assume for purposes of rewriting
  *              xlink:href attributes.
  */
-bool
-sp_repr_save_rebased_file(Document *doc, gchar const *const filename, gchar const *default_ns,
+bool sp_repr_save_rebased_file(Document *doc, gchar const *const filename, gchar const *default_ns,
                           gchar const *old_base, gchar const *for_filename)
 {
     if (!filename) {
@@ -710,28 +729,23 @@ sp_repr_save_rebased_file(Document *doc, gchar const *const filename, gchar cons
         return false;
     }
 
-    gchar *old_href_abs_base = NULL;
-    gchar *new_href_abs_base = NULL;
+    Glib::ustring old_href_abs_base;
+    Glib::ustring new_href_abs_base;
     if (for_filename) {
         old_href_abs_base = calc_abs_doc_base(old_base);
-        if (g_path_is_absolute(for_filename)) {
-            new_href_abs_base = g_path_get_dirname(for_filename);
+        if (Glib::path_is_absolute(for_filename)) {
+            new_href_abs_base = Glib::path_get_dirname(for_filename);
         } else {
-            gchar *const cwd = g_get_current_dir();
-            gchar *const for_abs_filename = g_build_filename(cwd, for_filename, NULL);
-            g_free(cwd);
-            new_href_abs_base = g_path_get_dirname(for_abs_filename);
-            g_free(for_abs_filename);
+            Glib::ustring const cwd = Glib::get_current_dir();
+            Glib::ustring const for_abs_filename = Glib::build_filename(cwd, for_filename);
+            new_href_abs_base = Glib::path_get_dirname(for_abs_filename);
         }
 
         /* effic: Once we're confident that we never need (or never want) to resort
          * to using sodipodi:absref instead of the xlink:href value,
          * then we should do `if streq() { free them and set both to NULL; }'. */
     }
-    sp_repr_save_stream(doc, file, default_ns, compress, old_href_abs_base, new_href_abs_base);
-
-    g_free(old_href_abs_base);
-    g_free(new_href_abs_base);
+    sp_repr_save_stream(doc, file, default_ns, compress, old_href_abs_base.c_str(), new_href_abs_base.c_str());
 
     if (fclose (file) != 0) {
         return false;
@@ -743,16 +757,14 @@ sp_repr_save_rebased_file(Document *doc, gchar const *const filename, gchar cons
 /**
  * Returns true iff file successfully saved.
  */
-bool
-sp_repr_save_file(Document *doc, gchar const *const filename, gchar const *default_ns)
+bool sp_repr_save_file(Document *doc, gchar const *const filename, gchar const *default_ns)
 {
     return sp_repr_save_rebased_file(doc, filename, default_ns, NULL, NULL);
 }
 
 
 /* (No doubt this function already exists elsewhere.) */
-static void
-repr_quote_write (Writer &out, const gchar * val)
+static void repr_quote_write (Writer &out, const gchar * val)
 {
     if (val) {
         for (; *val != '\0'; val++) {
@@ -849,8 +861,8 @@ void populate_ns_map(NSMap &ns_map, Node &repr) {
                 add_ns_map_entry(ns_map, prefix);
             }
         }
-        for ( Node *child=sp_repr_children(&repr) ;
-              child ; child = sp_repr_next(child) )
+        for ( Node *child=repr.firstChild() ;
+              child ; child = child->next() )
         {
             populate_ns_map(ns_map, *child);
         }
@@ -859,8 +871,7 @@ void populate_ns_map(NSMap &ns_map, Node &repr) {
 
 }
 
-static void
-sp_repr_write_stream_root_element(Node *repr, Writer &out,
+static void sp_repr_write_stream_root_element(Node *repr, Writer &out,
                                   bool add_whitespace, gchar const *default_ns,
                                   int inlineattrs, int indent,
                                   gchar const *const old_href_base,
@@ -869,6 +880,12 @@ sp_repr_write_stream_root_element(Node *repr, Writer &out,
     using Inkscape::Util::ptr_shared;
 
     g_assert(repr != NULL);
+
+    // Clean unnecessary attributes and stype properties. (Controlled by preferences.)
+    Inkscape::Preferences *prefs = Inkscape::Preferences::get();
+    bool clean = prefs->getBool("/options/svgoutput/check_on_writing");
+    if (clean) sp_attribute_clean_tree( repr );
+
     Glib::QueryQuark xml_prefix=g_quark_from_static_string("xml");
 
     NSMap ns_map;
@@ -915,7 +932,12 @@ void sp_repr_write_stream( Node *repr, Writer &out, gint indent_level,
 {
     switch (repr->type()) {
         case Inkscape::XML::TEXT_NODE: {
-            repr_quote_write( out, repr->content() );
+            if( dynamic_cast<const Inkscape::XML::TextNode *>(repr)->is_CData() ) {
+                // Preserve CDATA sections, not converting '&' to &amp;, etc.
+                out.printf( "<![CDATA[%s]]>", repr->content() );
+            } else {
+                repr_quote_write( out, repr->content() );
+            }
             break;
         }
         case Inkscape::XML::COMMENT_NODE: {
@@ -945,17 +967,16 @@ void sp_repr_write_stream( Node *repr, Writer &out, gint indent_level,
 }
 
 
-static void
-sp_repr_write_stream_element (Node * repr, Writer & out, gint indent_level,
-                              bool add_whitespace,
-                              Glib::QueryQuark elide_prefix,
-                              List<AttributeRecord const> attributes, 
-                              int inlineattrs, int indent,
-                              gchar const *const old_href_base,
-                              gchar const *const new_href_base)
+void sp_repr_write_stream_element( Node * repr, Writer & out,
+                                   gint indent_level, bool add_whitespace,
+                                   Glib::QueryQuark elide_prefix,
+                                   List<AttributeRecord const> attributes, 
+                                   int inlineattrs, int indent,
+                                   gchar const *old_href_base,
+                                   gchar const *new_href_base )
 {
-    Node *child;
-    bool loose;
+    Node *child = 0;
+    bool loose = false;
 
     g_return_if_fail (repr != NULL);
 
@@ -987,6 +1008,28 @@ sp_repr_write_stream_element (Node * repr, Writer & out, gint indent_level,
         add_whitespace = false;
     }
 
+    // THIS DOESN'T APPEAR TO DO ANYTHING. Can it be commented out or deleted?
+    {
+        GQuark const href_key = g_quark_from_static_string("xlink:href");
+        //GQuark const absref_key = g_quark_from_static_string("sodipodi:absref");
+
+        gchar const *xxHref = 0;
+        //gchar const *xxAbsref = 0;
+        for ( List<AttributeRecord const> ai(attributes); ai; ++ai ) {
+            if ( ai->key == href_key ) {
+                xxHref = ai->value;
+            //} else if ( ai->key == absref_key ) {
+                //xxAbsref = ai->value;
+            }
+        }
+
+        // Might add a special case for absref but no href.
+        if ( old_href_base && new_href_base && xxHref ) {
+            //g_message("href rebase test with [%s] and [%s]", xxHref, xxAbsref);
+            //std::string newOne = rebase_href_attrs( old_href_base, new_href_base, xxHref, xxAbsref );
+        }
+    }
+
     for ( List<AttributeRecord const> iter = rebase_href_attrs(old_href_base, new_href_base,
                                                                attributes);
           iter ; ++iter )
@@ -1054,4 +1097,4 @@ sp_repr_write_stream_element (Node * repr, Writer & out, gint indent_level,
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
