@@ -1,12 +1,12 @@
-#define __SP_REPR_UTIL_C__
-
-/** \file
+/**
+ * @file
  * Miscellaneous helpers for reprs.
  */
 
 /*
  * Authors:
  *   Lauris Kaplinski <lauris@ximian.com>
+ *   Jon A. Cruz <jon@joncruz.org>
  *
  * Copyright (C) 1999-2000 Lauris Kaplinski
  * Copyright (C) 2000-2001 Ximian, Inc.
@@ -68,74 +68,6 @@ struct SPXMLNs {
 static void sp_xml_ns_register_defaults();
 static char *sp_xml_ns_auto_prefix(char const *uri);
 
-/*#####################
-# UTILITY
-#####################*/
-
-/**
- * Locale-independent double to string conversion
- */
-unsigned int
-sp_xml_dtoa(gchar *buf, double val, unsigned int tprec, unsigned int fprec, unsigned int padf)
-{
-    double dival, fval, epsilon;
-    int idigits, ival, i;
-    i = 0;
-    if (val < 0.0) {
-        buf[i++] = '-';
-        val = -val;
-    }
-    /* Determine number of integral digits */
-    if (val >= 1.0) {
-        idigits = (int) floor(log10(val));
-    } else {
-        idigits = 0;
-    }
-    /* Determine the actual number of fractional digits */
-    fprec = MAX(fprec, tprec - idigits);
-    /* Find epsilon */
-    epsilon = 0.5 * pow(10.0, - (double) fprec);
-    /* Round value */
-    val += epsilon;
-    /* Extract integral and fractional parts */
-    dival = floor(val);
-    ival = (int) dival;
-    fval = val - dival;
-    /* Write integra */
-    if (ival > 0) {
-        char c[32];
-        int j;
-        j = 0;
-        while (ival > 0) {
-            c[32 - (++j)] = '0' + (ival % 10);
-            ival /= 10;
-        }
-        memcpy(buf + i, &c[32 - j], j);
-        i += j;
-        tprec -= j;
-    } else {
-        buf[i++] = '0';
-        tprec -= 1;
-    }
-    if ((fprec > 0) && (padf || (fval > epsilon))) {
-        buf[i++] = '.';
-        while ((fprec > 0) && (padf || (fval > epsilon))) {
-            fval *= 10.0;
-            dival = floor(fval);
-            fval -= dival;
-            buf[i++] = '0' + (int) dival;
-            fprec -= 1;
-        }
-
-    }
-    buf[i] = 0;
-    return i;
-}
-
-
-
-
-
 /*#####################
 # MAIN
 #####################*/
@@ -150,8 +82,7 @@ static SPXMLNs *namespaces=NULL;
  * There are the prefixes to use for the XML namespaces defined
  * in repr.h
  */
-static void
-sp_xml_ns_register_defaults()
+static void sp_xml_ns_register_defaults()
 {
     static SPXMLNs defaults[11];
 
@@ -213,8 +144,7 @@ sp_xml_ns_register_defaults()
     namespaces = &defaults[0];
 }
 
-char *
-sp_xml_ns_auto_prefix(char const *uri)
+char *sp_xml_ns_auto_prefix(char const *uri)
 {
     char const *start, *end;
     char *new_prefix;
@@ -240,8 +170,7 @@ sp_xml_ns_auto_prefix(char const *uri)
     return new_prefix;
 }
 
-gchar const *
-sp_xml_ns_uri_prefix(gchar const *uri, gchar const *suggested)
+gchar const *sp_xml_ns_uri_prefix(gchar const *uri, gchar const *suggested)
 {
     char const *prefix;
 
@@ -267,8 +196,13 @@ sp_xml_ns_uri_prefix(gchar const *uri, gchar const *suggested)
             GQuark const prefix_key=g_quark_from_string(suggested);
 
             SPXMLNs *found=namespaces;
-            while ( found && found->prefix != prefix_key ) {
-                found = found->next;
+            while (found) {
+                if (found->prefix != prefix_key) {
+                    found = found->next;
+                }
+                else {
+                    break;
+                }
             }
 
             if (found) { // prefix already used?
@@ -296,8 +230,7 @@ sp_xml_ns_uri_prefix(gchar const *uri, gchar const *suggested)
     return prefix;
 }
 
-gchar const *
-sp_xml_ns_prefix_uri(gchar const *prefix)
+gchar const *sp_xml_ns_prefix_uri(gchar const *prefix)
 {
     SPXMLNs *iter;
     char const *uri;
@@ -319,34 +252,6 @@ sp_xml_ns_prefix_uri(gchar const *prefix)
     return uri;
 }
 
-double sp_repr_get_double_attribute(Inkscape::XML::Node *repr, char const *key, double def)
-{
-    char *result;
-
-    g_return_val_if_fail(repr != NULL, def);
-    g_return_val_if_fail(key != NULL, def);
-
-    result = (char *) repr->attribute(key);
-
-    if (result == NULL) return def;
-
-    return g_ascii_strtod(result, NULL);
-}
-
-long long int sp_repr_get_int_attribute(Inkscape::XML::Node *repr, char const *key, long long int def)
-{
-    char *result;
-
-    g_return_val_if_fail(repr != NULL, def);
-    g_return_val_if_fail(key != NULL, def);
-
-    result = (char *) repr->attribute(key);
-
-    if (result == NULL) return def;
-
-    return atoll(result);
-}
-
 /** 
  *  Works for different-parent objects, so long as they have a common ancestor. Return value:
  *    0    positions are equivalent
@@ -354,11 +259,10 @@ long long int sp_repr_get_int_attribute(Inkscape::XML::Node *repr, char const *k
  *   -1    first object's position is less than the second
  * @todo Rewrite this function's description to be understandable
  */
-int
-sp_repr_compare_position(Inkscape::XML::Node *first, Inkscape::XML::Node *second)
+int sp_repr_compare_position(Inkscape::XML::Node const *first, Inkscape::XML::Node const *second)
 {
     int p1, p2;
-    if (sp_repr_parent(first) == sp_repr_parent(second)) {
+    if (first->parent() == second->parent()) {
         /* Basic case - first and second have same parent */
         p1 = first->position();
         p2 = second->position();
@@ -368,7 +272,7 @@ sp_repr_compare_position(Inkscape::XML::Node *first, Inkscape::XML::Node *second
            instance. */
 
         // Find the lowest common ancestor(LCA)
-        Inkscape::XML::Node *ancestor = LCA(first, second);
+        Inkscape::XML::Node const *ancestor = LCA(first, second);
         g_assert(ancestor != NULL);
 
         if (ancestor == first) {
@@ -378,7 +282,7 @@ sp_repr_compare_position(Inkscape::XML::Node *first, Inkscape::XML::Node *second
         } else {
             Inkscape::XML::Node const *to_first = AncetreFils(first, ancestor);
             Inkscape::XML::Node const *to_second = AncetreFils(second, ancestor);
-            g_assert(sp_repr_parent(to_second) == sp_repr_parent(to_first));
+            g_assert(to_second->parent() == to_first->parent());
             p1 = to_first->position();
             p2 = to_second->position();
         }
@@ -406,7 +310,7 @@ sp_repr_compare_position(Inkscape::XML::Node *first, Inkscape::XML::Node *second
 }
 
 /**
- * @brief Find an element node using an unique attribute
+ * Find an element node using an unique attribute.
  *
  * This function returns the first child of the specified node that has the attribute
  * @c key equal to @c value. Note that this function does not recurse.
@@ -416,10 +320,9 @@ sp_repr_compare_position(Inkscape::XML::Node *first, Inkscape::XML::Node *second
  * @param value The value of the attribute to look for
  * @relatesalso Inkscape::XML::Node
  */
-Inkscape::XML::Node *
-sp_repr_lookup_child(Inkscape::XML::Node *repr,
-                     gchar const *key,
-                     gchar const *value)
+Inkscape::XML::Node *sp_repr_lookup_child(Inkscape::XML::Node *repr,
+                                          gchar const *key,
+                                          gchar const *value)
 {
     g_return_val_if_fail(repr != NULL, NULL);
     for ( Inkscape::XML::Node *child = repr->firstChild() ; child ; child = child->next() ) {
@@ -433,45 +336,39 @@ sp_repr_lookup_child(Inkscape::XML::Node *repr,
     return NULL;
 }
 
-/**
- * @brief Find an element node with the given name
- *
- * This function searches the descendants of the specified node depth-first for
- * the first XML node with the specified name.
- *
- * @param repr The node to start from
- * @param name The name of the element node to find
- * @param maxdepth Maximum search depth, or -1 for an unlimited depth
- * @return  A pointer to the matching Inkscape::XML::Node
- * @relatesalso Inkscape::XML::Node
- */
-Inkscape::XML::Node *
-sp_repr_lookup_name( Inkscape::XML::Node *repr, gchar const *name, gint maxdepth )
+Inkscape::XML::Node const *sp_repr_lookup_name( Inkscape::XML::Node const *repr, gchar const *name, gint maxdepth )
 {
+    Inkscape::XML::Node const *found = 0;
     g_return_val_if_fail(repr != NULL, NULL);
     g_return_val_if_fail(name != NULL, NULL);
 
     GQuark const quark = g_quark_from_string(name);
 
-    if ( (GQuark)repr->code() == quark ) return repr;
-    if ( maxdepth == 0 ) return NULL;
+    if ( (GQuark)repr->code() == quark ) {
+        found = repr;
+    } else if ( maxdepth != 0 ) {
+        // maxdepth == -1 means unlimited
+        if ( maxdepth == -1 ) {
+            maxdepth = 0;
+        }
 
-    // maxdepth == -1 means unlimited
-    if ( maxdepth == -1 ) maxdepth = 0;
-
-    Inkscape::XML::Node *found = NULL;
-    for (Inkscape::XML::Node *child = repr->firstChild() ; child && !found; child = child->next() ) {
-        found = sp_repr_lookup_name( child, name, maxdepth-1 );
+        for (Inkscape::XML::Node const *child = repr->firstChild() ; child && !found; child = child->next() ) {
+            found = sp_repr_lookup_name( child, name, maxdepth - 1 );
+        }
     }
-
     return found;
 }
 
+Inkscape::XML::Node *sp_repr_lookup_name( Inkscape::XML::Node *repr, gchar const *name, gint maxdepth )
+{
+    Inkscape::XML::Node const *found = sp_repr_lookup_name( const_cast<Inkscape::XML::Node const *>(repr), name, maxdepth );
+    return const_cast<Inkscape::XML::Node *>(found);
+}
+
 /**
  * Determine if the node is a 'title', 'desc' or 'metadata' element.
  */
-bool
-sp_repr_is_meta_element(const Inkscape::XML::Node *node)
+bool sp_repr_is_meta_element(const Inkscape::XML::Node *node)
 {
     if (node == NULL) return false;
     if (node->type() != Inkscape::XML::ELEMENT_NODE) return false;
@@ -489,8 +386,7 @@ sp_repr_is_meta_element(const Inkscape::XML::Node *node)
  *
  * \return TRUE if the attr was set, FALSE otherwise.
  */
-unsigned int
-sp_repr_get_boolean(Inkscape::XML::Node *repr, gchar const *key, unsigned int *val)
+unsigned int sp_repr_get_boolean(Inkscape::XML::Node *repr, gchar const *key, unsigned int *val)
 {
     gchar const *v;
 
@@ -516,8 +412,7 @@ sp_repr_get_boolean(Inkscape::XML::Node *repr, gchar const *key, unsigned int *v
     }
 }
 
-unsigned int
-sp_repr_get_int(Inkscape::XML::Node *repr, gchar const *key, int *val)
+unsigned int sp_repr_get_int(Inkscape::XML::Node *repr, gchar const *key, int *val)
 {
     gchar const *v;
 
@@ -535,16 +430,13 @@ sp_repr_get_int(Inkscape::XML::Node *repr, gchar const *key, int *val)
     return FALSE;
 }
 
-unsigned int
-sp_repr_get_double(Inkscape::XML::Node *repr, gchar const *key, double *val)
+unsigned int sp_repr_get_double(Inkscape::XML::Node *repr, gchar const *key, double *val)
 {
-    gchar const *v;
-
     g_return_val_if_fail(repr != NULL, FALSE);
     g_return_val_if_fail(key != NULL, FALSE);
     g_return_val_if_fail(val != NULL, FALSE);
 
-    v = repr->attribute(key);
+    gchar const *v = repr->attribute(key);
 
     if (v != NULL) {
         *val = g_ascii_strtod(v, NULL);
@@ -554,8 +446,7 @@ sp_repr_get_double(Inkscape::XML::Node *repr, gchar const *key, double *val)
     return FALSE;
 }
 
-unsigned int
-sp_repr_set_boolean(Inkscape::XML::Node *repr, gchar const *key, unsigned int val)
+unsigned int sp_repr_set_boolean(Inkscape::XML::Node *repr, gchar const *key, unsigned int val)
 {
     g_return_val_if_fail(repr != NULL, FALSE);
     g_return_val_if_fail(key != NULL, FALSE);
@@ -564,8 +455,7 @@ sp_repr_set_boolean(Inkscape::XML::Node *repr, gchar const *key, unsigned int va
     return true;
 }
 
-unsigned int
-sp_repr_set_int(Inkscape::XML::Node *repr, gchar const *key, int val)
+unsigned int sp_repr_set_int(Inkscape::XML::Node *repr, gchar const *key, int val)
 {
     gchar c[32];
 
@@ -583,8 +473,7 @@ sp_repr_set_int(Inkscape::XML::Node *repr, gchar const *key, int val)
  * required for CSS properties: in particular, it never uses exponent
  * notation.
  */
-unsigned int
-sp_repr_set_css_double(Inkscape::XML::Node *repr, gchar const *key, double val)
+unsigned int sp_repr_set_css_double(Inkscape::XML::Node *repr, gchar const *key, double val)
 {
     g_return_val_if_fail(repr != NULL, FALSE);
     g_return_val_if_fail(key != NULL, FALSE);
@@ -601,8 +490,7 @@ sp_repr_set_css_double(Inkscape::XML::Node *repr, gchar const *key, double val)
  *
  * Not suitable for property attributes (fill-opacity, font-size etc.).
  */
-unsigned int
-sp_repr_set_svg_double(Inkscape::XML::Node *repr, gchar const *key, double val)
+unsigned int sp_repr_set_svg_double(Inkscape::XML::Node *repr, gchar const *key, double val)
 {
     g_return_val_if_fail(repr != NULL, FALSE);
     g_return_val_if_fail(key != NULL, FALSE);
@@ -626,8 +514,7 @@ unsigned sp_repr_set_point(Inkscape::XML::Node *repr, gchar const *key, Geom::Po
     return true;
 }
 
-unsigned int
-sp_repr_get_point(Inkscape::XML::Node *repr, gchar const *key, Geom::Point *val)
+unsigned int sp_repr_get_point(Inkscape::XML::Node *repr, gchar const *key, Geom::Point *val)
 {
     g_return_val_if_fail(repr != NULL, FALSE);
     g_return_val_if_fail(key != NULL, FALSE);
@@ -635,6 +522,8 @@ sp_repr_get_point(Inkscape::XML::Node *repr, gchar const *key, Geom::Point *val)
 
     gchar const *v = repr->attribute(key);
 
+    g_return_val_if_fail(v != NULL, FALSE);
+
     gchar ** strarray = g_strsplit(v, ",", 2);
 
     if (strarray && strarray[0] && strarray[1]) {
@@ -659,4 +548,4 @@ sp_repr_get_point(Inkscape::XML::Node *repr, gchar const *key, Geom::Point *val)
   fill-column:99
   End:
 */
-// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=99 :
+// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:fileencoding=utf-8:textwidth=99 :
